/*****************************************************************************
 *
 * \file
 *
 * \brief FIR coefficients for the dsp16_resampling function from the DPSLib.
 *        These coefficients have the following caracterizations:
 *          - Input sampling frequency: 32000 Hz
 *          - Output sampling frequency: 44100 Hz
 *          - Re-sampling order: 20
 *          - Coefficients are normalized
 *          - Memory foot print: 17640 bytes
 *         It matches the following parameters:
 *         (dsp16_resampling_options_t::dsp16_custom_filter)
 *          - Cut-off frequency (fc_hz): 16000
 *          - Sampling frequency (fs_hz): 14112000
 *          - Actual FIR filter order (order): 8820
 *
 * Copyright (c) 2010 Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an
 *    Atmel microcontroller product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 ******************************************************************************/

A_ALIGNED static const dsp16_t dsp16_resampling_fircoef_ord20_32000_44100_norm[] = {
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00000141),
  DSP16_Q(-0.00000618),
  DSP16_Q(0.00001525),
  DSP16_Q(-0.00002982),
  DSP16_Q(0.00005178),
  DSP16_Q(-0.00008473),
  DSP16_Q(0.00013704),
  DSP16_Q(-0.00023424),
  DSP16_Q(0.00050553),
  DSP16_Q(0.45651822),
  DSP16_Q(-0.00050432),
  DSP16_Q(0.00023392),
  DSP16_Q(-0.00013689),
  DSP16_Q(0.00008464),
  DSP16_Q(-0.00005172),
  DSP16_Q(0.00002978),
  DSP16_Q(-0.00001522),
  DSP16_Q(0.00000617),
  DSP16_Q(-0.00000140),
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00000424),
  DSP16_Q(-0.00001859),
  DSP16_Q(0.00004582),
  DSP16_Q(-0.00008957),
  DSP16_Q(0.00015552),
  DSP16_Q(-0.00025446),
  DSP16_Q(0.00041158),
  DSP16_Q(-0.00070367),
  DSP16_Q(0.00152018),
  DSP16_Q(0.45651038),
  DSP16_Q(-0.00150936),
  DSP16_Q(0.00070079),
  DSP16_Q(-0.00041020),
  DSP16_Q(0.00025363),
  DSP16_Q(-0.00015498),
  DSP16_Q(0.00008920),
  DSP16_Q(-0.00004558),
  DSP16_Q(0.00001845),
  DSP16_Q(-0.00000418),
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00000711),
  DSP16_Q(-0.00003105),
  DSP16_Q(0.00007649),
  DSP16_Q(-0.00014948),
  DSP16_Q(0.00025949),
  DSP16_Q(-0.00042455),
  DSP16_Q(0.00068670),
  DSP16_Q(-0.00117434),
  DSP16_Q(0.00253960),
  DSP16_Q(0.45649471),
  DSP16_Q(-0.00250956),
  DSP16_Q(0.00116635),
  DSP16_Q(-0.00068287),
  DSP16_Q(0.00042225),
  DSP16_Q(-0.00025798),
  DSP16_Q(0.00014846),
  DSP16_Q(-0.00007583),
  DSP16_Q(0.00003067),
  DSP16_Q(-0.00000694),
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00001000),
  DSP16_Q(-0.00004358),
  DSP16_Q(0.00010726),
  DSP16_Q(-0.00020954),
  DSP16_Q(0.00036370),
  DSP16_Q(-0.00059498),
  DSP16_Q(0.00096241),
  DSP16_Q(-0.00164624),
  DSP16_Q(0.00356379),
  DSP16_Q(0.45647119),
  DSP16_Q(-0.00350490),
  DSP16_Q(0.00163059),
  DSP16_Q(-0.00095491),
  DSP16_Q(0.00059048),
  DSP16_Q(-0.00036074),
  DSP16_Q(0.00020756),
  DSP16_Q(-0.00010598),
  DSP16_Q(0.00004283),
  DSP16_Q(-0.00000967),
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00001291),
  DSP16_Q(-0.00005617),
  DSP16_Q(0.00013813),
  DSP16_Q(-0.00026976),
  DSP16_Q(0.00046812),
  DSP16_Q(-0.00076575),
  DSP16_Q(0.00123869),
  DSP16_Q(-0.00211936),
  DSP16_Q(0.00459272),
  DSP16_Q(0.45643984),
  DSP16_Q(-0.00449537),
  DSP16_Q(0.00209347),
  DSP16_Q(-0.00122628),
  DSP16_Q(0.00075831),
  DSP16_Q(-0.00046323),
  DSP16_Q(0.00026648),
  DSP16_Q(-0.00013601),
  DSP16_Q(0.00005493),
  DSP16_Q(-0.00001237),
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00001585),
  DSP16_Q(-0.00006881),
  DSP16_Q(0.00016911),
  DSP16_Q(-0.00033012),
  DSP16_Q(0.00057277),
  DSP16_Q(-0.00093686),
  DSP16_Q(0.00151553),
  DSP16_Q(-0.00259366),
  DSP16_Q(0.00562636),
  DSP16_Q(0.45640065),
  DSP16_Q(-0.00548094),
  DSP16_Q(0.00255499),
  DSP16_Q(-0.00149699),
  DSP16_Q(0.00092574),
  DSP16_Q(-0.00056547),
  DSP16_Q(0.00032522),
  DSP16_Q(-0.00016593),
  DSP16_Q(0.00006696),
  DSP16_Q(-0.00001504),
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00001882),
  DSP16_Q(-0.00008152),
  DSP16_Q(0.00020018),
  DSP16_Q(-0.00039063),
  DSP16_Q(0.00067763),
  DSP16_Q(-0.00110828),
  DSP16_Q(0.00179290),
  DSP16_Q(-0.00306913),
  DSP16_Q(0.00666471),
  DSP16_Q(0.45635363),
  DSP16_Q(-0.00646161),
  DSP16_Q(0.00301513),
  DSP16_Q(-0.00176701),
  DSP16_Q(0.00109276),
  DSP16_Q(-0.00066743),
  DSP16_Q(0.00038379),
  DSP16_Q(-0.00019574),
  DSP16_Q(0.00007893),
  DSP16_Q(-0.00001769),
  DSP16_Q(-0.00000000),
  DSP16_Q(0.00002182),
  DSP16_Q(-0.00009428),
  DSP16_Q(0.00023134),
  DSP16_Q(-0.00045129),
  DSP16_Q(0.00078269),
  DSP16_Q(-0.00128002),
  DSP16_Q(0.00207081),
  DSP16_Q(-0.00354574),
  DSP16_Q(0.00770773),
  DSP16_Q(0.45629878),
  DSP16_Q(-0.00743734),
  DSP16_Q(0.00347386),
  DSP16_Q(-0.00203634),
  DSP16_Q(0.00125937),
  DSP16_Q(-0.00076912),
  DSP16_Q(0.00044218),
  DSP16_Q(-0.00022544),
  DSP16_Q(0.00009084),
  DSP16_Q(-0.00002031),
  DSP16_Q(-0.00000001),
  DSP16_Q(0.00002484),
  DSP16_Q(-0.00010710),
  DSP16_Q(0.00026260),
  DSP16_Q(-0.00051208),
  DSP16_Q(0.00088797),
  DSP16_Q(-0.00145207),
  DSP16_Q(0.00234923),
  DSP16_Q(-0.00402349),
  DSP16_Q(0.00875541),
  DSP16_Q(0.45623609),
  DSP16_Q(-0.00840813),
  DSP16_Q(0.00393117),
  DSP16_Q(-0.00230496),
  DSP16_Q(0.00142554),
  DSP16_Q(-0.00087054),
  DSP16_Q(0.00050039),
  DSP16_Q(-0.00025502),
  DSP16_Q(0.00010268),
  DSP16_Q(-0.00002291),
  DSP16_Q(-0.00000001),
  DSP16_Q(0.00002789),
  DSP16_Q(-0.00011997),
  DSP16_Q(0.00029395),
  DSP16_Q(-0.00057301),
  DSP16_Q(0.00099344),
  DSP16_Q(-0.00162441),
  DSP16_Q(0.00262815),
  DSP16_Q(-0.00450234),
  DSP16_Q(0.00980772),
  DSP16_Q(0.45616558),
  DSP16_Q(-0.00937396),
  DSP16_Q(0.00438704),
  DSP16_Q(-0.00257287),
  DSP16_Q(0.00159128),
  DSP16_Q(-0.00097167),
  DSP16_Q(0.00055841),
  DSP16_Q(-0.00028449),
  DSP16_Q(0.00011445),
  DSP16_Q(-0.00002548),
  DSP16_Q(-0.00000001),
  DSP16_Q(0.00003096),
  DSP16_Q(-0.00013290),
  DSP16_Q(0.00032540),
  DSP16_Q(-0.00063408),
  DSP16_Q(0.00109910),
  DSP16_Q(-0.00179705),
  DSP16_Q(0.00290757),
  DSP16_Q(-0.00498227),
  DSP16_Q(0.01086466),
  DSP16_Q(0.45608723),
  DSP16_Q(-0.01033480),
  DSP16_Q(0.00484145),
  DSP16_Q(-0.00284004),
  DSP16_Q(0.00175658),
  DSP16_Q(-0.00107251),
  DSP16_Q(0.00061624),
  DSP16_Q(-0.00031384),
  DSP16_Q(0.00012617),
  DSP16_Q(-0.00002802),
  DSP16_Q(-0.00000002),
  DSP16_Q(0.00003406),
  DSP16_Q(-0.00014589),
  DSP16_Q(0.00035693),
  DSP16_Q(-0.00069528),
  DSP16_Q(0.00120495),
  DSP16_Q(-0.00196996),
  DSP16_Q(0.00318746),
  DSP16_Q(-0.00546327),
  DSP16_Q(0.01192618),
  DSP16_Q(0.45600107),
  DSP16_Q(-0.01129064),
  DSP16_Q(0.00529437),
  DSP16_Q(-0.00310648),
  DSP16_Q(0.00192143),
  DSP16_Q(-0.00117307),
  DSP16_Q(0.00067389),
  DSP16_Q(-0.00034307),
  DSP16_Q(0.00013781),
  DSP16_Q(-0.00003053),
  DSP16_Q(-0.00000002),
  DSP16_Q(0.00003719),
  DSP16_Q(-0.00015893),
  DSP16_Q(0.00038856),
  DSP16_Q(-0.00075661),
  DSP16_Q(0.00131099),
  DSP16_Q(-0.00214315),
  DSP16_Q(0.00346782),
  DSP16_Q(-0.00594532),
  DSP16_Q(0.01299228),
  DSP16_Q(0.45590708),
  DSP16_Q(-0.01224147),
  DSP16_Q(0.00574580),
  DSP16_Q(-0.00337215),
  DSP16_Q(0.00208582),
  DSP16_Q(-0.00127332),
  DSP16_Q(0.00073134),
  DSP16_Q(-0.00037218),
  DSP16_Q(0.00014939),
  DSP16_Q(-0.00003302),
  DSP16_Q(-0.00000003),
  DSP16_Q(0.00004034),
  DSP16_Q(-0.00017203),
  DSP16_Q(0.00042026),
  DSP16_Q(-0.00081806),
  DSP16_Q(0.00141720),
  DSP16_Q(-0.00231660),
  DSP16_Q(0.00374862),
  DSP16_Q(-0.00642839),
  DSP16_Q(0.01406293),
  DSP16_Q(0.45580527),
  DSP16_Q(-0.01318726),
  DSP16_Q(0.00619572),
  DSP16_Q(-0.00363706),
  DSP16_Q(0.00224974),
  DSP16_Q(-0.00137328),
  DSP16_Q(0.00078859),
  DSP16_Q(-0.00040116),
  DSP16_Q(0.00016090),
  DSP16_Q(-0.00003548),
  DSP16_Q(-0.00000004),
  DSP16_Q(0.00004352),
  DSP16_Q(-0.00018518),
  DSP16_Q(0.00045206),
  DSP16_Q(-0.00087964),
  DSP16_Q(0.00152359),
  DSP16_Q(-0.00249031),
  DSP16_Q(0.00402987),
  DSP16_Q(-0.00691246),
  DSP16_Q(0.01513811),
  DSP16_Q(0.45569564),
  DSP16_Q(-0.01412801),
  DSP16_Q(0.00664410),
  DSP16_Q(-0.00390120),
  DSP16_Q(0.00241320),
  DSP16_Q(-0.00147293),
  DSP16_Q(0.00084565),
  DSP16_Q(-0.00043003),
  DSP16_Q(0.00017234),
  DSP16_Q(-0.00003791),
  DSP16_Q(-0.00000005),
  DSP16_Q(0.00004672),
  DSP16_Q(-0.00019838),
  DSP16_Q(0.00048394),
  DSP16_Q(-0.00094133),
  DSP16_Q(0.00163014),
  DSP16_Q(-0.00266426),
  DSP16_Q(0.00431153),
  DSP16_Q(-0.00739751),
  DSP16_Q(0.01621780),
  DSP16_Q(0.45557821),
  DSP16_Q(-0.01506368),
  DSP16_Q(0.00709092),
  DSP16_Q(-0.00416454),
  DSP16_Q(0.00257617),
  DSP16_Q(-0.00157227),
  DSP16_Q(0.00090251),
  DSP16_Q(-0.00045877),
  DSP16_Q(0.00018372),
  DSP16_Q(-0.00004031),
  DSP16_Q(-0.00000006),
  DSP16_Q(0.00004995),
  DSP16_Q(-0.00021164),
  DSP16_Q(0.00051590),
  DSP16_Q(-0.00100315),
  DSP16_Q(0.00173686),
  DSP16_Q(-0.00283846),
  DSP16_Q(0.00459361),
  DSP16_Q(-0.00788353),
  DSP16_Q(0.01730197),
  DSP16_Q(0.45545296),
  DSP16_Q(-0.01599428),
  DSP16_Q(0.00753618),
  DSP16_Q(-0.00442708),
  DSP16_Q(0.00273866),
  DSP16_Q(-0.00167129),
  DSP16_Q(0.00095916),
  DSP16_Q(-0.00048739),
  DSP16_Q(0.00019503),
  DSP16_Q(-0.00004269),
  DSP16_Q(-0.00000007),
  DSP16_Q(0.00005320),
  DSP16_Q(-0.00022495),
  DSP16_Q(0.00054794),
  DSP16_Q(-0.00106507),
  DSP16_Q(0.00184373),
  DSP16_Q(-0.00301288),
  DSP16_Q(0.00487608),
  DSP16_Q(-0.00837048),
  DSP16_Q(0.01839060),
  DSP16_Q(0.45531991),
  DSP16_Q(-0.01691978),
  DSP16_Q(0.00797985),
  DSP16_Q(-0.00468881),
  DSP16_Q(0.00290065),
  DSP16_Q(-0.00177000),
  DSP16_Q(0.00101560),
  DSP16_Q(-0.00051588),
  DSP16_Q(0.00020626),
  DSP16_Q(-0.00004504),
  DSP16_Q(-0.00000008),
  DSP16_Q(0.00005649),
  DSP16_Q(-0.00023831),
  DSP16_Q(0.00058006),
  DSP16_Q(-0.00112711),
  DSP16_Q(0.00195076),
  DSP16_Q(-0.00318753),
  DSP16_Q(0.00515894),
  DSP16_Q(-0.00885834),
  DSP16_Q(0.01948368),
  DSP16_Q(0.45517906),
  DSP16_Q(-0.01784016),
  DSP16_Q(0.00842191),
  DSP16_Q(-0.00494971),
  DSP16_Q(0.00306214),
  DSP16_Q(-0.00186838),
  DSP16_Q(0.00107184),
  DSP16_Q(-0.00054424),
  DSP16_Q(0.00021744),
  DSP16_Q(-0.00004737),
  DSP16_Q(-0.00000009),
  DSP16_Q(0.00005979),
  DSP16_Q(-0.00025172),
  DSP16_Q(0.00061226),
  DSP16_Q(-0.00118925),
  DSP16_Q(0.00205793),
  DSP16_Q(-0.00336239),
  DSP16_Q(0.00544216),
  DSP16_Q(-0.00934710),
  DSP16_Q(0.02058117),
  DSP16_Q(0.45503041),
  DSP16_Q(-0.01875541),
  DSP16_Q(0.00886235),
  DSP16_Q(-0.00520977),
  DSP16_Q(0.00322312),
  DSP16_Q(-0.00196643),
  DSP16_Q(0.00112787),
  DSP16_Q(-0.00057248),
  DSP16_Q(0.00022854),
  DSP16_Q(-0.00004967),
  DSP16_Q(-0.00000011),
  DSP16_Q(0.00006312),
  DSP16_Q(-0.00026518),
  DSP16_Q(0.00064454),
  DSP16_Q(-0.00125150),
  DSP16_Q(0.00216524),
  DSP16_Q(-0.00353745),
  DSP16_Q(0.00572574),
  DSP16_Q(-0.00983674),
  DSP16_Q(0.02168306),
  DSP16_Q(0.45487397),
  DSP16_Q(-0.01966551),
  DSP16_Q(0.00930115),
  DSP16_Q(-0.00546899),
  DSP16_Q(0.00338359),
  DSP16_Q(-0.00206416),
  DSP16_Q(0.00118369),
  DSP16_Q(-0.00060058),
  DSP16_Q(0.00023957),
  DSP16_Q(-0.00005194),
  DSP16_Q(-0.00000012),
  DSP16_Q(0.00006648),
  DSP16_Q(-0.00027870),
  DSP16_Q(0.00067689),
  DSP16_Q(-0.00131385),
  DSP16_Q(0.00227269),
  DSP16_Q(-0.00371271),
  DSP16_Q(0.00600966),
  DSP16_Q(-0.01032722),
  DSP16_Q(0.02278932),
  DSP16_Q(0.45470975),
  DSP16_Q(-0.02057046),
  DSP16_Q(0.00973829),
  DSP16_Q(-0.00572734),
  DSP16_Q(0.00354353),
  DSP16_Q(-0.00216154),
  DSP16_Q(0.00123928),
  DSP16_Q(-0.00062855),
  DSP16_Q(0.00025053),
  DSP16_Q(-0.00005418),
  DSP16_Q(-0.00000014),
  DSP16_Q(0.00006986),
  DSP16_Q(-0.00029226),
  DSP16_Q(0.00070931),
  DSP16_Q(-0.00137630),
  DSP16_Q(0.00238027),
  DSP16_Q(-0.00388816),
  DSP16_Q(0.00629391),
  DSP16_Q(-0.01081854),
  DSP16_Q(0.02389992),
  DSP16_Q(0.45453774),
  DSP16_Q(-0.02147023),
  DSP16_Q(0.01017375),
  DSP16_Q(-0.00598483),
  DSP16_Q(0.00370293),
  DSP16_Q(-0.00225858),
  DSP16_Q(0.00129466),
  DSP16_Q(-0.00065640),
  DSP16_Q(0.00026142),
  DSP16_Q(-0.00005640),
  DSP16_Q(-0.00000016),
  DSP16_Q(0.00007327),
  DSP16_Q(-0.00030587),
  DSP16_Q(0.00074180),
  DSP16_Q(-0.00143884),
  DSP16_Q(0.00248797),
  DSP16_Q(-0.00406378),
  DSP16_Q(0.00657848),
  DSP16_Q(-0.01131066),
  DSP16_Q(0.02501485),
  DSP16_Q(0.45435796),
  DSP16_Q(-0.02236480),
  DSP16_Q(0.01060752),
  DSP16_Q(-0.00624143),
  DSP16_Q(0.00386180),
  DSP16_Q(-0.00235528),
  DSP16_Q(0.00134982),
  DSP16_Q(-0.00068410),
  DSP16_Q(0.00027224),
  DSP16_Q(-0.00005859),
  DSP16_Q(-0.00000019),
  DSP16_Q(0.00007670),
  DSP16_Q(-0.00031952),
  DSP16_Q(0.00077437),
  DSP16_Q(-0.00150148),
  DSP16_Q(0.00259580),
  DSP16_Q(-0.00423957),
  DSP16_Q(0.00686334),
  DSP16_Q(-0.01180357),
  DSP16_Q(0.02613409),
  DSP16_Q(0.45417040),
  DSP16_Q(-0.02325418),
  DSP16_Q(0.01103958),
  DSP16_Q(-0.00649714),
  DSP16_Q(0.00402013),
  DSP16_Q(-0.00245163),
  DSP16_Q(0.00140476),
  DSP16_Q(-0.00071168),
  DSP16_Q(0.00028299),
  DSP16_Q(-0.00006075),
  DSP16_Q(-0.00000021),
  DSP16_Q(0.00008016),
  DSP16_Q(-0.00033323),
  DSP16_Q(0.00080700),
  DSP16_Q(-0.00156420),
  DSP16_Q(0.00270373),
  DSP16_Q(-0.00441553),
  DSP16_Q(0.00714849),
  DSP16_Q(-0.01229724),
  DSP16_Q(0.02725759),
  DSP16_Q(0.45397509),
  DSP16_Q(-0.02413833),
  DSP16_Q(0.01146992),
  DSP16_Q(-0.00675195),
  DSP16_Q(0.00417790),
  DSP16_Q(-0.00254762),
  DSP16_Q(0.00145947),
  DSP16_Q(-0.00073912),
  DSP16_Q(0.00029367),
  DSP16_Q(-0.00006288),
  DSP16_Q(-0.00000024),
  DSP16_Q(0.00008364),
  DSP16_Q(-0.00034698),
  DSP16_Q(0.00083970),
  DSP16_Q(-0.00162701),
  DSP16_Q(0.00281178),
  DSP16_Q(-0.00459163),
  DSP16_Q(0.00743392),
  DSP16_Q(-0.01279165),
  DSP16_Q(0.02838536),
  DSP16_Q(0.45377201),
  DSP16_Q(-0.02501725),
  DSP16_Q(0.01189851),
  DSP16_Q(-0.00700584),
  DSP16_Q(0.00433512),
  DSP16_Q(-0.00264326),
  DSP16_Q(0.00151395),
  DSP16_Q(-0.00076642),
  DSP16_Q(0.00030427),
  DSP16_Q(-0.00006499),
  DSP16_Q(-0.00000026),
  DSP16_Q(0.00008715),
  DSP16_Q(-0.00036077),
  DSP16_Q(0.00087246),
  DSP16_Q(-0.00168990),
  DSP16_Q(0.00291993),
  DSP16_Q(-0.00476788),
  DSP16_Q(0.00771960),
  DSP16_Q(-0.01328678),
  DSP16_Q(0.02951735),
  DSP16_Q(0.45356118),
  DSP16_Q(-0.02589092),
  DSP16_Q(0.01232534),
  DSP16_Q(-0.00725880),
  DSP16_Q(0.00449176),
  DSP16_Q(-0.00273853),
  DSP16_Q(0.00156821),
  DSP16_Q(-0.00079358),
  DSP16_Q(0.00031481),
  DSP16_Q(-0.00006707),
  DSP16_Q(-0.00000029),
  DSP16_Q(0.00009068),
  DSP16_Q(-0.00037462),
  DSP16_Q(0.00090529),
  DSP16_Q(-0.00175287),
  DSP16_Q(0.00302818),
  DSP16_Q(-0.00494426),
  DSP16_Q(0.00800553),
  DSP16_Q(-0.01378261),
  DSP16_Q(0.03065354),
  DSP16_Q(0.45334261),
  DSP16_Q(-0.02675933),
  DSP16_Q(0.01275039),
  DSP16_Q(-0.00751083),
  DSP16_Q(0.00464783),
  DSP16_Q(-0.00283343),
  DSP16_Q(0.00162223),
  DSP16_Q(-0.00082061),
  DSP16_Q(0.00032527),
  DSP16_Q(-0.00006913),
  DSP16_Q(-0.00000033),
  DSP16_Q(0.00009424),
  DSP16_Q(-0.00038850),
  DSP16_Q(0.00093818),
  DSP16_Q(-0.00181592),
  DSP16_Q(0.00313652),
  DSP16_Q(-0.00512077),
  DSP16_Q(0.00829168),
  DSP16_Q(-0.01427911),
  DSP16_Q(0.03179391),
  DSP16_Q(0.45311630),
  DSP16_Q(-0.02762246),
  DSP16_Q(0.01317365),
  DSP16_Q(-0.00776191),
  DSP16_Q(0.00480333),
  DSP16_Q(-0.00292797),
  DSP16_Q(0.00167601),
  DSP16_Q(-0.00084750),
  DSP16_Q(0.00033566),
  DSP16_Q(-0.00007116),
  DSP16_Q(-0.00000036),
  DSP16_Q(0.00009782),
  DSP16_Q(-0.00040243),
  DSP16_Q(0.00097112),
  DSP16_Q(-0.00187904),
  DSP16_Q(0.00324494),
  DSP16_Q(-0.00529740),
  DSP16_Q(0.00857805),
  DSP16_Q(-0.01477626),
  DSP16_Q(0.03293844),
  DSP16_Q(0.45288226),
  DSP16_Q(-0.02848030),
  DSP16_Q(0.01359511),
  DSP16_Q(-0.00801203),
  DSP16_Q(0.00495823),
  DSP16_Q(-0.00302212),
  DSP16_Q(0.00172956),
  DSP16_Q(-0.00087424),
  DSP16_Q(0.00034597),
  DSP16_Q(-0.00007316),
  DSP16_Q(-0.00000040),
  DSP16_Q(0.00010143),
  DSP16_Q(-0.00041641),
  DSP16_Q(0.00100413),
  DSP16_Q(-0.00194223),
  DSP16_Q(0.00335345),
  DSP16_Q(-0.00547413),
  DSP16_Q(0.00886463),
  DSP16_Q(-0.01527404),
  DSP16_Q(0.03408710),
  DSP16_Q(0.45264049),
  DSP16_Q(-0.02933285),
  DSP16_Q(0.01401473),
  DSP16_Q(-0.00826117),
  DSP16_Q(0.00511254),
  DSP16_Q(-0.00311590),
  DSP16_Q(0.00178287),
  DSP16_Q(-0.00090084),
  DSP16_Q(0.00035622),
  DSP16_Q(-0.00007513),
  DSP16_Q(-0.00000044),
  DSP16_Q(0.00010505),
  DSP16_Q(-0.00043043),
  DSP16_Q(0.00103719),
  DSP16_Q(-0.00200549),
  DSP16_Q(0.00346203),
  DSP16_Q(-0.00565096),
  DSP16_Q(0.00915139),
  DSP16_Q(-0.01577242),
  DSP16_Q(0.03523986),
  DSP16_Q(0.45239101),
  DSP16_Q(-0.03018007),
  DSP16_Q(0.01443252),
  DSP16_Q(-0.00850934),
  DSP16_Q(0.00526626),
  DSP16_Q(-0.00320930),
  DSP16_Q(0.00183594),
  DSP16_Q(-0.00092730),
  DSP16_Q(0.00036639),
  DSP16_Q(-0.00007708),
  DSP16_Q(-0.00000048),
  DSP16_Q(0.00010871),
  DSP16_Q(-0.00044448),
  DSP16_Q(0.00107031),
  DSP16_Q(-0.00206881),
  DSP16_Q(0.00357068),
  DSP16_Q(-0.00582788),
  DSP16_Q(0.00943833),
  DSP16_Q(-0.01627139),
  DSP16_Q(0.03639670),
  DSP16_Q(0.45213382),
  DSP16_Q(-0.03102197),
  DSP16_Q(0.01484845),
  DSP16_Q(-0.00875652),
  DSP16_Q(0.00541936),
  DSP16_Q(-0.00330230),
  DSP16_Q(0.00188877),
  DSP16_Q(-0.00095362),
  DSP16_Q(0.00037648),
  DSP16_Q(-0.00007900),
  DSP16_Q(-0.00000052),
  DSP16_Q(0.00011238),
  DSP16_Q(-0.00045858),
  DSP16_Q(0.00110348),
  DSP16_Q(-0.00213219),
  DSP16_Q(0.00367940),
  DSP16_Q(-0.00600488),
  DSP16_Q(0.00972542),
  DSP16_Q(-0.01677091),
  DSP16_Q(0.03755760),
  DSP16_Q(0.45186892),
  DSP16_Q(-0.03185853),
  DSP16_Q(0.01526250),
  DSP16_Q(-0.00900270),
  DSP16_Q(0.00557185),
  DSP16_Q(-0.00339492),
  DSP16_Q(0.00194135),
  DSP16_Q(-0.00097979),
  DSP16_Q(0.00038650),
  DSP16_Q(-0.00008090),
  DSP16_Q(-0.00000057),
  DSP16_Q(0.00011609),
  DSP16_Q(-0.00047272),
  DSP16_Q(0.00113671),
  DSP16_Q(-0.00219562),
  DSP16_Q(0.00378817),
  DSP16_Q(-0.00618195),
  DSP16_Q(0.01001266),
  DSP16_Q(-0.01727097),
  DSP16_Q(0.03872252),
  DSP16_Q(0.45159634),
  DSP16_Q(-0.03268974),
  DSP16_Q(0.01567467),
  DSP16_Q(-0.00924786),
  DSP16_Q(0.00572372),
  DSP16_Q(-0.00348714),
  DSP16_Q(0.00199368),
  DSP16_Q(-0.00100582),
  DSP16_Q(0.00039645),
  DSP16_Q(-0.00008277),
  DSP16_Q(-0.00000062),
  DSP16_Q(0.00011981),
  DSP16_Q(-0.00048691),
  DSP16_Q(0.00116998),
  DSP16_Q(-0.00225911),
  DSP16_Q(0.00389700),
  DSP16_Q(-0.00635908),
  DSP16_Q(0.01030003),
  DSP16_Q(-0.01777154),
  DSP16_Q(0.03989144),
  DSP16_Q(0.45131607),
  DSP16_Q(-0.03351558),
  DSP16_Q(0.01608493),
  DSP16_Q(-0.00949200),
  DSP16_Q(0.00587496),
  DSP16_Q(-0.00357896),
  DSP16_Q(0.00204576),
  DSP16_Q(-0.00103170),
  DSP16_Q(0.00040632),
  DSP16_Q(-0.00008461),
  DSP16_Q(-0.00000067),
  DSP16_Q(0.00012356),
  DSP16_Q(-0.00050112),
  DSP16_Q(0.00120330),
  DSP16_Q(-0.00232265),
  DSP16_Q(0.00400588),
  DSP16_Q(-0.00653627),
  DSP16_Q(0.01058752),
  DSP16_Q(-0.01827260),
  DSP16_Q(0.04106435),
  DSP16_Q(0.45102813),
  DSP16_Q(-0.03433605),
  DSP16_Q(0.01649327),
  DSP16_Q(-0.00973511),
  DSP16_Q(0.00602557),
  DSP16_Q(-0.00367038),
  DSP16_Q(0.00209758),
  DSP16_Q(-0.00105743),
  DSP16_Q(0.00041611),
  DSP16_Q(-0.00008642),
  DSP16_Q(-0.00000073),
  DSP16_Q(0.00012733),
  DSP16_Q(-0.00051538),
  DSP16_Q(0.00123667),
  DSP16_Q(-0.00238623),
  DSP16_Q(0.00411479),
  DSP16_Q(-0.00671350),
  DSP16_Q(0.01087511),
  DSP16_Q(-0.01877413),
  DSP16_Q(0.04224120),
  DSP16_Q(0.45073253),
  DSP16_Q(-0.03515113),
  DSP16_Q(0.01689968),
  DSP16_Q(-0.00997718),
  DSP16_Q(0.00617553),
  DSP16_Q(-0.00376139),
  DSP16_Q(0.00214916),
  DSP16_Q(-0.00108301),
  DSP16_Q(0.00042583),
  DSP16_Q(-0.00008821),
  DSP16_Q(-0.00000079),
  DSP16_Q(0.00013112),
  DSP16_Q(-0.00052968),
  DSP16_Q(0.00127008),
  DSP16_Q(-0.00244986),
  DSP16_Q(0.00422375),
  DSP16_Q(-0.00689076),
  DSP16_Q(0.01116278),
  DSP16_Q(-0.01927610),
  DSP16_Q(0.04342197),
  DSP16_Q(0.45042926),
  DSP16_Q(-0.03596081),
  DSP16_Q(0.01730414),
  DSP16_Q(-0.01021820),
  DSP16_Q(0.00632485),
  DSP16_Q(-0.00385199),
  DSP16_Q(0.00220047),
  DSP16_Q(-0.00110844),
  DSP16_Q(0.00043548),
  DSP16_Q(-0.00008997),
  DSP16_Q(-0.00000085),
  DSP16_Q(0.00013494),
  DSP16_Q(-0.00054401),
  DSP16_Q(0.00130354),
  DSP16_Q(-0.00251353),
  DSP16_Q(0.00433273),
  DSP16_Q(-0.00706806),
  DSP16_Q(0.01145053),
  DSP16_Q(-0.01977849),
  DSP16_Q(0.04460664),
  DSP16_Q(0.45011835),
  DSP16_Q(-0.03676507),
  DSP16_Q(0.01770663),
  DSP16_Q(-0.01045815),
  DSP16_Q(0.00647352),
  DSP16_Q(-0.00394217),
  DSP16_Q(0.00225153),
  DSP16_Q(-0.00113372),
  DSP16_Q(0.00044505),
  DSP16_Q(-0.00009171),
  DSP16_Q(-0.00000091),
  DSP16_Q(0.00013878),
  DSP16_Q(-0.00055838),
  DSP16_Q(0.00133703),
  DSP16_Q(-0.00257724),
  DSP16_Q(0.00444174),
  DSP16_Q(-0.00724536),
  DSP16_Q(0.01173833),
  DSP16_Q(-0.02028128),
  DSP16_Q(0.04579518),
  DSP16_Q(0.44979981),
  DSP16_Q(-0.03756392),
  DSP16_Q(0.01810714),
  DSP16_Q(-0.01069703),
  DSP16_Q(0.00662153),
  DSP16_Q(-0.00403194),
  DSP16_Q(0.00230233),
  DSP16_Q(-0.00115885),
  DSP16_Q(0.00045454),
  DSP16_Q(-0.00009342),
  DSP16_Q(-0.00000098),
  DSP16_Q(0.00014264),
  DSP16_Q(-0.00057278),
  DSP16_Q(0.00137057),
  DSP16_Q(-0.00264097),
  DSP16_Q(0.00455077),
  DSP16_Q(-0.00742268),
  DSP16_Q(0.01202618),
  DSP16_Q(-0.02078443),
  DSP16_Q(0.04698756),
  DSP16_Q(0.44947363),
  DSP16_Q(-0.03835733),
  DSP16_Q(0.01850566),
  DSP16_Q(-0.01093482),
  DSP16_Q(0.00676887),
  DSP16_Q(-0.00412128),
  DSP16_Q(0.00235286),
  DSP16_Q(-0.00118383),
  DSP16_Q(0.00046396),
  DSP16_Q(-0.00009510),
  DSP16_Q(-0.00000105),
  DSP16_Q(0.00014653),
  DSP16_Q(-0.00058722),
  DSP16_Q(0.00140414),
  DSP16_Q(-0.00270474),
  DSP16_Q(0.00465981),
  DSP16_Q(-0.00759999),
  DSP16_Q(0.01231406),
  DSP16_Q(-0.02128794),
  DSP16_Q(0.04818376),
  DSP16_Q(0.44913984),
  DSP16_Q(-0.03914530),
  DSP16_Q(0.01890217),
  DSP16_Q(-0.01117153),
  DSP16_Q(0.00691554),
  DSP16_Q(-0.00421019),
  DSP16_Q(0.00240313),
  DSP16_Q(-0.00120865),
  DSP16_Q(0.00047330),
  DSP16_Q(-0.00009676),
  DSP16_Q(-0.00000112),
  DSP16_Q(0.00015044),
  DSP16_Q(-0.00060170),
  DSP16_Q(0.00143775),
  DSP16_Q(-0.00276854),
  DSP16_Q(0.00476886),
  DSP16_Q(-0.00777729),
  DSP16_Q(0.01260195),
  DSP16_Q(-0.02179177),
  DSP16_Q(0.04938375),
  DSP16_Q(0.44879845),
  DSP16_Q(-0.03992781),
  DSP16_Q(0.01929666),
  DSP16_Q(-0.01140713),
  DSP16_Q(0.00706153),
  DSP16_Q(-0.00429868),
  DSP16_Q(0.00245313),
  DSP16_Q(-0.00123332),
  DSP16_Q(0.00048256),
  DSP16_Q(-0.00009839),
  DSP16_Q(-0.00000120),
  DSP16_Q(0.00015437),
  DSP16_Q(-0.00061620),
  DSP16_Q(0.00147139),
  DSP16_Q(-0.00283236),
  DSP16_Q(0.00487791),
  DSP16_Q(-0.00795457),
  DSP16_Q(0.01288984),
  DSP16_Q(-0.02229589),
  DSP16_Q(0.05058750),
  DSP16_Q(0.44844946),
  DSP16_Q(-0.04070487),
  DSP16_Q(0.01968911),
  DSP16_Q(-0.01164162),
  DSP16_Q(0.00720684),
  DSP16_Q(-0.00438673),
  DSP16_Q(0.00250286),
  DSP16_Q(-0.00125784),
  DSP16_Q(0.00049175),
  DSP16_Q(-0.00010000),
  DSP16_Q(-0.00000128),
  DSP16_Q(0.00015832),
  DSP16_Q(-0.00063074),
  DSP16_Q(0.00150507),
  DSP16_Q(-0.00289619),
  DSP16_Q(0.00498695),
  DSP16_Q(-0.00813183),
  DSP16_Q(0.01317772),
  DSP16_Q(-0.02280030),
  DSP16_Q(0.05179498),
  DSP16_Q(0.44809288),
  DSP16_Q(-0.04147644),
  DSP16_Q(0.02007951),
  DSP16_Q(-0.01187499),
  DSP16_Q(0.00735146),
  DSP16_Q(-0.00447434),
  DSP16_Q(0.00255232),
  DSP16_Q(-0.00128219),
  DSP16_Q(0.00050086),
  DSP16_Q(-0.00010158),
  DSP16_Q(-0.00000137),
  DSP16_Q(0.00016229),
  DSP16_Q(-0.00064531),
  DSP16_Q(0.00153878),
  DSP16_Q(-0.00296005),
  DSP16_Q(0.00509599),
  DSP16_Q(-0.00830904),
  DSP16_Q(0.01346556),
  DSP16_Q(-0.02330495),
  DSP16_Q(0.05300617),
  DSP16_Q(0.44772873),
  DSP16_Q(-0.04224253),
  DSP16_Q(0.02046784),
  DSP16_Q(-0.01210723),
  DSP16_Q(0.00749538),
  DSP16_Q(-0.00456152),
  DSP16_Q(0.00260151),
  DSP16_Q(-0.00130639),
  DSP16_Q(0.00050989),
  DSP16_Q(-0.00010313),
  DSP16_Q(-0.00000145),
  DSP16_Q(0.00016629),
  DSP16_Q(-0.00065992),
  DSP16_Q(0.00157251),
  DSP16_Q(-0.00302392),
  DSP16_Q(0.00520501),
  DSP16_Q(-0.00848620),
  DSP16_Q(0.01375336),
  DSP16_Q(-0.02380984),
  DSP16_Q(0.05422104),
  DSP16_Q(0.44735702),
  DSP16_Q(-0.04300313),
  DSP16_Q(0.02085410),
  DSP16_Q(-0.01233833),
  DSP16_Q(0.00763860),
  DSP16_Q(-0.00464824),
  DSP16_Q(0.00265043),
  DSP16_Q(-0.00133044),
  DSP16_Q(0.00051884),
  DSP16_Q(-0.00010466),
  DSP16_Q(-0.00000155),
  DSP16_Q(0.00017031),
  DSP16_Q(-0.00067455),
  DSP16_Q(0.00160627),
  DSP16_Q(-0.00308779),
  DSP16_Q(0.00531401),
  DSP16_Q(-0.00866331),
  DSP16_Q(0.01404110),
  DSP16_Q(-0.02431492),
  DSP16_Q(0.05543956),
  DSP16_Q(0.44697776),
  DSP16_Q(-0.04375823),
  DSP16_Q(0.02123826),
  DSP16_Q(-0.01256828),
  DSP16_Q(0.00778111),
  DSP16_Q(-0.00473452),
  DSP16_Q(0.00269907),
  DSP16_Q(-0.00135432),
  DSP16_Q(0.00052772),
  DSP16_Q(-0.00010616),
  DSP16_Q(-0.00000164),
  DSP16_Q(0.00017435),
  DSP16_Q(-0.00068921),
  DSP16_Q(0.00164006),
  DSP16_Q(-0.00315168),
  DSP16_Q(0.00542299),
  DSP16_Q(-0.00884035),
  DSP16_Q(0.01432877),
  DSP16_Q(-0.02482019),
  DSP16_Q(0.05666169),
  DSP16_Q(0.44659096),
  DSP16_Q(-0.04450781),
  DSP16_Q(0.02162031),
  DSP16_Q(-0.01279707),
  DSP16_Q(0.00792290),
  DSP16_Q(-0.00482035),
  DSP16_Q(0.00274742),
  DSP16_Q(-0.00137805),
  DSP16_Q(0.00053651),
  DSP16_Q(-0.00010764),
  DSP16_Q(-0.00000174),
  DSP16_Q(0.00017841),
  DSP16_Q(-0.00070390),
  DSP16_Q(0.00167387),
  DSP16_Q(-0.00321556),
  DSP16_Q(0.00553193),
  DSP16_Q(-0.00901731),
  DSP16_Q(0.01461635),
  DSP16_Q(-0.02532561),
  DSP16_Q(0.05788742),
  DSP16_Q(0.44619663),
  DSP16_Q(-0.04525188),
  DSP16_Q(0.02200024),
  DSP16_Q(-0.01302469),
  DSP16_Q(0.00806398),
  DSP16_Q(-0.00490572),
  DSP16_Q(0.00279550),
  DSP16_Q(-0.00140161),
  DSP16_Q(0.00054523),
  DSP16_Q(-0.00010909),
  DSP16_Q(-0.00000184),
  DSP16_Q(0.00018249),
  DSP16_Q(-0.00071862),
  DSP16_Q(0.00170770),
  DSP16_Q(-0.00327945),
  DSP16_Q(0.00564083),
  DSP16_Q(-0.00919419),
  DSP16_Q(0.01490382),
  DSP16_Q(-0.02583116),
  DSP16_Q(0.05911672),
  DSP16_Q(0.44579479),
  DSP16_Q(-0.04599041),
  DSP16_Q(0.02237804),
  DSP16_Q(-0.01325114),
  DSP16_Q(0.00820432),
  DSP16_Q(-0.00499064),
  DSP16_Q(0.00284330),
  DSP16_Q(-0.00142501),
  DSP16_Q(0.00055387),
  DSP16_Q(-0.00011051),
  DSP16_Q(-0.00000195),
  DSP16_Q(0.00018659),
  DSP16_Q(-0.00073336),
  DSP16_Q(0.00174155),
  DSP16_Q(-0.00334333),
  DSP16_Q(0.00574968),
  DSP16_Q(-0.00937098),
  DSP16_Q(0.01519117),
  DSP16_Q(-0.02633682),
  DSP16_Q(0.06034955),
  DSP16_Q(0.44538545),
  DSP16_Q(-0.04672340),
  DSP16_Q(0.02275369),
  DSP16_Q(-0.01347640),
  DSP16_Q(0.00834394),
  DSP16_Q(-0.00507509),
  DSP16_Q(0.00289081),
  DSP16_Q(-0.00144826),
  DSP16_Q(0.00056244),
  DSP16_Q(-0.00011191),
  DSP16_Q(-0.00000206),
  DSP16_Q(0.00019071),
  DSP16_Q(-0.00074814),
  DSP16_Q(0.00177542),
  DSP16_Q(-0.00340720),
  DSP16_Q(0.00585849),
  DSP16_Q(-0.00954765),
  DSP16_Q(0.01547839),
  DSP16_Q(-0.02684255),
  DSP16_Q(0.06158589),
  DSP16_Q(0.44496863),
  DSP16_Q(-0.04745085),
  DSP16_Q(0.02312719),
  DSP16_Q(-0.01370046),
  DSP16_Q(0.00848282),
  DSP16_Q(-0.00515908),
  DSP16_Q(0.00293803),
  DSP16_Q(-0.00147134),
  DSP16_Q(0.00057092),
  DSP16_Q(-0.00011329),
  DSP16_Q(-0.00000218),
  DSP16_Q(0.00019486),
  DSP16_Q(-0.00076293),
  DSP16_Q(0.00180930),
  DSP16_Q(-0.00347106),
  DSP16_Q(0.00596724),
  DSP16_Q(-0.00972422),
  DSP16_Q(0.01576546),
  DSP16_Q(-0.02734835),
  DSP16_Q(0.06282571),
  DSP16_Q(0.44454432),
  DSP16_Q(-0.04817274),
  DSP16_Q(0.02349850),
  DSP16_Q(-0.01392332),
  DSP16_Q(0.00862096),
  DSP16_Q(-0.00524259),
  DSP16_Q(0.00298497),
  DSP16_Q(-0.00149425),
  DSP16_Q(0.00057932),
  DSP16_Q(-0.00011464),
  DSP16_Q(-0.00000229),
  DSP16_Q(0.00019902),
  DSP16_Q(-0.00077776),
  DSP16_Q(0.00184320),
  DSP16_Q(-0.00353491),
  DSP16_Q(0.00607593),
  DSP16_Q(-0.00990066),
  DSP16_Q(0.01605236),
  DSP16_Q(-0.02785417),
  DSP16_Q(0.06406897),
  DSP16_Q(0.44411256),
  DSP16_Q(-0.04888907),
  DSP16_Q(0.02386764),
  DSP16_Q(-0.01414497),
  DSP16_Q(0.00875835),
  DSP16_Q(-0.00532564),
  DSP16_Q(0.00303162),
  DSP16_Q(-0.00151700),
  DSP16_Q(0.00058765),
  DSP16_Q(-0.00011596),
  DSP16_Q(-0.00000242),
  DSP16_Q(0.00020320),
  DSP16_Q(-0.00079260),
  DSP16_Q(0.00187711),
  DSP16_Q(-0.00359874),
  DSP16_Q(0.00618455),
  DSP16_Q(-0.01007697),
  DSP16_Q(0.01633909),
  DSP16_Q(-0.02836000),
  DSP16_Q(0.06531565),
  DSP16_Q(0.44367334),
  DSP16_Q(-0.04959983),
  DSP16_Q(0.02423457),
  DSP16_Q(-0.01436539),
  DSP16_Q(0.00889498),
  DSP16_Q(-0.00540821),
  DSP16_Q(0.00307798),
  DSP16_Q(-0.00153959),
  DSP16_Q(0.00059590),
  DSP16_Q(-0.00011726),
  DSP16_Q(-0.00000254),
  DSP16_Q(0.00020741),
  DSP16_Q(-0.00080747),
  DSP16_Q(0.00191103),
  DSP16_Q(-0.00366255),
  DSP16_Q(0.00629309),
  DSP16_Q(-0.01025314),
  DSP16_Q(0.01662562),
  DSP16_Q(-0.02886582),
  DSP16_Q(0.06656573),
  DSP16_Q(0.44322670),
  DSP16_Q(-0.05030502),
  DSP16_Q(0.02459929),
  DSP16_Q(-0.01458459),
  DSP16_Q(0.00903086),
  DSP16_Q(-0.00549030),
  DSP16_Q(0.00312404),
  DSP16_Q(-0.00156201),
  DSP16_Q(0.00060406),
  DSP16_Q(-0.00011853),
  DSP16_Q(-0.00000268),
  DSP16_Q(0.00021163),
  DSP16_Q(-0.00082236),
  DSP16_Q(0.00194496),
  DSP16_Q(-0.00372633),
  DSP16_Q(0.00640156),
  DSP16_Q(-0.01042915),
  DSP16_Q(0.01691194),
  DSP16_Q(-0.02937159),
  DSP16_Q(0.06781916),
  DSP16_Q(0.44277263),
  DSP16_Q(-0.05100461),
  DSP16_Q(0.02496179),
  DSP16_Q(-0.01480255),
  DSP16_Q(0.00916597),
  DSP16_Q(-0.00557190),
  DSP16_Q(0.00316981),
  DSP16_Q(-0.00158427),
  DSP16_Q(0.00061215),
  DSP16_Q(-0.00011978),
  DSP16_Q(-0.00000281),
  DSP16_Q(0.00021587),
  DSP16_Q(-0.00083728),
  DSP16_Q(0.00197890),
  DSP16_Q(-0.00379008),
  DSP16_Q(0.00650994),
  DSP16_Q(-0.01060500),
  DSP16_Q(0.01719804),
  DSP16_Q(-0.02987729),
  DSP16_Q(0.06907592),
  DSP16_Q(0.44231116),
  DSP16_Q(-0.05169862),
  DSP16_Q(0.02532206),
  DSP16_Q(-0.01501926),
  DSP16_Q(0.00930031),
  DSP16_Q(-0.00565303),
  DSP16_Q(0.00321528),
  DSP16_Q(-0.00160635),
  DSP16_Q(0.00062016),
  DSP16_Q(-0.00012101),
  DSP16_Q(-0.00000295),
  DSP16_Q(0.00022013),
  DSP16_Q(-0.00085221),
  DSP16_Q(0.00201284),
  DSP16_Q(-0.00385380),
  DSP16_Q(0.00661823),
  DSP16_Q(-0.01078069),
  DSP16_Q(0.01748390),
  DSP16_Q(-0.03038290),
  DSP16_Q(0.07033598),
  DSP16_Q(0.44184229),
  DSP16_Q(-0.05238702),
  DSP16_Q(0.02568007),
  DSP16_Q(-0.01523471),
  DSP16_Q(0.00943387),
  DSP16_Q(-0.00573366),
  DSP16_Q(0.00326046),
  DSP16_Q(-0.00162828),
  DSP16_Q(0.00062808),
  DSP16_Q(-0.00012221),
  DSP16_Q(-0.00000310),
  DSP16_Q(0.00022441),
  DSP16_Q(-0.00086716),
  DSP16_Q(0.00204678),
  DSP16_Q(-0.00391749),
  DSP16_Q(0.00672642),
  DSP16_Q(-0.01095619),
  DSP16_Q(0.01776950),
  DSP16_Q(-0.03088839),
  DSP16_Q(0.07159931),
  DSP16_Q(0.44136605),
  DSP16_Q(-0.05306983),
  DSP16_Q(0.02603583),
  DSP16_Q(-0.01544891),
  DSP16_Q(0.00956665),
  DSP16_Q(-0.00581380),
  DSP16_Q(0.00330533),
  DSP16_Q(-0.00165003),
  DSP16_Q(0.00063593),
  DSP16_Q(-0.00012338),
  DSP16_Q(-0.00000325),
  DSP16_Q(0.00022871),
  DSP16_Q(-0.00088213),
  DSP16_Q(0.00208073),
  DSP16_Q(-0.00398113),
  DSP16_Q(0.00683450),
  DSP16_Q(-0.01113150),
  DSP16_Q(0.01805483),
  DSP16_Q(-0.03139373),
  DSP16_Q(0.07286588),
  DSP16_Q(0.44088244),
  DSP16_Q(-0.05374701),
  DSP16_Q(0.02638931),
  DSP16_Q(-0.01566183),
  DSP16_Q(0.00969865),
  DSP16_Q(-0.00589345),
  DSP16_Q(0.00334991),
  DSP16_Q(-0.00167161),
  DSP16_Q(0.00064370),
  DSP16_Q(-0.00012453),
  DSP16_Q(-0.00000340),
  DSP16_Q(0.00023303),
  DSP16_Q(-0.00089713),
  DSP16_Q(0.00211467),
  DSP16_Q(-0.00404474),
  DSP16_Q(0.00694248),
  DSP16_Q(-0.01130662),
  DSP16_Q(0.01833988),
  DSP16_Q(-0.03189891),
  DSP16_Q(0.07413566),
  DSP16_Q(0.44039149),
  DSP16_Q(-0.05441858),
  DSP16_Q(0.02674052),
  DSP16_Q(-0.01587347),
  DSP16_Q(0.00982985),
  DSP16_Q(-0.00597260),
  DSP16_Q(0.00339418),
  DSP16_Q(-0.00169302),
  DSP16_Q(0.00065139),
  DSP16_Q(-0.00012566),
  DSP16_Q(-0.00000356),
  DSP16_Q(0.00023737),
  DSP16_Q(-0.00091213),
  DSP16_Q(0.00214861),
  DSP16_Q(-0.00410830),
  DSP16_Q(0.00705034),
  DSP16_Q(-0.01148152),
  DSP16_Q(0.01862462),
  DSP16_Q(-0.03240389),
  DSP16_Q(0.07540862),
  DSP16_Q(0.43989321),
  DSP16_Q(-0.05508453),
  DSP16_Q(0.02708942),
  DSP16_Q(-0.01608382),
  DSP16_Q(0.00996026),
  DSP16_Q(-0.00605125),
  DSP16_Q(0.00343814),
  DSP16_Q(-0.00171427),
  DSP16_Q(0.00065899),
  DSP16_Q(-0.00012676),
  DSP16_Q(-0.00000372),
  DSP16_Q(0.00024172),
  DSP16_Q(-0.00092716),
  DSP16_Q(0.00218255),
  DSP16_Q(-0.00417180),
  DSP16_Q(0.00715808),
  DSP16_Q(-0.01165621),
  DSP16_Q(0.01890905),
  DSP16_Q(-0.03290866),
  DSP16_Q(0.07668473),
  DSP16_Q(0.43938761),
  DSP16_Q(-0.05574485),
  DSP16_Q(0.02743602),
  DSP16_Q(-0.01629287),
  DSP16_Q(0.01008986),
  DSP16_Q(-0.00612940),
  DSP16_Q(0.00348181),
  DSP16_Q(-0.00173534),
  DSP16_Q(0.00066652),
  DSP16_Q(-0.00012783),
  DSP16_Q(-0.00000389),
  DSP16_Q(0.00024610),
  DSP16_Q(-0.00094220),
  DSP16_Q(0.00221647),
  DSP16_Q(-0.00423526),
  DSP16_Q(0.00726569),
  DSP16_Q(-0.01183067),
  DSP16_Q(0.01919315),
  DSP16_Q(-0.03341318),
  DSP16_Q(0.07796395),
  DSP16_Q(0.43887471),
  DSP16_Q(-0.05639953),
  DSP16_Q(0.02778030),
  DSP16_Q(-0.01650063),
  DSP16_Q(0.01021866),
  DSP16_Q(-0.00620704),
  DSP16_Q(0.00352516),
  DSP16_Q(-0.00175624),
  DSP16_Q(0.00067397),
  DSP16_Q(-0.00012889),
  DSP16_Q(-0.00000406),
  DSP16_Q(0.00025049),
  DSP16_Q(-0.00095725),
  DSP16_Q(0.00225039),
  DSP16_Q(-0.00429865),
  DSP16_Q(0.00737317),
  DSP16_Q(-0.01200489),
  DSP16_Q(0.01947690),
  DSP16_Q(-0.03391742),
  DSP16_Q(0.07924625),
  DSP16_Q(0.43835453),
  DSP16_Q(-0.05704857),
  DSP16_Q(0.02812226),
  DSP16_Q(-0.01670707),
  DSP16_Q(0.01034665),
  DSP16_Q(-0.00628417),
  DSP16_Q(0.00356820),
  DSP16_Q(-0.00177697),
  DSP16_Q(0.00068133),
  DSP16_Q(-0.00012992),
  DSP16_Q(-0.00000424),
  DSP16_Q(0.00025490),
  DSP16_Q(-0.00097232),
  DSP16_Q(0.00228430),
  DSP16_Q(-0.00436199),
  DSP16_Q(0.00748051),
  DSP16_Q(-0.01217886),
  DSP16_Q(0.01976029),
  DSP16_Q(-0.03442137),
  DSP16_Q(0.08053162),
  DSP16_Q(0.43782708),
  DSP16_Q(-0.05769196),
  DSP16_Q(0.02846187),
  DSP16_Q(-0.01691219),
  DSP16_Q(0.01047382),
  DSP16_Q(-0.00636078),
  DSP16_Q(0.00361093),
  DSP16_Q(-0.00179753),
  DSP16_Q(0.00068862),
  DSP16_Q(-0.00013092),
  DSP16_Q(-0.00000442),
  DSP16_Q(0.00025932),
  DSP16_Q(-0.00098740),
  DSP16_Q(0.00231820),
  DSP16_Q(-0.00442526),
  DSP16_Q(0.00758770),
  DSP16_Q(-0.01235258),
  DSP16_Q(0.02004331),
  DSP16_Q(-0.03492500),
  DSP16_Q(0.08182000),
  DSP16_Q(0.43729238),
  DSP16_Q(-0.05832971),
  DSP16_Q(0.02879913),
  DSP16_Q(-0.01711598),
  DSP16_Q(0.01060017),
  DSP16_Q(-0.00643688),
  DSP16_Q(0.00365335),
  DSP16_Q(-0.00181791),
  DSP16_Q(0.00069582),
  DSP16_Q(-0.00013190),
  DSP16_Q(-0.00000461),
  DSP16_Q(0.00026376),
  DSP16_Q(-0.00100250),
  DSP16_Q(0.00235208),
  DSP16_Q(-0.00448846),
  DSP16_Q(0.00769474),
  DSP16_Q(-0.01252603),
  DSP16_Q(0.02032593),
  DSP16_Q(-0.03542828),
  DSP16_Q(0.08311137),
  DSP16_Q(0.43675044),
  DSP16_Q(-0.05896179),
  DSP16_Q(0.02913403),
  DSP16_Q(-0.01731844),
  DSP16_Q(0.01072569),
  DSP16_Q(-0.00651247),
  DSP16_Q(0.00369546),
  DSP16_Q(-0.00183812),
  DSP16_Q(0.00070295),
  DSP16_Q(-0.00013286),
  DSP16_Q(-0.00000480),
  DSP16_Q(0.00026822),
  DSP16_Q(-0.00101760),
  DSP16_Q(0.00238594),
  DSP16_Q(-0.00455160),
  DSP16_Q(0.00780163),
  DSP16_Q(-0.01269921),
  DSP16_Q(0.02060814),
  DSP16_Q(-0.03593119),
  DSP16_Q(0.08440571),
  DSP16_Q(0.43620129),
  DSP16_Q(-0.05958822),
  DSP16_Q(0.02946656),
  DSP16_Q(-0.01751956),
  DSP16_Q(0.01085038),
  DSP16_Q(-0.00658753),
  DSP16_Q(0.00373725),
  DSP16_Q(-0.00185815),
  DSP16_Q(0.00070999),
  DSP16_Q(-0.00013379),
  DSP16_Q(-0.00000500),
  DSP16_Q(0.00027270),
  DSP16_Q(-0.00103272),
  DSP16_Q(0.00241979),
  DSP16_Q(-0.00461465),
  DSP16_Q(0.00790835),
  DSP16_Q(-0.01287210),
  DSP16_Q(0.02088994),
  DSP16_Q(-0.03643370),
  DSP16_Q(0.08570297),
  DSP16_Q(0.43564493),
  DSP16_Q(-0.06020898),
  DSP16_Q(0.02979671),
  DSP16_Q(-0.01771933),
  DSP16_Q(0.01097424),
  DSP16_Q(-0.00666207),
  DSP16_Q(0.00377873),
  DSP16_Q(-0.00187801),
  DSP16_Q(0.00071695),
  DSP16_Q(-0.00013471),
  DSP16_Q(-0.00000520),
  DSP16_Q(0.00027719),
  DSP16_Q(-0.00104784),
  DSP16_Q(0.00245362),
  DSP16_Q(-0.00467763),
  DSP16_Q(0.00801490),
  DSP16_Q(-0.01304469),
  DSP16_Q(0.02117129),
  DSP16_Q(-0.03693578),
  DSP16_Q(0.08700312),
  DSP16_Q(0.43508140),
  DSP16_Q(-0.06082408),
  DSP16_Q(0.03012446),
  DSP16_Q(-0.01791774),
  DSP16_Q(0.01109725),
  DSP16_Q(-0.00673608),
  DSP16_Q(0.00381989),
  DSP16_Q(-0.00189770),
  DSP16_Q(0.00072384),
  DSP16_Q(-0.00013559),
  DSP16_Q(-0.00000541),
  DSP16_Q(0.00028170),
  DSP16_Q(-0.00106297),
  DSP16_Q(0.00248742),
  DSP16_Q(-0.00474053),
  DSP16_Q(0.00812128),
  DSP16_Q(-0.01321698),
  DSP16_Q(0.02145219),
  DSP16_Q(-0.03743742),
  DSP16_Q(0.08830614),
  DSP16_Q(0.43451070),
  DSP16_Q(-0.06143350),
  DSP16_Q(0.03044981),
  DSP16_Q(-0.01811479),
  DSP16_Q(0.01121942),
  DSP16_Q(-0.00680956),
  DSP16_Q(0.00386073),
  DSP16_Q(-0.00191720),
  DSP16_Q(0.00073064),
  DSP16_Q(-0.00013646),
  DSP16_Q(-0.00000562),
  DSP16_Q(0.00028622),
  DSP16_Q(-0.00107811),
  DSP16_Q(0.00252120),
  DSP16_Q(-0.00480334),
  DSP16_Q(0.00822747),
  DSP16_Q(-0.01338896),
  DSP16_Q(0.02173262),
  DSP16_Q(-0.03793857),
  DSP16_Q(0.08961200),
  DSP16_Q(0.43393285),
  DSP16_Q(-0.06203725),
  DSP16_Q(0.03077275),
  DSP16_Q(-0.01831046),
  DSP16_Q(0.01134074),
  DSP16_Q(-0.00688251),
  DSP16_Q(0.00390125),
  DSP16_Q(-0.00193654),
  DSP16_Q(0.00073736),
  DSP16_Q(-0.00013730),
  DSP16_Q(-0.00000584),
  DSP16_Q(0.00029076),
  DSP16_Q(-0.00109326),
  DSP16_Q(0.00255495),
  DSP16_Q(-0.00486606),
  DSP16_Q(0.00833348),
  DSP16_Q(-0.01356061),
  DSP16_Q(0.02201256),
  DSP16_Q(-0.03843922),
  DSP16_Q(0.09092065),
  DSP16_Q(0.43334787),
  DSP16_Q(-0.06263531),
  DSP16_Q(0.03109327),
  DSP16_Q(-0.01850476),
  DSP16_Q(0.01146121),
  DSP16_Q(-0.00695493),
  DSP16_Q(0.00394144),
  DSP16_Q(-0.00195569),
  DSP16_Q(0.00074399),
  DSP16_Q(-0.00013812),
  DSP16_Q(-0.00000606),
  DSP16_Q(0.00029532),
  DSP16_Q(-0.00110842),
  DSP16_Q(0.00258867),
  DSP16_Q(-0.00492869),
  DSP16_Q(0.00843929),
  DSP16_Q(-0.01373193),
  DSP16_Q(0.02229201),
  DSP16_Q(-0.03893935),
  DSP16_Q(0.09223206),
  DSP16_Q(0.43275578),
  DSP16_Q(-0.06322770),
  DSP16_Q(0.03141135),
  DSP16_Q(-0.01869767),
  DSP16_Q(0.01158081),
  DSP16_Q(-0.00702680),
  DSP16_Q(0.00398132),
  DSP16_Q(-0.00197467),
  DSP16_Q(0.00075055),
  DSP16_Q(-0.00013891),
  DSP16_Q(-0.00000629),
  DSP16_Q(0.00029989),
  DSP16_Q(-0.00112357),
  DSP16_Q(0.00262237),
  DSP16_Q(-0.00499122),
  DSP16_Q(0.00854491),
  DSP16_Q(-0.01390290),
  DSP16_Q(0.02257094),
  DSP16_Q(-0.03943892),
  DSP16_Q(0.09354621),
  DSP16_Q(0.43215660),
  DSP16_Q(-0.06381439),
  DSP16_Q(0.03172699),
  DSP16_Q(-0.01888920),
  DSP16_Q(0.01169956),
  DSP16_Q(-0.00709814),
  DSP16_Q(0.00402087),
  DSP16_Q(-0.00199347),
  DSP16_Q(0.00075703),
  DSP16_Q(-0.00013968),
  DSP16_Q(-0.00000652),
  DSP16_Q(0.00030447),
  DSP16_Q(-0.00113874),
  DSP16_Q(0.00265603),
  DSP16_Q(-0.00505365),
  DSP16_Q(0.00865031),
  DSP16_Q(-0.01407353),
  DSP16_Q(0.02284934),
  DSP16_Q(-0.03993790),
  DSP16_Q(0.09486306),
  DSP16_Q(0.43155035),
  DSP16_Q(-0.06439540),
  DSP16_Q(0.03204018),
  DSP16_Q(-0.01907932),
  DSP16_Q(0.01181743),
  DSP16_Q(-0.00716894),
  DSP16_Q(0.00406009),
  DSP16_Q(-0.00201209),
  DSP16_Q(0.00076342),
  DSP16_Q(-0.00014043),
  DSP16_Q(-0.00000676),
  DSP16_Q(0.00030907),
  DSP16_Q(-0.00115390),
  DSP16_Q(0.00268966),
  DSP16_Q(-0.00511598),
  DSP16_Q(0.00875551),
  DSP16_Q(-0.01424379),
  DSP16_Q(0.02312719),
  DSP16_Q(-0.04043628),
  DSP16_Q(0.09618258),
  DSP16_Q(0.43093704),
  DSP16_Q(-0.06497072),
  DSP16_Q(0.03235090),
  DSP16_Q(-0.01926804),
  DSP16_Q(0.01193444),
  DSP16_Q(-0.00723919),
  DSP16_Q(0.00409899),
  DSP16_Q(-0.00203053),
  DSP16_Q(0.00076974),
  DSP16_Q(-0.00014116),
  DSP16_Q(-0.00000700),
  DSP16_Q(0.00031368),
  DSP16_Q(-0.00116907),
  DSP16_Q(0.00272325),
  DSP16_Q(-0.00517820),
  DSP16_Q(0.00886049),
  DSP16_Q(-0.01441368),
  DSP16_Q(0.02340448),
  DSP16_Q(-0.04093403),
  DSP16_Q(0.09750473),
  DSP16_Q(0.43031670),
  DSP16_Q(-0.06554034),
  DSP16_Q(0.03265916),
  DSP16_Q(-0.01945534),
  DSP16_Q(0.01205057),
  DSP16_Q(-0.00730890),
  DSP16_Q(0.00413756),
  DSP16_Q(-0.00204879),
  DSP16_Q(0.00077597),
  DSP16_Q(-0.00014186),
  DSP16_Q(-0.00000725),
  DSP16_Q(0.00031831),
  DSP16_Q(-0.00118424),
  DSP16_Q(0.00275680),
  DSP16_Q(-0.00524031),
  DSP16_Q(0.00896524),
  DSP16_Q(-0.01458318),
  DSP16_Q(0.02368120),
  DSP16_Q(-0.04143112),
  DSP16_Q(0.09882949),
  DSP16_Q(0.42968934),
  DSP16_Q(-0.06610427),
  DSP16_Q(0.03296494),
  DSP16_Q(-0.01964123),
  DSP16_Q(0.01216582),
  DSP16_Q(-0.00737805),
  DSP16_Q(0.00417579),
  DSP16_Q(-0.00206688),
  DSP16_Q(0.00078212),
  DSP16_Q(-0.00014255),
  DSP16_Q(-0.00000751),
  DSP16_Q(0.00032295),
  DSP16_Q(-0.00119941),
  DSP16_Q(0.00279031),
  DSP16_Q(-0.00530230),
  DSP16_Q(0.00906977),
  DSP16_Q(-0.01475230),
  DSP16_Q(0.02395732),
  DSP16_Q(-0.04192752),
  DSP16_Q(0.10015681),
  DSP16_Q(0.42905498),
  DSP16_Q(-0.06666250),
  DSP16_Q(0.03326823),
  DSP16_Q(-0.01982569),
  DSP16_Q(0.01228019),
  DSP16_Q(-0.00744666),
  DSP16_Q(0.00421370),
  DSP16_Q(-0.00208478),
  DSP16_Q(0.00078819),
  DSP16_Q(-0.00014321),
  DSP16_Q(-0.00000777),
  DSP16_Q(0.00032761),
  DSP16_Q(-0.00121458),
  DSP16_Q(0.00282378),
  DSP16_Q(-0.00536417),
  DSP16_Q(0.00917406),
  DSP16_Q(-0.01492101),
  DSP16_Q(0.02423284),
  DSP16_Q(-0.04242320),
  DSP16_Q(0.10148667),
  DSP16_Q(0.42841365),
  DSP16_Q(-0.06721503),
  DSP16_Q(0.03356902),
  DSP16_Q(-0.02000872),
  DSP16_Q(0.01239366),
  DSP16_Q(-0.00751471),
  DSP16_Q(0.00425128),
  DSP16_Q(-0.00210250),
  DSP16_Q(0.00079418),
  DSP16_Q(-0.00014385),
  DSP16_Q(-0.00000804),
  DSP16_Q(0.00033227),
  DSP16_Q(-0.00122975),
  DSP16_Q(0.00285721),
  DSP16_Q(-0.00542593),
  DSP16_Q(0.00927810),
  DSP16_Q(-0.01508932),
  DSP16_Q(0.02450773),
  DSP16_Q(-0.04291815),
  DSP16_Q(0.10281903),
  DSP16_Q(0.42776536),
  DSP16_Q(-0.06776186),
  DSP16_Q(0.03386731),
  DSP16_Q(-0.02019032),
  DSP16_Q(0.01250625),
  DSP16_Q(-0.00758220),
  DSP16_Q(0.00428853),
  DSP16_Q(-0.00212005),
  DSP16_Q(0.00080009),
  DSP16_Q(-0.00014446),
  DSP16_Q(-0.00000831),
  DSP16_Q(0.00033695),
  DSP16_Q(-0.00124491),
  DSP16_Q(0.00289058),
  DSP16_Q(-0.00548755),
  DSP16_Q(0.00938190),
  DSP16_Q(-0.01525720),
  DSP16_Q(0.02478198),
  DSP16_Q(-0.04341233),
  DSP16_Q(0.10415385),
  DSP16_Q(0.42711014),
  DSP16_Q(-0.06830299),
  DSP16_Q(0.03416308),
  DSP16_Q(-0.02037046),
  DSP16_Q(0.01261794),
  DSP16_Q(-0.00764913),
  DSP16_Q(0.00432544),
  DSP16_Q(-0.00213741),
  DSP16_Q(0.00080592),
  DSP16_Q(-0.00014506),
  DSP16_Q(-0.00000859),
  DSP16_Q(0.00034165),
  DSP16_Q(-0.00126007),
  DSP16_Q(0.00292391),
  DSP16_Q(-0.00554905),
  DSP16_Q(0.00948545),
  DSP16_Q(-0.01542465),
  DSP16_Q(0.02505558),
  DSP16_Q(-0.04390573),
  DSP16_Q(0.10549111),
  DSP16_Q(0.42644800),
  DSP16_Q(-0.06883842),
  DSP16_Q(0.03445633),
  DSP16_Q(-0.02054916),
  DSP16_Q(0.01272873),
  DSP16_Q(-0.00771551),
  DSP16_Q(0.00436201),
  DSP16_Q(-0.00215459),
  DSP16_Q(0.00081166),
  DSP16_Q(-0.00014563),
  DSP16_Q(-0.00000887),
  DSP16_Q(0.00034635),
  DSP16_Q(-0.00127523),
  DSP16_Q(0.00295719),
  DSP16_Q(-0.00561041),
  DSP16_Q(0.00958873),
  DSP16_Q(-0.01559167),
  DSP16_Q(0.02532851),
  DSP16_Q(-0.04439830),
  DSP16_Q(0.10683077),
  DSP16_Q(0.42577896),
  DSP16_Q(-0.06936813),
  DSP16_Q(0.03474705),
  DSP16_Q(-0.02072641),
  DSP16_Q(0.01283861),
  DSP16_Q(-0.00778132),
  DSP16_Q(0.00439825),
  DSP16_Q(-0.00217158),
  DSP16_Q(0.00081733),
  DSP16_Q(-0.00014618),
  DSP16_Q(-0.00000916),
  DSP16_Q(0.00035107),
  DSP16_Q(-0.00129038),
  DSP16_Q(0.00299042),
  DSP16_Q(-0.00567164),
  DSP16_Q(0.00969175),
  DSP16_Q(-0.01575823),
  DSP16_Q(0.02560075),
  DSP16_Q(-0.04489003),
  DSP16_Q(0.10817280),
  DSP16_Q(0.42510305),
  DSP16_Q(-0.06989215),
  DSP16_Q(0.03503523),
  DSP16_Q(-0.02090219),
  DSP16_Q(0.01294759),
  DSP16_Q(-0.00784656),
  DSP16_Q(0.00443415),
  DSP16_Q(-0.00218840),
  DSP16_Q(0.00082291),
  DSP16_Q(-0.00014671),
  DSP16_Q(-0.00000946),
  DSP16_Q(0.00035580),
  DSP16_Q(-0.00130553),
  DSP16_Q(0.00302359),
  DSP16_Q(-0.00573273),
  DSP16_Q(0.00979450),
  DSP16_Q(-0.01592434),
  DSP16_Q(0.02587229),
  DSP16_Q(-0.04538088),
  DSP16_Q(0.10951715),
  DSP16_Q(0.42442029),
  DSP16_Q(-0.07041046),
  DSP16_Q(0.03532087),
  DSP16_Q(-0.02107650),
  DSP16_Q(0.01305566),
  DSP16_Q(-0.00791124),
  DSP16_Q(0.00446972),
  DSP16_Q(-0.00220503),
  DSP16_Q(0.00082842),
  DSP16_Q(-0.00014722),
  DSP16_Q(-0.00000976),
  DSP16_Q(0.00036054),
  DSP16_Q(-0.00132067),
  DSP16_Q(0.00305671),
  DSP16_Q(-0.00579367),
  DSP16_Q(0.00989697),
  DSP16_Q(-0.01608998),
  DSP16_Q(0.02614312),
  DSP16_Q(-0.04587084),
  DSP16_Q(0.11086380),
  DSP16_Q(0.42373069),
  DSP16_Q(-0.07092306),
  DSP16_Q(0.03560395),
  DSP16_Q(-0.02124935),
  DSP16_Q(0.01316281),
  DSP16_Q(-0.00797535),
  DSP16_Q(0.00450494),
  DSP16_Q(-0.00222148),
  DSP16_Q(0.00083384),
  DSP16_Q(-0.00014771),
  DSP16_Q(-0.00001006),
  DSP16_Q(0.00036529),
  DSP16_Q(-0.00133580),
  DSP16_Q(0.00308977),
  DSP16_Q(-0.00585447),
  DSP16_Q(0.00999915),
  DSP16_Q(-0.01625514),
  DSP16_Q(0.02641321),
  DSP16_Q(-0.04635987),
  DSP16_Q(0.11221272),
  DSP16_Q(0.42303429),
  DSP16_Q(-0.07142995),
  DSP16_Q(0.03588447),
  DSP16_Q(-0.02142071),
  DSP16_Q(0.01326905),
  DSP16_Q(-0.00803888),
  DSP16_Q(0.00453983),
  DSP16_Q(-0.00223775),
  DSP16_Q(0.00083918),
  DSP16_Q(-0.00014818),
  DSP16_Q(-0.00001038),
  DSP16_Q(0.00037005),
  DSP16_Q(-0.00135092),
  DSP16_Q(0.00312276),
  DSP16_Q(-0.00591511),
  DSP16_Q(0.01010105),
  DSP16_Q(-0.01641981),
  DSP16_Q(0.02668256),
  DSP16_Q(-0.04684796),
  DSP16_Q(0.11356386),
  DSP16_Q(0.42233109),
  DSP16_Q(-0.07193114),
  DSP16_Q(0.03616242),
  DSP16_Q(-0.02159059),
  DSP16_Q(0.01337436),
  DSP16_Q(-0.00810184),
  DSP16_Q(0.00457437),
  DSP16_Q(-0.00225383),
  DSP16_Q(0.00084444),
  DSP16_Q(-0.00014862),
  DSP16_Q(-0.00001070),
  DSP16_Q(0.00037483),
  DSP16_Q(-0.00136603),
  DSP16_Q(0.00315570),
  DSP16_Q(-0.00597560),
  DSP16_Q(0.01020265),
  DSP16_Q(-0.01658399),
  DSP16_Q(0.02695114),
  DSP16_Q(-0.04733506),
  DSP16_Q(0.11491720),
  DSP16_Q(0.42162113),
  DSP16_Q(-0.07242662),
  DSP16_Q(0.03643779),
  DSP16_Q(-0.02175897),
  DSP16_Q(0.01347874),
  DSP16_Q(-0.00816423),
  DSP16_Q(0.00460858),
  DSP16_Q(-0.00226973),
  DSP16_Q(0.00084962),
  DSP16_Q(-0.00014905),
  DSP16_Q(-0.00001102),
  DSP16_Q(0.00037961),
  DSP16_Q(-0.00138113),
  DSP16_Q(0.00318857),
  DSP16_Q(-0.00603593),
  DSP16_Q(0.01030394),
  DSP16_Q(-0.01674765),
  DSP16_Q(0.02721895),
  DSP16_Q(-0.04782116),
  DSP16_Q(0.11627269),
  DSP16_Q(0.42090443),
  DSP16_Q(-0.07291640),
  DSP16_Q(0.03671058),
  DSP16_Q(-0.02192587),
  DSP16_Q(0.01358220),
  DSP16_Q(-0.00822604),
  DSP16_Q(0.00464244),
  DSP16_Q(-0.00228545),
  DSP16_Q(0.00085472),
  DSP16_Q(-0.00014946),
  DSP16_Q(-0.00001135),
  DSP16_Q(0.00038440),
  DSP16_Q(-0.00139622),
  DSP16_Q(0.00322137),
  DSP16_Q(-0.00609610),
  DSP16_Q(0.01040493),
  DSP16_Q(-0.01691081),
  DSP16_Q(0.02748596),
  DSP16_Q(-0.04830622),
  DSP16_Q(0.11763031),
  DSP16_Q(0.42018100),
  DSP16_Q(-0.07340047),
  DSP16_Q(0.03698078),
  DSP16_Q(-0.02209126),
  DSP16_Q(0.01368473),
  DSP16_Q(-0.00828727),
  DSP16_Q(0.00467595),
  DSP16_Q(-0.00230098),
  DSP16_Q(0.00085974),
  DSP16_Q(-0.00014984),
  DSP16_Q(-0.00001169),
  DSP16_Q(0.00038921),
  DSP16_Q(-0.00141130),
  DSP16_Q(0.00325410),
  DSP16_Q(-0.00615610),
  DSP16_Q(0.01050560),
  DSP16_Q(-0.01707343),
  DSP16_Q(0.02775217),
  DSP16_Q(-0.04879023),
  DSP16_Q(0.11899001),
  DSP16_Q(0.41945087),
  DSP16_Q(-0.07387884),
  DSP16_Q(0.03724838),
  DSP16_Q(-0.02225514),
  DSP16_Q(0.01378632),
  DSP16_Q(-0.00834791),
  DSP16_Q(0.00470912),
  DSP16_Q(-0.00231632),
  DSP16_Q(0.00086468),
  DSP16_Q(-0.00015021),
  DSP16_Q(-0.00001203),
  DSP16_Q(0.00039402),
  DSP16_Q(-0.00142636),
  DSP16_Q(0.00328677),
  DSP16_Q(-0.00621593),
  DSP16_Q(0.01060595),
  DSP16_Q(-0.01723552),
  DSP16_Q(0.02801755),
  DSP16_Q(-0.04927316),
  DSP16_Q(0.12035177),
  DSP16_Q(0.41871407),
  DSP16_Q(-0.07435151),
  DSP16_Q(0.03751337),
  DSP16_Q(-0.02241752),
  DSP16_Q(0.01388697),
  DSP16_Q(-0.00840798),
  DSP16_Q(0.00474195),
  DSP16_Q(-0.00233149),
  DSP16_Q(0.00086954),
  DSP16_Q(-0.00015055),
  DSP16_Q(-0.00001238),
  DSP16_Q(0.00039884),
  DSP16_Q(-0.00144141),
  DSP16_Q(0.00331936),
  DSP16_Q(-0.00627559),
  DSP16_Q(0.01070598),
  DSP16_Q(-0.01739707),
  DSP16_Q(0.02828208),
  DSP16_Q(-0.04975497),
  DSP16_Q(0.12171554),
  DSP16_Q(0.41797061),
  DSP16_Q(-0.07481847),
  DSP16_Q(0.03777576),
  DSP16_Q(-0.02257838),
  DSP16_Q(0.01398667),
  DSP16_Q(-0.00846746),
  DSP16_Q(0.00477443),
  DSP16_Q(-0.00234646),
  DSP16_Q(0.00087432),
  DSP16_Q(-0.00015088),
  DSP16_Q(-0.00001273),
  DSP16_Q(0.00040367),
  DSP16_Q(-0.00145644),
  DSP16_Q(0.00335187),
  DSP16_Q(-0.00633507),
  DSP16_Q(0.01080567),
  DSP16_Q(-0.01755806),
  DSP16_Q(0.02854577),
  DSP16_Q(-0.05023564),
  DSP16_Q(0.12308130),
  DSP16_Q(0.41722052),
  DSP16_Q(-0.07527974),
  DSP16_Q(0.03803552),
  DSP16_Q(-0.02273771),
  DSP16_Q(0.01408544),
  DSP16_Q(-0.00852635),
  DSP16_Q(0.00480656),
  DSP16_Q(-0.00236126),
  DSP16_Q(0.00087902),
  DSP16_Q(-0.00015118),
  DSP16_Q(-0.00001310),
  DSP16_Q(0.00040851),
  DSP16_Q(-0.00147145),
  DSP16_Q(0.00338431),
  DSP16_Q(-0.00639437),
  DSP16_Q(0.01090503),
  DSP16_Q(-0.01771849),
  DSP16_Q(0.02880858),
  DSP16_Q(-0.05071514),
  DSP16_Q(0.12444900),
  DSP16_Q(0.41646383),
  DSP16_Q(-0.07573531),
  DSP16_Q(0.03829266),
  DSP16_Q(-0.02289552),
  DSP16_Q(0.01418325),
  DSP16_Q(-0.00858465),
  DSP16_Q(0.00483834),
  DSP16_Q(-0.00237586),
  DSP16_Q(0.00088364),
  DSP16_Q(-0.00015147),
  DSP16_Q(-0.00001346),
  DSP16_Q(0.00041335),
  DSP16_Q(-0.00148645),
  DSP16_Q(0.00341667),
  DSP16_Q(-0.00645348),
  DSP16_Q(0.01100404),
  DSP16_Q(-0.01787834),
  DSP16_Q(0.02907051),
  DSP16_Q(-0.05119346),
  DSP16_Q(0.12581862),
  DSP16_Q(0.41570055),
  DSP16_Q(-0.07618518),
  DSP16_Q(0.03854718),
  DSP16_Q(-0.02305180),
  DSP16_Q(0.01428011),
  DSP16_Q(-0.00864237),
  DSP16_Q(0.00486978),
  DSP16_Q(-0.00239028),
  DSP16_Q(0.00088817),
  DSP16_Q(-0.00015173),
  DSP16_Q(-0.00001384),
  DSP16_Q(0.00041821),
  DSP16_Q(-0.00150143),
  DSP16_Q(0.00344894),
  DSP16_Q(-0.00651241),
  DSP16_Q(0.01110270),
  DSP16_Q(-0.01803761),
  DSP16_Q(0.02933154),
  DSP16_Q(-0.05167055),
  DSP16_Q(0.12719011),
  DSP16_Q(0.41493071),
  DSP16_Q(-0.07662935),
  DSP16_Q(0.03879905),
  DSP16_Q(-0.02320655),
  DSP16_Q(0.01437602),
  DSP16_Q(-0.00869949),
  DSP16_Q(0.00490087),
  DSP16_Q(-0.00240452),
  DSP16_Q(0.00089263),
  DSP16_Q(-0.00015198),
  DSP16_Q(-0.00001421),
  DSP16_Q(0.00042307),
  DSP16_Q(-0.00151639),
  DSP16_Q(0.00348114),
  DSP16_Q(-0.00657115),
  DSP16_Q(0.01120100),
  DSP16_Q(-0.01819629),
  DSP16_Q(0.02959165),
  DSP16_Q(-0.05214640),
  DSP16_Q(0.12856343),
  DSP16_Q(0.41415433),
  DSP16_Q(-0.07706784),
  DSP16_Q(0.03904829),
  DSP16_Q(-0.02335975),
  DSP16_Q(0.01447097),
  DSP16_Q(-0.00875602),
  DSP16_Q(0.00493160),
  DSP16_Q(-0.00241857),
  DSP16_Q(0.00089701),
  DSP16_Q(-0.00015221),
  DSP16_Q(-0.00001460),
  DSP16_Q(0.00042794),
  DSP16_Q(-0.00153132),
  DSP16_Q(0.00351325),
  DSP16_Q(-0.00662969),
  DSP16_Q(0.01129894),
  DSP16_Q(-0.01835436),
  DSP16_Q(0.02985082),
  DSP16_Q(-0.05262097),
  DSP16_Q(0.12993857),
  DSP16_Q(0.41337144),
  DSP16_Q(-0.07750063),
  DSP16_Q(0.03929487),
  DSP16_Q(-0.02351142),
  DSP16_Q(0.01456496),
  DSP16_Q(-0.00881195),
  DSP16_Q(0.00496199),
  DSP16_Q(-0.00243243),
  DSP16_Q(0.00090131),
  DSP16_Q(-0.00015242),
  DSP16_Q(-0.00001499),
  DSP16_Q(0.00043281),
  DSP16_Q(-0.00154624),
  DSP16_Q(0.00354527),
  DSP16_Q(-0.00668803),
  DSP16_Q(0.01139652),
  DSP16_Q(-0.01851182),
  DSP16_Q(0.03010905),
  DSP16_Q(-0.05309424),
  DSP16_Q(0.13131546),
  DSP16_Q(0.41258207),
  DSP16_Q(-0.07792774),
  DSP16_Q(0.03953880),
  DSP16_Q(-0.02366153),
  DSP16_Q(0.01465799),
  DSP16_Q(-0.00886728),
  DSP16_Q(0.00499202),
  DSP16_Q(-0.00244611),
  DSP16_Q(0.00090553),
  DSP16_Q(-0.00015261),
  DSP16_Q(-0.00001539),
  DSP16_Q(0.00043769),
  DSP16_Q(-0.00156113),
  DSP16_Q(0.00357720),
  DSP16_Q(-0.00674617),
  DSP16_Q(0.01149372),
  DSP16_Q(-0.01866866),
  DSP16_Q(0.03036632),
  DSP16_Q(-0.05356619),
  DSP16_Q(0.13269409),
  DSP16_Q(0.41178623),
  DSP16_Q(-0.07834917),
  DSP16_Q(0.03978008),
  DSP16_Q(-0.02381009),
  DSP16_Q(0.01475005),
  DSP16_Q(-0.00892202),
  DSP16_Q(0.00502170),
  DSP16_Q(-0.00245960),
  DSP16_Q(0.00090967),
  DSP16_Q(-0.00015278),
  DSP16_Q(-0.00001579),
  DSP16_Q(0.00044258),
  DSP16_Q(-0.00157600),
  DSP16_Q(0.00360904),
  DSP16_Q(-0.00680410),
  DSP16_Q(0.01159053),
  DSP16_Q(-0.01882487),
  DSP16_Q(0.03062261),
  DSP16_Q(-0.05403678),
  DSP16_Q(0.13407442),
  DSP16_Q(0.41098395),
  DSP16_Q(-0.07876491),
  DSP16_Q(0.04001869),
  DSP16_Q(-0.02395709),
  DSP16_Q(0.01484114),
  DSP16_Q(-0.00897616),
  DSP16_Q(0.00505103),
  DSP16_Q(-0.00247291),
  DSP16_Q(0.00091373),
  DSP16_Q(-0.00015293),
  DSP16_Q(-0.00001620),
  DSP16_Q(0.00044747),
  DSP16_Q(-0.00159084),
  DSP16_Q(0.00364079),
  DSP16_Q(-0.00686182),
  DSP16_Q(0.01168696),
  DSP16_Q(-0.01898044),
  DSP16_Q(0.03087790),
  DSP16_Q(-0.05450598),
  DSP16_Q(0.13545640),
  DSP16_Q(0.41017526),
  DSP16_Q(-0.07917498),
  DSP16_Q(0.04025462),
  DSP16_Q(-0.02410253),
  DSP16_Q(0.01493126),
  DSP16_Q(-0.00902970),
  DSP16_Q(0.00508001),
  DSP16_Q(-0.00248602),
  DSP16_Q(0.00091771),
  DSP16_Q(-0.00015307),
  DSP16_Q(-0.00001662),
  DSP16_Q(0.00045237),
  DSP16_Q(-0.00160566),
  DSP16_Q(0.00367244),
  DSP16_Q(-0.00691933),
  DSP16_Q(0.01178300),
  DSP16_Q(-0.01913536),
  DSP16_Q(0.03113219),
  DSP16_Q(-0.05497378),
  DSP16_Q(0.13684001),
  DSP16_Q(0.40936019),
  DSP16_Q(-0.07957938),
  DSP16_Q(0.04048789),
  DSP16_Q(-0.02424640),
  DSP16_Q(0.01502041),
  DSP16_Q(-0.00908263),
  DSP16_Q(0.00510863),
  DSP16_Q(-0.00249896),
  DSP16_Q(0.00092161),
  DSP16_Q(-0.00015318),
  DSP16_Q(-0.00001704),
  DSP16_Q(0.00045727),
  DSP16_Q(-0.00162045),
  DSP16_Q(0.00370399),
  DSP16_Q(-0.00697663),
  DSP16_Q(0.01187864),
  DSP16_Q(-0.01928962),
  DSP16_Q(0.03138545),
  DSP16_Q(-0.05544014),
  DSP16_Q(0.13822520),
  DSP16_Q(0.40853874),
  DSP16_Q(-0.07997810),
  DSP16_Q(0.04071847),
  DSP16_Q(-0.02438870),
  DSP16_Q(0.01510858),
  DSP16_Q(-0.00913496),
  DSP16_Q(0.00513689),
  DSP16_Q(-0.00251170),
  DSP16_Q(0.00092543),
  DSP16_Q(-0.00015328),
  DSP16_Q(-0.00001747),
  DSP16_Q(0.00046218),
  DSP16_Q(-0.00163521),
  DSP16_Q(0.00373545),
  DSP16_Q(-0.00703370),
  DSP16_Q(0.01197388),
  DSP16_Q(-0.01944321),
  DSP16_Q(0.03163767),
  DSP16_Q(-0.05590504),
  DSP16_Q(0.13961194),
  DSP16_Q(0.40771097),
  DSP16_Q(-0.08037116),
  DSP16_Q(0.04094637),
  DSP16_Q(-0.02452943),
  DSP16_Q(0.01519577),
  DSP16_Q(-0.00918668),
  DSP16_Q(0.00516481),
  DSP16_Q(-0.00252426),
  DSP16_Q(0.00092918),
  DSP16_Q(-0.00015336),
  DSP16_Q(-0.00001791),
  DSP16_Q(0.00046709),
  DSP16_Q(-0.00164995),
  DSP16_Q(0.00376680),
  DSP16_Q(-0.00709054),
  DSP16_Q(0.01206871),
  DSP16_Q(-0.01959613),
  DSP16_Q(0.03188884),
  DSP16_Q(-0.05636845),
  DSP16_Q(0.14100019),
  DSP16_Q(0.40687687),
  DSP16_Q(-0.08075856),
  DSP16_Q(0.04117158),
  DSP16_Q(-0.02466859),
  DSP16_Q(0.01528198),
  DSP16_Q(-0.00923780),
  DSP16_Q(0.00519236),
  DSP16_Q(-0.00253663),
  DSP16_Q(0.00093284),
  DSP16_Q(-0.00015343),
  DSP16_Q(-0.00001835),
  DSP16_Q(0.00047201),
  DSP16_Q(-0.00166465),
  DSP16_Q(0.00379804),
  DSP16_Q(-0.00714716),
  DSP16_Q(0.01216312),
  DSP16_Q(-0.01974835),
  DSP16_Q(0.03213894),
  DSP16_Q(-0.05683035),
  DSP16_Q(0.14238991),
  DSP16_Q(0.40603650),
  DSP16_Q(-0.08114030),
  DSP16_Q(0.04139410),
  DSP16_Q(-0.02480616),
  DSP16_Q(0.01536721),
  DSP16_Q(-0.00928831),
  DSP16_Q(0.00521956),
  DSP16_Q(-0.00254882),
  DSP16_Q(0.00093643),
  DSP16_Q(-0.00015347),
  DSP16_Q(-0.00001880),
  DSP16_Q(0.00047693),
  DSP16_Q(-0.00167932),
  DSP16_Q(0.00382919),
  DSP16_Q(-0.00720355),
  DSP16_Q(0.01225711),
  DSP16_Q(-0.01989988),
  DSP16_Q(0.03238795),
  DSP16_Q(-0.05729070),
  DSP16_Q(0.14378108),
  DSP16_Q(0.40518986),
  DSP16_Q(-0.08151639),
  DSP16_Q(0.04161392),
  DSP16_Q(-0.02494214),
  DSP16_Q(0.01545145),
  DSP16_Q(-0.00933821),
  DSP16_Q(0.00524640),
  DSP16_Q(-0.00256082),
  DSP16_Q(0.00093994),
  DSP16_Q(-0.00015350),
  DSP16_Q(-0.00001925),
  DSP16_Q(0.00048185),
  DSP16_Q(-0.00169397),
  DSP16_Q(0.00386022),
  DSP16_Q(-0.00725970),
  DSP16_Q(0.01235067),
  DSP16_Q(-0.02005071),
  DSP16_Q(0.03263586),
  DSP16_Q(-0.05774947),
  DSP16_Q(0.14517364),
  DSP16_Q(0.40433699),
  DSP16_Q(-0.08188682),
  DSP16_Q(0.04183104),
  DSP16_Q(-0.02507654),
  DSP16_Q(0.01553470),
  DSP16_Q(-0.00938750),
  DSP16_Q(0.00527289),
  DSP16_Q(-0.00257263),
  DSP16_Q(0.00094337),
  DSP16_Q(-0.00015351),
  DSP16_Q(-0.00001971),
  DSP16_Q(0.00048678),
  DSP16_Q(-0.00170857),
  DSP16_Q(0.00389114),
  DSP16_Q(-0.00731561),
  DSP16_Q(0.01244379),
  DSP16_Q(-0.02020081),
  DSP16_Q(0.03288266),
  DSP16_Q(-0.05820665),
  DSP16_Q(0.14656757),
  DSP16_Q(0.40347792),
  DSP16_Q(-0.08225162),
  DSP16_Q(0.04204545),
  DSP16_Q(-0.02520935),
  DSP16_Q(0.01561696),
  DSP16_Q(-0.00943618),
  DSP16_Q(0.00529902),
  DSP16_Q(-0.00258425),
  DSP16_Q(0.00094673),
  DSP16_Q(-0.00015350),
  DSP16_Q(-0.00002018),
  DSP16_Q(0.00049171),
  DSP16_Q(-0.00172315),
  DSP16_Q(0.00392195),
  DSP16_Q(-0.00737127),
  DSP16_Q(0.01253648),
  DSP16_Q(-0.02035019),
  DSP16_Q(0.03312832),
  DSP16_Q(-0.05866221),
  DSP16_Q(0.14796283),
  DSP16_Q(0.40261266),
  DSP16_Q(-0.08261078),
  DSP16_Q(0.04225715),
  DSP16_Q(-0.02534056),
  DSP16_Q(0.01569823),
  DSP16_Q(-0.00948424),
  DSP16_Q(0.00532479),
  DSP16_Q(-0.00259569),
  DSP16_Q(0.00095000),
  DSP16_Q(-0.00015348),
  DSP16_Q(-0.00002065),
  DSP16_Q(0.00049664),
  DSP16_Q(-0.00173769),
  DSP16_Q(0.00395265),
  DSP16_Q(-0.00742670),
  DSP16_Q(0.01262872),
  DSP16_Q(-0.02049884),
  DSP16_Q(0.03337284),
  DSP16_Q(-0.05911611),
  DSP16_Q(0.14935937),
  DSP16_Q(0.40174125),
  DSP16_Q(-0.08296430),
  DSP16_Q(0.04246614),
  DSP16_Q(-0.02547017),
  DSP16_Q(0.01577851),
  DSP16_Q(-0.00953170),
  DSP16_Q(0.00535020),
  DSP16_Q(-0.00260694),
  DSP16_Q(0.00095320),
  DSP16_Q(-0.00015344),
  DSP16_Q(-0.00002113),
  DSP16_Q(0.00050157),
  DSP16_Q(-0.00175219),
  DSP16_Q(0.00398323),
  DSP16_Q(-0.00748187),
  DSP16_Q(0.01272051),
  DSP16_Q(-0.02064675),
  DSP16_Q(0.03361620),
  DSP16_Q(-0.05956833),
  DSP16_Q(0.15075717),
  DSP16_Q(0.40086371),
  DSP16_Q(-0.08331220),
  DSP16_Q(0.04267241),
  DSP16_Q(-0.02559819),
  DSP16_Q(0.01585778),
  DSP16_Q(-0.00957853),
  DSP16_Q(0.00537525),
  DSP16_Q(-0.00261801),
  DSP16_Q(0.00095632),
  DSP16_Q(-0.00015338),
  DSP16_Q(-0.00002161),
  DSP16_Q(0.00050650),
  DSP16_Q(-0.00176666),
  DSP16_Q(0.00401369),
  DSP16_Q(-0.00753679),
  DSP16_Q(0.01281184),
  DSP16_Q(-0.02079390),
  DSP16_Q(0.03385838),
  DSP16_Q(-0.06001884),
  DSP16_Q(0.15215618),
  DSP16_Q(0.39998008),
  DSP16_Q(-0.08365448),
  DSP16_Q(0.04287596),
  DSP16_Q(-0.02572459),
  DSP16_Q(0.01593606),
  DSP16_Q(-0.00962475),
  DSP16_Q(0.00539995),
  DSP16_Q(-0.00262888),
  DSP16_Q(0.00095937),
  DSP16_Q(-0.00015331),
  DSP16_Q(-0.00002210),
  DSP16_Q(0.00051144),
  DSP16_Q(-0.00178108),
  DSP16_Q(0.00404403),
  DSP16_Q(-0.00759145),
  DSP16_Q(0.01290271),
  DSP16_Q(-0.02094029),
  DSP16_Q(0.03409937),
  DSP16_Q(-0.06046762),
  DSP16_Q(0.15355636),
  DSP16_Q(0.39909037),
  DSP16_Q(-0.08399114),
  DSP16_Q(0.04307679),
  DSP16_Q(-0.02584939),
  DSP16_Q(0.01601334),
  DSP16_Q(-0.00967036),
  DSP16_Q(0.00542428),
  DSP16_Q(-0.00263957),
  DSP16_Q(0.00096234),
  DSP16_Q(-0.00015322),
  DSP16_Q(-0.00002260),
  DSP16_Q(0.00051637),
  DSP16_Q(-0.00179547),
  DSP16_Q(0.00407425),
  DSP16_Q(-0.00764585),
  DSP16_Q(0.01299311),
  DSP16_Q(-0.02108590),
  DSP16_Q(0.03433916),
  DSP16_Q(-0.06091464),
  DSP16_Q(0.15495769),
  DSP16_Q(0.39819461),
  DSP16_Q(-0.08432219),
  DSP16_Q(0.04327488),
  DSP16_Q(-0.02597258),
  DSP16_Q(0.01608962),
  DSP16_Q(-0.00971534),
  DSP16_Q(0.00544826),
  DSP16_Q(-0.00265008),
  DSP16_Q(0.00096523),
  DSP16_Q(-0.00015312),
  DSP16_Q(-0.00002311),
  DSP16_Q(0.00052131),
  DSP16_Q(-0.00180982),
  DSP16_Q(0.00410435),
  DSP16_Q(-0.00769998),
  DSP16_Q(0.01308303),
  DSP16_Q(-0.02123074),
  DSP16_Q(0.03457772),
  DSP16_Q(-0.06135987),
  DSP16_Q(0.15636011),
  DSP16_Q(0.39729284),
  DSP16_Q(-0.08464764),
  DSP16_Q(0.04347025),
  DSP16_Q(-0.02609415),
  DSP16_Q(0.01616489),
  DSP16_Q(-0.00975971),
  DSP16_Q(0.00547187),
  DSP16_Q(-0.00266039),
  DSP16_Q(0.00096804),
  DSP16_Q(-0.00015300),
  DSP16_Q(-0.00002362),
  DSP16_Q(0.00052625),
  DSP16_Q(-0.00182413),
  DSP16_Q(0.00413431),
  DSP16_Q(-0.00775385),
  DSP16_Q(0.01317248),
  DSP16_Q(-0.02137479),
  DSP16_Q(0.03481505),
  DSP16_Q(-0.06180328),
  DSP16_Q(0.15776360),
  DSP16_Q(0.39638508),
  DSP16_Q(-0.08496749),
  DSP16_Q(0.04366288),
  DSP16_Q(-0.02621411),
  DSP16_Q(0.01623915),
  DSP16_Q(-0.00980346),
  DSP16_Q(0.00549512),
  DSP16_Q(-0.00267053),
  DSP16_Q(0.00097078),
  DSP16_Q(-0.00015286),
  DSP16_Q(-0.00002413),
  DSP16_Q(0.00053118),
  DSP16_Q(-0.00183839),
  DSP16_Q(0.00416415),
  DSP16_Q(-0.00780745),
  DSP16_Q(0.01326144),
  DSP16_Q(-0.02151804),
  DSP16_Q(0.03505113),
  DSP16_Q(-0.06224486),
  DSP16_Q(0.15916812),
  DSP16_Q(0.39547136),
  DSP16_Q(-0.08528176),
  DSP16_Q(0.04385278),
  DSP16_Q(-0.02633245),
  DSP16_Q(0.01631241),
  DSP16_Q(-0.00984659),
  DSP16_Q(0.00551802),
  DSP16_Q(-0.00268047),
  DSP16_Q(0.00097344),
  DSP16_Q(-0.00015271),
  DSP16_Q(-0.00002465),
  DSP16_Q(0.00053612),
  DSP16_Q(-0.00185261),
  DSP16_Q(0.00419386),
  DSP16_Q(-0.00786077),
  DSP16_Q(0.01334990),
  DSP16_Q(-0.02166048),
  DSP16_Q(0.03528594),
  DSP16_Q(-0.06268456),
  DSP16_Q(0.16057362),
  DSP16_Q(0.39455171),
  DSP16_Q(-0.08559044),
  DSP16_Q(0.04403994),
  DSP16_Q(-0.02644917),
  DSP16_Q(0.01638466),
  DSP16_Q(-0.00988909),
  DSP16_Q(0.00554055),
  DSP16_Q(-0.00269023),
  DSP16_Q(0.00097603),
  DSP16_Q(-0.00015254),
  DSP16_Q(-0.00002518),
  DSP16_Q(0.00054105),
  DSP16_Q(-0.00186679),
  DSP16_Q(0.00422343),
  DSP16_Q(-0.00791381),
  DSP16_Q(0.01343787),
  DSP16_Q(-0.02180210),
  DSP16_Q(0.03551947),
  DSP16_Q(-0.06312236),
  DSP16_Q(0.16198007),
  DSP16_Q(0.39362616),
  DSP16_Q(-0.08589354),
  DSP16_Q(0.04422435),
  DSP16_Q(-0.02656427),
  DSP16_Q(0.01645590),
  DSP16_Q(-0.00993098),
  DSP16_Q(0.00556272),
  DSP16_Q(-0.00269980),
  DSP16_Q(0.00097854),
  DSP16_Q(-0.00015236),
  DSP16_Q(-0.00002572),
  DSP16_Q(0.00054598),
  DSP16_Q(-0.00188092),
  DSP16_Q(0.00425287),
  DSP16_Q(-0.00796657),
  DSP16_Q(0.01352533),
  DSP16_Q(-0.02194290),
  DSP16_Q(0.03575170),
  DSP16_Q(-0.06355825),
  DSP16_Q(0.16338744),
  DSP16_Q(0.39269472),
  DSP16_Q(-0.08619107),
  DSP16_Q(0.04440603),
  DSP16_Q(-0.02667774),
  DSP16_Q(0.01652612),
  DSP16_Q(-0.00997224),
  DSP16_Q(0.00558453),
  DSP16_Q(-0.00270918),
  DSP16_Q(0.00098097),
  DSP16_Q(-0.00015216),
  DSP16_Q(-0.00002626),
  DSP16_Q(0.00055091),
  DSP16_Q(-0.00189501),
  DSP16_Q(0.00428217),
  DSP16_Q(-0.00801905),
  DSP16_Q(0.01361229),
  DSP16_Q(-0.02208286),
  DSP16_Q(0.03598262),
  DSP16_Q(-0.06399218),
  DSP16_Q(0.16479567),
  DSP16_Q(0.39175744),
  DSP16_Q(-0.08648305),
  DSP16_Q(0.04458495),
  DSP16_Q(-0.02678958),
  DSP16_Q(0.01659533),
  DSP16_Q(-0.01001287),
  DSP16_Q(0.00560598),
  DSP16_Q(-0.00271838),
  DSP16_Q(0.00098333),
  DSP16_Q(-0.00015194),
  DSP16_Q(-0.00002681),
  DSP16_Q(0.00055584),
  DSP16_Q(-0.00190904),
  DSP16_Q(0.00431134),
  DSP16_Q(-0.00807123),
  DSP16_Q(0.01369873),
  DSP16_Q(-0.02222197),
  DSP16_Q(0.03621222),
  DSP16_Q(-0.06442413),
  DSP16_Q(0.16620474),
  DSP16_Q(0.39081435),
  DSP16_Q(-0.08676947),
  DSP16_Q(0.04476113),
  DSP16_Q(-0.02689979),
  DSP16_Q(0.01666353),
  DSP16_Q(-0.01005289),
  DSP16_Q(0.00562707),
  DSP16_Q(-0.00272740),
  DSP16_Q(0.00098562),
  DSP16_Q(-0.00015172),
  DSP16_Q(-0.00002736),
  DSP16_Q(0.00056076),
  DSP16_Q(-0.00192303),
  DSP16_Q(0.00434036),
  DSP16_Q(-0.00812312),
  DSP16_Q(0.01378465),
  DSP16_Q(-0.02236024),
  DSP16_Q(0.03644048),
  DSP16_Q(-0.06485408),
  DSP16_Q(0.16761461),
  DSP16_Q(0.38986546),
  DSP16_Q(-0.08705034),
  DSP16_Q(0.04493456),
  DSP16_Q(-0.02700836),
  DSP16_Q(0.01673071),
  DSP16_Q(-0.01009228),
  DSP16_Q(0.00564779),
  DSP16_Q(-0.00273623),
  DSP16_Q(0.00098783),
  DSP16_Q(-0.00015147),
  DSP16_Q(-0.00002792),
  DSP16_Q(0.00056569),
  DSP16_Q(-0.00193697),
  DSP16_Q(0.00436924),
  DSP16_Q(-0.00817472),
  DSP16_Q(0.01387004),
  DSP16_Q(-0.02249764),
  DSP16_Q(0.03666738),
  DSP16_Q(-0.06528199),
  DSP16_Q(0.16902523),
  DSP16_Q(0.38891082),
  DSP16_Q(-0.08732568),
  DSP16_Q(0.04510523),
  DSP16_Q(-0.02711530),
  DSP16_Q(0.01679687),
  DSP16_Q(-0.01013104),
  DSP16_Q(0.00566816),
  DSP16_Q(-0.00274487),
  DSP16_Q(0.00098996),
  DSP16_Q(-0.00015122),
  DSP16_Q(-0.00002849),
  DSP16_Q(0.00057060),
  DSP16_Q(-0.00195086),
  DSP16_Q(0.00439797),
  DSP16_Q(-0.00822601),
  DSP16_Q(0.01395490),
  DSP16_Q(-0.02263417),
  DSP16_Q(0.03689291),
  DSP16_Q(-0.06570785),
  DSP16_Q(0.17043658),
  DSP16_Q(0.38795044),
  DSP16_Q(-0.08759548),
  DSP16_Q(0.04527315),
  DSP16_Q(-0.02722061),
  DSP16_Q(0.01686201),
  DSP16_Q(-0.01016918),
  DSP16_Q(0.00568816),
  DSP16_Q(-0.00275333),
  DSP16_Q(0.00099202),
  DSP16_Q(-0.00015094),
  DSP16_Q(-0.00002906),
  DSP16_Q(0.00057552),
  DSP16_Q(-0.00196470),
  DSP16_Q(0.00442656),
  DSP16_Q(-0.00827700),
  DSP16_Q(0.01403922),
  DSP16_Q(-0.02276981),
  DSP16_Q(0.03711706),
  DSP16_Q(-0.06613162),
  DSP16_Q(0.17184860),
  DSP16_Q(0.38698437),
  DSP16_Q(-0.08785977),
  DSP16_Q(0.04543832),
  DSP16_Q(-0.02732427),
  DSP16_Q(0.01692613),
  DSP16_Q(-0.01020669),
  DSP16_Q(0.00570780),
  DSP16_Q(-0.00276160),
  DSP16_Q(0.00099401),
  DSP16_Q(-0.00015066),
  DSP16_Q(-0.00002964),
  DSP16_Q(0.00058043),
  DSP16_Q(-0.00197848),
  DSP16_Q(0.00445499),
  DSP16_Q(-0.00832769),
  DSP16_Q(0.01412299),
  DSP16_Q(-0.02290457),
  DSP16_Q(0.03733981),
  DSP16_Q(-0.06655328),
  DSP16_Q(0.17326127),
  DSP16_Q(0.38601262),
  DSP16_Q(-0.08811854),
  DSP16_Q(0.04560073),
  DSP16_Q(-0.02742630),
  DSP16_Q(0.01698923),
  DSP16_Q(-0.01024357),
  DSP16_Q(0.00572707),
  DSP16_Q(-0.00276969),
  DSP16_Q(0.00099593),
  DSP16_Q(-0.00015036),
  DSP16_Q(-0.00003022),
  DSP16_Q(0.00058533),
  DSP16_Q(-0.00199221),
  DSP16_Q(0.00448328),
  DSP16_Q(-0.00837806),
  DSP16_Q(0.01420622),
  DSP16_Q(-0.02303843),
  DSP16_Q(0.03756115),
  DSP16_Q(-0.06697280),
  DSP16_Q(0.17467453),
  DSP16_Q(0.38503524),
  DSP16_Q(-0.08837181),
  DSP16_Q(0.04576038),
  DSP16_Q(-0.02752668),
  DSP16_Q(0.01705130),
  DSP16_Q(-0.01027982),
  DSP16_Q(0.00574599),
  DSP16_Q(-0.00277759),
  DSP16_Q(0.00099777),
  DSP16_Q(-0.00015005),
  DSP16_Q(-0.00003081),
  DSP16_Q(0.00059023),
  DSP16_Q(-0.00200589),
  DSP16_Q(0.00451141),
  DSP16_Q(-0.00842812),
  DSP16_Q(0.01428890),
  DSP16_Q(-0.02317139),
  DSP16_Q(0.03778105),
  DSP16_Q(-0.06739015),
  DSP16_Q(0.17608837),
  DSP16_Q(0.38405224),
  DSP16_Q(-0.08861958),
  DSP16_Q(0.04591727),
  DSP16_Q(-0.02762542),
  DSP16_Q(0.01711236),
  DSP16_Q(-0.01031545),
  DSP16_Q(0.00576454),
  DSP16_Q(-0.00278531),
  DSP16_Q(0.00099954),
  DSP16_Q(-0.00014972),
  DSP16_Q(-0.00003141),
  DSP16_Q(0.00059513),
  DSP16_Q(-0.00201951),
  DSP16_Q(0.00453938),
  DSP16_Q(-0.00847787),
  DSP16_Q(0.01437101),
  DSP16_Q(-0.02330342),
  DSP16_Q(0.03799952),
  DSP16_Q(-0.06780531),
  DSP16_Q(0.17750273),
  DSP16_Q(0.38306366),
  DSP16_Q(-0.08886187),
  DSP16_Q(0.04607140),
  DSP16_Q(-0.02772252),
  DSP16_Q(0.01717239),
  DSP16_Q(-0.01035045),
  DSP16_Q(0.00578273),
  DSP16_Q(-0.00279284),
  DSP16_Q(0.00100123),
  DSP16_Q(-0.00014938),
  DSP16_Q(-0.00003201),
  DSP16_Q(0.00060002),
  DSP16_Q(-0.00203307),
  DSP16_Q(0.00456720),
  DSP16_Q(-0.00852729),
  DSP16_Q(0.01445255),
  DSP16_Q(-0.02343454),
  DSP16_Q(0.03821653),
  DSP16_Q(-0.06821824),
  DSP16_Q(0.17891757),
  DSP16_Q(0.38206953),
  DSP16_Q(-0.08909868),
  DSP16_Q(0.04622277),
  DSP16_Q(-0.02781796),
  DSP16_Q(0.01723139),
  DSP16_Q(-0.01038482),
  DSP16_Q(0.00580056),
  DSP16_Q(-0.00280019),
  DSP16_Q(0.00100285),
  DSP16_Q(-0.00014903),
  DSP16_Q(-0.00003262),
  DSP16_Q(0.00060490),
  DSP16_Q(-0.00204658),
  DSP16_Q(0.00459486),
  DSP16_Q(-0.00857638),
  DSP16_Q(0.01453353),
  DSP16_Q(-0.02356471),
  DSP16_Q(0.03843206),
  DSP16_Q(-0.06862893),
  DSP16_Q(0.18033287),
  DSP16_Q(0.38106989),
  DSP16_Q(-0.08933003),
  DSP16_Q(0.04637138),
  DSP16_Q(-0.02791176),
  DSP16_Q(0.01728936),
  DSP16_Q(-0.01041856),
  DSP16_Q(0.00581803),
  DSP16_Q(-0.00280735),
  DSP16_Q(0.00100440),
  DSP16_Q(-0.00014866),
  DSP16_Q(-0.00003323),
  DSP16_Q(0.00060977),
  DSP16_Q(-0.00206002),
  DSP16_Q(0.00462235),
  DSP16_Q(-0.00862515),
  DSP16_Q(0.01461393),
  DSP16_Q(-0.02369395),
  DSP16_Q(0.03864612),
  DSP16_Q(-0.06903734),
  DSP16_Q(0.18174857),
  DSP16_Q(0.38006475),
  DSP16_Q(-0.08955591),
  DSP16_Q(0.04651723),
  DSP16_Q(-0.02800391),
  DSP16_Q(0.01734631),
  DSP16_Q(-0.01045168),
  DSP16_Q(0.00583513),
  DSP16_Q(-0.00281434),
  DSP16_Q(0.00100588),
  DSP16_Q(-0.00014828),
  DSP16_Q(-0.00003385),
  DSP16_Q(0.00061464),
  DSP16_Q(-0.00207341),
  DSP16_Q(0.00464969),
  DSP16_Q(-0.00867359),
  DSP16_Q(0.01469374),
  DSP16_Q(-0.02382223),
  DSP16_Q(0.03885867),
  DSP16_Q(-0.06944346),
  DSP16_Q(0.18316464),
  DSP16_Q(0.37905416),
  DSP16_Q(-0.08977635),
  DSP16_Q(0.04666031),
  DSP16_Q(-0.02809441),
  DSP16_Q(0.01740224),
  DSP16_Q(-0.01048416),
  DSP16_Q(0.00585188),
  DSP16_Q(-0.00282114),
  DSP16_Q(0.00100729),
  DSP16_Q(-0.00014789),
  DSP16_Q(-0.00003448),
  DSP16_Q(0.00061950),
  DSP16_Q(-0.00208673),
  DSP16_Q(0.00467685),
  DSP16_Q(-0.00872169),
  DSP16_Q(0.01477297),
  DSP16_Q(-0.02394956),
  DSP16_Q(0.03906970),
  DSP16_Q(-0.06984724),
  DSP16_Q(0.18458104),
  DSP16_Q(0.37803814),
  DSP16_Q(-0.08999136),
  DSP16_Q(0.04680063),
  DSP16_Q(-0.02818326),
  DSP16_Q(0.01745713),
  DSP16_Q(-0.01051601),
  DSP16_Q(0.00586826),
  DSP16_Q(-0.00282775),
  DSP16_Q(0.00100862),
  DSP16_Q(-0.00014749),
  DSP16_Q(-0.00003511),
  DSP16_Q(0.00062436),
  DSP16_Q(-0.00209999),
  DSP16_Q(0.00470385),
  DSP16_Q(-0.00876946),
  DSP16_Q(0.01485160),
  DSP16_Q(-0.02407591),
  DSP16_Q(0.03927921),
  DSP16_Q(-0.07024867),
  DSP16_Q(0.18599773),
  DSP16_Q(0.37701673),
  DSP16_Q(-0.09020093),
  DSP16_Q(0.04693819),
  DSP16_Q(-0.02827046),
  DSP16_Q(0.01751099),
  DSP16_Q(-0.01054723),
  DSP16_Q(0.00588428),
  DSP16_Q(-0.00283418),
  DSP16_Q(0.00100989),
  DSP16_Q(-0.00014707),
  DSP16_Q(-0.00003574),
  DSP16_Q(0.00062920),
  DSP16_Q(-0.00211319),
  DSP16_Q(0.00473068),
  DSP16_Q(-0.00881688),
  DSP16_Q(0.01492964),
  DSP16_Q(-0.02420129),
  DSP16_Q(0.03948717),
  DSP16_Q(-0.07064772),
  DSP16_Q(0.18741468),
  DSP16_Q(0.37598995),
  DSP16_Q(-0.09040509),
  DSP16_Q(0.04707298),
  DSP16_Q(-0.02835600),
  DSP16_Q(0.01756383),
  DSP16_Q(-0.01057783),
  DSP16_Q(0.00589994),
  DSP16_Q(-0.00284043),
  DSP16_Q(0.00101108),
  DSP16_Q(-0.00014664),
  DSP16_Q(-0.00003639),
  DSP16_Q(0.00063404),
  DSP16_Q(-0.00212633),
  DSP16_Q(0.00475734),
  DSP16_Q(-0.00886396),
  DSP16_Q(0.01500706),
  DSP16_Q(-0.02432568),
  DSP16_Q(0.03969358),
  DSP16_Q(-0.07104435),
  DSP16_Q(0.18883183),
  DSP16_Q(0.37495784),
  DSP16_Q(-0.09060384),
  DSP16_Q(0.04720501),
  DSP16_Q(-0.02843989),
  DSP16_Q(0.01761563),
  DSP16_Q(-0.01060779),
  DSP16_Q(0.00591524),
  DSP16_Q(-0.00284650),
  DSP16_Q(0.00101220),
  DSP16_Q(-0.00014620),
  DSP16_Q(-0.00003704),
  DSP16_Q(0.00063886),
  DSP16_Q(-0.00213940),
  DSP16_Q(0.00478382),
  DSP16_Q(-0.00891068),
  DSP16_Q(0.01508388),
  DSP16_Q(-0.02444908),
  DSP16_Q(0.03989841),
  DSP16_Q(-0.07143856),
  DSP16_Q(0.19024916),
  DSP16_Q(0.37392043),
  DSP16_Q(-0.09079720),
  DSP16_Q(0.04733428),
  DSP16_Q(-0.02852212),
  DSP16_Q(0.01766640),
  DSP16_Q(-0.01063712),
  DSP16_Q(0.00593017),
  DSP16_Q(-0.00285239),
  DSP16_Q(0.00101325),
  DSP16_Q(-0.00014575),
  DSP16_Q(-0.00003769),
  DSP16_Q(0.00064368),
  DSP16_Q(-0.00215240),
  DSP16_Q(0.00481013),
  DSP16_Q(-0.00895706),
  DSP16_Q(0.01516009),
  DSP16_Q(-0.02457148),
  DSP16_Q(0.04010166),
  DSP16_Q(-0.07183030),
  DSP16_Q(0.19166663),
  DSP16_Q(0.37287775),
  DSP16_Q(-0.09098517),
  DSP16_Q(0.04746078),
  DSP16_Q(-0.02860270),
  DSP16_Q(0.01771614),
  DSP16_Q(-0.01066582),
  DSP16_Q(0.00594475),
  DSP16_Q(-0.00285809),
  DSP16_Q(0.00101423),
  DSP16_Q(-0.00014528),
  DSP16_Q(-0.00003835),
  DSP16_Q(0.00064849),
  DSP16_Q(-0.00216533),
  DSP16_Q(0.00483626),
  DSP16_Q(-0.00900308),
  DSP16_Q(0.01523567),
  DSP16_Q(-0.02469286),
  DSP16_Q(0.04030331),
  DSP16_Q(-0.07221955),
  DSP16_Q(0.19308418),
  DSP16_Q(0.37182983),
  DSP16_Q(-0.09116777),
  DSP16_Q(0.04758452),
  DSP16_Q(-0.02868162),
  DSP16_Q(0.01776485),
  DSP16_Q(-0.01069390),
  DSP16_Q(0.00595896),
  DSP16_Q(-0.00286362),
  DSP16_Q(0.00101515),
  DSP16_Q(-0.00014481),
  DSP16_Q(-0.00003902),
  DSP16_Q(0.00065328),
  DSP16_Q(-0.00217820),
  DSP16_Q(0.00486221),
  DSP16_Q(-0.00904875),
  DSP16_Q(0.01531062),
  DSP16_Q(-0.02481323),
  DSP16_Q(0.04050335),
  DSP16_Q(-0.07260629),
  DSP16_Q(0.19450180),
  DSP16_Q(0.37077671),
  DSP16_Q(-0.09134500),
  DSP16_Q(0.04770550),
  DSP16_Q(-0.02875888),
  DSP16_Q(0.01781253),
  DSP16_Q(-0.01072134),
  DSP16_Q(0.00597282),
  DSP16_Q(-0.00286896),
  DSP16_Q(0.00101599),
  DSP16_Q(-0.00014432),
  DSP16_Q(-0.00003969),
  DSP16_Q(0.00065807),
  DSP16_Q(-0.00219100),
  DSP16_Q(0.00488798),
  DSP16_Q(-0.00909405),
  DSP16_Q(0.01538495),
  DSP16_Q(-0.02493256),
  DSP16_Q(0.04070176),
  DSP16_Q(-0.07299048),
  DSP16_Q(0.19591942),
  DSP16_Q(0.36971842),
  DSP16_Q(-0.09151689),
  DSP16_Q(0.04782372),
  DSP16_Q(-0.02883449),
  DSP16_Q(0.01785918),
  DSP16_Q(-0.01074815),
  DSP16_Q(0.00598631),
  DSP16_Q(-0.00287412),
  DSP16_Q(0.00101676),
  DSP16_Q(-0.00014382),
  DSP16_Q(-0.00004036),
  DSP16_Q(0.00066284),
  DSP16_Q(-0.00220372),
  DSP16_Q(0.00491356),
  DSP16_Q(-0.00913898),
  DSP16_Q(0.01545864),
  DSP16_Q(-0.02505086),
  DSP16_Q(0.04089853),
  DSP16_Q(-0.07337211),
  DSP16_Q(0.19733703),
  DSP16_Q(0.36865499),
  DSP16_Q(-0.09168343),
  DSP16_Q(0.04793918),
  DSP16_Q(-0.02890844),
  DSP16_Q(0.01790479),
  DSP16_Q(-0.01077433),
  DSP16_Q(0.00599944),
  DSP16_Q(-0.00287911),
  DSP16_Q(0.00101747),
  DSP16_Q(-0.00014331),
  DSP16_Q(-0.00004104),
  DSP16_Q(0.00066760),
  DSP16_Q(-0.00221638),
  DSP16_Q(0.00493896),
  DSP16_Q(-0.00918355),
  DSP16_Q(0.01553168),
  DSP16_Q(-0.02516811),
  DSP16_Q(0.04109364),
  DSP16_Q(-0.07375114),
  DSP16_Q(0.19875457),
  DSP16_Q(0.36758644),
  DSP16_Q(-0.09184465),
  DSP16_Q(0.04805188),
  DSP16_Q(-0.02898073),
  DSP16_Q(0.01794937),
  DSP16_Q(-0.01079988),
  DSP16_Q(0.00601222),
  DSP16_Q(-0.00288391),
  DSP16_Q(0.00101810),
  DSP16_Q(-0.00014279),
  DSP16_Q(-0.00004173),
  DSP16_Q(0.00067235),
  DSP16_Q(-0.00222896),
  DSP16_Q(0.00496417),
  DSP16_Q(-0.00922774),
  DSP16_Q(0.01560408),
  DSP16_Q(-0.02528431),
  DSP16_Q(0.04128708),
  DSP16_Q(-0.07412755),
  DSP16_Q(0.20017200),
  DSP16_Q(0.36651283),
  DSP16_Q(-0.09200056),
  DSP16_Q(0.04816183),
  DSP16_Q(-0.02905137),
  DSP16_Q(0.01799292),
  DSP16_Q(-0.01082480),
  DSP16_Q(0.00602463),
  DSP16_Q(-0.00288853),
  DSP16_Q(0.00101867),
  DSP16_Q(-0.00014226),
  DSP16_Q(-0.00004242),
  DSP16_Q(0.00067709),
  DSP16_Q(-0.00224147),
  DSP16_Q(0.00498919),
  DSP16_Q(-0.00927156),
  DSP16_Q(0.01567582),
  DSP16_Q(-0.02539945),
  DSP16_Q(0.04147884),
  DSP16_Q(-0.07450131),
  DSP16_Q(0.20158930),
  DSP16_Q(0.36543417),
  DSP16_Q(-0.09215116),
  DSP16_Q(0.04826901),
  DSP16_Q(-0.02912034),
  DSP16_Q(0.01803544),
  DSP16_Q(-0.01084909),
  DSP16_Q(0.00603669),
  DSP16_Q(-0.00289298),
  DSP16_Q(0.00101917),
  DSP16_Q(-0.00014172),
  DSP16_Q(-0.00004312),
  DSP16_Q(0.00068181),
  DSP16_Q(-0.00225390),
  DSP16_Q(0.00501402),
  DSP16_Q(-0.00931500),
  DSP16_Q(0.01574691),
  DSP16_Q(-0.02551352),
  DSP16_Q(0.04166891),
  DSP16_Q(-0.07487240),
  DSP16_Q(0.20300641),
  DSP16_Q(0.36435050),
  DSP16_Q(-0.09229647),
  DSP16_Q(0.04837344),
  DSP16_Q(-0.02918766),
  DSP16_Q(0.01807692),
  DSP16_Q(-0.01087275),
  DSP16_Q(0.00604839),
  DSP16_Q(-0.00289724),
  DSP16_Q(0.00101961),
  DSP16_Q(-0.00014117),
  DSP16_Q(-0.00004383),
  DSP16_Q(0.00068652),
  DSP16_Q(-0.00226626),
  DSP16_Q(0.00503865),
  DSP16_Q(-0.00935806),
  DSP16_Q(0.01581733),
  DSP16_Q(-0.02562651),
  DSP16_Q(0.04185726),
  DSP16_Q(-0.07524079),
  DSP16_Q(0.20442329),
  DSP16_Q(0.36326186),
  DSP16_Q(-0.09243650),
  DSP16_Q(0.04847512),
  DSP16_Q(-0.02925333),
  DSP16_Q(0.01811737),
  DSP16_Q(-0.01089578),
  DSP16_Q(0.00605973),
  DSP16_Q(-0.00290133),
  DSP16_Q(0.00101997),
  DSP16_Q(-0.00014061),
  DSP16_Q(-0.00004453),
  DSP16_Q(0.00069122),
  DSP16_Q(-0.00227854),
  DSP16_Q(0.00506309),
  DSP16_Q(-0.00940073),
  DSP16_Q(0.01588709),
  DSP16_Q(-0.02573841),
  DSP16_Q(0.04204389),
  DSP16_Q(-0.07560645),
  DSP16_Q(0.20583992),
  DSP16_Q(0.36216828),
  DSP16_Q(-0.09257127),
  DSP16_Q(0.04857405),
  DSP16_Q(-0.02931733),
  DSP16_Q(0.01815679),
  DSP16_Q(-0.01091819),
  DSP16_Q(0.00607071),
  DSP16_Q(-0.00290524),
  DSP16_Q(0.00102027),
  DSP16_Q(-0.00014004),
  DSP16_Q(-0.00004525),
  DSP16_Q(0.00069590),
  DSP16_Q(-0.00229074),
  DSP16_Q(0.00508733),
  DSP16_Q(-0.00944301),
  DSP16_Q(0.01595617),
  DSP16_Q(-0.02584922),
  DSP16_Q(0.04222878),
  DSP16_Q(-0.07596936),
  DSP16_Q(0.20725624),
  DSP16_Q(0.36106978),
  DSP16_Q(-0.09270078),
  DSP16_Q(0.04867022),
  DSP16_Q(-0.02937968),
  DSP16_Q(0.01819518),
  DSP16_Q(-0.01093996),
  DSP16_Q(0.00608134),
  DSP16_Q(-0.00290897),
  DSP16_Q(0.00102051),
  DSP16_Q(-0.00013946),
  DSP16_Q(-0.00004597),
  DSP16_Q(0.00070057),
  DSP16_Q(-0.00230286),
  DSP16_Q(0.00511137),
  DSP16_Q(-0.00948491),
  DSP16_Q(0.01602457),
  DSP16_Q(-0.02595892),
  DSP16_Q(0.04241193),
  DSP16_Q(-0.07632949),
  DSP16_Q(0.20867221),
  DSP16_Q(0.35996641),
  DSP16_Q(-0.09282505),
  DSP16_Q(0.04876365),
  DSP16_Q(-0.02944037),
  DSP16_Q(0.01823253),
  DSP16_Q(-0.01096111),
  DSP16_Q(0.00609161),
  DSP16_Q(-0.00291253),
  DSP16_Q(0.00102067),
  DSP16_Q(-0.00013887),
  DSP16_Q(-0.00004669),
  DSP16_Q(0.00070522),
  DSP16_Q(-0.00231490),
  DSP16_Q(0.00513521),
  DSP16_Q(-0.00952640),
  DSP16_Q(0.01609229),
  DSP16_Q(-0.02606752),
  DSP16_Q(0.04259331),
  DSP16_Q(-0.07668681),
  DSP16_Q(0.21008781),
  DSP16_Q(0.35885820),
  DSP16_Q(-0.09294410),
  DSP16_Q(0.04885433),
  DSP16_Q(-0.02949940),
  DSP16_Q(0.01826886),
  DSP16_Q(-0.01098163),
  DSP16_Q(0.00610152),
  DSP16_Q(-0.00291591),
  DSP16_Q(0.00102078),
  DSP16_Q(-0.00013827),
  DSP16_Q(-0.00004742),
  DSP16_Q(0.00070986),
  DSP16_Q(-0.00232687),
  DSP16_Q(0.00515885),
  DSP16_Q(-0.00956750),
  DSP16_Q(0.01615932),
  DSP16_Q(-0.02617499),
  DSP16_Q(0.04277291),
  DSP16_Q(-0.07704130),
  DSP16_Q(0.21150298),
  DSP16_Q(0.35774518),
  DSP16_Q(-0.09305793),
  DSP16_Q(0.04894227),
  DSP16_Q(-0.02955678),
  DSP16_Q(0.01830415),
  DSP16_Q(-0.01100152),
  DSP16_Q(0.00611107),
  DSP16_Q(-0.00291911),
  DSP16_Q(0.00102081),
  DSP16_Q(-0.00013766),
  DSP16_Q(-0.00004815),
  DSP16_Q(0.00071448),
  DSP16_Q(-0.00233875),
  DSP16_Q(0.00518228),
  DSP16_Q(-0.00960820),
  DSP16_Q(0.01622566),
  DSP16_Q(-0.02628134),
  DSP16_Q(0.04295072),
  DSP16_Q(-0.07739294),
  DSP16_Q(0.21291769),
  DSP16_Q(0.35662739),
  DSP16_Q(-0.09316657),
  DSP16_Q(0.04902746),
  DSP16_Q(-0.02961251),
  DSP16_Q(0.01833841),
  DSP16_Q(-0.01102078),
  DSP16_Q(0.00612027),
  DSP16_Q(-0.00292214),
  DSP16_Q(0.00102078),
  DSP16_Q(-0.00013704),
  DSP16_Q(-0.00004889),
  DSP16_Q(0.00071908),
  DSP16_Q(-0.00235054),
  DSP16_Q(0.00520551),
  DSP16_Q(-0.00964849),
  DSP16_Q(0.01629129),
  DSP16_Q(-0.02638656),
  DSP16_Q(0.04312672),
  DSP16_Q(-0.07774169),
  DSP16_Q(0.21433190),
  DSP16_Q(0.35550486),
  DSP16_Q(-0.09327002),
  DSP16_Q(0.04910992),
  DSP16_Q(-0.02966658),
  DSP16_Q(0.01837164),
  DSP16_Q(-0.01103942),
  DSP16_Q(0.00612912),
  DSP16_Q(-0.00292499),
  DSP16_Q(0.00102069),
  DSP16_Q(-0.00013641),
  DSP16_Q(-0.00004963),
  DSP16_Q(0.00072367),
  DSP16_Q(-0.00236225),
  DSP16_Q(0.00522852),
  DSP16_Q(-0.00968838),
  DSP16_Q(0.01635622),
  DSP16_Q(-0.02649063),
  DSP16_Q(0.04330091),
  DSP16_Q(-0.07808753),
  DSP16_Q(0.21574557),
  DSP16_Q(0.35437762),
  DSP16_Q(-0.09336829),
  DSP16_Q(0.04918964),
  DSP16_Q(-0.02971899),
  DSP16_Q(0.01840385),
  DSP16_Q(-0.01105743),
  DSP16_Q(0.00613761),
  DSP16_Q(-0.00292767),
  DSP16_Q(0.00102053),
  DSP16_Q(-0.00013577),
  DSP16_Q(-0.00005038),
  DSP16_Q(0.00072824),
  DSP16_Q(-0.00237388),
  DSP16_Q(0.00525132),
  DSP16_Q(-0.00972785),
  DSP16_Q(0.01642045),
  DSP16_Q(-0.02659355),
  DSP16_Q(0.04347327),
  DSP16_Q(-0.07843044),
  DSP16_Q(0.21715865),
  DSP16_Q(0.35324571),
  DSP16_Q(-0.09346141),
  DSP16_Q(0.04926662),
  DSP16_Q(-0.02976976),
  DSP16_Q(0.01843502),
  DSP16_Q(-0.01107481),
  DSP16_Q(0.00614575),
  DSP16_Q(-0.00293018),
  DSP16_Q(0.00102031),
  DSP16_Q(-0.00013513),
  DSP16_Q(-0.00005114),
  DSP16_Q(0.00073279),
  DSP16_Q(-0.00238542),
  DSP16_Q(0.00527391),
  DSP16_Q(-0.00976691),
  DSP16_Q(0.01648396),
  DSP16_Q(-0.02669531),
  DSP16_Q(0.04364378),
  DSP16_Q(-0.07877038),
  DSP16_Q(0.21857111),
  DSP16_Q(0.35210916),
  DSP16_Q(-0.09354939),
  DSP16_Q(0.04934088),
  DSP16_Q(-0.02981887),
  DSP16_Q(0.01846516),
  DSP16_Q(-0.01109157),
  DSP16_Q(0.00615353),
  DSP16_Q(-0.00293251),
  DSP16_Q(0.00102003),
  DSP16_Q(-0.00013447),
  DSP16_Q(-0.00005189),
  DSP16_Q(0.00073732),
  DSP16_Q(-0.00239687),
  DSP16_Q(0.00529629),
  DSP16_Q(-0.00980556),
  DSP16_Q(0.01654675),
  DSP16_Q(-0.02679591),
  DSP16_Q(0.04381244),
  DSP16_Q(-0.07910734),
  DSP16_Q(0.21998291),
  DSP16_Q(0.35096801),
  DSP16_Q(-0.09363223),
  DSP16_Q(0.04941241),
  DSP16_Q(-0.02986633),
  DSP16_Q(0.01849428),
  DSP16_Q(-0.01110771),
  DSP16_Q(0.00616097),
  DSP16_Q(-0.00293467),
  DSP16_Q(0.00101968),
  DSP16_Q(-0.00013381),
  DSP16_Q(-0.00005265),
  DSP16_Q(0.00074184),
  DSP16_Q(-0.00240823),
  DSP16_Q(0.00531844),
  DSP16_Q(-0.00984378),
  DSP16_Q(0.01660881),
  DSP16_Q(-0.02689533),
  DSP16_Q(0.04397923),
  DSP16_Q(-0.07944128),
  DSP16_Q(0.22139401),
  DSP16_Q(0.34982229),
  DSP16_Q(-0.09370996),
  DSP16_Q(0.04948121),
  DSP16_Q(-0.02991214),
  DSP16_Q(0.01852237),
  DSP16_Q(-0.01112322),
  DSP16_Q(0.00616805),
  DSP16_Q(-0.00293665),
  DSP16_Q(0.00101927),
  DSP16_Q(-0.00013314),
  DSP16_Q(-0.00005342),
  DSP16_Q(0.00074633),
  DSP16_Q(-0.00241951),
  DSP16_Q(0.00534038),
  DSP16_Q(-0.00988158),
  DSP16_Q(0.01667015),
  DSP16_Q(-0.02699357),
  DSP16_Q(0.04414414),
  DSP16_Q(-0.07977219),
  DSP16_Q(0.22280436),
  DSP16_Q(0.34867204),
  DSP16_Q(-0.09378259),
  DSP16_Q(0.04954729),
  DSP16_Q(-0.02995631),
  DSP16_Q(0.01854943),
  DSP16_Q(-0.01113811),
  DSP16_Q(0.00617477),
  DSP16_Q(-0.00293846),
  DSP16_Q(0.00101879),
  DSP16_Q(-0.00013246),
  DSP16_Q(-0.00005419),
  DSP16_Q(0.00075081),
  DSP16_Q(-0.00243069),
  DSP16_Q(0.00536210),
  DSP16_Q(-0.00991895),
  DSP16_Q(0.01673075),
  DSP16_Q(-0.02709063),
  DSP16_Q(0.04430715),
  DSP16_Q(-0.08010003),
  DSP16_Q(0.22421393),
  DSP16_Q(0.34751730),
  DSP16_Q(-0.09385013),
  DSP16_Q(0.04961065),
  DSP16_Q(-0.02999883),
  DSP16_Q(0.01857547),
  DSP16_Q(-0.01115238),
  DSP16_Q(0.00618115),
  DSP16_Q(-0.00294011),
  DSP16_Q(0.00101826),
  DSP16_Q(-0.00013178),
  DSP16_Q(-0.00005497),
  DSP16_Q(0.00075526),
  DSP16_Q(-0.00244178),
  DSP16_Q(0.00538360),
  DSP16_Q(-0.00995589),
  DSP16_Q(0.01679062),
  DSP16_Q(-0.02718648),
  DSP16_Q(0.04446826),
  DSP16_Q(-0.08042478),
  DSP16_Q(0.22562267),
  DSP16_Q(0.34635809),
  DSP16_Q(-0.09391260),
  DSP16_Q(0.04967130),
  DSP16_Q(-0.03003970),
  DSP16_Q(0.01860048),
  DSP16_Q(-0.01116603),
  DSP16_Q(0.00618718),
  DSP16_Q(-0.00294158),
  DSP16_Q(0.00101766),
  DSP16_Q(-0.00013109),
  DSP16_Q(-0.00005575),
  DSP16_Q(0.00075970),
  DSP16_Q(-0.00245278),
  DSP16_Q(0.00540487),
  DSP16_Q(-0.00999240),
  DSP16_Q(0.01684974),
  DSP16_Q(-0.02728113),
  DSP16_Q(0.04462744),
  DSP16_Q(-0.08074641),
  DSP16_Q(0.22703055),
  DSP16_Q(0.34519445),
  DSP16_Q(-0.09397002),
  DSP16_Q(0.04972923),
  DSP16_Q(-0.03007892),
  DSP16_Q(0.01862447),
  DSP16_Q(-0.01117905),
  DSP16_Q(0.00619286),
  DSP16_Q(-0.00294288),
  DSP16_Q(0.00101700),
  DSP16_Q(-0.00013039),
  DSP16_Q(-0.00005653),
  DSP16_Q(0.00076411),
  DSP16_Q(-0.00246369),
  DSP16_Q(0.00542591),
  DSP16_Q(-0.01002848),
  DSP16_Q(0.01690811),
  DSP16_Q(-0.02737457),
  DSP16_Q(0.04478469),
  DSP16_Q(-0.08106491),
  DSP16_Q(0.22843753),
  DSP16_Q(0.34402642),
  DSP16_Q(-0.09402240),
  DSP16_Q(0.04978446),
  DSP16_Q(-0.03011651),
  DSP16_Q(0.01864744),
  DSP16_Q(-0.01119146),
  DSP16_Q(0.00619819),
  DSP16_Q(-0.00294401),
  DSP16_Q(0.00101628),
  DSP16_Q(-0.00012968),
  DSP16_Q(-0.00005732),
  DSP16_Q(0.00076850),
  DSP16_Q(-0.00247450),
  DSP16_Q(0.00544673),
  DSP16_Q(-0.01006411),
  DSP16_Q(0.01696574),
  DSP16_Q(-0.02746679),
  DSP16_Q(0.04494000),
  DSP16_Q(-0.08138024),
  DSP16_Q(0.22984356),
  DSP16_Q(0.34285403),
  DSP16_Q(-0.09406975),
  DSP16_Q(0.04983698),
  DSP16_Q(-0.03015245),
  DSP16_Q(0.01866938),
  DSP16_Q(-0.01120324),
  DSP16_Q(0.00620317),
  DSP16_Q(-0.00294497),
  DSP16_Q(0.00101550),
  DSP16_Q(-0.00012896),
  DSP16_Q(-0.00005811),
  DSP16_Q(0.00077287),
  DSP16_Q(-0.00248522),
  DSP16_Q(0.00546732),
  DSP16_Q(-0.01009930),
  DSP16_Q(0.01702260),
  DSP16_Q(-0.02755779),
  DSP16_Q(0.04509334),
  DSP16_Q(-0.08169237),
  DSP16_Q(0.23124861),
  DSP16_Q(0.34167732),
  DSP16_Q(-0.09411209),
  DSP16_Q(0.04988681),
  DSP16_Q(-0.03018676),
  DSP16_Q(0.01869030),
  DSP16_Q(-0.01121441),
  DSP16_Q(0.00620780),
  DSP16_Q(-0.00294577),
  DSP16_Q(0.00101466),
  DSP16_Q(-0.00012824),
  DSP16_Q(-0.00005890),
  DSP16_Q(0.00077722),
  DSP16_Q(-0.00249583),
  DSP16_Q(0.00548767),
  DSP16_Q(-0.01013405),
  DSP16_Q(0.01707870),
  DSP16_Q(-0.02764755),
  DSP16_Q(0.04524472),
  DSP16_Q(-0.08200129),
  DSP16_Q(0.23265264),
  DSP16_Q(0.34049633),
  DSP16_Q(-0.09414944),
  DSP16_Q(0.04993394),
  DSP16_Q(-0.03021942),
  DSP16_Q(0.01871021),
  DSP16_Q(-0.01122496),
  DSP16_Q(0.00621209),
  DSP16_Q(-0.00294639),
  DSP16_Q(0.00101376),
  DSP16_Q(-0.00012751),
  DSP16_Q(-0.00005970),
  DSP16_Q(0.00078154),
  DSP16_Q(-0.00250636),
  DSP16_Q(0.00550779),
  DSP16_Q(-0.01016835),
  DSP16_Q(0.01713403),
  DSP16_Q(-0.02773606),
  DSP16_Q(0.04539411),
  DSP16_Q(-0.08230697),
  DSP16_Q(0.23405560),
  DSP16_Q(0.33931108),
  DSP16_Q(-0.09418181),
  DSP16_Q(0.04997837),
  DSP16_Q(-0.03025046),
  DSP16_Q(0.01872909),
  DSP16_Q(-0.01123490),
  DSP16_Q(0.00621603),
  DSP16_Q(-0.00294685),
  DSP16_Q(0.00101280),
  DSP16_Q(-0.00012678),
  DSP16_Q(-0.00006051),
  DSP16_Q(0.00078584),
  DSP16_Q(-0.00251678),
  DSP16_Q(0.00552768),
  DSP16_Q(-0.01020220),
  DSP16_Q(0.01718860),
  DSP16_Q(-0.02782333),
  DSP16_Q(0.04554151),
  DSP16_Q(-0.08260938),
  DSP16_Q(0.23545745),
  DSP16_Q(0.33812163),
  DSP16_Q(-0.09420923),
  DSP16_Q(0.05002012),
  DSP16_Q(-0.03027985),
  DSP16_Q(0.01874696),
  DSP16_Q(-0.01124422),
  DSP16_Q(0.00621963),
  DSP16_Q(-0.00294714),
  DSP16_Q(0.00101178),
  DSP16_Q(-0.00012604),
  DSP16_Q(-0.00006131),
  DSP16_Q(0.00079012),
  DSP16_Q(-0.00252710),
  DSP16_Q(0.00554732),
  DSP16_Q(-0.01023560),
  DSP16_Q(0.01724239),
  DSP16_Q(-0.02790935),
  DSP16_Q(0.04568690),
  DSP16_Q(-0.08290850),
  DSP16_Q(0.23685816),
  DSP16_Q(0.33692799),
  DSP16_Q(-0.09423169),
  DSP16_Q(0.05005919),
  DSP16_Q(-0.03030762),
  DSP16_Q(0.01876382),
  DSP16_Q(-0.01125292),
  DSP16_Q(0.00622288),
  DSP16_Q(-0.00294726),
  DSP16_Q(0.00101071),
  DSP16_Q(-0.00012529),
  DSP16_Q(-0.00006212),
  DSP16_Q(0.00079438),
  DSP16_Q(-0.00253732),
  DSP16_Q(0.00556673),
  DSP16_Q(-0.01026854),
  DSP16_Q(0.01729539),
  DSP16_Q(-0.02799410),
  DSP16_Q(0.04583028),
  DSP16_Q(-0.08320430),
  DSP16_Q(0.23825769),
  DSP16_Q(0.33573021),
  DSP16_Q(-0.09424923),
  DSP16_Q(0.05009558),
  DSP16_Q(-0.03033375),
  DSP16_Q(0.01877965),
  DSP16_Q(-0.01126101),
  DSP16_Q(0.00622579),
  DSP16_Q(-0.00294722),
  DSP16_Q(0.00100957),
  DSP16_Q(-0.00012453),
  DSP16_Q(-0.00006294),
  DSP16_Q(0.00079860),
  DSP16_Q(-0.00254744),
  DSP16_Q(0.00558590),
  DSP16_Q(-0.01030102),
  DSP16_Q(0.01734761),
  DSP16_Q(-0.02807758),
  DSP16_Q(0.04597162),
  DSP16_Q(-0.08349676),
  DSP16_Q(0.23965599),
  DSP16_Q(0.33452833),
  DSP16_Q(-0.09426185),
  DSP16_Q(0.05012929),
  DSP16_Q(-0.03035826),
  DSP16_Q(0.01879448),
  DSP16_Q(-0.01126849),
  DSP16_Q(0.00622836),
  DSP16_Q(-0.00294702),
  DSP16_Q(0.00100838),
  DSP16_Q(-0.00012377),
  DSP16_Q(-0.00006375),
  DSP16_Q(0.00080281),
  DSP16_Q(-0.00255746),
  DSP16_Q(0.00560482),
  DSP16_Q(-0.01033304),
  DSP16_Q(0.01739904),
  DSP16_Q(-0.02815978),
  DSP16_Q(0.04611091),
  DSP16_Q(-0.08378585),
  DSP16_Q(0.24105302),
  DSP16_Q(0.33332238),
  DSP16_Q(-0.09426958),
  DSP16_Q(0.05016034),
  DSP16_Q(-0.03038115),
  DSP16_Q(0.01880829),
  DSP16_Q(-0.01127536),
  DSP16_Q(0.00623058),
  DSP16_Q(-0.00294665),
  DSP16_Q(0.00100713),
  DSP16_Q(-0.00012301),
  DSP16_Q(-0.00006457),
  DSP16_Q(0.00080698),
  DSP16_Q(-0.00256737),
  DSP16_Q(0.00562350),
  DSP16_Q(-0.01036459),
  DSP16_Q(0.01744968),
  DSP16_Q(-0.02824070),
  DSP16_Q(0.04624816),
  DSP16_Q(-0.08407156),
  DSP16_Q(0.24244875),
  DSP16_Q(0.33211239),
  DSP16_Q(-0.09427243),
  DSP16_Q(0.05018872),
  DSP16_Q(-0.03040241),
  DSP16_Q(0.01882110),
  DSP16_Q(-0.01128162),
  DSP16_Q(0.00623246),
  DSP16_Q(-0.00294611),
  DSP16_Q(0.00100582),
  DSP16_Q(-0.00012224),
  DSP16_Q(-0.00006540),
  DSP16_Q(0.00081114),
  DSP16_Q(-0.00257718),
  DSP16_Q(0.00564194),
  DSP16_Q(-0.01039567),
  DSP16_Q(0.01749952),
  DSP16_Q(-0.02832033),
  DSP16_Q(0.04638333),
  DSP16_Q(-0.08435384),
  DSP16_Q(0.24384313),
  DSP16_Q(0.33089841),
  DSP16_Q(-0.09427042),
  DSP16_Q(0.05021445),
  DSP16_Q(-0.03042205),
  DSP16_Q(0.01883289),
  DSP16_Q(-0.01128727),
  DSP16_Q(0.00623401),
  DSP16_Q(-0.00294542),
  DSP16_Q(0.00100446),
  DSP16_Q(-0.00012146),
  DSP16_Q(-0.00006623),
  DSP16_Q(0.00081526),
  DSP16_Q(-0.00258688),
  DSP16_Q(0.00566012),
  DSP16_Q(-0.01042629),
  DSP16_Q(0.01754856),
  DSP16_Q(-0.02839866),
  DSP16_Q(0.04651643),
  DSP16_Q(-0.08463268),
  DSP16_Q(0.24523612),
  DSP16_Q(0.32968047),
  DSP16_Q(-0.09426356),
  DSP16_Q(0.05023752),
  DSP16_Q(-0.03044007),
  DSP16_Q(0.01884368),
  DSP16_Q(-0.01129231),
  DSP16_Q(0.00623521),
  DSP16_Q(-0.00294456),
  DSP16_Q(0.00100304),
  DSP16_Q(-0.00012068),
  DSP16_Q(-0.00006706),
  DSP16_Q(0.00081936),
  DSP16_Q(-0.00259647),
  DSP16_Q(0.00567806),
  DSP16_Q(-0.01045643),
  DSP16_Q(0.01759679),
  DSP16_Q(-0.02847569),
  DSP16_Q(0.04664743),
  DSP16_Q(-0.08490806),
  DSP16_Q(0.24662769),
  DSP16_Q(0.32845861),
  DSP16_Q(-0.09425187),
  DSP16_Q(0.05025794),
  DSP16_Q(-0.03045648),
  DSP16_Q(0.01885346),
  DSP16_Q(-0.01129674),
  DSP16_Q(0.00623608),
  DSP16_Q(-0.00294354),
  DSP16_Q(0.00100156),
  DSP16_Q(-0.00011989),
  DSP16_Q(-0.00006789),
  DSP16_Q(0.00082343),
  DSP16_Q(-0.00260596),
  DSP16_Q(0.00569574),
  DSP16_Q(-0.01048609),
  DSP16_Q(0.01764421),
  DSP16_Q(-0.02855140),
  DSP16_Q(0.04677633),
  DSP16_Q(-0.08517995),
  DSP16_Q(0.24801779),
  DSP16_Q(0.32723285),
  DSP16_Q(-0.09423538),
  DSP16_Q(0.05027572),
  DSP16_Q(-0.03047127),
  DSP16_Q(0.01886224),
  DSP16_Q(-0.01130057),
  DSP16_Q(0.00623661),
  DSP16_Q(-0.00294236),
  DSP16_Q(0.00100003),
  DSP16_Q(-0.00011910),
  DSP16_Q(-0.00006872),
  DSP16_Q(0.00082747),
  DSP16_Q(-0.00261534),
  DSP16_Q(0.00571317),
  DSP16_Q(-0.01051527),
  DSP16_Q(0.01769081),
  DSP16_Q(-0.02862580),
  DSP16_Q(0.04690312),
  DSP16_Q(-0.08544832),
  DSP16_Q(0.24940638),
  DSP16_Q(0.32600326),
  DSP16_Q(-0.09421409),
  DSP16_Q(0.05029086),
  DSP16_Q(-0.03048446),
  DSP16_Q(0.01887002),
  DSP16_Q(-0.01130380),
  DSP16_Q(0.00623681),
  DSP16_Q(-0.00294101),
  DSP16_Q(0.00099844),
  DSP16_Q(-0.00011830),
  DSP16_Q(-0.00006956),
  DSP16_Q(0.00083149),
  DSP16_Q(-0.00262461),
  DSP16_Q(0.00573035),
  DSP16_Q(-0.01054397),
  DSP16_Q(0.01773659),
  DSP16_Q(-0.02869887),
  DSP16_Q(0.04702779),
  DSP16_Q(-0.08571315),
  DSP16_Q(0.25079342),
  DSP16_Q(0.32476984),
  DSP16_Q(-0.09418803),
  DSP16_Q(0.05030337),
  DSP16_Q(-0.03049604),
  DSP16_Q(0.01887680),
  DSP16_Q(-0.01130642),
  DSP16_Q(0.00623666),
  DSP16_Q(-0.00293951),
  DSP16_Q(0.00099680),
  DSP16_Q(-0.00011750),
  DSP16_Q(-0.00007040),
  DSP16_Q(0.00083547),
  DSP16_Q(-0.00263377),
  DSP16_Q(0.00574727),
  DSP16_Q(-0.01057219),
  DSP16_Q(0.01778155),
  DSP16_Q(-0.02877060),
  DSP16_Q(0.04715031),
  DSP16_Q(-0.08597443),
  DSP16_Q(0.25217888),
  DSP16_Q(0.32353266),
  DSP16_Q(-0.09415721),
  DSP16_Q(0.05031326),
  DSP16_Q(-0.03050601),
  DSP16_Q(0.01888258),
  DSP16_Q(-0.01130844),
  DSP16_Q(0.00623619),
  DSP16_Q(-0.00293785),
  DSP16_Q(0.00099511),
  DSP16_Q(-0.00011669),
  DSP16_Q(-0.00007125),
  DSP16_Q(0.00083943),
  DSP16_Q(-0.00264281),
  DSP16_Q(0.00576393),
  DSP16_Q(-0.01059992),
  DSP16_Q(0.01782567),
  DSP16_Q(-0.02884100),
  DSP16_Q(0.04727069),
  DSP16_Q(-0.08623211),
  DSP16_Q(0.25356271),
  DSP16_Q(0.32229173),
  DSP16_Q(-0.09412165),
  DSP16_Q(0.05032052),
  DSP16_Q(-0.03051438),
  DSP16_Q(0.01888736),
  DSP16_Q(-0.01130986),
  DSP16_Q(0.00623538),
  DSP16_Q(-0.00293603),
  DSP16_Q(0.00099336),
  DSP16_Q(-0.00011588),
  DSP16_Q(-0.00007210),
  DSP16_Q(0.00084336),
  DSP16_Q(-0.00265175),
  DSP16_Q(0.00578033),
  DSP16_Q(-0.01062716),
  DSP16_Q(0.01786897),
  DSP16_Q(-0.02891006),
  DSP16_Q(0.04738891),
  DSP16_Q(-0.08648618),
  DSP16_Q(0.25494487),
  DSP16_Q(0.32104711),
  DSP16_Q(-0.09408137),
  DSP16_Q(0.05032517),
  DSP16_Q(-0.03052115),
  DSP16_Q(0.01889115),
  DSP16_Q(-0.01131068),
  DSP16_Q(0.00623424),
  DSP16_Q(-0.00293405),
  DSP16_Q(0.00099156),
  DSP16_Q(-0.00011507),
  DSP16_Q(-0.00007294),
  DSP16_Q(0.00084725),
  DSP16_Q(-0.00266056),
  DSP16_Q(0.00579647),
  DSP16_Q(-0.01065390),
  DSP16_Q(0.01791143),
  DSP16_Q(-0.02897776),
  DSP16_Q(0.04750496),
  DSP16_Q(-0.08673662),
  DSP16_Q(0.25632532),
  DSP16_Q(0.31979882),
  DSP16_Q(-0.09403639),
  DSP16_Q(0.05032722),
  DSP16_Q(-0.03052633),
  DSP16_Q(0.01889395),
  DSP16_Q(-0.01131091),
  DSP16_Q(0.00623277),
  DSP16_Q(-0.00293192),
  DSP16_Q(0.00098971),
  DSP16_Q(-0.00011425),
  DSP16_Q(-0.00007380),
  DSP16_Q(0.00085112),
  DSP16_Q(-0.00266927),
  DSP16_Q(0.00581234),
  DSP16_Q(-0.01068015),
  DSP16_Q(0.01795305),
  DSP16_Q(-0.02904411),
  DSP16_Q(0.04761883),
  DSP16_Q(-0.08698340),
  DSP16_Q(0.25770402),
  DSP16_Q(0.31854691),
  DSP16_Q(-0.09398672),
  DSP16_Q(0.05032666),
  DSP16_Q(-0.03052992),
  DSP16_Q(0.01889575),
  DSP16_Q(-0.01131054),
  DSP16_Q(0.00623096),
  DSP16_Q(-0.00292963),
  DSP16_Q(0.00098780),
  DSP16_Q(-0.00011343),
  DSP16_Q(-0.00007465),
  DSP16_Q(0.00085495),
  DSP16_Q(-0.00267786),
  DSP16_Q(0.00582795),
  DSP16_Q(-0.01070590),
  DSP16_Q(0.01799382),
  DSP16_Q(-0.02910909),
  DSP16_Q(0.04773051),
  DSP16_Q(-0.08722649),
  DSP16_Q(0.25908094),
  DSP16_Q(0.31729141),
  DSP16_Q(-0.09393239),
  DSP16_Q(0.05032350),
  DSP16_Q(-0.03053192),
  DSP16_Q(0.01889657),
  DSP16_Q(-0.01130957),
  DSP16_Q(0.00622883),
  DSP16_Q(-0.00292718),
  DSP16_Q(0.00098584),
  DSP16_Q(-0.00011260),
  DSP16_Q(-0.00007550),
  DSP16_Q(0.00085876),
  DSP16_Q(-0.00268633),
  DSP16_Q(0.00584330),
  DSP16_Q(-0.01073115),
  DSP16_Q(0.01803374),
  DSP16_Q(-0.02917270),
  DSP16_Q(0.04783998),
  DSP16_Q(-0.08746588),
  DSP16_Q(0.26045602),
  DSP16_Q(0.31603235),
  DSP16_Q(-0.09387341),
  DSP16_Q(0.05031776),
  DSP16_Q(-0.03053233),
  DSP16_Q(0.01889640),
  DSP16_Q(-0.01130801),
  DSP16_Q(0.00622637),
  DSP16_Q(-0.00292458),
  DSP16_Q(0.00098383),
  DSP16_Q(-0.00011177),
  DSP16_Q(-0.00007636),
  DSP16_Q(0.00086253),
  DSP16_Q(-0.00269468),
  DSP16_Q(0.00585837),
  DSP16_Q(-0.01075590),
  DSP16_Q(0.01807282),
  DSP16_Q(-0.02923494),
  DSP16_Q(0.04794724),
  DSP16_Q(-0.08770154),
  DSP16_Q(0.26182924),
  DSP16_Q(0.31476979),
  DSP16_Q(-0.09380980),
  DSP16_Q(0.05030943),
  DSP16_Q(-0.03053116),
  DSP16_Q(0.01889525),
  DSP16_Q(-0.01130586),
  DSP16_Q(0.00622359),
  DSP16_Q(-0.00292183),
  DSP16_Q(0.00098177),
  DSP16_Q(-0.00011093),
  DSP16_Q(-0.00007722),
  DSP16_Q(0.00086626),
  DSP16_Q(-0.00270292),
  DSP16_Q(0.00587317),
  DSP16_Q(-0.01078014),
  DSP16_Q(0.01811103),
  DSP16_Q(-0.02929579),
  DSP16_Q(0.04805228),
  DSP16_Q(-0.08793344),
  DSP16_Q(0.26320055),
  DSP16_Q(0.31350375),
  DSP16_Q(-0.09374158),
  DSP16_Q(0.05029853),
  DSP16_Q(-0.03052841),
  DSP16_Q(0.01889311),
  DSP16_Q(-0.01130312),
  DSP16_Q(0.00622047),
  DSP16_Q(-0.00291892),
  DSP16_Q(0.00097966),
  DSP16_Q(-0.00011010),
  DSP16_Q(-0.00007808),
  DSP16_Q(0.00086997),
  DSP16_Q(-0.00271104),
  DSP16_Q(0.00588771),
  DSP16_Q(-0.01080387),
  DSP16_Q(0.01814838),
  DSP16_Q(-0.02935525),
  DSP16_Q(0.04815508),
  DSP16_Q(-0.08816157),
  DSP16_Q(0.26456991),
  DSP16_Q(0.31223427),
  DSP16_Q(-0.09366877),
  DSP16_Q(0.05028506),
  DSP16_Q(-0.03052409),
  DSP16_Q(0.01889000),
  DSP16_Q(-0.01129980),
  DSP16_Q(0.00621704),
  DSP16_Q(-0.00291586),
  DSP16_Q(0.00097750),
  DSP16_Q(-0.00010926),
  DSP16_Q(-0.00007895),
  DSP16_Q(0.00087364),
  DSP16_Q(-0.00271903),
  DSP16_Q(0.00590197),
  DSP16_Q(-0.01082709),
  DSP16_Q(0.01818487),
  DSP16_Q(-0.02941332),
  DSP16_Q(0.04825563),
  DSP16_Q(-0.08838590),
  DSP16_Q(0.26593728),
  DSP16_Q(0.31096140),
  DSP16_Q(-0.09359139),
  DSP16_Q(0.05026903),
  DSP16_Q(-0.03051819),
  DSP16_Q(0.01888591),
  DSP16_Q(-0.01129588),
  DSP16_Q(0.00621327),
  DSP16_Q(-0.00291265),
  DSP16_Q(0.00097528),
  DSP16_Q(-0.00010841),
  DSP16_Q(-0.00007981),
  DSP16_Q(0.00087728),
  DSP16_Q(-0.00272691),
  DSP16_Q(0.00591595),
  DSP16_Q(-0.01084980),
  DSP16_Q(0.01822049),
  DSP16_Q(-0.02946998),
  DSP16_Q(0.04835393),
  DSP16_Q(-0.08860640),
  DSP16_Q(0.26730262),
  DSP16_Q(0.30968516),
  DSP16_Q(-0.09350946),
  DSP16_Q(0.05025044),
  DSP16_Q(-0.03051073),
  DSP16_Q(0.01888084),
  DSP16_Q(-0.01129138),
  DSP16_Q(0.00620919),
  DSP16_Q(-0.00290929),
  DSP16_Q(0.00097302),
  DSP16_Q(-0.00010757),
  DSP16_Q(-0.00008068),
  DSP16_Q(0.00088088),
  DSP16_Q(-0.00273466),
  DSP16_Q(0.00592966),
  DSP16_Q(-0.01087199),
  DSP16_Q(0.01825524),
  DSP16_Q(-0.02952524),
  DSP16_Q(0.04844996),
  DSP16_Q(-0.08882305),
  DSP16_Q(0.26866589),
  DSP16_Q(0.30840560),
  DSP16_Q(-0.09342299),
  DSP16_Q(0.05022931),
  DSP16_Q(-0.03050170),
  DSP16_Q(0.01887480),
  DSP16_Q(-0.01128630),
  DSP16_Q(0.00620479),
  DSP16_Q(-0.00290578),
  DSP16_Q(0.00097071),
  DSP16_Q(-0.00010672),
  DSP16_Q(-0.00008154),
  DSP16_Q(0.00088445),
  DSP16_Q(-0.00274229),
  DSP16_Q(0.00594308),
  DSP16_Q(-0.01089366),
  DSP16_Q(0.01828911),
  DSP16_Q(-0.02957909),
  DSP16_Q(0.04854372),
  DSP16_Q(-0.08903584),
  DSP16_Q(0.27002706),
  DSP16_Q(0.30712276),
  DSP16_Q(-0.09333202),
  DSP16_Q(0.05020563),
  DSP16_Q(-0.03049111),
  DSP16_Q(0.01886779),
  DSP16_Q(-0.01128064),
  DSP16_Q(0.00620006),
  DSP16_Q(-0.00290211),
  DSP16_Q(0.00096835),
  DSP16_Q(-0.00010587),
  DSP16_Q(-0.00008241),
  DSP16_Q(0.00088798),
  DSP16_Q(-0.00274979),
  DSP16_Q(0.00595623),
  DSP16_Q(-0.01091482),
  DSP16_Q(0.01832210),
  DSP16_Q(-0.02963152),
  DSP16_Q(0.04863518),
  DSP16_Q(-0.08924473),
  DSP16_Q(0.27138607),
  DSP16_Q(0.30583667),
  DSP16_Q(-0.09323655),
  DSP16_Q(0.05017942),
  DSP16_Q(-0.03047897),
  DSP16_Q(0.01885982),
  DSP16_Q(-0.01127439),
  DSP16_Q(0.00619502),
  DSP16_Q(-0.00289830),
  DSP16_Q(0.00096595),
  DSP16_Q(-0.00010501),
  DSP16_Q(-0.00008328),
  DSP16_Q(0.00089148),
  DSP16_Q(-0.00275717),
  DSP16_Q(0.00596910),
  DSP16_Q(-0.01093544),
  DSP16_Q(0.01835421),
  DSP16_Q(-0.02968252),
  DSP16_Q(0.04872435),
  DSP16_Q(-0.08944970),
  DSP16_Q(0.27274290),
  DSP16_Q(0.30454738),
  DSP16_Q(-0.09313660),
  DSP16_Q(0.05015068),
  DSP16_Q(-0.03046528),
  DSP16_Q(0.01885088),
  DSP16_Q(-0.01126757),
  DSP16_Q(0.00618966),
  DSP16_Q(-0.00289435),
  DSP16_Q(0.00096349),
  DSP16_Q(-0.00010416),
  DSP16_Q(-0.00008415),
  DSP16_Q(0.00089494),
  DSP16_Q(-0.00276442),
  DSP16_Q(0.00598169),
  DSP16_Q(-0.01095555),
  DSP16_Q(0.01838543),
  DSP16_Q(-0.02973209),
  DSP16_Q(0.04881122),
  DSP16_Q(-0.08965074),
  DSP16_Q(0.27409750),
  DSP16_Q(0.30325491),
  DSP16_Q(-0.09303221),
  DSP16_Q(0.05011943),
  DSP16_Q(-0.03045003),
  DSP16_Q(0.01884097),
  DSP16_Q(-0.01126017),
  DSP16_Q(0.00618398),
  DSP16_Q(-0.00289024),
  DSP16_Q(0.00096099),
  DSP16_Q(-0.00010330),
  DSP16_Q(-0.00008502),
  DSP16_Q(0.00089837),
  DSP16_Q(-0.00277155),
  DSP16_Q(0.00599399),
  DSP16_Q(-0.01097512),
  DSP16_Q(0.01841576),
  DSP16_Q(-0.02978023),
  DSP16_Q(0.04889576),
  DSP16_Q(-0.08984782),
  DSP16_Q(0.27544983),
  DSP16_Q(0.30195931),
  DSP16_Q(-0.09292338),
  DSP16_Q(0.05008566),
  DSP16_Q(-0.03043324),
  DSP16_Q(0.01883011),
  DSP16_Q(-0.01125220),
  DSP16_Q(0.00617799),
  DSP16_Q(-0.00288599),
  DSP16_Q(0.00095844),
  DSP16_Q(-0.00010244),
  DSP16_Q(-0.00008590),
  DSP16_Q(0.00090175),
  DSP16_Q(-0.00277855),
  DSP16_Q(0.00600600),
  DSP16_Q(-0.01099416),
  DSP16_Q(0.01844520),
  DSP16_Q(-0.02982692),
  DSP16_Q(0.04897798),
  DSP16_Q(-0.09004091),
  DSP16_Q(0.27679985),
  DSP16_Q(0.30066063),
  DSP16_Q(-0.09281014),
  DSP16_Q(0.05004939),
  DSP16_Q(-0.03041492),
  DSP16_Q(0.01881829),
  DSP16_Q(-0.01124366),
  DSP16_Q(0.00617168),
  DSP16_Q(-0.00288159),
  DSP16_Q(0.00095585),
  DSP16_Q(-0.00010158),
  DSP16_Q(-0.00008677),
  DSP16_Q(0.00090510),
  DSP16_Q(-0.00278541),
  DSP16_Q(0.00601773),
  DSP16_Q(-0.01101267),
  DSP16_Q(0.01847374),
  DSP16_Q(-0.02987217),
  DSP16_Q(0.04905786),
  DSP16_Q(-0.09022999),
  DSP16_Q(0.27814753),
  DSP16_Q(0.29935888),
  DSP16_Q(-0.09269250),
  DSP16_Q(0.05001063),
  DSP16_Q(-0.03039506),
  DSP16_Q(0.01880552),
  DSP16_Q(-0.01123454),
  DSP16_Q(0.00616506),
  DSP16_Q(-0.00287705),
  DSP16_Q(0.00095321),
  DSP16_Q(-0.00010071),
  DSP16_Q(-0.00008764),
  DSP16_Q(0.00090842),
  DSP16_Q(-0.00279215),
  DSP16_Q(0.00602916),
  DSP16_Q(-0.01103065),
  DSP16_Q(0.01850137),
  DSP16_Q(-0.02991596),
  DSP16_Q(0.04913540),
  DSP16_Q(-0.09041505),
  DSP16_Q(0.27949282),
  DSP16_Q(0.29805413),
  DSP16_Q(-0.09257049),
  DSP16_Q(0.04996938),
  DSP16_Q(-0.03037366),
  DSP16_Q(0.01879179),
  DSP16_Q(-0.01122486),
  DSP16_Q(0.00615813),
  DSP16_Q(-0.00287236),
  DSP16_Q(0.00095052),
  DSP16_Q(-0.00009985),
  DSP16_Q(-0.00008851),
  DSP16_Q(0.00091169),
  DSP16_Q(-0.00279876),
  DSP16_Q(0.00604031),
  DSP16_Q(-0.01104809),
  DSP16_Q(0.01852811),
  DSP16_Q(-0.02995830),
  DSP16_Q(0.04921058),
  DSP16_Q(-0.09059605),
  DSP16_Q(0.28083568),
  DSP16_Q(0.29674639),
  DSP16_Q(-0.09244413),
  DSP16_Q(0.04992565),
  DSP16_Q(-0.03035074),
  DSP16_Q(0.01877712),
  DSP16_Q(-0.01121460),
  DSP16_Q(0.00615089),
  DSP16_Q(-0.00286754),
  DSP16_Q(0.00094779),
  DSP16_Q(-0.00009898),
  DSP16_Q(-0.00008939),
  DSP16_Q(0.00091492),
  DSP16_Q(-0.00280524),
  DSP16_Q(0.00605117),
  DSP16_Q(-0.01106498),
  DSP16_Q(0.01855393),
  DSP16_Q(-0.02999917),
  DSP16_Q(0.04928340),
  DSP16_Q(-0.09077298),
  DSP16_Q(0.28217608),
  DSP16_Q(0.29543572),
  DSP16_Q(-0.09231344),
  DSP16_Q(0.04987945),
  DSP16_Q(-0.03032630),
  DSP16_Q(0.01876149),
  DSP16_Q(-0.01120379),
  DSP16_Q(0.00614334),
  DSP16_Q(-0.00286257),
  DSP16_Q(0.00094502),
  DSP16_Q(-0.00009811),
  DSP16_Q(-0.00009026),
  DSP16_Q(0.00091812),
  DSP16_Q(-0.00281158),
  DSP16_Q(0.00606173),
  DSP16_Q(-0.01108134),
  DSP16_Q(0.01857884),
  DSP16_Q(-0.03003858),
  DSP16_Q(0.04935384),
  DSP16_Q(-0.09094581),
  DSP16_Q(0.28351397),
  DSP16_Q(0.29412215),
  DSP16_Q(-0.09217844),
  DSP16_Q(0.04983078),
  DSP16_Q(-0.03030034),
  DSP16_Q(0.01874493),
  DSP16_Q(-0.01119241),
  DSP16_Q(0.00613549),
  DSP16_Q(-0.00285745),
  DSP16_Q(0.00094220),
  DSP16_Q(-0.00009724),
  DSP16_Q(-0.00009114),
  DSP16_Q(0.00092127),
  DSP16_Q(-0.00281779),
  DSP16_Q(0.00607200),
  DSP16_Q(-0.01109715),
  DSP16_Q(0.01860284),
  DSP16_Q(-0.03007651),
  DSP16_Q(0.04942189),
  DSP16_Q(-0.09111453),
  DSP16_Q(0.28484931),
  DSP16_Q(0.29280573),
  DSP16_Q(-0.09203915),
  DSP16_Q(0.04977967),
  DSP16_Q(-0.03027286),
  DSP16_Q(0.01872743),
  DSP16_Q(-0.01118047),
  DSP16_Q(0.00612732),
  DSP16_Q(-0.00285220),
  DSP16_Q(0.00093934),
  DSP16_Q(-0.00009637),
  DSP16_Q(-0.00009201),
  DSP16_Q(0.00092439),
  DSP16_Q(-0.00282387),
  DSP16_Q(0.00608197),
  DSP16_Q(-0.01111241),
  DSP16_Q(0.01862592),
  DSP16_Q(-0.03011296),
  DSP16_Q(0.04948756),
  DSP16_Q(-0.09127911),
  DSP16_Q(0.28618207),
  DSP16_Q(0.29148648),
  DSP16_Q(-0.09189560),
  DSP16_Q(0.04972610),
  DSP16_Q(-0.03024388),
  DSP16_Q(0.01870899),
  DSP16_Q(-0.01116796),
  DSP16_Q(0.00611886),
  DSP16_Q(-0.00284681),
  DSP16_Q(0.00093643),
  DSP16_Q(-0.00009550),
  DSP16_Q(-0.00009288),
  DSP16_Q(0.00092746),
  DSP16_Q(-0.00282981),
  DSP16_Q(0.00609164),
  DSP16_Q(-0.01112713),
  DSP16_Q(0.01864808),
  DSP16_Q(-0.03014793),
  DSP16_Q(0.04955082),
  DSP16_Q(-0.09143953),
  DSP16_Q(0.28751221),
  DSP16_Q(0.29016445),
  DSP16_Q(-0.09174779),
  DSP16_Q(0.04967010),
  DSP16_Q(-0.03021339),
  DSP16_Q(0.01868961),
  DSP16_Q(-0.01115491),
  DSP16_Q(0.00611009),
  DSP16_Q(-0.00284128),
  DSP16_Q(0.00093348),
  DSP16_Q(-0.00009463),
  DSP16_Q(-0.00009376),
  DSP16_Q(0.00093049),
  DSP16_Q(-0.00283561),
  DSP16_Q(0.00610101),
  DSP16_Q(-0.01114130),
  DSP16_Q(0.01866931),
  DSP16_Q(-0.03018141),
  DSP16_Q(0.04961167),
  DSP16_Q(-0.09159576),
  DSP16_Q(0.28883968),
  DSP16_Q(0.28883968),
  DSP16_Q(-0.09159576),
  DSP16_Q(0.04961167),
  DSP16_Q(-0.03018141),
  DSP16_Q(0.01866931),
  DSP16_Q(-0.01114130),
  DSP16_Q(0.00610101),
  DSP16_Q(-0.00283561),
  DSP16_Q(0.00093049),
  DSP16_Q(-0.00009376),
  DSP16_Q(-0.00009463),
  DSP16_Q(0.00093348),
  DSP16_Q(-0.00284128),
  DSP16_Q(0.00611009),
  DSP16_Q(-0.01115491),
  DSP16_Q(0.01868961),
  DSP16_Q(-0.03021339),
  DSP16_Q(0.04967010),
  DSP16_Q(-0.09174779),
  DSP16_Q(0.29016445),
  DSP16_Q(0.28751221),
  DSP16_Q(-0.09143953),
  DSP16_Q(0.04955082),
  DSP16_Q(-0.03014793),
  DSP16_Q(0.01864808),
  DSP16_Q(-0.01112713),
  DSP16_Q(0.00609164),
  DSP16_Q(-0.00282981),
  DSP16_Q(0.00092746),
  DSP16_Q(-0.00009288),
  DSP16_Q(-0.00009550),
  DSP16_Q(0.00093643),
  DSP16_Q(-0.00284681),
  DSP16_Q(0.00611886),
  DSP16_Q(-0.01116796),
  DSP16_Q(0.01870899),
  DSP16_Q(-0.03024388),
  DSP16_Q(0.04972610),
  DSP16_Q(-0.09189560),
  DSP16_Q(0.29148648),
  DSP16_Q(0.28618207),
  DSP16_Q(-0.09127911),
  DSP16_Q(0.04948756),
  DSP16_Q(-0.03011296),
  DSP16_Q(0.01862592),
  DSP16_Q(-0.01111241),
  DSP16_Q(0.00608197),
  DSP16_Q(-0.00282387),
  DSP16_Q(0.00092439),
  DSP16_Q(-0.00009201),
  DSP16_Q(-0.00009637),
  DSP16_Q(0.00093934),
  DSP16_Q(-0.00285220),
  DSP16_Q(0.00612732),
  DSP16_Q(-0.01118047),
  DSP16_Q(0.01872743),
  DSP16_Q(-0.03027286),
  DSP16_Q(0.04977967),
  DSP16_Q(-0.09203915),
  DSP16_Q(0.29280573),
  DSP16_Q(0.28484931),
  DSP16_Q(-0.09111453),
  DSP16_Q(0.04942189),
  DSP16_Q(-0.03007651),
  DSP16_Q(0.01860284),
  DSP16_Q(-0.01109715),
  DSP16_Q(0.00607200),
  DSP16_Q(-0.00281779),
  DSP16_Q(0.00092127),
  DSP16_Q(-0.00009114),
  DSP16_Q(-0.00009724),
  DSP16_Q(0.00094220),
  DSP16_Q(-0.00285745),
  DSP16_Q(0.00613549),
  DSP16_Q(-0.01119241),
  DSP16_Q(0.01874493),
  DSP16_Q(-0.03030034),
  DSP16_Q(0.04983078),
  DSP16_Q(-0.09217844),
  DSP16_Q(0.29412215),
  DSP16_Q(0.28351397),
  DSP16_Q(-0.09094581),
  DSP16_Q(0.04935384),
  DSP16_Q(-0.03003858),
  DSP16_Q(0.01857884),
  DSP16_Q(-0.01108134),
  DSP16_Q(0.00606173),
  DSP16_Q(-0.00281158),
  DSP16_Q(0.00091812),
  DSP16_Q(-0.00009026),
  DSP16_Q(-0.00009811),
  DSP16_Q(0.00094502),
  DSP16_Q(-0.00286257),
  DSP16_Q(0.00614334),
  DSP16_Q(-0.01120379),
  DSP16_Q(0.01876149),
  DSP16_Q(-0.03032630),
  DSP16_Q(0.04987945),
  DSP16_Q(-0.09231344),
  DSP16_Q(0.29543572),
  DSP16_Q(0.28217608),
  DSP16_Q(-0.09077298),
  DSP16_Q(0.04928340),
  DSP16_Q(-0.02999917),
  DSP16_Q(0.01855393),
  DSP16_Q(-0.01106498),
  DSP16_Q(0.00605117),
  DSP16_Q(-0.00280524),
  DSP16_Q(0.00091492),
  DSP16_Q(-0.00008939),
  DSP16_Q(-0.00009898),
  DSP16_Q(0.00094779),
  DSP16_Q(-0.00286754),
  DSP16_Q(0.00615089),
  DSP16_Q(-0.01121460),
  DSP16_Q(0.01877712),
  DSP16_Q(-0.03035074),
  DSP16_Q(0.04992565),
  DSP16_Q(-0.09244413),
  DSP16_Q(0.29674639),
  DSP16_Q(0.28083568),
  DSP16_Q(-0.09059605),
  DSP16_Q(0.04921058),
  DSP16_Q(-0.02995830),
  DSP16_Q(0.01852811),
  DSP16_Q(-0.01104809),
  DSP16_Q(0.00604031),
  DSP16_Q(-0.00279876),
  DSP16_Q(0.00091169),
  DSP16_Q(-0.00008851),
  DSP16_Q(-0.00009985),
  DSP16_Q(0.00095052),
  DSP16_Q(-0.00287236),
  DSP16_Q(0.00615813),
  DSP16_Q(-0.01122486),
  DSP16_Q(0.01879179),
  DSP16_Q(-0.03037366),
  DSP16_Q(0.04996938),
  DSP16_Q(-0.09257049),
  DSP16_Q(0.29805413),
  DSP16_Q(0.27949282),
  DSP16_Q(-0.09041505),
  DSP16_Q(0.04913540),
  DSP16_Q(-0.02991596),
  DSP16_Q(0.01850137),
  DSP16_Q(-0.01103065),
  DSP16_Q(0.00602916),
  DSP16_Q(-0.00279215),
  DSP16_Q(0.00090842),
  DSP16_Q(-0.00008764),
  DSP16_Q(-0.00010071),
  DSP16_Q(0.00095321),
  DSP16_Q(-0.00287705),
  DSP16_Q(0.00616506),
  DSP16_Q(-0.01123454),
  DSP16_Q(0.01880552),
  DSP16_Q(-0.03039506),
  DSP16_Q(0.05001063),
  DSP16_Q(-0.09269250),
  DSP16_Q(0.29935888),
  DSP16_Q(0.27814753),
  DSP16_Q(-0.09022999),
  DSP16_Q(0.04905786),
  DSP16_Q(-0.02987217),
  DSP16_Q(0.01847374),
  DSP16_Q(-0.01101267),
  DSP16_Q(0.00601773),
  DSP16_Q(-0.00278541),
  DSP16_Q(0.00090510),
  DSP16_Q(-0.00008677),
  DSP16_Q(-0.00010158),
  DSP16_Q(0.00095585),
  DSP16_Q(-0.00288159),
  DSP16_Q(0.00617168),
  DSP16_Q(-0.01124366),
  DSP16_Q(0.01881829),
  DSP16_Q(-0.03041492),
  DSP16_Q(0.05004939),
  DSP16_Q(-0.09281014),
  DSP16_Q(0.30066063),
  DSP16_Q(0.27679985),
  DSP16_Q(-0.09004091),
  DSP16_Q(0.04897798),
  DSP16_Q(-0.02982692),
  DSP16_Q(0.01844520),
  DSP16_Q(-0.01099416),
  DSP16_Q(0.00600600),
  DSP16_Q(-0.00277855),
  DSP16_Q(0.00090175),
  DSP16_Q(-0.00008590),
  DSP16_Q(-0.00010244),
  DSP16_Q(0.00095844),
  DSP16_Q(-0.00288599),
  DSP16_Q(0.00617799),
  DSP16_Q(-0.01125220),
  DSP16_Q(0.01883011),
  DSP16_Q(-0.03043324),
  DSP16_Q(0.05008566),
  DSP16_Q(-0.09292338),
  DSP16_Q(0.30195931),
  DSP16_Q(0.27544983),
  DSP16_Q(-0.08984782),
  DSP16_Q(0.04889576),
  DSP16_Q(-0.02978023),
  DSP16_Q(0.01841576),
  DSP16_Q(-0.01097512),
  DSP16_Q(0.00599399),
  DSP16_Q(-0.00277155),
  DSP16_Q(0.00089837),
  DSP16_Q(-0.00008502),
  DSP16_Q(-0.00010330),
  DSP16_Q(0.00096099),
  DSP16_Q(-0.00289024),
  DSP16_Q(0.00618398),
  DSP16_Q(-0.01126017),
  DSP16_Q(0.01884097),
  DSP16_Q(-0.03045003),
  DSP16_Q(0.05011943),
  DSP16_Q(-0.09303221),
  DSP16_Q(0.30325491),
  DSP16_Q(0.27409750),
  DSP16_Q(-0.08965074),
  DSP16_Q(0.04881122),
  DSP16_Q(-0.02973209),
  DSP16_Q(0.01838543),
  DSP16_Q(-0.01095555),
  DSP16_Q(0.00598169),
  DSP16_Q(-0.00276442),
  DSP16_Q(0.00089494),
  DSP16_Q(-0.00008415),
  DSP16_Q(-0.00010416),
  DSP16_Q(0.00096349),
  DSP16_Q(-0.00289435),
  DSP16_Q(0.00618966),
  DSP16_Q(-0.01126757),
  DSP16_Q(0.01885088),
  DSP16_Q(-0.03046528),
  DSP16_Q(0.05015068),
  DSP16_Q(-0.09313660),
  DSP16_Q(0.30454738),
  DSP16_Q(0.27274290),
  DSP16_Q(-0.08944970),
  DSP16_Q(0.04872435),
  DSP16_Q(-0.02968252),
  DSP16_Q(0.01835421),
  DSP16_Q(-0.01093544),
  DSP16_Q(0.00596910),
  DSP16_Q(-0.00275717),
  DSP16_Q(0.00089148),
  DSP16_Q(-0.00008328),
  DSP16_Q(-0.00010501),
  DSP16_Q(0.00096595),
  DSP16_Q(-0.00289830),
  DSP16_Q(0.00619502),
  DSP16_Q(-0.01127439),
  DSP16_Q(0.01885982),
  DSP16_Q(-0.03047897),
  DSP16_Q(0.05017942),
  DSP16_Q(-0.09323655),
  DSP16_Q(0.30583667),
  DSP16_Q(0.27138607),
  DSP16_Q(-0.08924473),
  DSP16_Q(0.04863518),
  DSP16_Q(-0.02963152),
  DSP16_Q(0.01832210),
  DSP16_Q(-0.01091482),
  DSP16_Q(0.00595623),
  DSP16_Q(-0.00274979),
  DSP16_Q(0.00088798),
  DSP16_Q(-0.00008241),
  DSP16_Q(-0.00010587),
  DSP16_Q(0.00096835),
  DSP16_Q(-0.00290211),
  DSP16_Q(0.00620006),
  DSP16_Q(-0.01128064),
  DSP16_Q(0.01886779),
  DSP16_Q(-0.03049111),
  DSP16_Q(0.05020563),
  DSP16_Q(-0.09333202),
  DSP16_Q(0.30712276),
  DSP16_Q(0.27002706),
  DSP16_Q(-0.08903584),
  DSP16_Q(0.04854372),
  DSP16_Q(-0.02957909),
  DSP16_Q(0.01828911),
  DSP16_Q(-0.01089366),
  DSP16_Q(0.00594308),
  DSP16_Q(-0.00274229),
  DSP16_Q(0.00088445),
  DSP16_Q(-0.00008154),
  DSP16_Q(-0.00010672),
  DSP16_Q(0.00097071),
  DSP16_Q(-0.00290578),
  DSP16_Q(0.00620479),
  DSP16_Q(-0.01128630),
  DSP16_Q(0.01887480),
  DSP16_Q(-0.03050170),
  DSP16_Q(0.05022931),
  DSP16_Q(-0.09342299),
  DSP16_Q(0.30840560),
  DSP16_Q(0.26866589),
  DSP16_Q(-0.08882305),
  DSP16_Q(0.04844996),
  DSP16_Q(-0.02952524),
  DSP16_Q(0.01825524),
  DSP16_Q(-0.01087199),
  DSP16_Q(0.00592966),
  DSP16_Q(-0.00273466),
  DSP16_Q(0.00088088),
  DSP16_Q(-0.00008068),
  DSP16_Q(-0.00010757),
  DSP16_Q(0.00097302),
  DSP16_Q(-0.00290929),
  DSP16_Q(0.00620919),
  DSP16_Q(-0.01129138),
  DSP16_Q(0.01888084),
  DSP16_Q(-0.03051073),
  DSP16_Q(0.05025044),
  DSP16_Q(-0.09350946),
  DSP16_Q(0.30968516),
  DSP16_Q(0.26730262),
  DSP16_Q(-0.08860640),
  DSP16_Q(0.04835393),
  DSP16_Q(-0.02946998),
  DSP16_Q(0.01822049),
  DSP16_Q(-0.01084980),
  DSP16_Q(0.00591595),
  DSP16_Q(-0.00272691),
  DSP16_Q(0.00087728),
  DSP16_Q(-0.00007981),
  DSP16_Q(-0.00010841),
  DSP16_Q(0.00097528),
  DSP16_Q(-0.00291265),
  DSP16_Q(0.00621327),
  DSP16_Q(-0.01129588),
  DSP16_Q(0.01888591),
  DSP16_Q(-0.03051819),
  DSP16_Q(0.05026903),
  DSP16_Q(-0.09359139),
  DSP16_Q(0.31096140),
  DSP16_Q(0.26593728),
  DSP16_Q(-0.08838590),
  DSP16_Q(0.04825563),
  DSP16_Q(-0.02941332),
  DSP16_Q(0.01818487),
  DSP16_Q(-0.01082709),
  DSP16_Q(0.00590197),
  DSP16_Q(-0.00271903),
  DSP16_Q(0.00087364),
  DSP16_Q(-0.00007895),
  DSP16_Q(-0.00010926),
  DSP16_Q(0.00097750),
  DSP16_Q(-0.00291586),
  DSP16_Q(0.00621704),
  DSP16_Q(-0.01129980),
  DSP16_Q(0.01889000),
  DSP16_Q(-0.03052409),
  DSP16_Q(0.05028506),
  DSP16_Q(-0.09366877),
  DSP16_Q(0.31223427),
  DSP16_Q(0.26456991),
  DSP16_Q(-0.08816157),
  DSP16_Q(0.04815508),
  DSP16_Q(-0.02935525),
  DSP16_Q(0.01814838),
  DSP16_Q(-0.01080387),
  DSP16_Q(0.00588771),
  DSP16_Q(-0.00271104),
  DSP16_Q(0.00086997),
  DSP16_Q(-0.00007808),
  DSP16_Q(-0.00011010),
  DSP16_Q(0.00097966),
  DSP16_Q(-0.00291892),
  DSP16_Q(0.00622047),
  DSP16_Q(-0.01130312),
  DSP16_Q(0.01889311),
  DSP16_Q(-0.03052841),
  DSP16_Q(0.05029853),
  DSP16_Q(-0.09374158),
  DSP16_Q(0.31350375),
  DSP16_Q(0.26320055),
  DSP16_Q(-0.08793344),
  DSP16_Q(0.04805228),
  DSP16_Q(-0.02929579),
  DSP16_Q(0.01811103),
  DSP16_Q(-0.01078014),
  DSP16_Q(0.00587317),
  DSP16_Q(-0.00270292),
  DSP16_Q(0.00086626),
  DSP16_Q(-0.00007722),
  DSP16_Q(-0.00011093),
  DSP16_Q(0.00098177),
  DSP16_Q(-0.00292183),
  DSP16_Q(0.00622359),
  DSP16_Q(-0.01130586),
  DSP16_Q(0.01889525),
  DSP16_Q(-0.03053116),
  DSP16_Q(0.05030943),
  DSP16_Q(-0.09380980),
  DSP16_Q(0.31476979),
  DSP16_Q(0.26182924),
  DSP16_Q(-0.08770154),
  DSP16_Q(0.04794724),
  DSP16_Q(-0.02923494),
  DSP16_Q(0.01807282),
  DSP16_Q(-0.01075590),
  DSP16_Q(0.00585837),
  DSP16_Q(-0.00269468),
  DSP16_Q(0.00086253),
  DSP16_Q(-0.00007636),
  DSP16_Q(-0.00011177),
  DSP16_Q(0.00098383),
  DSP16_Q(-0.00292458),
  DSP16_Q(0.00622637),
  DSP16_Q(-0.01130801),
  DSP16_Q(0.01889640),
  DSP16_Q(-0.03053233),
  DSP16_Q(0.05031776),
  DSP16_Q(-0.09387341),
  DSP16_Q(0.31603235),
  DSP16_Q(0.26045602),
  DSP16_Q(-0.08746588),
  DSP16_Q(0.04783998),
  DSP16_Q(-0.02917270),
  DSP16_Q(0.01803374),
  DSP16_Q(-0.01073115),
  DSP16_Q(0.00584330),
  DSP16_Q(-0.00268633),
  DSP16_Q(0.00085876),
  DSP16_Q(-0.00007550),
  DSP16_Q(-0.00011260),
  DSP16_Q(0.00098584),
  DSP16_Q(-0.00292718),
  DSP16_Q(0.00622883),
  DSP16_Q(-0.01130957),
  DSP16_Q(0.01889657),
  DSP16_Q(-0.03053192),
  DSP16_Q(0.05032350),
  DSP16_Q(-0.09393239),
  DSP16_Q(0.31729141),
  DSP16_Q(0.25908094),
  DSP16_Q(-0.08722649),
  DSP16_Q(0.04773051),
  DSP16_Q(-0.02910909),
  DSP16_Q(0.01799382),
  DSP16_Q(-0.01070590),
  DSP16_Q(0.00582795),
  DSP16_Q(-0.00267786),
  DSP16_Q(0.00085495),
  DSP16_Q(-0.00007465),
  DSP16_Q(-0.00011343),
  DSP16_Q(0.00098780),
  DSP16_Q(-0.00292963),
  DSP16_Q(0.00623096),
  DSP16_Q(-0.01131054),
  DSP16_Q(0.01889575),
  DSP16_Q(-0.03052992),
  DSP16_Q(0.05032666),
  DSP16_Q(-0.09398672),
  DSP16_Q(0.31854691),
  DSP16_Q(0.25770402),
  DSP16_Q(-0.08698340),
  DSP16_Q(0.04761883),
  DSP16_Q(-0.02904411),
  DSP16_Q(0.01795305),
  DSP16_Q(-0.01068015),
  DSP16_Q(0.00581234),
  DSP16_Q(-0.00266927),
  DSP16_Q(0.00085112),
  DSP16_Q(-0.00007380),
  DSP16_Q(-0.00011425),
  DSP16_Q(0.00098971),
  DSP16_Q(-0.00293192),
  DSP16_Q(0.00623277),
  DSP16_Q(-0.01131091),
  DSP16_Q(0.01889395),
  DSP16_Q(-0.03052633),
  DSP16_Q(0.05032722),
  DSP16_Q(-0.09403639),
  DSP16_Q(0.31979882),
  DSP16_Q(0.25632532),
  DSP16_Q(-0.08673662),
  DSP16_Q(0.04750496),
  DSP16_Q(-0.02897776),
  DSP16_Q(0.01791143),
  DSP16_Q(-0.01065390),
  DSP16_Q(0.00579647),
  DSP16_Q(-0.00266056),
  DSP16_Q(0.00084725),
  DSP16_Q(-0.00007294),
  DSP16_Q(-0.00011507),
  DSP16_Q(0.00099156),
  DSP16_Q(-0.00293405),
  DSP16_Q(0.00623424),
  DSP16_Q(-0.01131068),
  DSP16_Q(0.01889115),
  DSP16_Q(-0.03052115),
  DSP16_Q(0.05032517),
  DSP16_Q(-0.09408137),
  DSP16_Q(0.32104711),
  DSP16_Q(0.25494487),
  DSP16_Q(-0.08648618),
  DSP16_Q(0.04738891),
  DSP16_Q(-0.02891006),
  DSP16_Q(0.01786897),
  DSP16_Q(-0.01062716),
  DSP16_Q(0.00578033),
  DSP16_Q(-0.00265175),
  DSP16_Q(0.00084336),
  DSP16_Q(-0.00007210),
  DSP16_Q(-0.00011588),
  DSP16_Q(0.00099336),
  DSP16_Q(-0.00293603),
  DSP16_Q(0.00623538),
  DSP16_Q(-0.01130986),
  DSP16_Q(0.01888736),
  DSP16_Q(-0.03051438),
  DSP16_Q(0.05032052),
  DSP16_Q(-0.09412165),
  DSP16_Q(0.32229173),
  DSP16_Q(0.25356271),
  DSP16_Q(-0.08623211),
  DSP16_Q(0.04727069),
  DSP16_Q(-0.02884100),
  DSP16_Q(0.01782567),
  DSP16_Q(-0.01059992),
  DSP16_Q(0.00576393),
  DSP16_Q(-0.00264281),
  DSP16_Q(0.00083943),
  DSP16_Q(-0.00007125),
  DSP16_Q(-0.00011669),
  DSP16_Q(0.00099511),
  DSP16_Q(-0.00293785),
  DSP16_Q(0.00623619),
  DSP16_Q(-0.01130844),
  DSP16_Q(0.01888258),
  DSP16_Q(-0.03050601),
  DSP16_Q(0.05031326),
  DSP16_Q(-0.09415721),
  DSP16_Q(0.32353266),
  DSP16_Q(0.25217888),
  DSP16_Q(-0.08597443),
  DSP16_Q(0.04715031),
  DSP16_Q(-0.02877060),
  DSP16_Q(0.01778155),
  DSP16_Q(-0.01057219),
  DSP16_Q(0.00574727),
  DSP16_Q(-0.00263377),
  DSP16_Q(0.00083547),
  DSP16_Q(-0.00007040),
  DSP16_Q(-0.00011750),
  DSP16_Q(0.00099680),
  DSP16_Q(-0.00293951),
  DSP16_Q(0.00623666),
  DSP16_Q(-0.01130642),
  DSP16_Q(0.01887680),
  DSP16_Q(-0.03049604),
  DSP16_Q(0.05030337),
  DSP16_Q(-0.09418803),
  DSP16_Q(0.32476984),
  DSP16_Q(0.25079342),
  DSP16_Q(-0.08571315),
  DSP16_Q(0.04702779),
  DSP16_Q(-0.02869887),
  DSP16_Q(0.01773659),
  DSP16_Q(-0.01054397),
  DSP16_Q(0.00573035),
  DSP16_Q(-0.00262461),
  DSP16_Q(0.00083149),
  DSP16_Q(-0.00006956),
  DSP16_Q(-0.00011830),
  DSP16_Q(0.00099844),
  DSP16_Q(-0.00294101),
  DSP16_Q(0.00623681),
  DSP16_Q(-0.01130380),
  DSP16_Q(0.01887002),
  DSP16_Q(-0.03048446),
  DSP16_Q(0.05029086),
  DSP16_Q(-0.09421409),
  DSP16_Q(0.32600326),
  DSP16_Q(0.24940638),
  DSP16_Q(-0.08544832),
  DSP16_Q(0.04690312),
  DSP16_Q(-0.02862580),
  DSP16_Q(0.01769081),
  DSP16_Q(-0.01051527),
  DSP16_Q(0.00571317),
  DSP16_Q(-0.00261534),
  DSP16_Q(0.00082747),
  DSP16_Q(-0.00006872),
  DSP16_Q(-0.00011910),
  DSP16_Q(0.00100003),
  DSP16_Q(-0.00294236),
  DSP16_Q(0.00623661),
  DSP16_Q(-0.01130057),
  DSP16_Q(0.01886224),
  DSP16_Q(-0.03047127),
  DSP16_Q(0.05027572),
  DSP16_Q(-0.09423538),
  DSP16_Q(0.32723285),
  DSP16_Q(0.24801779),
  DSP16_Q(-0.08517995),
  DSP16_Q(0.04677633),
  DSP16_Q(-0.02855140),
  DSP16_Q(0.01764421),
  DSP16_Q(-0.01048609),
  DSP16_Q(0.00569574),
  DSP16_Q(-0.00260596),
  DSP16_Q(0.00082343),
  DSP16_Q(-0.00006789),
  DSP16_Q(-0.00011989),
  DSP16_Q(0.00100156),
  DSP16_Q(-0.00294354),
  DSP16_Q(0.00623608),
  DSP16_Q(-0.01129674),
  DSP16_Q(0.01885346),
  DSP16_Q(-0.03045648),
  DSP16_Q(0.05025794),
  DSP16_Q(-0.09425187),
  DSP16_Q(0.32845861),
  DSP16_Q(0.24662769),
  DSP16_Q(-0.08490806),
  DSP16_Q(0.04664743),
  DSP16_Q(-0.02847569),
  DSP16_Q(0.01759679),
  DSP16_Q(-0.01045643),
  DSP16_Q(0.00567806),
  DSP16_Q(-0.00259647),
  DSP16_Q(0.00081936),
  DSP16_Q(-0.00006706),
  DSP16_Q(-0.00012068),
  DSP16_Q(0.00100304),
  DSP16_Q(-0.00294456),
  DSP16_Q(0.00623521),
  DSP16_Q(-0.01129231),
  DSP16_Q(0.01884368),
  DSP16_Q(-0.03044007),
  DSP16_Q(0.05023752),
  DSP16_Q(-0.09426356),
  DSP16_Q(0.32968047),
  DSP16_Q(0.24523612),
  DSP16_Q(-0.08463268),
  DSP16_Q(0.04651643),
  DSP16_Q(-0.02839866),
  DSP16_Q(0.01754856),
  DSP16_Q(-0.01042629),
  DSP16_Q(0.00566012),
  DSP16_Q(-0.00258688),
  DSP16_Q(0.00081526),
  DSP16_Q(-0.00006623),
  DSP16_Q(-0.00012146),
  DSP16_Q(0.00100446),
  DSP16_Q(-0.00294542),
  DSP16_Q(0.00623401),
  DSP16_Q(-0.01128727),
  DSP16_Q(0.01883289),
  DSP16_Q(-0.03042205),
  DSP16_Q(0.05021445),
  DSP16_Q(-0.09427042),
  DSP16_Q(0.33089841),
  DSP16_Q(0.24384313),
  DSP16_Q(-0.08435384),
  DSP16_Q(0.04638333),
  DSP16_Q(-0.02832033),
  DSP16_Q(0.01749952),
  DSP16_Q(-0.01039567),
  DSP16_Q(0.00564194),
  DSP16_Q(-0.00257718),
  DSP16_Q(0.00081114),
  DSP16_Q(-0.00006540),
  DSP16_Q(-0.00012224),
  DSP16_Q(0.00100582),
  DSP16_Q(-0.00294611),
  DSP16_Q(0.00623246),
  DSP16_Q(-0.01128162),
  DSP16_Q(0.01882110),
  DSP16_Q(-0.03040241),
  DSP16_Q(0.05018872),
  DSP16_Q(-0.09427243),
  DSP16_Q(0.33211239),
  DSP16_Q(0.24244875),
  DSP16_Q(-0.08407156),
  DSP16_Q(0.04624816),
  DSP16_Q(-0.02824070),
  DSP16_Q(0.01744968),
  DSP16_Q(-0.01036459),
  DSP16_Q(0.00562350),
  DSP16_Q(-0.00256737),
  DSP16_Q(0.00080698),
  DSP16_Q(-0.00006457),
  DSP16_Q(-0.00012301),
  DSP16_Q(0.00100713),
  DSP16_Q(-0.00294665),
  DSP16_Q(0.00623058),
  DSP16_Q(-0.01127536),
  DSP16_Q(0.01880829),
  DSP16_Q(-0.03038115),
  DSP16_Q(0.05016034),
  DSP16_Q(-0.09426958),
  DSP16_Q(0.33332238),
  DSP16_Q(0.24105302),
  DSP16_Q(-0.08378585),
  DSP16_Q(0.04611091),
  DSP16_Q(-0.02815978),
  DSP16_Q(0.01739904),
  DSP16_Q(-0.01033304),
  DSP16_Q(0.00560482),
  DSP16_Q(-0.00255746),
  DSP16_Q(0.00080281),
  DSP16_Q(-0.00006375),
  DSP16_Q(-0.00012377),
  DSP16_Q(0.00100838),
  DSP16_Q(-0.00294702),
  DSP16_Q(0.00622836),
  DSP16_Q(-0.01126849),
  DSP16_Q(0.01879448),
  DSP16_Q(-0.03035826),
  DSP16_Q(0.05012929),
  DSP16_Q(-0.09426185),
  DSP16_Q(0.33452833),
  DSP16_Q(0.23965599),
  DSP16_Q(-0.08349676),
  DSP16_Q(0.04597162),
  DSP16_Q(-0.02807758),
  DSP16_Q(0.01734761),
  DSP16_Q(-0.01030102),
  DSP16_Q(0.00558590),
  DSP16_Q(-0.00254744),
  DSP16_Q(0.00079860),
  DSP16_Q(-0.00006294),
  DSP16_Q(-0.00012453),
  DSP16_Q(0.00100957),
  DSP16_Q(-0.00294722),
  DSP16_Q(0.00622579),
  DSP16_Q(-0.01126101),
  DSP16_Q(0.01877965),
  DSP16_Q(-0.03033375),
  DSP16_Q(0.05009558),
  DSP16_Q(-0.09424923),
  DSP16_Q(0.33573021),
  DSP16_Q(0.23825769),
  DSP16_Q(-0.08320430),
  DSP16_Q(0.04583028),
  DSP16_Q(-0.02799410),
  DSP16_Q(0.01729539),
  DSP16_Q(-0.01026854),
  DSP16_Q(0.00556673),
  DSP16_Q(-0.00253732),
  DSP16_Q(0.00079438),
  DSP16_Q(-0.00006212),
  DSP16_Q(-0.00012529),
  DSP16_Q(0.00101071),
  DSP16_Q(-0.00294726),
  DSP16_Q(0.00622288),
  DSP16_Q(-0.01125292),
  DSP16_Q(0.01876382),
  DSP16_Q(-0.03030762),
  DSP16_Q(0.05005919),
  DSP16_Q(-0.09423169),
  DSP16_Q(0.33692799),
  DSP16_Q(0.23685816),
  DSP16_Q(-0.08290850),
  DSP16_Q(0.04568690),
  DSP16_Q(-0.02790935),
  DSP16_Q(0.01724239),
  DSP16_Q(-0.01023560),
  DSP16_Q(0.00554732),
  DSP16_Q(-0.00252710),
  DSP16_Q(0.00079012),
  DSP16_Q(-0.00006131),
  DSP16_Q(-0.00012604),
  DSP16_Q(0.00101178),
  DSP16_Q(-0.00294714),
  DSP16_Q(0.00621963),
  DSP16_Q(-0.01124422),
  DSP16_Q(0.01874696),
  DSP16_Q(-0.03027985),
  DSP16_Q(0.05002012),
  DSP16_Q(-0.09420923),
  DSP16_Q(0.33812163),
  DSP16_Q(0.23545745),
  DSP16_Q(-0.08260938),
  DSP16_Q(0.04554151),
  DSP16_Q(-0.02782333),
  DSP16_Q(0.01718860),
  DSP16_Q(-0.01020220),
  DSP16_Q(0.00552768),
  DSP16_Q(-0.00251678),
  DSP16_Q(0.00078584),
  DSP16_Q(-0.00006051),
  DSP16_Q(-0.00012678),
  DSP16_Q(0.00101280),
  DSP16_Q(-0.00294685),
  DSP16_Q(0.00621603),
  DSP16_Q(-0.01123490),
  DSP16_Q(0.01872909),
  DSP16_Q(-0.03025046),
  DSP16_Q(0.04997837),
  DSP16_Q(-0.09418181),
  DSP16_Q(0.33931108),
  DSP16_Q(0.23405560),
  DSP16_Q(-0.08230697),
  DSP16_Q(0.04539411),
  DSP16_Q(-0.02773606),
  DSP16_Q(0.01713403),
  DSP16_Q(-0.01016835),
  DSP16_Q(0.00550779),
  DSP16_Q(-0.00250636),
  DSP16_Q(0.00078154),
  DSP16_Q(-0.00005970),
  DSP16_Q(-0.00012751),
  DSP16_Q(0.00101376),
  DSP16_Q(-0.00294639),
  DSP16_Q(0.00621209),
  DSP16_Q(-0.01122496),
  DSP16_Q(0.01871021),
  DSP16_Q(-0.03021942),
  DSP16_Q(0.04993394),
  DSP16_Q(-0.09414944),
  DSP16_Q(0.34049633),
  DSP16_Q(0.23265264),
  DSP16_Q(-0.08200129),
  DSP16_Q(0.04524472),
  DSP16_Q(-0.02764755),
  DSP16_Q(0.01707870),
  DSP16_Q(-0.01013405),
  DSP16_Q(0.00548767),
  DSP16_Q(-0.00249583),
  DSP16_Q(0.00077722),
  DSP16_Q(-0.00005890),
  DSP16_Q(-0.00012824),
  DSP16_Q(0.00101466),
  DSP16_Q(-0.00294577),
  DSP16_Q(0.00620780),
  DSP16_Q(-0.01121441),
  DSP16_Q(0.01869030),
  DSP16_Q(-0.03018676),
  DSP16_Q(0.04988681),
  DSP16_Q(-0.09411209),
  DSP16_Q(0.34167732),
  DSP16_Q(0.23124861),
  DSP16_Q(-0.08169237),
  DSP16_Q(0.04509334),
  DSP16_Q(-0.02755779),
  DSP16_Q(0.01702260),
  DSP16_Q(-0.01009930),
  DSP16_Q(0.00546732),
  DSP16_Q(-0.00248522),
  DSP16_Q(0.00077287),
  DSP16_Q(-0.00005811),
  DSP16_Q(-0.00012896),
  DSP16_Q(0.00101550),
  DSP16_Q(-0.00294497),
  DSP16_Q(0.00620317),
  DSP16_Q(-0.01120324),
  DSP16_Q(0.01866938),
  DSP16_Q(-0.03015245),
  DSP16_Q(0.04983698),
  DSP16_Q(-0.09406975),
  DSP16_Q(0.34285403),
  DSP16_Q(0.22984356),
  DSP16_Q(-0.08138024),
  DSP16_Q(0.04494000),
  DSP16_Q(-0.02746679),
  DSP16_Q(0.01696574),
  DSP16_Q(-0.01006411),
  DSP16_Q(0.00544673),
  DSP16_Q(-0.00247450),
  DSP16_Q(0.00076850),
  DSP16_Q(-0.00005732),
  DSP16_Q(-0.00012968),
  DSP16_Q(0.00101628),
  DSP16_Q(-0.00294401),
  DSP16_Q(0.00619819),
  DSP16_Q(-0.01119146),
  DSP16_Q(0.01864744),
  DSP16_Q(-0.03011651),
  DSP16_Q(0.04978446),
  DSP16_Q(-0.09402240),
  DSP16_Q(0.34402642),
  DSP16_Q(0.22843753),
  DSP16_Q(-0.08106491),
  DSP16_Q(0.04478469),
  DSP16_Q(-0.02737457),
  DSP16_Q(0.01690811),
  DSP16_Q(-0.01002848),
  DSP16_Q(0.00542591),
  DSP16_Q(-0.00246369),
  DSP16_Q(0.00076411),
  DSP16_Q(-0.00005653),
  DSP16_Q(-0.00013039),
  DSP16_Q(0.00101700),
  DSP16_Q(-0.00294288),
  DSP16_Q(0.00619286),
  DSP16_Q(-0.01117905),
  DSP16_Q(0.01862447),
  DSP16_Q(-0.03007892),
  DSP16_Q(0.04972923),
  DSP16_Q(-0.09397002),
  DSP16_Q(0.34519445),
  DSP16_Q(0.22703055),
  DSP16_Q(-0.08074641),
  DSP16_Q(0.04462744),
  DSP16_Q(-0.02728113),
  DSP16_Q(0.01684974),
  DSP16_Q(-0.00999240),
  DSP16_Q(0.00540487),
  DSP16_Q(-0.00245278),
  DSP16_Q(0.00075970),
  DSP16_Q(-0.00005575),
  DSP16_Q(-0.00013109),
  DSP16_Q(0.00101766),
  DSP16_Q(-0.00294158),
  DSP16_Q(0.00618718),
  DSP16_Q(-0.01116603),
  DSP16_Q(0.01860048),
  DSP16_Q(-0.03003970),
  DSP16_Q(0.04967130),
  DSP16_Q(-0.09391260),
  DSP16_Q(0.34635809),
  DSP16_Q(0.22562267),
  DSP16_Q(-0.08042478),
  DSP16_Q(0.04446826),
  DSP16_Q(-0.02718648),
  DSP16_Q(0.01679062),
  DSP16_Q(-0.00995589),
  DSP16_Q(0.00538360),
  DSP16_Q(-0.00244178),
  DSP16_Q(0.00075526),
  DSP16_Q(-0.00005497),
  DSP16_Q(-0.00013178),
  DSP16_Q(0.00101826),
  DSP16_Q(-0.00294011),
  DSP16_Q(0.00618115),
  DSP16_Q(-0.01115238),
  DSP16_Q(0.01857547),
  DSP16_Q(-0.02999883),
  DSP16_Q(0.04961065),
  DSP16_Q(-0.09385013),
  DSP16_Q(0.34751730),
  DSP16_Q(0.22421393),
  DSP16_Q(-0.08010003),
  DSP16_Q(0.04430715),
  DSP16_Q(-0.02709063),
  DSP16_Q(0.01673075),
  DSP16_Q(-0.00991895),
  DSP16_Q(0.00536210),
  DSP16_Q(-0.00243069),
  DSP16_Q(0.00075081),
  DSP16_Q(-0.00005419),
  DSP16_Q(-0.00013246),
  DSP16_Q(0.00101879),
  DSP16_Q(-0.00293846),
  DSP16_Q(0.00617477),
  DSP16_Q(-0.01113811),
  DSP16_Q(0.01854943),
  DSP16_Q(-0.02995631),
  DSP16_Q(0.04954729),
  DSP16_Q(-0.09378259),
  DSP16_Q(0.34867204),
  DSP16_Q(0.22280436),
  DSP16_Q(-0.07977219),
  DSP16_Q(0.04414414),
  DSP16_Q(-0.02699357),
  DSP16_Q(0.01667015),
  DSP16_Q(-0.00988158),
  DSP16_Q(0.00534038),
  DSP16_Q(-0.00241951),
  DSP16_Q(0.00074633),
  DSP16_Q(-0.00005342),
  DSP16_Q(-0.00013314),
  DSP16_Q(0.00101927),
  DSP16_Q(-0.00293665),
  DSP16_Q(0.00616805),
  DSP16_Q(-0.01112322),
  DSP16_Q(0.01852237),
  DSP16_Q(-0.02991214),
  DSP16_Q(0.04948121),
  DSP16_Q(-0.09370996),
  DSP16_Q(0.34982229),
  DSP16_Q(0.22139401),
  DSP16_Q(-0.07944128),
  DSP16_Q(0.04397923),
  DSP16_Q(-0.02689533),
  DSP16_Q(0.01660881),
  DSP16_Q(-0.00984378),
  DSP16_Q(0.00531844),
  DSP16_Q(-0.00240823),
  DSP16_Q(0.00074184),
  DSP16_Q(-0.00005265),
  DSP16_Q(-0.00013381),
  DSP16_Q(0.00101968),
  DSP16_Q(-0.00293467),
  DSP16_Q(0.00616097),
  DSP16_Q(-0.01110771),
  DSP16_Q(0.01849428),
  DSP16_Q(-0.02986633),
  DSP16_Q(0.04941241),
  DSP16_Q(-0.09363223),
  DSP16_Q(0.35096801),
  DSP16_Q(0.21998291),
  DSP16_Q(-0.07910734),
  DSP16_Q(0.04381244),
  DSP16_Q(-0.02679591),
  DSP16_Q(0.01654675),
  DSP16_Q(-0.00980556),
  DSP16_Q(0.00529629),
  DSP16_Q(-0.00239687),
  DSP16_Q(0.00073732),
  DSP16_Q(-0.00005189),
  DSP16_Q(-0.00013447),
  DSP16_Q(0.00102003),
  DSP16_Q(-0.00293251),
  DSP16_Q(0.00615353),
  DSP16_Q(-0.01109157),
  DSP16_Q(0.01846516),
  DSP16_Q(-0.02981887),
  DSP16_Q(0.04934088),
  DSP16_Q(-0.09354939),
  DSP16_Q(0.35210916),
  DSP16_Q(0.21857111),
  DSP16_Q(-0.07877038),
  DSP16_Q(0.04364378),
  DSP16_Q(-0.02669531),
  DSP16_Q(0.01648396),
  DSP16_Q(-0.00976691),
  DSP16_Q(0.00527391),
  DSP16_Q(-0.00238542),
  DSP16_Q(0.00073279),
  DSP16_Q(-0.00005114),
  DSP16_Q(-0.00013513),
  DSP16_Q(0.00102031),
  DSP16_Q(-0.00293018),
  DSP16_Q(0.00614575),
  DSP16_Q(-0.01107481),
  DSP16_Q(0.01843502),
  DSP16_Q(-0.02976976),
  DSP16_Q(0.04926662),
  DSP16_Q(-0.09346141),
  DSP16_Q(0.35324571),
  DSP16_Q(0.21715865),
  DSP16_Q(-0.07843044),
  DSP16_Q(0.04347327),
  DSP16_Q(-0.02659355),
  DSP16_Q(0.01642045),
  DSP16_Q(-0.00972785),
  DSP16_Q(0.00525132),
  DSP16_Q(-0.00237388),
  DSP16_Q(0.00072824),
  DSP16_Q(-0.00005038),
  DSP16_Q(-0.00013577),
  DSP16_Q(0.00102053),
  DSP16_Q(-0.00292767),
  DSP16_Q(0.00613761),
  DSP16_Q(-0.01105743),
  DSP16_Q(0.01840385),
  DSP16_Q(-0.02971899),
  DSP16_Q(0.04918964),
  DSP16_Q(-0.09336829),
  DSP16_Q(0.35437762),
  DSP16_Q(0.21574557),
  DSP16_Q(-0.07808753),
  DSP16_Q(0.04330091),
  DSP16_Q(-0.02649063),
  DSP16_Q(0.01635622),
  DSP16_Q(-0.00968838),
  DSP16_Q(0.00522852),
  DSP16_Q(-0.00236225),
  DSP16_Q(0.00072367),
  DSP16_Q(-0.00004963),
  DSP16_Q(-0.00013641),
  DSP16_Q(0.00102069),
  DSP16_Q(-0.00292499),
  DSP16_Q(0.00612912),
  DSP16_Q(-0.01103942),
  DSP16_Q(0.01837164),
  DSP16_Q(-0.02966658),
  DSP16_Q(0.04910992),
  DSP16_Q(-0.09327002),
  DSP16_Q(0.35550486),
  DSP16_Q(0.21433190),
  DSP16_Q(-0.07774169),
  DSP16_Q(0.04312672),
  DSP16_Q(-0.02638656),
  DSP16_Q(0.01629129),
  DSP16_Q(-0.00964849),
  DSP16_Q(0.00520551),
  DSP16_Q(-0.00235054),
  DSP16_Q(0.00071908),
  DSP16_Q(-0.00004889),
  DSP16_Q(-0.00013704),
  DSP16_Q(0.00102078),
  DSP16_Q(-0.00292214),
  DSP16_Q(0.00612027),
  DSP16_Q(-0.01102078),
  DSP16_Q(0.01833841),
  DSP16_Q(-0.02961251),
  DSP16_Q(0.04902746),
  DSP16_Q(-0.09316657),
  DSP16_Q(0.35662739),
  DSP16_Q(0.21291769),
  DSP16_Q(-0.07739294),
  DSP16_Q(0.04295072),
  DSP16_Q(-0.02628134),
  DSP16_Q(0.01622566),
  DSP16_Q(-0.00960820),
  DSP16_Q(0.00518228),
  DSP16_Q(-0.00233875),
  DSP16_Q(0.00071448),
  DSP16_Q(-0.00004815),
  DSP16_Q(-0.00013766),
  DSP16_Q(0.00102081),
  DSP16_Q(-0.00291911),
  DSP16_Q(0.00611107),
  DSP16_Q(-0.01100152),
  DSP16_Q(0.01830415),
  DSP16_Q(-0.02955678),
  DSP16_Q(0.04894227),
  DSP16_Q(-0.09305793),
  DSP16_Q(0.35774518),
  DSP16_Q(0.21150298),
  DSP16_Q(-0.07704130),
  DSP16_Q(0.04277291),
  DSP16_Q(-0.02617499),
  DSP16_Q(0.01615932),
  DSP16_Q(-0.00956750),
  DSP16_Q(0.00515885),
  DSP16_Q(-0.00232687),
  DSP16_Q(0.00070986),
  DSP16_Q(-0.00004742),
  DSP16_Q(-0.00013827),
  DSP16_Q(0.00102078),
  DSP16_Q(-0.00291591),
  DSP16_Q(0.00610152),
  DSP16_Q(-0.01098163),
  DSP16_Q(0.01826886),
  DSP16_Q(-0.02949940),
  DSP16_Q(0.04885433),
  DSP16_Q(-0.09294410),
  DSP16_Q(0.35885820),
  DSP16_Q(0.21008781),
  DSP16_Q(-0.07668681),
  DSP16_Q(0.04259331),
  DSP16_Q(-0.02606752),
  DSP16_Q(0.01609229),
  DSP16_Q(-0.00952640),
  DSP16_Q(0.00513521),
  DSP16_Q(-0.00231490),
  DSP16_Q(0.00070522),
  DSP16_Q(-0.00004669),
  DSP16_Q(-0.00013887),
  DSP16_Q(0.00102067),
  DSP16_Q(-0.00291253),
  DSP16_Q(0.00609161),
  DSP16_Q(-0.01096111),
  DSP16_Q(0.01823253),
  DSP16_Q(-0.02944037),
  DSP16_Q(0.04876365),
  DSP16_Q(-0.09282505),
  DSP16_Q(0.35996641),
  DSP16_Q(0.20867221),
  DSP16_Q(-0.07632949),
  DSP16_Q(0.04241193),
  DSP16_Q(-0.02595892),
  DSP16_Q(0.01602457),
  DSP16_Q(-0.00948491),
  DSP16_Q(0.00511137),
  DSP16_Q(-0.00230286),
  DSP16_Q(0.00070057),
  DSP16_Q(-0.00004597),
  DSP16_Q(-0.00013946),
  DSP16_Q(0.00102051),
  DSP16_Q(-0.00290897),
  DSP16_Q(0.00608134),
  DSP16_Q(-0.01093996),
  DSP16_Q(0.01819518),
  DSP16_Q(-0.02937968),
  DSP16_Q(0.04867022),
  DSP16_Q(-0.09270078),
  DSP16_Q(0.36106978),
  DSP16_Q(0.20725624),
  DSP16_Q(-0.07596936),
  DSP16_Q(0.04222878),
  DSP16_Q(-0.02584922),
  DSP16_Q(0.01595617),
  DSP16_Q(-0.00944301),
  DSP16_Q(0.00508733),
  DSP16_Q(-0.00229074),
  DSP16_Q(0.00069590),
  DSP16_Q(-0.00004525),
  DSP16_Q(-0.00014004),
  DSP16_Q(0.00102027),
  DSP16_Q(-0.00290524),
  DSP16_Q(0.00607071),
  DSP16_Q(-0.01091819),
  DSP16_Q(0.01815679),
  DSP16_Q(-0.02931733),
  DSP16_Q(0.04857405),
  DSP16_Q(-0.09257127),
  DSP16_Q(0.36216828),
  DSP16_Q(0.20583992),
  DSP16_Q(-0.07560645),
  DSP16_Q(0.04204389),
  DSP16_Q(-0.02573841),
  DSP16_Q(0.01588709),
  DSP16_Q(-0.00940073),
  DSP16_Q(0.00506309),
  DSP16_Q(-0.00227854),
  DSP16_Q(0.00069122),
  DSP16_Q(-0.00004453),
  DSP16_Q(-0.00014061),
  DSP16_Q(0.00101997),
  DSP16_Q(-0.00290133),
  DSP16_Q(0.00605973),
  DSP16_Q(-0.01089578),
  DSP16_Q(0.01811737),
  DSP16_Q(-0.02925333),
  DSP16_Q(0.04847512),
  DSP16_Q(-0.09243650),
  DSP16_Q(0.36326186),
  DSP16_Q(0.20442329),
  DSP16_Q(-0.07524079),
  DSP16_Q(0.04185726),
  DSP16_Q(-0.02562651),
  DSP16_Q(0.01581733),
  DSP16_Q(-0.00935806),
  DSP16_Q(0.00503865),
  DSP16_Q(-0.00226626),
  DSP16_Q(0.00068652),
  DSP16_Q(-0.00004383),
  DSP16_Q(-0.00014117),
  DSP16_Q(0.00101961),
  DSP16_Q(-0.00289724),
  DSP16_Q(0.00604839),
  DSP16_Q(-0.01087275),
  DSP16_Q(0.01807692),
  DSP16_Q(-0.02918766),
  DSP16_Q(0.04837344),
  DSP16_Q(-0.09229647),
  DSP16_Q(0.36435050),
  DSP16_Q(0.20300641),
  DSP16_Q(-0.07487240),
  DSP16_Q(0.04166891),
  DSP16_Q(-0.02551352),
  DSP16_Q(0.01574691),
  DSP16_Q(-0.00931500),
  DSP16_Q(0.00501402),
  DSP16_Q(-0.00225390),
  DSP16_Q(0.00068181),
  DSP16_Q(-0.00004312),
  DSP16_Q(-0.00014172),
  DSP16_Q(0.00101917),
  DSP16_Q(-0.00289298),
  DSP16_Q(0.00603669),
  DSP16_Q(-0.01084909),
  DSP16_Q(0.01803544),
  DSP16_Q(-0.02912034),
  DSP16_Q(0.04826901),
  DSP16_Q(-0.09215116),
  DSP16_Q(0.36543417),
  DSP16_Q(0.20158930),
  DSP16_Q(-0.07450131),
  DSP16_Q(0.04147884),
  DSP16_Q(-0.02539945),
  DSP16_Q(0.01567582),
  DSP16_Q(-0.00927156),
  DSP16_Q(0.00498919),
  DSP16_Q(-0.00224147),
  DSP16_Q(0.00067709),
  DSP16_Q(-0.00004242),
  DSP16_Q(-0.00014226),
  DSP16_Q(0.00101867),
  DSP16_Q(-0.00288853),
  DSP16_Q(0.00602463),
  DSP16_Q(-0.01082480),
  DSP16_Q(0.01799292),
  DSP16_Q(-0.02905137),
  DSP16_Q(0.04816183),
  DSP16_Q(-0.09200056),
  DSP16_Q(0.36651283),
  DSP16_Q(0.20017200),
  DSP16_Q(-0.07412755),
  DSP16_Q(0.04128708),
  DSP16_Q(-0.02528431),
  DSP16_Q(0.01560408),
  DSP16_Q(-0.00922774),
  DSP16_Q(0.00496417),
  DSP16_Q(-0.00222896),
  DSP16_Q(0.00067235),
  DSP16_Q(-0.00004173),
  DSP16_Q(-0.00014279),
  DSP16_Q(0.00101810),
  DSP16_Q(-0.00288391),
  DSP16_Q(0.00601222),
  DSP16_Q(-0.01079988),
  DSP16_Q(0.01794937),
  DSP16_Q(-0.02898073),
  DSP16_Q(0.04805188),
  DSP16_Q(-0.09184465),
  DSP16_Q(0.36758644),
  DSP16_Q(0.19875457),
  DSP16_Q(-0.07375114),
  DSP16_Q(0.04109364),
  DSP16_Q(-0.02516811),
  DSP16_Q(0.01553168),
  DSP16_Q(-0.00918355),
  DSP16_Q(0.00493896),
  DSP16_Q(-0.00221638),
  DSP16_Q(0.00066760),
  DSP16_Q(-0.00004104),
  DSP16_Q(-0.00014331),
  DSP16_Q(0.00101747),
  DSP16_Q(-0.00287911),
  DSP16_Q(0.00599944),
  DSP16_Q(-0.01077433),
  DSP16_Q(0.01790479),
  DSP16_Q(-0.02890844),
  DSP16_Q(0.04793918),
  DSP16_Q(-0.09168343),
  DSP16_Q(0.36865499),
  DSP16_Q(0.19733703),
  DSP16_Q(-0.07337211),
  DSP16_Q(0.04089853),
  DSP16_Q(-0.02505086),
  DSP16_Q(0.01545864),
  DSP16_Q(-0.00913898),
  DSP16_Q(0.00491356),
  DSP16_Q(-0.00220372),
  DSP16_Q(0.00066284),
  DSP16_Q(-0.00004036),
  DSP16_Q(-0.00014382),
  DSP16_Q(0.00101676),
  DSP16_Q(-0.00287412),
  DSP16_Q(0.00598631),
  DSP16_Q(-0.01074815),
  DSP16_Q(0.01785918),
  DSP16_Q(-0.02883449),
  DSP16_Q(0.04782372),
  DSP16_Q(-0.09151689),
  DSP16_Q(0.36971842),
  DSP16_Q(0.19591942),
  DSP16_Q(-0.07299048),
  DSP16_Q(0.04070176),
  DSP16_Q(-0.02493256),
  DSP16_Q(0.01538495),
  DSP16_Q(-0.00909405),
  DSP16_Q(0.00488798),
  DSP16_Q(-0.00219100),
  DSP16_Q(0.00065807),
  DSP16_Q(-0.00003969),
  DSP16_Q(-0.00014432),
  DSP16_Q(0.00101599),
  DSP16_Q(-0.00286896),
  DSP16_Q(0.00597282),
  DSP16_Q(-0.01072134),
  DSP16_Q(0.01781253),
  DSP16_Q(-0.02875888),
  DSP16_Q(0.04770550),
  DSP16_Q(-0.09134500),
  DSP16_Q(0.37077671),
  DSP16_Q(0.19450180),
  DSP16_Q(-0.07260629),
  DSP16_Q(0.04050335),
  DSP16_Q(-0.02481323),
  DSP16_Q(0.01531062),
  DSP16_Q(-0.00904875),
  DSP16_Q(0.00486221),
  DSP16_Q(-0.00217820),
  DSP16_Q(0.00065328),
  DSP16_Q(-0.00003902),
  DSP16_Q(-0.00014481),
  DSP16_Q(0.00101515),
  DSP16_Q(-0.00286362),
  DSP16_Q(0.00595896),
  DSP16_Q(-0.01069390),
  DSP16_Q(0.01776485),
  DSP16_Q(-0.02868162),
  DSP16_Q(0.04758452),
  DSP16_Q(-0.09116777),
  DSP16_Q(0.37182983),
  DSP16_Q(0.19308418),
  DSP16_Q(-0.07221955),
  DSP16_Q(0.04030331),
  DSP16_Q(-0.02469286),
  DSP16_Q(0.01523567),
  DSP16_Q(-0.00900308),
  DSP16_Q(0.00483626),
  DSP16_Q(-0.00216533),
  DSP16_Q(0.00064849),
  DSP16_Q(-0.00003835),
  DSP16_Q(-0.00014528),
  DSP16_Q(0.00101423),
  DSP16_Q(-0.00285809),
  DSP16_Q(0.00594475),
  DSP16_Q(-0.01066582),
  DSP16_Q(0.01771614),
  DSP16_Q(-0.02860270),
  DSP16_Q(0.04746078),
  DSP16_Q(-0.09098517),
  DSP16_Q(0.37287775),
  DSP16_Q(0.19166663),
  DSP16_Q(-0.07183030),
  DSP16_Q(0.04010166),
  DSP16_Q(-0.02457148),
  DSP16_Q(0.01516009),
  DSP16_Q(-0.00895706),
  DSP16_Q(0.00481013),
  DSP16_Q(-0.00215240),
  DSP16_Q(0.00064368),
  DSP16_Q(-0.00003769),
  DSP16_Q(-0.00014575),
  DSP16_Q(0.00101325),
  DSP16_Q(-0.00285239),
  DSP16_Q(0.00593017),
  DSP16_Q(-0.01063712),
  DSP16_Q(0.01766640),
  DSP16_Q(-0.02852212),
  DSP16_Q(0.04733428),
  DSP16_Q(-0.09079720),
  DSP16_Q(0.37392043),
  DSP16_Q(0.19024916),
  DSP16_Q(-0.07143856),
  DSP16_Q(0.03989841),
  DSP16_Q(-0.02444908),
  DSP16_Q(0.01508388),
  DSP16_Q(-0.00891068),
  DSP16_Q(0.00478382),
  DSP16_Q(-0.00213940),
  DSP16_Q(0.00063886),
  DSP16_Q(-0.00003704),
  DSP16_Q(-0.00014620),
  DSP16_Q(0.00101220),
  DSP16_Q(-0.00284650),
  DSP16_Q(0.00591524),
  DSP16_Q(-0.01060779),
  DSP16_Q(0.01761563),
  DSP16_Q(-0.02843989),
  DSP16_Q(0.04720501),
  DSP16_Q(-0.09060384),
  DSP16_Q(0.37495784),
  DSP16_Q(0.18883183),
  DSP16_Q(-0.07104435),
  DSP16_Q(0.03969358),
  DSP16_Q(-0.02432568),
  DSP16_Q(0.01500706),
  DSP16_Q(-0.00886396),
  DSP16_Q(0.00475734),
  DSP16_Q(-0.00212633),
  DSP16_Q(0.00063404),
  DSP16_Q(-0.00003639),
  DSP16_Q(-0.00014664),
  DSP16_Q(0.00101108),
  DSP16_Q(-0.00284043),
  DSP16_Q(0.00589994),
  DSP16_Q(-0.01057783),
  DSP16_Q(0.01756383),
  DSP16_Q(-0.02835600),
  DSP16_Q(0.04707298),
  DSP16_Q(-0.09040509),
  DSP16_Q(0.37598995),
  DSP16_Q(0.18741468),
  DSP16_Q(-0.07064772),
  DSP16_Q(0.03948717),
  DSP16_Q(-0.02420129),
  DSP16_Q(0.01492964),
  DSP16_Q(-0.00881688),
  DSP16_Q(0.00473068),
  DSP16_Q(-0.00211319),
  DSP16_Q(0.00062920),
  DSP16_Q(-0.00003574),
  DSP16_Q(-0.00014707),
  DSP16_Q(0.00100989),
  DSP16_Q(-0.00283418),
  DSP16_Q(0.00588428),
  DSP16_Q(-0.01054723),
  DSP16_Q(0.01751099),
  DSP16_Q(-0.02827046),
  DSP16_Q(0.04693819),
  DSP16_Q(-0.09020093),
  DSP16_Q(0.37701673),
  DSP16_Q(0.18599773),
  DSP16_Q(-0.07024867),
  DSP16_Q(0.03927921),
  DSP16_Q(-0.02407591),
  DSP16_Q(0.01485160),
  DSP16_Q(-0.00876946),
  DSP16_Q(0.00470385),
  DSP16_Q(-0.00209999),
  DSP16_Q(0.00062436),
  DSP16_Q(-0.00003511),
  DSP16_Q(-0.00014749),
  DSP16_Q(0.00100862),
  DSP16_Q(-0.00282775),
  DSP16_Q(0.00586826),
  DSP16_Q(-0.01051601),
  DSP16_Q(0.01745713),
  DSP16_Q(-0.02818326),
  DSP16_Q(0.04680063),
  DSP16_Q(-0.08999136),
  DSP16_Q(0.37803814),
  DSP16_Q(0.18458104),
  DSP16_Q(-0.06984724),
  DSP16_Q(0.03906970),
  DSP16_Q(-0.02394956),
  DSP16_Q(0.01477297),
  DSP16_Q(-0.00872169),
  DSP16_Q(0.00467685),
  DSP16_Q(-0.00208673),
  DSP16_Q(0.00061950),
  DSP16_Q(-0.00003448),
  DSP16_Q(-0.00014789),
  DSP16_Q(0.00100729),
  DSP16_Q(-0.00282114),
  DSP16_Q(0.00585188),
  DSP16_Q(-0.01048416),
  DSP16_Q(0.01740224),
  DSP16_Q(-0.02809441),
  DSP16_Q(0.04666031),
  DSP16_Q(-0.08977635),
  DSP16_Q(0.37905416),
  DSP16_Q(0.18316464),
  DSP16_Q(-0.06944346),
  DSP16_Q(0.03885867),
  DSP16_Q(-0.02382223),
  DSP16_Q(0.01469374),
  DSP16_Q(-0.00867359),
  DSP16_Q(0.00464969),
  DSP16_Q(-0.00207341),
  DSP16_Q(0.00061464),
  DSP16_Q(-0.00003385),
  DSP16_Q(-0.00014828),
  DSP16_Q(0.00100588),
  DSP16_Q(-0.00281434),
  DSP16_Q(0.00583513),
  DSP16_Q(-0.01045168),
  DSP16_Q(0.01734631),
  DSP16_Q(-0.02800391),
  DSP16_Q(0.04651723),
  DSP16_Q(-0.08955591),
  DSP16_Q(0.38006475),
  DSP16_Q(0.18174857),
  DSP16_Q(-0.06903734),
  DSP16_Q(0.03864612),
  DSP16_Q(-0.02369395),
  DSP16_Q(0.01461393),
  DSP16_Q(-0.00862515),
  DSP16_Q(0.00462235),
  DSP16_Q(-0.00206002),
  DSP16_Q(0.00060977),
  DSP16_Q(-0.00003323),
  DSP16_Q(-0.00014866),
  DSP16_Q(0.00100440),
  DSP16_Q(-0.00280735),
  DSP16_Q(0.00581803),
  DSP16_Q(-0.01041856),
  DSP16_Q(0.01728936),
  DSP16_Q(-0.02791176),
  DSP16_Q(0.04637138),
  DSP16_Q(-0.08933003),
  DSP16_Q(0.38106989),
  DSP16_Q(0.18033287),
  DSP16_Q(-0.06862893),
  DSP16_Q(0.03843206),
  DSP16_Q(-0.02356471),
  DSP16_Q(0.01453353),
  DSP16_Q(-0.00857638),
  DSP16_Q(0.00459486),
  DSP16_Q(-0.00204658),
  DSP16_Q(0.00060490),
  DSP16_Q(-0.00003262),
  DSP16_Q(-0.00014903),
  DSP16_Q(0.00100285),
  DSP16_Q(-0.00280019),
  DSP16_Q(0.00580056),
  DSP16_Q(-0.01038482),
  DSP16_Q(0.01723139),
  DSP16_Q(-0.02781796),
  DSP16_Q(0.04622277),
  DSP16_Q(-0.08909868),
  DSP16_Q(0.38206953),
  DSP16_Q(0.17891757),
  DSP16_Q(-0.06821824),
  DSP16_Q(0.03821653),
  DSP16_Q(-0.02343454),
  DSP16_Q(0.01445255),
  DSP16_Q(-0.00852729),
  DSP16_Q(0.00456720),
  DSP16_Q(-0.00203307),
  DSP16_Q(0.00060002),
  DSP16_Q(-0.00003201),
  DSP16_Q(-0.00014938),
  DSP16_Q(0.00100123),
  DSP16_Q(-0.00279284),
  DSP16_Q(0.00578273),
  DSP16_Q(-0.01035045),
  DSP16_Q(0.01717239),
  DSP16_Q(-0.02772252),
  DSP16_Q(0.04607140),
  DSP16_Q(-0.08886187),
  DSP16_Q(0.38306366),
  DSP16_Q(0.17750273),
  DSP16_Q(-0.06780531),
  DSP16_Q(0.03799952),
  DSP16_Q(-0.02330342),
  DSP16_Q(0.01437101),
  DSP16_Q(-0.00847787),
  DSP16_Q(0.00453938),
  DSP16_Q(-0.00201951),
  DSP16_Q(0.00059513),
  DSP16_Q(-0.00003141),
  DSP16_Q(-0.00014972),
  DSP16_Q(0.00099954),
  DSP16_Q(-0.00278531),
  DSP16_Q(0.00576454),
  DSP16_Q(-0.01031545),
  DSP16_Q(0.01711236),
  DSP16_Q(-0.02762542),
  DSP16_Q(0.04591727),
  DSP16_Q(-0.08861958),
  DSP16_Q(0.38405224),
  DSP16_Q(0.17608837),
  DSP16_Q(-0.06739015),
  DSP16_Q(0.03778105),
  DSP16_Q(-0.02317139),
  DSP16_Q(0.01428890),
  DSP16_Q(-0.00842812),
  DSP16_Q(0.00451141),
  DSP16_Q(-0.00200589),
  DSP16_Q(0.00059023),
  DSP16_Q(-0.00003081),
  DSP16_Q(-0.00015005),
  DSP16_Q(0.00099777),
  DSP16_Q(-0.00277759),
  DSP16_Q(0.00574599),
  DSP16_Q(-0.01027982),
  DSP16_Q(0.01705130),
  DSP16_Q(-0.02752668),
  DSP16_Q(0.04576038),
  DSP16_Q(-0.08837181),
  DSP16_Q(0.38503524),
  DSP16_Q(0.17467453),
  DSP16_Q(-0.06697280),
  DSP16_Q(0.03756115),
  DSP16_Q(-0.02303843),
  DSP16_Q(0.01420622),
  DSP16_Q(-0.00837806),
  DSP16_Q(0.00448328),
  DSP16_Q(-0.00199221),
  DSP16_Q(0.00058533),
  DSP16_Q(-0.00003022),
  DSP16_Q(-0.00015036),
  DSP16_Q(0.00099593),
  DSP16_Q(-0.00276969),
  DSP16_Q(0.00572707),
  DSP16_Q(-0.01024357),
  DSP16_Q(0.01698923),
  DSP16_Q(-0.02742630),
  DSP16_Q(0.04560073),
  DSP16_Q(-0.08811854),
  DSP16_Q(0.38601262),
  DSP16_Q(0.17326127),
  DSP16_Q(-0.06655328),
  DSP16_Q(0.03733981),
  DSP16_Q(-0.02290457),
  DSP16_Q(0.01412299),
  DSP16_Q(-0.00832769),
  DSP16_Q(0.00445499),
  DSP16_Q(-0.00197848),
  DSP16_Q(0.00058043),
  DSP16_Q(-0.00002964),
  DSP16_Q(-0.00015066),
  DSP16_Q(0.00099401),
  DSP16_Q(-0.00276160),
  DSP16_Q(0.00570780),
  DSP16_Q(-0.01020669),
  DSP16_Q(0.01692613),
  DSP16_Q(-0.02732427),
  DSP16_Q(0.04543832),
  DSP16_Q(-0.08785977),
  DSP16_Q(0.38698437),
  DSP16_Q(0.17184860),
  DSP16_Q(-0.06613162),
  DSP16_Q(0.03711706),
  DSP16_Q(-0.02276981),
  DSP16_Q(0.01403922),
  DSP16_Q(-0.00827700),
  DSP16_Q(0.00442656),
  DSP16_Q(-0.00196470),
  DSP16_Q(0.00057552),
  DSP16_Q(-0.00002906),
  DSP16_Q(-0.00015094),
  DSP16_Q(0.00099202),
  DSP16_Q(-0.00275333),
  DSP16_Q(0.00568816),
  DSP16_Q(-0.01016918),
  DSP16_Q(0.01686201),
  DSP16_Q(-0.02722061),
  DSP16_Q(0.04527315),
  DSP16_Q(-0.08759548),
  DSP16_Q(0.38795044),
  DSP16_Q(0.17043658),
  DSP16_Q(-0.06570785),
  DSP16_Q(0.03689291),
  DSP16_Q(-0.02263417),
  DSP16_Q(0.01395490),
  DSP16_Q(-0.00822601),
  DSP16_Q(0.00439797),
  DSP16_Q(-0.00195086),
  DSP16_Q(0.00057060),
  DSP16_Q(-0.00002849),
  DSP16_Q(-0.00015122),
  DSP16_Q(0.00098996),
  DSP16_Q(-0.00274487),
  DSP16_Q(0.00566816),
  DSP16_Q(-0.01013104),
  DSP16_Q(0.01679687),
  DSP16_Q(-0.02711530),
  DSP16_Q(0.04510523),
  DSP16_Q(-0.08732568),
  DSP16_Q(0.38891082),
  DSP16_Q(0.16902523),
  DSP16_Q(-0.06528199),
  DSP16_Q(0.03666738),
  DSP16_Q(-0.02249764),
  DSP16_Q(0.01387004),
  DSP16_Q(-0.00817472),
  DSP16_Q(0.00436924),
  DSP16_Q(-0.00193697),
  DSP16_Q(0.00056569),
  DSP16_Q(-0.00002792),
  DSP16_Q(-0.00015147),
  DSP16_Q(0.00098783),
  DSP16_Q(-0.00273623),
  DSP16_Q(0.00564779),
  DSP16_Q(-0.01009228),
  DSP16_Q(0.01673071),
  DSP16_Q(-0.02700836),
  DSP16_Q(0.04493456),
  DSP16_Q(-0.08705034),
  DSP16_Q(0.38986546),
  DSP16_Q(0.16761461),
  DSP16_Q(-0.06485408),
  DSP16_Q(0.03644048),
  DSP16_Q(-0.02236024),
  DSP16_Q(0.01378465),
  DSP16_Q(-0.00812312),
  DSP16_Q(0.00434036),
  DSP16_Q(-0.00192303),
  DSP16_Q(0.00056076),
  DSP16_Q(-0.00002736),
  DSP16_Q(-0.00015172),
  DSP16_Q(0.00098562),
  DSP16_Q(-0.00272740),
  DSP16_Q(0.00562707),
  DSP16_Q(-0.01005289),
  DSP16_Q(0.01666353),
  DSP16_Q(-0.02689979),
  DSP16_Q(0.04476113),
  DSP16_Q(-0.08676947),
  DSP16_Q(0.39081435),
  DSP16_Q(0.16620474),
  DSP16_Q(-0.06442413),
  DSP16_Q(0.03621222),
  DSP16_Q(-0.02222197),
  DSP16_Q(0.01369873),
  DSP16_Q(-0.00807123),
  DSP16_Q(0.00431134),
  DSP16_Q(-0.00190904),
  DSP16_Q(0.00055584),
  DSP16_Q(-0.00002681),
  DSP16_Q(-0.00015194),
  DSP16_Q(0.00098333),
  DSP16_Q(-0.00271838),
  DSP16_Q(0.00560598),
  DSP16_Q(-0.01001287),
  DSP16_Q(0.01659533),
  DSP16_Q(-0.02678958),
  DSP16_Q(0.04458495),
  DSP16_Q(-0.08648305),
  DSP16_Q(0.39175744),
  DSP16_Q(0.16479567),
  DSP16_Q(-0.06399218),
  DSP16_Q(0.03598262),
  DSP16_Q(-0.02208286),
  DSP16_Q(0.01361229),
  DSP16_Q(-0.00801905),
  DSP16_Q(0.00428217),
  DSP16_Q(-0.00189501),
  DSP16_Q(0.00055091),
  DSP16_Q(-0.00002626),
  DSP16_Q(-0.00015216),
  DSP16_Q(0.00098097),
  DSP16_Q(-0.00270918),
  DSP16_Q(0.00558453),
  DSP16_Q(-0.00997224),
  DSP16_Q(0.01652612),
  DSP16_Q(-0.02667774),
  DSP16_Q(0.04440603),
  DSP16_Q(-0.08619107),
  DSP16_Q(0.39269472),
  DSP16_Q(0.16338744),
  DSP16_Q(-0.06355825),
  DSP16_Q(0.03575170),
  DSP16_Q(-0.02194290),
  DSP16_Q(0.01352533),
  DSP16_Q(-0.00796657),
  DSP16_Q(0.00425287),
  DSP16_Q(-0.00188092),
  DSP16_Q(0.00054598),
  DSP16_Q(-0.00002572),
  DSP16_Q(-0.00015236),
  DSP16_Q(0.00097854),
  DSP16_Q(-0.00269980),
  DSP16_Q(0.00556272),
  DSP16_Q(-0.00993098),
  DSP16_Q(0.01645590),
  DSP16_Q(-0.02656427),
  DSP16_Q(0.04422435),
  DSP16_Q(-0.08589354),
  DSP16_Q(0.39362616),
  DSP16_Q(0.16198007),
  DSP16_Q(-0.06312236),
  DSP16_Q(0.03551947),
  DSP16_Q(-0.02180210),
  DSP16_Q(0.01343787),
  DSP16_Q(-0.00791381),
  DSP16_Q(0.00422343),
  DSP16_Q(-0.00186679),
  DSP16_Q(0.00054105),
  DSP16_Q(-0.00002518),
  DSP16_Q(-0.00015254),
  DSP16_Q(0.00097603),
  DSP16_Q(-0.00269023),
  DSP16_Q(0.00554055),
  DSP16_Q(-0.00988909),
  DSP16_Q(0.01638466),
  DSP16_Q(-0.02644917),
  DSP16_Q(0.04403994),
  DSP16_Q(-0.08559044),
  DSP16_Q(0.39455171),
  DSP16_Q(0.16057362),
  DSP16_Q(-0.06268456),
  DSP16_Q(0.03528594),
  DSP16_Q(-0.02166048),
  DSP16_Q(0.01334990),
  DSP16_Q(-0.00786077),
  DSP16_Q(0.00419386),
  DSP16_Q(-0.00185261),
  DSP16_Q(0.00053612),
  DSP16_Q(-0.00002465),
  DSP16_Q(-0.00015271),
  DSP16_Q(0.00097344),
  DSP16_Q(-0.00268047),
  DSP16_Q(0.00551802),
  DSP16_Q(-0.00984659),
  DSP16_Q(0.01631241),
  DSP16_Q(-0.02633245),
  DSP16_Q(0.04385278),
  DSP16_Q(-0.08528176),
  DSP16_Q(0.39547136),
  DSP16_Q(0.15916812),
  DSP16_Q(-0.06224486),
  DSP16_Q(0.03505113),
  DSP16_Q(-0.02151804),
  DSP16_Q(0.01326144),
  DSP16_Q(-0.00780745),
  DSP16_Q(0.00416415),
  DSP16_Q(-0.00183839),
  DSP16_Q(0.00053118),
  DSP16_Q(-0.00002413),
  DSP16_Q(-0.00015286),
  DSP16_Q(0.00097078),
  DSP16_Q(-0.00267053),
  DSP16_Q(0.00549512),
  DSP16_Q(-0.00980346),
  DSP16_Q(0.01623915),
  DSP16_Q(-0.02621411),
  DSP16_Q(0.04366288),
  DSP16_Q(-0.08496749),
  DSP16_Q(0.39638508),
  DSP16_Q(0.15776360),
  DSP16_Q(-0.06180328),
  DSP16_Q(0.03481505),
  DSP16_Q(-0.02137479),
  DSP16_Q(0.01317248),
  DSP16_Q(-0.00775385),
  DSP16_Q(0.00413431),
  DSP16_Q(-0.00182413),
  DSP16_Q(0.00052625),
  DSP16_Q(-0.00002362),
  DSP16_Q(-0.00015300),
  DSP16_Q(0.00096804),
  DSP16_Q(-0.00266039),
  DSP16_Q(0.00547187),
  DSP16_Q(-0.00975971),
  DSP16_Q(0.01616489),
  DSP16_Q(-0.02609415),
  DSP16_Q(0.04347025),
  DSP16_Q(-0.08464764),
  DSP16_Q(0.39729284),
  DSP16_Q(0.15636011),
  DSP16_Q(-0.06135987),
  DSP16_Q(0.03457772),
  DSP16_Q(-0.02123074),
  DSP16_Q(0.01308303),
  DSP16_Q(-0.00769998),
  DSP16_Q(0.00410435),
  DSP16_Q(-0.00180982),
  DSP16_Q(0.00052131),
  DSP16_Q(-0.00002311),
  DSP16_Q(-0.00015312),
  DSP16_Q(0.00096523),
  DSP16_Q(-0.00265008),
  DSP16_Q(0.00544826),
  DSP16_Q(-0.00971534),
  DSP16_Q(0.01608962),
  DSP16_Q(-0.02597258),
  DSP16_Q(0.04327488),
  DSP16_Q(-0.08432219),
  DSP16_Q(0.39819461),
  DSP16_Q(0.15495769),
  DSP16_Q(-0.06091464),
  DSP16_Q(0.03433916),
  DSP16_Q(-0.02108590),
  DSP16_Q(0.01299311),
  DSP16_Q(-0.00764585),
  DSP16_Q(0.00407425),
  DSP16_Q(-0.00179547),
  DSP16_Q(0.00051637),
  DSP16_Q(-0.00002260),
  DSP16_Q(-0.00015322),
  DSP16_Q(0.00096234),
  DSP16_Q(-0.00263957),
  DSP16_Q(0.00542428),
  DSP16_Q(-0.00967036),
  DSP16_Q(0.01601334),
  DSP16_Q(-0.02584939),
  DSP16_Q(0.04307679),
  DSP16_Q(-0.08399114),
  DSP16_Q(0.39909037),
  DSP16_Q(0.15355636),
  DSP16_Q(-0.06046762),
  DSP16_Q(0.03409937),
  DSP16_Q(-0.02094029),
  DSP16_Q(0.01290271),
  DSP16_Q(-0.00759145),
  DSP16_Q(0.00404403),
  DSP16_Q(-0.00178108),
  DSP16_Q(0.00051144),
  DSP16_Q(-0.00002210),
  DSP16_Q(-0.00015331),
  DSP16_Q(0.00095937),
  DSP16_Q(-0.00262888),
  DSP16_Q(0.00539995),
  DSP16_Q(-0.00962475),
  DSP16_Q(0.01593606),
  DSP16_Q(-0.02572459),
  DSP16_Q(0.04287596),
  DSP16_Q(-0.08365448),
  DSP16_Q(0.39998008),
  DSP16_Q(0.15215618),
  DSP16_Q(-0.06001884),
  DSP16_Q(0.03385838),
  DSP16_Q(-0.02079390),
  DSP16_Q(0.01281184),
  DSP16_Q(-0.00753679),
  DSP16_Q(0.00401369),
  DSP16_Q(-0.00176666),
  DSP16_Q(0.00050650),
  DSP16_Q(-0.00002161),
  DSP16_Q(-0.00015338),
  DSP16_Q(0.00095632),
  DSP16_Q(-0.00261801),
  DSP16_Q(0.00537525),
  DSP16_Q(-0.00957853),
  DSP16_Q(0.01585778),
  DSP16_Q(-0.02559819),
  DSP16_Q(0.04267241),
  DSP16_Q(-0.08331220),
  DSP16_Q(0.40086371),
  DSP16_Q(0.15075717),
  DSP16_Q(-0.05956833),
  DSP16_Q(0.03361620),
  DSP16_Q(-0.02064675),
  DSP16_Q(0.01272051),
  DSP16_Q(-0.00748187),
  DSP16_Q(0.00398323),
  DSP16_Q(-0.00175219),
  DSP16_Q(0.00050157),
  DSP16_Q(-0.00002113),
  DSP16_Q(-0.00015344),
  DSP16_Q(0.00095320),
  DSP16_Q(-0.00260694),
  DSP16_Q(0.00535020),
  DSP16_Q(-0.00953170),
  DSP16_Q(0.01577851),
  DSP16_Q(-0.02547017),
  DSP16_Q(0.04246614),
  DSP16_Q(-0.08296430),
  DSP16_Q(0.40174125),
  DSP16_Q(0.14935937),
  DSP16_Q(-0.05911611),
  DSP16_Q(0.03337284),
  DSP16_Q(-0.02049884),
  DSP16_Q(0.01262872),
  DSP16_Q(-0.00742670),
  DSP16_Q(0.00395265),
  DSP16_Q(-0.00173769),
  DSP16_Q(0.00049664),
  DSP16_Q(-0.00002065),
  DSP16_Q(-0.00015348),
  DSP16_Q(0.00095000),
  DSP16_Q(-0.00259569),
  DSP16_Q(0.00532479),
  DSP16_Q(-0.00948424),
  DSP16_Q(0.01569823),
  DSP16_Q(-0.02534056),
  DSP16_Q(0.04225715),
  DSP16_Q(-0.08261078),
  DSP16_Q(0.40261266),
  DSP16_Q(0.14796283),
  DSP16_Q(-0.05866221),
  DSP16_Q(0.03312832),
  DSP16_Q(-0.02035019),
  DSP16_Q(0.01253648),
  DSP16_Q(-0.00737127),
  DSP16_Q(0.00392195),
  DSP16_Q(-0.00172315),
  DSP16_Q(0.00049171),
  DSP16_Q(-0.00002018),
  DSP16_Q(-0.00015350),
  DSP16_Q(0.00094673),
  DSP16_Q(-0.00258425),
  DSP16_Q(0.00529902),
  DSP16_Q(-0.00943618),
  DSP16_Q(0.01561696),
  DSP16_Q(-0.02520935),
  DSP16_Q(0.04204545),
  DSP16_Q(-0.08225162),
  DSP16_Q(0.40347792),
  DSP16_Q(0.14656757),
  DSP16_Q(-0.05820665),
  DSP16_Q(0.03288266),
  DSP16_Q(-0.02020081),
  DSP16_Q(0.01244379),
  DSP16_Q(-0.00731561),
  DSP16_Q(0.00389114),
  DSP16_Q(-0.00170857),
  DSP16_Q(0.00048678),
  DSP16_Q(-0.00001971),
  DSP16_Q(-0.00015351),
  DSP16_Q(0.00094337),
  DSP16_Q(-0.00257263),
  DSP16_Q(0.00527289),
  DSP16_Q(-0.00938750),
  DSP16_Q(0.01553470),
  DSP16_Q(-0.02507654),
  DSP16_Q(0.04183104),
  DSP16_Q(-0.08188682),
  DSP16_Q(0.40433699),
  DSP16_Q(0.14517364),
  DSP16_Q(-0.05774947),
  DSP16_Q(0.03263586),
  DSP16_Q(-0.02005071),
  DSP16_Q(0.01235067),
  DSP16_Q(-0.00725970),
  DSP16_Q(0.00386022),
  DSP16_Q(-0.00169397),
  DSP16_Q(0.00048185),
  DSP16_Q(-0.00001925),
  DSP16_Q(-0.00015350),
  DSP16_Q(0.00093994),
  DSP16_Q(-0.00256082),
  DSP16_Q(0.00524640),
  DSP16_Q(-0.00933821),
  DSP16_Q(0.01545145),
  DSP16_Q(-0.02494214),
  DSP16_Q(0.04161392),
  DSP16_Q(-0.08151639),
  DSP16_Q(0.40518986),
  DSP16_Q(0.14378108),
  DSP16_Q(-0.05729070),
  DSP16_Q(0.03238795),
  DSP16_Q(-0.01989988),
  DSP16_Q(0.01225711),
  DSP16_Q(-0.00720355),
  DSP16_Q(0.00382919),
  DSP16_Q(-0.00167932),
  DSP16_Q(0.00047693),
  DSP16_Q(-0.00001880),
  DSP16_Q(-0.00015347),
  DSP16_Q(0.00093643),
  DSP16_Q(-0.00254882),
  DSP16_Q(0.00521956),
  DSP16_Q(-0.00928831),
  DSP16_Q(0.01536721),
  DSP16_Q(-0.02480616),
  DSP16_Q(0.04139410),
  DSP16_Q(-0.08114030),
  DSP16_Q(0.40603650),
  DSP16_Q(0.14238991),
  DSP16_Q(-0.05683035),
  DSP16_Q(0.03213894),
  DSP16_Q(-0.01974835),
  DSP16_Q(0.01216312),
  DSP16_Q(-0.00714716),
  DSP16_Q(0.00379804),
  DSP16_Q(-0.00166465),
  DSP16_Q(0.00047201),
  DSP16_Q(-0.00001835),
  DSP16_Q(-0.00015343),
  DSP16_Q(0.00093284),
  DSP16_Q(-0.00253663),
  DSP16_Q(0.00519236),
  DSP16_Q(-0.00923780),
  DSP16_Q(0.01528198),
  DSP16_Q(-0.02466859),
  DSP16_Q(0.04117158),
  DSP16_Q(-0.08075856),
  DSP16_Q(0.40687687),
  DSP16_Q(0.14100019),
  DSP16_Q(-0.05636845),
  DSP16_Q(0.03188884),
  DSP16_Q(-0.01959613),
  DSP16_Q(0.01206871),
  DSP16_Q(-0.00709054),
  DSP16_Q(0.00376680),
  DSP16_Q(-0.00164995),
  DSP16_Q(0.00046709),
  DSP16_Q(-0.00001791),
  DSP16_Q(-0.00015336),
  DSP16_Q(0.00092918),
  DSP16_Q(-0.00252426),
  DSP16_Q(0.00516481),
  DSP16_Q(-0.00918668),
  DSP16_Q(0.01519577),
  DSP16_Q(-0.02452943),
  DSP16_Q(0.04094637),
  DSP16_Q(-0.08037116),
  DSP16_Q(0.40771097),
  DSP16_Q(0.13961194),
  DSP16_Q(-0.05590504),
  DSP16_Q(0.03163767),
  DSP16_Q(-0.01944321),
  DSP16_Q(0.01197388),
  DSP16_Q(-0.00703370),
  DSP16_Q(0.00373545),
  DSP16_Q(-0.00163521),
  DSP16_Q(0.00046218),
  DSP16_Q(-0.00001747),
  DSP16_Q(-0.00015328),
  DSP16_Q(0.00092543),
  DSP16_Q(-0.00251170),
  DSP16_Q(0.00513689),
  DSP16_Q(-0.00913496),
  DSP16_Q(0.01510858),
  DSP16_Q(-0.02438870),
  DSP16_Q(0.04071847),
  DSP16_Q(-0.07997810),
  DSP16_Q(0.40853874),
  DSP16_Q(0.13822520),
  DSP16_Q(-0.05544014),
  DSP16_Q(0.03138545),
  DSP16_Q(-0.01928962),
  DSP16_Q(0.01187864),
  DSP16_Q(-0.00697663),
  DSP16_Q(0.00370399),
  DSP16_Q(-0.00162045),
  DSP16_Q(0.00045727),
  DSP16_Q(-0.00001704),
  DSP16_Q(-0.00015318),
  DSP16_Q(0.00092161),
  DSP16_Q(-0.00249896),
  DSP16_Q(0.00510863),
  DSP16_Q(-0.00908263),
  DSP16_Q(0.01502041),
  DSP16_Q(-0.02424640),
  DSP16_Q(0.04048789),
  DSP16_Q(-0.07957938),
  DSP16_Q(0.40936019),
  DSP16_Q(0.13684001),
  DSP16_Q(-0.05497378),
  DSP16_Q(0.03113219),
  DSP16_Q(-0.01913536),
  DSP16_Q(0.01178300),
  DSP16_Q(-0.00691933),
  DSP16_Q(0.00367244),
  DSP16_Q(-0.00160566),
  DSP16_Q(0.00045237),
  DSP16_Q(-0.00001662),
  DSP16_Q(-0.00015307),
  DSP16_Q(0.00091771),
  DSP16_Q(-0.00248602),
  DSP16_Q(0.00508001),
  DSP16_Q(-0.00902970),
  DSP16_Q(0.01493126),
  DSP16_Q(-0.02410253),
  DSP16_Q(0.04025462),
  DSP16_Q(-0.07917498),
  DSP16_Q(0.41017526),
  DSP16_Q(0.13545640),
  DSP16_Q(-0.05450598),
  DSP16_Q(0.03087790),
  DSP16_Q(-0.01898044),
  DSP16_Q(0.01168696),
  DSP16_Q(-0.00686182),
  DSP16_Q(0.00364079),
  DSP16_Q(-0.00159084),
  DSP16_Q(0.00044747),
  DSP16_Q(-0.00001620),
  DSP16_Q(-0.00015293),
  DSP16_Q(0.00091373),
  DSP16_Q(-0.00247291),
  DSP16_Q(0.00505103),
  DSP16_Q(-0.00897616),
  DSP16_Q(0.01484114),
  DSP16_Q(-0.02395709),
  DSP16_Q(0.04001869),
  DSP16_Q(-0.07876491),
  DSP16_Q(0.41098395),
  DSP16_Q(0.13407442),
  DSP16_Q(-0.05403678),
  DSP16_Q(0.03062261),
  DSP16_Q(-0.01882487),
  DSP16_Q(0.01159053),
  DSP16_Q(-0.00680410),
  DSP16_Q(0.00360904),
  DSP16_Q(-0.00157600),
  DSP16_Q(0.00044258),
  DSP16_Q(-0.00001579),
  DSP16_Q(-0.00015278),
  DSP16_Q(0.00090967),
  DSP16_Q(-0.00245960),
  DSP16_Q(0.00502170),
  DSP16_Q(-0.00892202),
  DSP16_Q(0.01475005),
  DSP16_Q(-0.02381009),
  DSP16_Q(0.03978008),
  DSP16_Q(-0.07834917),
  DSP16_Q(0.41178623),
  DSP16_Q(0.13269409),
  DSP16_Q(-0.05356619),
  DSP16_Q(0.03036632),
  DSP16_Q(-0.01866866),
  DSP16_Q(0.01149372),
  DSP16_Q(-0.00674617),
  DSP16_Q(0.00357720),
  DSP16_Q(-0.00156113),
  DSP16_Q(0.00043769),
  DSP16_Q(-0.00001539),
  DSP16_Q(-0.00015261),
  DSP16_Q(0.00090553),
  DSP16_Q(-0.00244611),
  DSP16_Q(0.00499202),
  DSP16_Q(-0.00886728),
  DSP16_Q(0.01465799),
  DSP16_Q(-0.02366153),
  DSP16_Q(0.03953880),
  DSP16_Q(-0.07792774),
  DSP16_Q(0.41258207),
  DSP16_Q(0.13131546),
  DSP16_Q(-0.05309424),
  DSP16_Q(0.03010905),
  DSP16_Q(-0.01851182),
  DSP16_Q(0.01139652),
  DSP16_Q(-0.00668803),
  DSP16_Q(0.00354527),
  DSP16_Q(-0.00154624),
  DSP16_Q(0.00043281),
  DSP16_Q(-0.00001499),
  DSP16_Q(-0.00015242),
  DSP16_Q(0.00090131),
  DSP16_Q(-0.00243243),
  DSP16_Q(0.00496199),
  DSP16_Q(-0.00881195),
  DSP16_Q(0.01456496),
  DSP16_Q(-0.02351142),
  DSP16_Q(0.03929487),
  DSP16_Q(-0.07750063),
  DSP16_Q(0.41337144),
  DSP16_Q(0.12993857),
  DSP16_Q(-0.05262097),
  DSP16_Q(0.02985082),
  DSP16_Q(-0.01835436),
  DSP16_Q(0.01129894),
  DSP16_Q(-0.00662969),
  DSP16_Q(0.00351325),
  DSP16_Q(-0.00153132),
  DSP16_Q(0.00042794),
  DSP16_Q(-0.00001460),
  DSP16_Q(-0.00015221),
  DSP16_Q(0.00089701),
  DSP16_Q(-0.00241857),
  DSP16_Q(0.00493160),
  DSP16_Q(-0.00875602),
  DSP16_Q(0.01447097),
  DSP16_Q(-0.02335975),
  DSP16_Q(0.03904829),
  DSP16_Q(-0.07706784),
  DSP16_Q(0.41415433),
  DSP16_Q(0.12856343),
  DSP16_Q(-0.05214640),
  DSP16_Q(0.02959165),
  DSP16_Q(-0.01819629),
  DSP16_Q(0.01120100),
  DSP16_Q(-0.00657115),
  DSP16_Q(0.00348114),
  DSP16_Q(-0.00151639),
  DSP16_Q(0.00042307),
  DSP16_Q(-0.00001421),
  DSP16_Q(-0.00015198),
  DSP16_Q(0.00089263),
  DSP16_Q(-0.00240452),
  DSP16_Q(0.00490087),
  DSP16_Q(-0.00869949),
  DSP16_Q(0.01437602),
  DSP16_Q(-0.02320655),
  DSP16_Q(0.03879905),
  DSP16_Q(-0.07662935),
  DSP16_Q(0.41493071),
  DSP16_Q(0.12719011),
  DSP16_Q(-0.05167055),
  DSP16_Q(0.02933154),
  DSP16_Q(-0.01803761),
  DSP16_Q(0.01110270),
  DSP16_Q(-0.00651241),
  DSP16_Q(0.00344894),
  DSP16_Q(-0.00150143),
  DSP16_Q(0.00041821),
  DSP16_Q(-0.00001384),
  DSP16_Q(-0.00015173),
  DSP16_Q(0.00088817),
  DSP16_Q(-0.00239028),
  DSP16_Q(0.00486978),
  DSP16_Q(-0.00864237),
  DSP16_Q(0.01428011),
  DSP16_Q(-0.02305180),
  DSP16_Q(0.03854718),
  DSP16_Q(-0.07618518),
  DSP16_Q(0.41570055),
  DSP16_Q(0.12581862),
  DSP16_Q(-0.05119346),
  DSP16_Q(0.02907051),
  DSP16_Q(-0.01787834),
  DSP16_Q(0.01100404),
  DSP16_Q(-0.00645348),
  DSP16_Q(0.00341667),
  DSP16_Q(-0.00148645),
  DSP16_Q(0.00041335),
  DSP16_Q(-0.00001346),
  DSP16_Q(-0.00015147),
  DSP16_Q(0.00088364),
  DSP16_Q(-0.00237586),
  DSP16_Q(0.00483834),
  DSP16_Q(-0.00858465),
  DSP16_Q(0.01418325),
  DSP16_Q(-0.02289552),
  DSP16_Q(0.03829266),
  DSP16_Q(-0.07573531),
  DSP16_Q(0.41646383),
  DSP16_Q(0.12444900),
  DSP16_Q(-0.05071514),
  DSP16_Q(0.02880858),
  DSP16_Q(-0.01771849),
  DSP16_Q(0.01090503),
  DSP16_Q(-0.00639437),
  DSP16_Q(0.00338431),
  DSP16_Q(-0.00147145),
  DSP16_Q(0.00040851),
  DSP16_Q(-0.00001310),
  DSP16_Q(-0.00015118),
  DSP16_Q(0.00087902),
  DSP16_Q(-0.00236126),
  DSP16_Q(0.00480656),
  DSP16_Q(-0.00852635),
  DSP16_Q(0.01408544),
  DSP16_Q(-0.02273771),
  DSP16_Q(0.03803552),
  DSP16_Q(-0.07527974),
  DSP16_Q(0.41722052),
  DSP16_Q(0.12308130),
  DSP16_Q(-0.05023564),
  DSP16_Q(0.02854577),
  DSP16_Q(-0.01755806),
  DSP16_Q(0.01080567),
  DSP16_Q(-0.00633507),
  DSP16_Q(0.00335187),
  DSP16_Q(-0.00145644),
  DSP16_Q(0.00040367),
  DSP16_Q(-0.00001273),
  DSP16_Q(-0.00015088),
  DSP16_Q(0.00087432),
  DSP16_Q(-0.00234646),
  DSP16_Q(0.00477443),
  DSP16_Q(-0.00846746),
  DSP16_Q(0.01398667),
  DSP16_Q(-0.02257838),
  DSP16_Q(0.03777576),
  DSP16_Q(-0.07481847),
  DSP16_Q(0.41797061),
  DSP16_Q(0.12171554),
  DSP16_Q(-0.04975497),
  DSP16_Q(0.02828208),
  DSP16_Q(-0.01739707),
  DSP16_Q(0.01070598),
  DSP16_Q(-0.00627559),
  DSP16_Q(0.00331936),
  DSP16_Q(-0.00144141),
  DSP16_Q(0.00039884),
  DSP16_Q(-0.00001238),
  DSP16_Q(-0.00015055),
  DSP16_Q(0.00086954),
  DSP16_Q(-0.00233149),
  DSP16_Q(0.00474195),
  DSP16_Q(-0.00840798),
  DSP16_Q(0.01388697),
  DSP16_Q(-0.02241752),
  DSP16_Q(0.03751337),
  DSP16_Q(-0.07435151),
  DSP16_Q(0.41871407),
  DSP16_Q(0.12035177),
  DSP16_Q(-0.04927316),
  DSP16_Q(0.02801755),
  DSP16_Q(-0.01723552),
  DSP16_Q(0.01060595),
  DSP16_Q(-0.00621593),
  DSP16_Q(0.00328677),
  DSP16_Q(-0.00142636),
  DSP16_Q(0.00039402),
  DSP16_Q(-0.00001203),
  DSP16_Q(-0.00015021),
  DSP16_Q(0.00086468),
  DSP16_Q(-0.00231632),
  DSP16_Q(0.00470912),
  DSP16_Q(-0.00834791),
  DSP16_Q(0.01378632),
  DSP16_Q(-0.02225514),
  DSP16_Q(0.03724838),
  DSP16_Q(-0.07387884),
  DSP16_Q(0.41945087),
  DSP16_Q(0.11899001),
  DSP16_Q(-0.04879023),
  DSP16_Q(0.02775217),
  DSP16_Q(-0.01707343),
  DSP16_Q(0.01050560),
  DSP16_Q(-0.00615610),
  DSP16_Q(0.00325410),
  DSP16_Q(-0.00141130),
  DSP16_Q(0.00038921),
  DSP16_Q(-0.00001169),
  DSP16_Q(-0.00014984),
  DSP16_Q(0.00085974),
  DSP16_Q(-0.00230098),
  DSP16_Q(0.00467595),
  DSP16_Q(-0.00828727),
  DSP16_Q(0.01368473),
  DSP16_Q(-0.02209126),
  DSP16_Q(0.03698078),
  DSP16_Q(-0.07340047),
  DSP16_Q(0.42018100),
  DSP16_Q(0.11763031),
  DSP16_Q(-0.04830622),
  DSP16_Q(0.02748596),
  DSP16_Q(-0.01691081),
  DSP16_Q(0.01040493),
  DSP16_Q(-0.00609610),
  DSP16_Q(0.00322137),
  DSP16_Q(-0.00139622),
  DSP16_Q(0.00038440),
  DSP16_Q(-0.00001135),
  DSP16_Q(-0.00014946),
  DSP16_Q(0.00085472),
  DSP16_Q(-0.00228545),
  DSP16_Q(0.00464244),
  DSP16_Q(-0.00822604),
  DSP16_Q(0.01358220),
  DSP16_Q(-0.02192587),
  DSP16_Q(0.03671058),
  DSP16_Q(-0.07291640),
  DSP16_Q(0.42090443),
  DSP16_Q(0.11627269),
  DSP16_Q(-0.04782116),
  DSP16_Q(0.02721895),
  DSP16_Q(-0.01674765),
  DSP16_Q(0.01030394),
  DSP16_Q(-0.00603593),
  DSP16_Q(0.00318857),
  DSP16_Q(-0.00138113),
  DSP16_Q(0.00037961),
  DSP16_Q(-0.00001102),
  DSP16_Q(-0.00014905),
  DSP16_Q(0.00084962),
  DSP16_Q(-0.00226973),
  DSP16_Q(0.00460858),
  DSP16_Q(-0.00816423),
  DSP16_Q(0.01347874),
  DSP16_Q(-0.02175897),
  DSP16_Q(0.03643779),
  DSP16_Q(-0.07242662),
  DSP16_Q(0.42162113),
  DSP16_Q(0.11491720),
  DSP16_Q(-0.04733506),
  DSP16_Q(0.02695114),
  DSP16_Q(-0.01658399),
  DSP16_Q(0.01020265),
  DSP16_Q(-0.00597560),
  DSP16_Q(0.00315570),
  DSP16_Q(-0.00136603),
  DSP16_Q(0.00037483),
  DSP16_Q(-0.00001070),
  DSP16_Q(-0.00014862),
  DSP16_Q(0.00084444),
  DSP16_Q(-0.00225383),
  DSP16_Q(0.00457437),
  DSP16_Q(-0.00810184),
  DSP16_Q(0.01337436),
  DSP16_Q(-0.02159059),
  DSP16_Q(0.03616242),
  DSP16_Q(-0.07193114),
  DSP16_Q(0.42233109),
  DSP16_Q(0.11356386),
  DSP16_Q(-0.04684796),
  DSP16_Q(0.02668256),
  DSP16_Q(-0.01641981),
  DSP16_Q(0.01010105),
  DSP16_Q(-0.00591511),
  DSP16_Q(0.00312276),
  DSP16_Q(-0.00135092),
  DSP16_Q(0.00037005),
  DSP16_Q(-0.00001038),
  DSP16_Q(-0.00014818),
  DSP16_Q(0.00083918),
  DSP16_Q(-0.00223775),
  DSP16_Q(0.00453983),
  DSP16_Q(-0.00803888),
  DSP16_Q(0.01326905),
  DSP16_Q(-0.02142071),
  DSP16_Q(0.03588447),
  DSP16_Q(-0.07142995),
  DSP16_Q(0.42303429),
  DSP16_Q(0.11221272),
  DSP16_Q(-0.04635987),
  DSP16_Q(0.02641321),
  DSP16_Q(-0.01625514),
  DSP16_Q(0.00999915),
  DSP16_Q(-0.00585447),
  DSP16_Q(0.00308977),
  DSP16_Q(-0.00133580),
  DSP16_Q(0.00036529),
  DSP16_Q(-0.00001006),
  DSP16_Q(-0.00014771),
  DSP16_Q(0.00083384),
  DSP16_Q(-0.00222148),
  DSP16_Q(0.00450494),
  DSP16_Q(-0.00797535),
  DSP16_Q(0.01316281),
  DSP16_Q(-0.02124935),
  DSP16_Q(0.03560395),
  DSP16_Q(-0.07092306),
  DSP16_Q(0.42373069),
  DSP16_Q(0.11086380),
  DSP16_Q(-0.04587084),
  DSP16_Q(0.02614312),
  DSP16_Q(-0.01608998),
  DSP16_Q(0.00989697),
  DSP16_Q(-0.00579367),
  DSP16_Q(0.00305671),
  DSP16_Q(-0.00132067),
  DSP16_Q(0.00036054),
  DSP16_Q(-0.00000976),
  DSP16_Q(-0.00014722),
  DSP16_Q(0.00082842),
  DSP16_Q(-0.00220503),
  DSP16_Q(0.00446972),
  DSP16_Q(-0.00791124),
  DSP16_Q(0.01305566),
  DSP16_Q(-0.02107650),
  DSP16_Q(0.03532087),
  DSP16_Q(-0.07041046),
  DSP16_Q(0.42442029),
  DSP16_Q(0.10951715),
  DSP16_Q(-0.04538088),
  DSP16_Q(0.02587229),
  DSP16_Q(-0.01592434),
  DSP16_Q(0.00979450),
  DSP16_Q(-0.00573273),
  DSP16_Q(0.00302359),
  DSP16_Q(-0.00130553),
  DSP16_Q(0.00035580),
  DSP16_Q(-0.00000946),
  DSP16_Q(-0.00014671),
  DSP16_Q(0.00082291),
  DSP16_Q(-0.00218840),
  DSP16_Q(0.00443415),
  DSP16_Q(-0.00784656),
  DSP16_Q(0.01294759),
  DSP16_Q(-0.02090219),
  DSP16_Q(0.03503523),
  DSP16_Q(-0.06989215),
  DSP16_Q(0.42510305),
  DSP16_Q(0.10817280),
  DSP16_Q(-0.04489003),
  DSP16_Q(0.02560075),
  DSP16_Q(-0.01575823),
  DSP16_Q(0.00969175),
  DSP16_Q(-0.00567164),
  DSP16_Q(0.00299042),
  DSP16_Q(-0.00129038),
  DSP16_Q(0.00035107),
  DSP16_Q(-0.00000916),
  DSP16_Q(-0.00014618),
  DSP16_Q(0.00081733),
  DSP16_Q(-0.00217158),
  DSP16_Q(0.00439825),
  DSP16_Q(-0.00778132),
  DSP16_Q(0.01283861),
  DSP16_Q(-0.02072641),
  DSP16_Q(0.03474705),
  DSP16_Q(-0.06936813),
  DSP16_Q(0.42577896),
  DSP16_Q(0.10683077),
  DSP16_Q(-0.04439830),
  DSP16_Q(0.02532851),
  DSP16_Q(-0.01559167),
  DSP16_Q(0.00958873),
  DSP16_Q(-0.00561041),
  DSP16_Q(0.00295719),
  DSP16_Q(-0.00127523),
  DSP16_Q(0.00034635),
  DSP16_Q(-0.00000887),
  DSP16_Q(-0.00014563),
  DSP16_Q(0.00081166),
  DSP16_Q(-0.00215459),
  DSP16_Q(0.00436201),
  DSP16_Q(-0.00771551),
  DSP16_Q(0.01272873),
  DSP16_Q(-0.02054916),
  DSP16_Q(0.03445633),
  DSP16_Q(-0.06883842),
  DSP16_Q(0.42644800),
  DSP16_Q(0.10549111),
  DSP16_Q(-0.04390573),
  DSP16_Q(0.02505558),
  DSP16_Q(-0.01542465),
  DSP16_Q(0.00948545),
  DSP16_Q(-0.00554905),
  DSP16_Q(0.00292391),
  DSP16_Q(-0.00126007),
  DSP16_Q(0.00034165),
  DSP16_Q(-0.00000859),
  DSP16_Q(-0.00014506),
  DSP16_Q(0.00080592),
  DSP16_Q(-0.00213741),
  DSP16_Q(0.00432544),
  DSP16_Q(-0.00764913),
  DSP16_Q(0.01261794),
  DSP16_Q(-0.02037046),
  DSP16_Q(0.03416308),
  DSP16_Q(-0.06830299),
  DSP16_Q(0.42711014),
  DSP16_Q(0.10415385),
  DSP16_Q(-0.04341233),
  DSP16_Q(0.02478198),
  DSP16_Q(-0.01525720),
  DSP16_Q(0.00938190),
  DSP16_Q(-0.00548755),
  DSP16_Q(0.00289058),
  DSP16_Q(-0.00124491),
  DSP16_Q(0.00033695),
  DSP16_Q(-0.00000831),
  DSP16_Q(-0.00014446),
  DSP16_Q(0.00080009),
  DSP16_Q(-0.00212005),
  DSP16_Q(0.00428853),
  DSP16_Q(-0.00758220),
  DSP16_Q(0.01250625),
  DSP16_Q(-0.02019032),
  DSP16_Q(0.03386731),
  DSP16_Q(-0.06776186),
  DSP16_Q(0.42776536),
  DSP16_Q(0.10281903),
  DSP16_Q(-0.04291815),
  DSP16_Q(0.02450773),
  DSP16_Q(-0.01508932),
  DSP16_Q(0.00927810),
  DSP16_Q(-0.00542593),
  DSP16_Q(0.00285721),
  DSP16_Q(-0.00122975),
  DSP16_Q(0.00033227),
  DSP16_Q(-0.00000804),
  DSP16_Q(-0.00014385),
  DSP16_Q(0.00079418),
  DSP16_Q(-0.00210250),
  DSP16_Q(0.00425128),
  DSP16_Q(-0.00751471),
  DSP16_Q(0.01239366),
  DSP16_Q(-0.02000872),
  DSP16_Q(0.03356902),
  DSP16_Q(-0.06721503),
  DSP16_Q(0.42841365),
  DSP16_Q(0.10148667),
  DSP16_Q(-0.04242320),
  DSP16_Q(0.02423284),
  DSP16_Q(-0.01492101),
  DSP16_Q(0.00917406),
  DSP16_Q(-0.00536417),
  DSP16_Q(0.00282378),
  DSP16_Q(-0.00121458),
  DSP16_Q(0.00032761),
  DSP16_Q(-0.00000777),
  DSP16_Q(-0.00014321),
  DSP16_Q(0.00078819),
  DSP16_Q(-0.00208478),
  DSP16_Q(0.00421370),
  DSP16_Q(-0.00744666),
  DSP16_Q(0.01228019),
  DSP16_Q(-0.01982569),
  DSP16_Q(0.03326823),
  DSP16_Q(-0.06666250),
  DSP16_Q(0.42905498),
  DSP16_Q(0.10015681),
  DSP16_Q(-0.04192752),
  DSP16_Q(0.02395732),
  DSP16_Q(-0.01475230),
  DSP16_Q(0.00906977),
  DSP16_Q(-0.00530230),
  DSP16_Q(0.00279031),
  DSP16_Q(-0.00119941),
  DSP16_Q(0.00032295),
  DSP16_Q(-0.00000751),
  DSP16_Q(-0.00014255),
  DSP16_Q(0.00078212),
  DSP16_Q(-0.00206688),
  DSP16_Q(0.00417579),
  DSP16_Q(-0.00737805),
  DSP16_Q(0.01216582),
  DSP16_Q(-0.01964123),
  DSP16_Q(0.03296494),
  DSP16_Q(-0.06610427),
  DSP16_Q(0.42968934),
  DSP16_Q(0.09882949),
  DSP16_Q(-0.04143112),
  DSP16_Q(0.02368120),
  DSP16_Q(-0.01458318),
  DSP16_Q(0.00896524),
  DSP16_Q(-0.00524031),
  DSP16_Q(0.00275680),
  DSP16_Q(-0.00118424),
  DSP16_Q(0.00031831),
  DSP16_Q(-0.00000725),
  DSP16_Q(-0.00014186),
  DSP16_Q(0.00077597),
  DSP16_Q(-0.00204879),
  DSP16_Q(0.00413756),
  DSP16_Q(-0.00730890),
  DSP16_Q(0.01205057),
  DSP16_Q(-0.01945534),
  DSP16_Q(0.03265916),
  DSP16_Q(-0.06554034),
  DSP16_Q(0.43031670),
  DSP16_Q(0.09750473),
  DSP16_Q(-0.04093403),
  DSP16_Q(0.02340448),
  DSP16_Q(-0.01441368),
  DSP16_Q(0.00886049),
  DSP16_Q(-0.00517820),
  DSP16_Q(0.00272325),
  DSP16_Q(-0.00116907),
  DSP16_Q(0.00031368),
  DSP16_Q(-0.00000700),
  DSP16_Q(-0.00014116),
  DSP16_Q(0.00076974),
  DSP16_Q(-0.00203053),
  DSP16_Q(0.00409899),
  DSP16_Q(-0.00723919),
  DSP16_Q(0.01193444),
  DSP16_Q(-0.01926804),
  DSP16_Q(0.03235090),
  DSP16_Q(-0.06497072),
  DSP16_Q(0.43093704),
  DSP16_Q(0.09618258),
  DSP16_Q(-0.04043628),
  DSP16_Q(0.02312719),
  DSP16_Q(-0.01424379),
  DSP16_Q(0.00875551),
  DSP16_Q(-0.00511598),
  DSP16_Q(0.00268966),
  DSP16_Q(-0.00115390),
  DSP16_Q(0.00030907),
  DSP16_Q(-0.00000676),
  DSP16_Q(-0.00014043),
  DSP16_Q(0.00076342),
  DSP16_Q(-0.00201209),
  DSP16_Q(0.00406009),
  DSP16_Q(-0.00716894),
  DSP16_Q(0.01181743),
  DSP16_Q(-0.01907932),
  DSP16_Q(0.03204018),
  DSP16_Q(-0.06439540),
  DSP16_Q(0.43155035),
  DSP16_Q(0.09486306),
  DSP16_Q(-0.03993790),
  DSP16_Q(0.02284934),
  DSP16_Q(-0.01407353),
  DSP16_Q(0.00865031),
  DSP16_Q(-0.00505365),
  DSP16_Q(0.00265603),
  DSP16_Q(-0.00113874),
  DSP16_Q(0.00030447),
  DSP16_Q(-0.00000652),
  DSP16_Q(-0.00013968),
  DSP16_Q(0.00075703),
  DSP16_Q(-0.00199347),
  DSP16_Q(0.00402087),
  DSP16_Q(-0.00709814),
  DSP16_Q(0.01169956),
  DSP16_Q(-0.01888920),
  DSP16_Q(0.03172699),
  DSP16_Q(-0.06381439),
  DSP16_Q(0.43215660),
  DSP16_Q(0.09354621),
  DSP16_Q(-0.03943892),
  DSP16_Q(0.02257094),
  DSP16_Q(-0.01390290),
  DSP16_Q(0.00854491),
  DSP16_Q(-0.00499122),
  DSP16_Q(0.00262237),
  DSP16_Q(-0.00112357),
  DSP16_Q(0.00029989),
  DSP16_Q(-0.00000629),
  DSP16_Q(-0.00013891),
  DSP16_Q(0.00075055),
  DSP16_Q(-0.00197467),
  DSP16_Q(0.00398132),
  DSP16_Q(-0.00702680),
  DSP16_Q(0.01158081),
  DSP16_Q(-0.01869767),
  DSP16_Q(0.03141135),
  DSP16_Q(-0.06322770),
  DSP16_Q(0.43275578),
  DSP16_Q(0.09223206),
  DSP16_Q(-0.03893935),
  DSP16_Q(0.02229201),
  DSP16_Q(-0.01373193),
  DSP16_Q(0.00843929),
  DSP16_Q(-0.00492869),
  DSP16_Q(0.00258867),
  DSP16_Q(-0.00110842),
  DSP16_Q(0.00029532),
  DSP16_Q(-0.00000606),
  DSP16_Q(-0.00013812),
  DSP16_Q(0.00074399),
  DSP16_Q(-0.00195569),
  DSP16_Q(0.00394144),
  DSP16_Q(-0.00695493),
  DSP16_Q(0.01146121),
  DSP16_Q(-0.01850476),
  DSP16_Q(0.03109327),
  DSP16_Q(-0.06263531),
  DSP16_Q(0.43334787),
  DSP16_Q(0.09092065),
  DSP16_Q(-0.03843922),
  DSP16_Q(0.02201256),
  DSP16_Q(-0.01356061),
  DSP16_Q(0.00833348),
  DSP16_Q(-0.00486606),
  DSP16_Q(0.00255495),
  DSP16_Q(-0.00109326),
  DSP16_Q(0.00029076),
  DSP16_Q(-0.00000584),
  DSP16_Q(-0.00013730),
  DSP16_Q(0.00073736),
  DSP16_Q(-0.00193654),
  DSP16_Q(0.00390125),
  DSP16_Q(-0.00688251),
  DSP16_Q(0.01134074),
  DSP16_Q(-0.01831046),
  DSP16_Q(0.03077275),
  DSP16_Q(-0.06203725),
  DSP16_Q(0.43393285),
  DSP16_Q(0.08961200),
  DSP16_Q(-0.03793857),
  DSP16_Q(0.02173262),
  DSP16_Q(-0.01338896),
  DSP16_Q(0.00822747),
  DSP16_Q(-0.00480334),
  DSP16_Q(0.00252120),
  DSP16_Q(-0.00107811),
  DSP16_Q(0.00028622),
  DSP16_Q(-0.00000562),
  DSP16_Q(-0.00013646),
  DSP16_Q(0.00073064),
  DSP16_Q(-0.00191720),
  DSP16_Q(0.00386073),
  DSP16_Q(-0.00680956),
  DSP16_Q(0.01121942),
  DSP16_Q(-0.01811479),
  DSP16_Q(0.03044981),
  DSP16_Q(-0.06143350),
  DSP16_Q(0.43451070),
  DSP16_Q(0.08830614),
  DSP16_Q(-0.03743742),
  DSP16_Q(0.02145219),
  DSP16_Q(-0.01321698),
  DSP16_Q(0.00812128),
  DSP16_Q(-0.00474053),
  DSP16_Q(0.00248742),
  DSP16_Q(-0.00106297),
  DSP16_Q(0.00028170),
  DSP16_Q(-0.00000541),
  DSP16_Q(-0.00013559),
  DSP16_Q(0.00072384),
  DSP16_Q(-0.00189770),
  DSP16_Q(0.00381989),
  DSP16_Q(-0.00673608),
  DSP16_Q(0.01109725),
  DSP16_Q(-0.01791774),
  DSP16_Q(0.03012446),
  DSP16_Q(-0.06082408),
  DSP16_Q(0.43508140),
  DSP16_Q(0.08700312),
  DSP16_Q(-0.03693578),
  DSP16_Q(0.02117129),
  DSP16_Q(-0.01304469),
  DSP16_Q(0.00801490),
  DSP16_Q(-0.00467763),
  DSP16_Q(0.00245362),
  DSP16_Q(-0.00104784),
  DSP16_Q(0.00027719),
  DSP16_Q(-0.00000520),
  DSP16_Q(-0.00013471),
  DSP16_Q(0.00071695),
  DSP16_Q(-0.00187801),
  DSP16_Q(0.00377873),
  DSP16_Q(-0.00666207),
  DSP16_Q(0.01097424),
  DSP16_Q(-0.01771933),
  DSP16_Q(0.02979671),
  DSP16_Q(-0.06020898),
  DSP16_Q(0.43564493),
  DSP16_Q(0.08570297),
  DSP16_Q(-0.03643370),
  DSP16_Q(0.02088994),
  DSP16_Q(-0.01287210),
  DSP16_Q(0.00790835),
  DSP16_Q(-0.00461465),
  DSP16_Q(0.00241979),
  DSP16_Q(-0.00103272),
  DSP16_Q(0.00027270),
  DSP16_Q(-0.00000500),
  DSP16_Q(-0.00013379),
  DSP16_Q(0.00070999),
  DSP16_Q(-0.00185815),
  DSP16_Q(0.00373725),
  DSP16_Q(-0.00658753),
  DSP16_Q(0.01085038),
  DSP16_Q(-0.01751956),
  DSP16_Q(0.02946656),
  DSP16_Q(-0.05958822),
  DSP16_Q(0.43620129),
  DSP16_Q(0.08440571),
  DSP16_Q(-0.03593119),
  DSP16_Q(0.02060814),
  DSP16_Q(-0.01269921),
  DSP16_Q(0.00780163),
  DSP16_Q(-0.00455160),
  DSP16_Q(0.00238594),
  DSP16_Q(-0.00101760),
  DSP16_Q(0.00026822),
  DSP16_Q(-0.00000480),
  DSP16_Q(-0.00013286),
  DSP16_Q(0.00070295),
  DSP16_Q(-0.00183812),
  DSP16_Q(0.00369546),
  DSP16_Q(-0.00651247),
  DSP16_Q(0.01072569),
  DSP16_Q(-0.01731844),
  DSP16_Q(0.02913403),
  DSP16_Q(-0.05896179),
  DSP16_Q(0.43675044),
  DSP16_Q(0.08311137),
  DSP16_Q(-0.03542828),
  DSP16_Q(0.02032593),
  DSP16_Q(-0.01252603),
  DSP16_Q(0.00769474),
  DSP16_Q(-0.00448846),
  DSP16_Q(0.00235208),
  DSP16_Q(-0.00100250),
  DSP16_Q(0.00026376),
  DSP16_Q(-0.00000461),
  DSP16_Q(-0.00013190),
  DSP16_Q(0.00069582),
  DSP16_Q(-0.00181791),
  DSP16_Q(0.00365335),
  DSP16_Q(-0.00643688),
  DSP16_Q(0.01060017),
  DSP16_Q(-0.01711598),
  DSP16_Q(0.02879913),
  DSP16_Q(-0.05832971),
  DSP16_Q(0.43729238),
  DSP16_Q(0.08182000),
  DSP16_Q(-0.03492500),
  DSP16_Q(0.02004331),
  DSP16_Q(-0.01235258),
  DSP16_Q(0.00758770),
  DSP16_Q(-0.00442526),
  DSP16_Q(0.00231820),
  DSP16_Q(-0.00098740),
  DSP16_Q(0.00025932),
  DSP16_Q(-0.00000442),
  DSP16_Q(-0.00013092),
  DSP16_Q(0.00068862),
  DSP16_Q(-0.00179753),
  DSP16_Q(0.00361093),
  DSP16_Q(-0.00636078),
  DSP16_Q(0.01047382),
  DSP16_Q(-0.01691219),
  DSP16_Q(0.02846187),
  DSP16_Q(-0.05769196),
  DSP16_Q(0.43782708),
  DSP16_Q(0.08053162),
  DSP16_Q(-0.03442137),
  DSP16_Q(0.01976029),
  DSP16_Q(-0.01217886),
  DSP16_Q(0.00748051),
  DSP16_Q(-0.00436199),
  DSP16_Q(0.00228430),
  DSP16_Q(-0.00097232),
  DSP16_Q(0.00025490),
  DSP16_Q(-0.00000424),
  DSP16_Q(-0.00012992),
  DSP16_Q(0.00068133),
  DSP16_Q(-0.00177697),
  DSP16_Q(0.00356820),
  DSP16_Q(-0.00628417),
  DSP16_Q(0.01034665),
  DSP16_Q(-0.01670707),
  DSP16_Q(0.02812226),
  DSP16_Q(-0.05704857),
  DSP16_Q(0.43835453),
  DSP16_Q(0.07924625),
  DSP16_Q(-0.03391742),
  DSP16_Q(0.01947690),
  DSP16_Q(-0.01200489),
  DSP16_Q(0.00737317),
  DSP16_Q(-0.00429865),
  DSP16_Q(0.00225039),
  DSP16_Q(-0.00095725),
  DSP16_Q(0.00025049),
  DSP16_Q(-0.00000406),
  DSP16_Q(-0.00012889),
  DSP16_Q(0.00067397),
  DSP16_Q(-0.00175624),
  DSP16_Q(0.00352516),
  DSP16_Q(-0.00620704),
  DSP16_Q(0.01021866),
  DSP16_Q(-0.01650063),
  DSP16_Q(0.02778030),
  DSP16_Q(-0.05639953),
  DSP16_Q(0.43887471),
  DSP16_Q(0.07796395),
  DSP16_Q(-0.03341318),
  DSP16_Q(0.01919315),
  DSP16_Q(-0.01183067),
  DSP16_Q(0.00726569),
  DSP16_Q(-0.00423526),
  DSP16_Q(0.00221647),
  DSP16_Q(-0.00094220),
  DSP16_Q(0.00024610),
  DSP16_Q(-0.00000389),
  DSP16_Q(-0.00012783),
  DSP16_Q(0.00066652),
  DSP16_Q(-0.00173534),
  DSP16_Q(0.00348181),
  DSP16_Q(-0.00612940),
  DSP16_Q(0.01008986),
  DSP16_Q(-0.01629287),
  DSP16_Q(0.02743602),
  DSP16_Q(-0.05574485),
  DSP16_Q(0.43938761),
  DSP16_Q(0.07668473),
  DSP16_Q(-0.03290866),
  DSP16_Q(0.01890905),
  DSP16_Q(-0.01165621),
  DSP16_Q(0.00715808),
  DSP16_Q(-0.00417180),
  DSP16_Q(0.00218255),
  DSP16_Q(-0.00092716),
  DSP16_Q(0.00024172),
  DSP16_Q(-0.00000372),
  DSP16_Q(-0.00012676),
  DSP16_Q(0.00065899),
  DSP16_Q(-0.00171427),
  DSP16_Q(0.00343814),
  DSP16_Q(-0.00605125),
  DSP16_Q(0.00996026),
  DSP16_Q(-0.01608382),
  DSP16_Q(0.02708942),
  DSP16_Q(-0.05508453),
  DSP16_Q(0.43989321),
  DSP16_Q(0.07540862),
  DSP16_Q(-0.03240389),
  DSP16_Q(0.01862462),
  DSP16_Q(-0.01148152),
  DSP16_Q(0.00705034),
  DSP16_Q(-0.00410830),
  DSP16_Q(0.00214861),
  DSP16_Q(-0.00091213),
  DSP16_Q(0.00023737),
  DSP16_Q(-0.00000356),
  DSP16_Q(-0.00012566),
  DSP16_Q(0.00065139),
  DSP16_Q(-0.00169302),
  DSP16_Q(0.00339418),
  DSP16_Q(-0.00597260),
  DSP16_Q(0.00982985),
  DSP16_Q(-0.01587347),
  DSP16_Q(0.02674052),
  DSP16_Q(-0.05441858),
  DSP16_Q(0.44039149),
  DSP16_Q(0.07413566),
  DSP16_Q(-0.03189891),
  DSP16_Q(0.01833988),
  DSP16_Q(-0.01130662),
  DSP16_Q(0.00694248),
  DSP16_Q(-0.00404474),
  DSP16_Q(0.00211467),
  DSP16_Q(-0.00089713),
  DSP16_Q(0.00023303),
  DSP16_Q(-0.00000340),
  DSP16_Q(-0.00012453),
  DSP16_Q(0.00064370),
  DSP16_Q(-0.00167161),
  DSP16_Q(0.00334991),
  DSP16_Q(-0.00589345),
  DSP16_Q(0.00969865),
  DSP16_Q(-0.01566183),
  DSP16_Q(0.02638931),
  DSP16_Q(-0.05374701),
  DSP16_Q(0.44088244),
  DSP16_Q(0.07286588),
  DSP16_Q(-0.03139373),
  DSP16_Q(0.01805483),
  DSP16_Q(-0.01113150),
  DSP16_Q(0.00683450),
  DSP16_Q(-0.00398113),
  DSP16_Q(0.00208073),
  DSP16_Q(-0.00088213),
  DSP16_Q(0.00022871),
  DSP16_Q(-0.00000325),
  DSP16_Q(-0.00012338),
  DSP16_Q(0.00063593),
  DSP16_Q(-0.00165003),
  DSP16_Q(0.00330533),
  DSP16_Q(-0.00581380),
  DSP16_Q(0.00956665),
  DSP16_Q(-0.01544891),
  DSP16_Q(0.02603583),
  DSP16_Q(-0.05306983),
  DSP16_Q(0.44136605),
  DSP16_Q(0.07159931),
  DSP16_Q(-0.03088839),
  DSP16_Q(0.01776950),
  DSP16_Q(-0.01095619),
  DSP16_Q(0.00672642),
  DSP16_Q(-0.00391749),
  DSP16_Q(0.00204678),
  DSP16_Q(-0.00086716),
  DSP16_Q(0.00022441),
  DSP16_Q(-0.00000310),
  DSP16_Q(-0.00012221),
  DSP16_Q(0.00062808),
  DSP16_Q(-0.00162828),
  DSP16_Q(0.00326046),
  DSP16_Q(-0.00573366),
  DSP16_Q(0.00943387),
  DSP16_Q(-0.01523471),
  DSP16_Q(0.02568007),
  DSP16_Q(-0.05238702),
  DSP16_Q(0.44184229),
  DSP16_Q(0.07033598),
  DSP16_Q(-0.03038290),
  DSP16_Q(0.01748390),
  DSP16_Q(-0.01078069),
  DSP16_Q(0.00661823),
  DSP16_Q(-0.00385380),
  DSP16_Q(0.00201284),
  DSP16_Q(-0.00085221),
  DSP16_Q(0.00022013),
  DSP16_Q(-0.00000295),
  DSP16_Q(-0.00012101),
  DSP16_Q(0.00062016),
  DSP16_Q(-0.00160635),
  DSP16_Q(0.00321528),
  DSP16_Q(-0.00565303),
  DSP16_Q(0.00930031),
  DSP16_Q(-0.01501926),
  DSP16_Q(0.02532206),
  DSP16_Q(-0.05169862),
  DSP16_Q(0.44231116),
  DSP16_Q(0.06907592),
  DSP16_Q(-0.02987729),
  DSP16_Q(0.01719804),
  DSP16_Q(-0.01060500),
  DSP16_Q(0.00650994),
  DSP16_Q(-0.00379008),
  DSP16_Q(0.00197890),
  DSP16_Q(-0.00083728),
  DSP16_Q(0.00021587),
  DSP16_Q(-0.00000281),
  DSP16_Q(-0.00011978),
  DSP16_Q(0.00061215),
  DSP16_Q(-0.00158427),
  DSP16_Q(0.00316981),
  DSP16_Q(-0.00557190),
  DSP16_Q(0.00916597),
  DSP16_Q(-0.01480255),
  DSP16_Q(0.02496179),
  DSP16_Q(-0.05100461),
  DSP16_Q(0.44277263),
  DSP16_Q(0.06781916),
  DSP16_Q(-0.02937159),
  DSP16_Q(0.01691194),
  DSP16_Q(-0.01042915),
  DSP16_Q(0.00640156),
  DSP16_Q(-0.00372633),
  DSP16_Q(0.00194496),
  DSP16_Q(-0.00082236),
  DSP16_Q(0.00021163),
  DSP16_Q(-0.00000268),
  DSP16_Q(-0.00011853),
  DSP16_Q(0.00060406),
  DSP16_Q(-0.00156201),
  DSP16_Q(0.00312404),
  DSP16_Q(-0.00549030),
  DSP16_Q(0.00903086),
  DSP16_Q(-0.01458459),
  DSP16_Q(0.02459929),
  DSP16_Q(-0.05030502),
  DSP16_Q(0.44322670),
  DSP16_Q(0.06656573),
  DSP16_Q(-0.02886582),
  DSP16_Q(0.01662562),
  DSP16_Q(-0.01025314),
  DSP16_Q(0.00629309),
  DSP16_Q(-0.00366255),
  DSP16_Q(0.00191103),
  DSP16_Q(-0.00080747),
  DSP16_Q(0.00020741),
  DSP16_Q(-0.00000254),
  DSP16_Q(-0.00011726),
  DSP16_Q(0.00059590),
  DSP16_Q(-0.00153959),
  DSP16_Q(0.00307798),
  DSP16_Q(-0.00540821),
  DSP16_Q(0.00889498),
  DSP16_Q(-0.01436539),
  DSP16_Q(0.02423457),
  DSP16_Q(-0.04959983),
  DSP16_Q(0.44367334),
  DSP16_Q(0.06531565),
  DSP16_Q(-0.02836000),
  DSP16_Q(0.01633909),
  DSP16_Q(-0.01007697),
  DSP16_Q(0.00618455),
  DSP16_Q(-0.00359874),
  DSP16_Q(0.00187711),
  DSP16_Q(-0.00079260),
  DSP16_Q(0.00020320),
  DSP16_Q(-0.00000242),
  DSP16_Q(-0.00011596),
  DSP16_Q(0.00058765),
  DSP16_Q(-0.00151700),
  DSP16_Q(0.00303162),
  DSP16_Q(-0.00532564),
  DSP16_Q(0.00875835),
  DSP16_Q(-0.01414497),
  DSP16_Q(0.02386764),
  DSP16_Q(-0.04888907),
  DSP16_Q(0.44411256),
  DSP16_Q(0.06406897),
  DSP16_Q(-0.02785417),
  DSP16_Q(0.01605236),
  DSP16_Q(-0.00990066),
  DSP16_Q(0.00607593),
  DSP16_Q(-0.00353491),
  DSP16_Q(0.00184320),
  DSP16_Q(-0.00077776),
  DSP16_Q(0.00019902),
  DSP16_Q(-0.00000229),
  DSP16_Q(-0.00011464),
  DSP16_Q(0.00057932),
  DSP16_Q(-0.00149425),
  DSP16_Q(0.00298497),
  DSP16_Q(-0.00524259),
  DSP16_Q(0.00862096),
  DSP16_Q(-0.01392332),
  DSP16_Q(0.02349850),
  DSP16_Q(-0.04817274),
  DSP16_Q(0.44454432),
  DSP16_Q(0.06282571),
  DSP16_Q(-0.02734835),
  DSP16_Q(0.01576546),
  DSP16_Q(-0.00972422),
  DSP16_Q(0.00596724),
  DSP16_Q(-0.00347106),
  DSP16_Q(0.00180930),
  DSP16_Q(-0.00076293),
  DSP16_Q(0.00019486),
  DSP16_Q(-0.00000218),
  DSP16_Q(-0.00011329),
  DSP16_Q(0.00057092),
  DSP16_Q(-0.00147134),
  DSP16_Q(0.00293803),
  DSP16_Q(-0.00515908),
  DSP16_Q(0.00848282),
  DSP16_Q(-0.01370046),
  DSP16_Q(0.02312719),
  DSP16_Q(-0.04745085),
  DSP16_Q(0.44496863),
  DSP16_Q(0.06158589),
  DSP16_Q(-0.02684255),
  DSP16_Q(0.01547839),
  DSP16_Q(-0.00954765),
  DSP16_Q(0.00585849),
  DSP16_Q(-0.00340720),
  DSP16_Q(0.00177542),
  DSP16_Q(-0.00074814),
  DSP16_Q(0.00019071),
  DSP16_Q(-0.00000206),
  DSP16_Q(-0.00011191),
  DSP16_Q(0.00056244),
  DSP16_Q(-0.00144826),
  DSP16_Q(0.00289081),
  DSP16_Q(-0.00507509),
  DSP16_Q(0.00834394),
  DSP16_Q(-0.01347640),
  DSP16_Q(0.02275369),
  DSP16_Q(-0.04672340),
  DSP16_Q(0.44538545),
  DSP16_Q(0.06034955),
  DSP16_Q(-0.02633682),
  DSP16_Q(0.01519117),
  DSP16_Q(-0.00937098),
  DSP16_Q(0.00574968),
  DSP16_Q(-0.00334333),
  DSP16_Q(0.00174155),
  DSP16_Q(-0.00073336),
  DSP16_Q(0.00018659),
  DSP16_Q(-0.00000195),
  DSP16_Q(-0.00011051),
  DSP16_Q(0.00055387),
  DSP16_Q(-0.00142501),
  DSP16_Q(0.00284330),
  DSP16_Q(-0.00499064),
  DSP16_Q(0.00820432),
  DSP16_Q(-0.01325114),
  DSP16_Q(0.02237804),
  DSP16_Q(-0.04599041),
  DSP16_Q(0.44579479),
  DSP16_Q(0.05911672),
  DSP16_Q(-0.02583116),
  DSP16_Q(0.01490382),
  DSP16_Q(-0.00919419),
  DSP16_Q(0.00564083),
  DSP16_Q(-0.00327945),
  DSP16_Q(0.00170770),
  DSP16_Q(-0.00071862),
  DSP16_Q(0.00018249),
  DSP16_Q(-0.00000184),
  DSP16_Q(-0.00010909),
  DSP16_Q(0.00054523),
  DSP16_Q(-0.00140161),
  DSP16_Q(0.00279550),
  DSP16_Q(-0.00490572),
  DSP16_Q(0.00806398),
  DSP16_Q(-0.01302469),
  DSP16_Q(0.02200024),
  DSP16_Q(-0.04525188),
  DSP16_Q(0.44619663),
  DSP16_Q(0.05788742),
  DSP16_Q(-0.02532561),
  DSP16_Q(0.01461635),
  DSP16_Q(-0.00901731),
  DSP16_Q(0.00553193),
  DSP16_Q(-0.00321556),
  DSP16_Q(0.00167387),
  DSP16_Q(-0.00070390),
  DSP16_Q(0.00017841),
  DSP16_Q(-0.00000174),
  DSP16_Q(-0.00010764),
  DSP16_Q(0.00053651),
  DSP16_Q(-0.00137805),
  DSP16_Q(0.00274742),
  DSP16_Q(-0.00482035),
  DSP16_Q(0.00792290),
  DSP16_Q(-0.01279707),
  DSP16_Q(0.02162031),
  DSP16_Q(-0.04450781),
  DSP16_Q(0.44659096),
  DSP16_Q(0.05666169),
  DSP16_Q(-0.02482019),
  DSP16_Q(0.01432877),
  DSP16_Q(-0.00884035),
  DSP16_Q(0.00542299),
  DSP16_Q(-0.00315168),
  DSP16_Q(0.00164006),
  DSP16_Q(-0.00068921),
  DSP16_Q(0.00017435),
  DSP16_Q(-0.00000164),
  DSP16_Q(-0.00010616),
  DSP16_Q(0.00052772),
  DSP16_Q(-0.00135432),
  DSP16_Q(0.00269907),
  DSP16_Q(-0.00473452),
  DSP16_Q(0.00778111),
  DSP16_Q(-0.01256828),
  DSP16_Q(0.02123826),
  DSP16_Q(-0.04375823),
  DSP16_Q(0.44697776),
  DSP16_Q(0.05543956),
  DSP16_Q(-0.02431492),
  DSP16_Q(0.01404110),
  DSP16_Q(-0.00866331),
  DSP16_Q(0.00531401),
  DSP16_Q(-0.00308779),
  DSP16_Q(0.00160627),
  DSP16_Q(-0.00067455),
  DSP16_Q(0.00017031),
  DSP16_Q(-0.00000155),
  DSP16_Q(-0.00010466),
  DSP16_Q(0.00051884),
  DSP16_Q(-0.00133044),
  DSP16_Q(0.00265043),
  DSP16_Q(-0.00464824),
  DSP16_Q(0.00763860),
  DSP16_Q(-0.01233833),
  DSP16_Q(0.02085410),
  DSP16_Q(-0.04300313),
  DSP16_Q(0.44735702),
  DSP16_Q(0.05422104),
  DSP16_Q(-0.02380984),
  DSP16_Q(0.01375336),
  DSP16_Q(-0.00848620),
  DSP16_Q(0.00520501),
  DSP16_Q(-0.00302392),
  DSP16_Q(0.00157251),
  DSP16_Q(-0.00065992),
  DSP16_Q(0.00016629),
  DSP16_Q(-0.00000145),
  DSP16_Q(-0.00010313),
  DSP16_Q(0.00050989),
  DSP16_Q(-0.00130639),
  DSP16_Q(0.00260151),
  DSP16_Q(-0.00456152),
  DSP16_Q(0.00749538),
  DSP16_Q(-0.01210723),
  DSP16_Q(0.02046784),
  DSP16_Q(-0.04224253),
  DSP16_Q(0.44772873),
  DSP16_Q(0.05300617),
  DSP16_Q(-0.02330495),
  DSP16_Q(0.01346556),
  DSP16_Q(-0.00830904),
  DSP16_Q(0.00509599),
  DSP16_Q(-0.00296005),
  DSP16_Q(0.00153878),
  DSP16_Q(-0.00064531),
  DSP16_Q(0.00016229),
  DSP16_Q(-0.00000137),
  DSP16_Q(-0.00010158),
  DSP16_Q(0.00050086),
  DSP16_Q(-0.00128219),
  DSP16_Q(0.00255232),
  DSP16_Q(-0.00447434),
  DSP16_Q(0.00735146),
  DSP16_Q(-0.01187499),
  DSP16_Q(0.02007951),
  DSP16_Q(-0.04147644),
  DSP16_Q(0.44809288),
  DSP16_Q(0.05179498),
  DSP16_Q(-0.02280030),
  DSP16_Q(0.01317772),
  DSP16_Q(-0.00813183),
  DSP16_Q(0.00498695),
  DSP16_Q(-0.00289619),
  DSP16_Q(0.00150507),
  DSP16_Q(-0.00063074),
  DSP16_Q(0.00015832),
  DSP16_Q(-0.00000128),
  DSP16_Q(-0.00010000),
  DSP16_Q(0.00049175),
  DSP16_Q(-0.00125784),
  DSP16_Q(0.00250286),
  DSP16_Q(-0.00438673),
  DSP16_Q(0.00720684),
  DSP16_Q(-0.01164162),
  DSP16_Q(0.01968911),
  DSP16_Q(-0.04070487),
  DSP16_Q(0.44844946),
  DSP16_Q(0.05058750),
  DSP16_Q(-0.02229589),
  DSP16_Q(0.01288984),
  DSP16_Q(-0.00795457),
  DSP16_Q(0.00487791),
  DSP16_Q(-0.00283236),
  DSP16_Q(0.00147139),
  DSP16_Q(-0.00061620),
  DSP16_Q(0.00015437),
  DSP16_Q(-0.00000120),
  DSP16_Q(-0.00009839),
  DSP16_Q(0.00048256),
  DSP16_Q(-0.00123332),
  DSP16_Q(0.00245313),
  DSP16_Q(-0.00429868),
  DSP16_Q(0.00706153),
  DSP16_Q(-0.01140713),
  DSP16_Q(0.01929666),
  DSP16_Q(-0.03992781),
  DSP16_Q(0.44879845),
  DSP16_Q(0.04938375),
  DSP16_Q(-0.02179177),
  DSP16_Q(0.01260195),
  DSP16_Q(-0.00777729),
  DSP16_Q(0.00476886),
  DSP16_Q(-0.00276854),
  DSP16_Q(0.00143775),
  DSP16_Q(-0.00060170),
  DSP16_Q(0.00015044),
  DSP16_Q(-0.00000112),
  DSP16_Q(-0.00009676),
  DSP16_Q(0.00047330),
  DSP16_Q(-0.00120865),
  DSP16_Q(0.00240313),
  DSP16_Q(-0.00421019),
  DSP16_Q(0.00691554),
  DSP16_Q(-0.01117153),
  DSP16_Q(0.01890217),
  DSP16_Q(-0.03914530),
  DSP16_Q(0.44913984),
  DSP16_Q(0.04818376),
  DSP16_Q(-0.02128794),
  DSP16_Q(0.01231406),
  DSP16_Q(-0.00759999),
  DSP16_Q(0.00465981),
  DSP16_Q(-0.00270474),
  DSP16_Q(0.00140414),
  DSP16_Q(-0.00058722),
  DSP16_Q(0.00014653),
  DSP16_Q(-0.00000105),
  DSP16_Q(-0.00009510),
  DSP16_Q(0.00046396),
  DSP16_Q(-0.00118383),
  DSP16_Q(0.00235286),
  DSP16_Q(-0.00412128),
  DSP16_Q(0.00676887),
  DSP16_Q(-0.01093482),
  DSP16_Q(0.01850566),
  DSP16_Q(-0.03835733),
  DSP16_Q(0.44947363),
  DSP16_Q(0.04698756),
  DSP16_Q(-0.02078443),
  DSP16_Q(0.01202618),
  DSP16_Q(-0.00742268),
  DSP16_Q(0.00455077),
  DSP16_Q(-0.00264097),
  DSP16_Q(0.00137057),
  DSP16_Q(-0.00057278),
  DSP16_Q(0.00014264),
  DSP16_Q(-0.00000098),
  DSP16_Q(-0.00009342),
  DSP16_Q(0.00045454),
  DSP16_Q(-0.00115885),
  DSP16_Q(0.00230233),
  DSP16_Q(-0.00403194),
  DSP16_Q(0.00662153),
  DSP16_Q(-0.01069703),
  DSP16_Q(0.01810714),
  DSP16_Q(-0.03756392),
  DSP16_Q(0.44979981),
  DSP16_Q(0.04579518),
  DSP16_Q(-0.02028128),
  DSP16_Q(0.01173833),
  DSP16_Q(-0.00724536),
  DSP16_Q(0.00444174),
  DSP16_Q(-0.00257724),
  DSP16_Q(0.00133703),
  DSP16_Q(-0.00055838),
  DSP16_Q(0.00013878),
  DSP16_Q(-0.00000091),
  DSP16_Q(-0.00009171),
  DSP16_Q(0.00044505),
  DSP16_Q(-0.00113372),
  DSP16_Q(0.00225153),
  DSP16_Q(-0.00394217),
  DSP16_Q(0.00647352),
  DSP16_Q(-0.01045815),
  DSP16_Q(0.01770663),
  DSP16_Q(-0.03676507),
  DSP16_Q(0.45011835),
  DSP16_Q(0.04460664),
  DSP16_Q(-0.01977849),
  DSP16_Q(0.01145053),
  DSP16_Q(-0.00706806),
  DSP16_Q(0.00433273),
  DSP16_Q(-0.00251353),
  DSP16_Q(0.00130354),
  DSP16_Q(-0.00054401),
  DSP16_Q(0.00013494),
  DSP16_Q(-0.00000085),
  DSP16_Q(-0.00008997),
  DSP16_Q(0.00043548),
  DSP16_Q(-0.00110844),
  DSP16_Q(0.00220047),
  DSP16_Q(-0.00385199),
  DSP16_Q(0.00632485),
  DSP16_Q(-0.01021820),
  DSP16_Q(0.01730414),
  DSP16_Q(-0.03596081),
  DSP16_Q(0.45042926),
  DSP16_Q(0.04342197),
  DSP16_Q(-0.01927610),
  DSP16_Q(0.01116278),
  DSP16_Q(-0.00689076),
  DSP16_Q(0.00422375),
  DSP16_Q(-0.00244986),
  DSP16_Q(0.00127008),
  DSP16_Q(-0.00052968),
  DSP16_Q(0.00013112),
  DSP16_Q(-0.00000079),
  DSP16_Q(-0.00008821),
  DSP16_Q(0.00042583),
  DSP16_Q(-0.00108301),
  DSP16_Q(0.00214916),
  DSP16_Q(-0.00376139),
  DSP16_Q(0.00617553),
  DSP16_Q(-0.00997718),
  DSP16_Q(0.01689968),
  DSP16_Q(-0.03515113),
  DSP16_Q(0.45073253),
  DSP16_Q(0.04224120),
  DSP16_Q(-0.01877413),
  DSP16_Q(0.01087511),
  DSP16_Q(-0.00671350),
  DSP16_Q(0.00411479),
  DSP16_Q(-0.00238623),
  DSP16_Q(0.00123667),
  DSP16_Q(-0.00051538),
  DSP16_Q(0.00012733),
  DSP16_Q(-0.00000073),
  DSP16_Q(-0.00008642),
  DSP16_Q(0.00041611),
  DSP16_Q(-0.00105743),
  DSP16_Q(0.00209758),
  DSP16_Q(-0.00367038),
  DSP16_Q(0.00602557),
  DSP16_Q(-0.00973511),
  DSP16_Q(0.01649327),
  DSP16_Q(-0.03433605),
  DSP16_Q(0.45102813),
  DSP16_Q(0.04106435),
  DSP16_Q(-0.01827260),
  DSP16_Q(0.01058752),
  DSP16_Q(-0.00653627),
  DSP16_Q(0.00400588),
  DSP16_Q(-0.00232265),
  DSP16_Q(0.00120330),
  DSP16_Q(-0.00050112),
  DSP16_Q(0.00012356),
  DSP16_Q(-0.00000067),
  DSP16_Q(-0.00008461),
  DSP16_Q(0.00040632),
  DSP16_Q(-0.00103170),
  DSP16_Q(0.00204576),
  DSP16_Q(-0.00357896),
  DSP16_Q(0.00587496),
  DSP16_Q(-0.00949200),
  DSP16_Q(0.01608493),
  DSP16_Q(-0.03351558),
  DSP16_Q(0.45131607),
  DSP16_Q(0.03989144),
  DSP16_Q(-0.01777154),
  DSP16_Q(0.01030003),
  DSP16_Q(-0.00635908),
  DSP16_Q(0.00389700),
  DSP16_Q(-0.00225911),
  DSP16_Q(0.00116998),
  DSP16_Q(-0.00048691),
  DSP16_Q(0.00011981),
  DSP16_Q(-0.00000062),
  DSP16_Q(-0.00008277),
  DSP16_Q(0.00039645),
  DSP16_Q(-0.00100582),
  DSP16_Q(0.00199368),
  DSP16_Q(-0.00348714),
  DSP16_Q(0.00572372),
  DSP16_Q(-0.00924786),
  DSP16_Q(0.01567467),
  DSP16_Q(-0.03268974),
  DSP16_Q(0.45159634),
  DSP16_Q(0.03872252),
  DSP16_Q(-0.01727097),
  DSP16_Q(0.01001266),
  DSP16_Q(-0.00618195),
  DSP16_Q(0.00378817),
  DSP16_Q(-0.00219562),
  DSP16_Q(0.00113671),
  DSP16_Q(-0.00047272),
  DSP16_Q(0.00011609),
  DSP16_Q(-0.00000057),
  DSP16_Q(-0.00008090),
  DSP16_Q(0.00038650),
  DSP16_Q(-0.00097979),
  DSP16_Q(0.00194135),
  DSP16_Q(-0.00339492),
  DSP16_Q(0.00557185),
  DSP16_Q(-0.00900270),
  DSP16_Q(0.01526250),
  DSP16_Q(-0.03185853),
  DSP16_Q(0.45186892),
  DSP16_Q(0.03755760),
  DSP16_Q(-0.01677091),
  DSP16_Q(0.00972542),
  DSP16_Q(-0.00600488),
  DSP16_Q(0.00367940),
  DSP16_Q(-0.00213219),
  DSP16_Q(0.00110348),
  DSP16_Q(-0.00045858),
  DSP16_Q(0.00011238),
  DSP16_Q(-0.00000052),
  DSP16_Q(-0.00007900),
  DSP16_Q(0.00037648),
  DSP16_Q(-0.00095362),
  DSP16_Q(0.00188877),
  DSP16_Q(-0.00330230),
  DSP16_Q(0.00541936),
  DSP16_Q(-0.00875652),
  DSP16_Q(0.01484845),
  DSP16_Q(-0.03102197),
  DSP16_Q(0.45213382),
  DSP16_Q(0.03639670),
  DSP16_Q(-0.01627139),
  DSP16_Q(0.00943833),
  DSP16_Q(-0.00582788),
  DSP16_Q(0.00357068),
  DSP16_Q(-0.00206881),
  DSP16_Q(0.00107031),
  DSP16_Q(-0.00044448),
  DSP16_Q(0.00010871),
  DSP16_Q(-0.00000048),
  DSP16_Q(-0.00007708),
  DSP16_Q(0.00036639),
  DSP16_Q(-0.00092730),
  DSP16_Q(0.00183594),
  DSP16_Q(-0.00320930),
  DSP16_Q(0.00526626),
  DSP16_Q(-0.00850934),
  DSP16_Q(0.01443252),
  DSP16_Q(-0.03018007),
  DSP16_Q(0.45239101),
  DSP16_Q(0.03523986),
  DSP16_Q(-0.01577242),
  DSP16_Q(0.00915139),
  DSP16_Q(-0.00565096),
  DSP16_Q(0.00346203),
  DSP16_Q(-0.00200549),
  DSP16_Q(0.00103719),
  DSP16_Q(-0.00043043),
  DSP16_Q(0.00010505),
  DSP16_Q(-0.00000044),
  DSP16_Q(-0.00007513),
  DSP16_Q(0.00035622),
  DSP16_Q(-0.00090084),
  DSP16_Q(0.00178287),
  DSP16_Q(-0.00311590),
  DSP16_Q(0.00511254),
  DSP16_Q(-0.00826117),
  DSP16_Q(0.01401473),
  DSP16_Q(-0.02933285),
  DSP16_Q(0.45264049),
  DSP16_Q(0.03408710),
  DSP16_Q(-0.01527404),
  DSP16_Q(0.00886463),
  DSP16_Q(-0.00547413),
  DSP16_Q(0.00335345),
  DSP16_Q(-0.00194223),
  DSP16_Q(0.00100413),
  DSP16_Q(-0.00041641),
  DSP16_Q(0.00010143),
  DSP16_Q(-0.00000040),
  DSP16_Q(-0.00007316),
  DSP16_Q(0.00034597),
  DSP16_Q(-0.00087424),
  DSP16_Q(0.00172956),
  DSP16_Q(-0.00302212),
  DSP16_Q(0.00495823),
  DSP16_Q(-0.00801203),
  DSP16_Q(0.01359511),
  DSP16_Q(-0.02848030),
  DSP16_Q(0.45288226),
  DSP16_Q(0.03293844),
  DSP16_Q(-0.01477626),
  DSP16_Q(0.00857805),
  DSP16_Q(-0.00529740),
  DSP16_Q(0.00324494),
  DSP16_Q(-0.00187904),
  DSP16_Q(0.00097112),
  DSP16_Q(-0.00040243),
  DSP16_Q(0.00009782),
  DSP16_Q(-0.00000036),
  DSP16_Q(-0.00007116),
  DSP16_Q(0.00033566),
  DSP16_Q(-0.00084750),
  DSP16_Q(0.00167601),
  DSP16_Q(-0.00292797),
  DSP16_Q(0.00480333),
  DSP16_Q(-0.00776191),
  DSP16_Q(0.01317365),
  DSP16_Q(-0.02762246),
  DSP16_Q(0.45311630),
  DSP16_Q(0.03179391),
  DSP16_Q(-0.01427911),
  DSP16_Q(0.00829168),
  DSP16_Q(-0.00512077),
  DSP16_Q(0.00313652),
  DSP16_Q(-0.00181592),
  DSP16_Q(0.00093818),
  DSP16_Q(-0.00038850),
  DSP16_Q(0.00009424),
  DSP16_Q(-0.00000033),
  DSP16_Q(-0.00006913),
  DSP16_Q(0.00032527),
  DSP16_Q(-0.00082061),
  DSP16_Q(0.00162223),
  DSP16_Q(-0.00283343),
  DSP16_Q(0.00464783),
  DSP16_Q(-0.00751083),
  DSP16_Q(0.01275039),
  DSP16_Q(-0.02675933),
  DSP16_Q(0.45334261),
  DSP16_Q(0.03065354),
  DSP16_Q(-0.01378261),
  DSP16_Q(0.00800553),
  DSP16_Q(-0.00494426),
  DSP16_Q(0.00302818),
  DSP16_Q(-0.00175287),
  DSP16_Q(0.00090529),
  DSP16_Q(-0.00037462),
  DSP16_Q(0.00009068),
  DSP16_Q(-0.00000029),
  DSP16_Q(-0.00006707),
  DSP16_Q(0.00031481),
  DSP16_Q(-0.00079358),
  DSP16_Q(0.00156821),
  DSP16_Q(-0.00273853),
  DSP16_Q(0.00449176),
  DSP16_Q(-0.00725880),
  DSP16_Q(0.01232534),
  DSP16_Q(-0.02589092),
  DSP16_Q(0.45356118),
  DSP16_Q(0.02951735),
  DSP16_Q(-0.01328678),
  DSP16_Q(0.00771960),
  DSP16_Q(-0.00476788),
  DSP16_Q(0.00291993),
  DSP16_Q(-0.00168990),
  DSP16_Q(0.00087246),
  DSP16_Q(-0.00036077),
  DSP16_Q(0.00008715),
  DSP16_Q(-0.00000026),
  DSP16_Q(-0.00006499),
  DSP16_Q(0.00030427),
  DSP16_Q(-0.00076642),
  DSP16_Q(0.00151395),
  DSP16_Q(-0.00264326),
  DSP16_Q(0.00433512),
  DSP16_Q(-0.00700584),
  DSP16_Q(0.01189851),
  DSP16_Q(-0.02501725),
  DSP16_Q(0.45377201),
  DSP16_Q(0.02838536),
  DSP16_Q(-0.01279165),
  DSP16_Q(0.00743392),
  DSP16_Q(-0.00459163),
  DSP16_Q(0.00281178),
  DSP16_Q(-0.00162701),
  DSP16_Q(0.00083970),
  DSP16_Q(-0.00034698),
  DSP16_Q(0.00008364),
  DSP16_Q(-0.00000024),
  DSP16_Q(-0.00006288),
  DSP16_Q(0.00029367),
  DSP16_Q(-0.00073912),
  DSP16_Q(0.00145947),
  DSP16_Q(-0.00254762),
  DSP16_Q(0.00417790),
  DSP16_Q(-0.00675195),
  DSP16_Q(0.01146992),
  DSP16_Q(-0.02413833),
  DSP16_Q(0.45397509),
  DSP16_Q(0.02725759),
  DSP16_Q(-0.01229724),
  DSP16_Q(0.00714849),
  DSP16_Q(-0.00441553),
  DSP16_Q(0.00270373),
  DSP16_Q(-0.00156420),
  DSP16_Q(0.00080700),
  DSP16_Q(-0.00033323),
  DSP16_Q(0.00008016),
  DSP16_Q(-0.00000021),
  DSP16_Q(-0.00006075),
  DSP16_Q(0.00028299),
  DSP16_Q(-0.00071168),
  DSP16_Q(0.00140476),
  DSP16_Q(-0.00245163),
  DSP16_Q(0.00402013),
  DSP16_Q(-0.00649714),
  DSP16_Q(0.01103958),
  DSP16_Q(-0.02325418),
  DSP16_Q(0.45417040),
  DSP16_Q(0.02613409),
  DSP16_Q(-0.01180357),
  DSP16_Q(0.00686334),
  DSP16_Q(-0.00423957),
  DSP16_Q(0.00259580),
  DSP16_Q(-0.00150148),
  DSP16_Q(0.00077437),
  DSP16_Q(-0.00031952),
  DSP16_Q(0.00007670),
  DSP16_Q(-0.00000019),
  DSP16_Q(-0.00005859),
  DSP16_Q(0.00027224),
  DSP16_Q(-0.00068410),
  DSP16_Q(0.00134982),
  DSP16_Q(-0.00235528),
  DSP16_Q(0.00386180),
  DSP16_Q(-0.00624143),
  DSP16_Q(0.01060752),
  DSP16_Q(-0.02236480),
  DSP16_Q(0.45435796),
  DSP16_Q(0.02501485),
  DSP16_Q(-0.01131066),
  DSP16_Q(0.00657848),
  DSP16_Q(-0.00406378),
  DSP16_Q(0.00248797),
  DSP16_Q(-0.00143884),
  DSP16_Q(0.00074180),
  DSP16_Q(-0.00030587),
  DSP16_Q(0.00007327),
  DSP16_Q(-0.00000016),
  DSP16_Q(-0.00005640),
  DSP16_Q(0.00026142),
  DSP16_Q(-0.00065640),
  DSP16_Q(0.00129466),
  DSP16_Q(-0.00225858),
  DSP16_Q(0.00370293),
  DSP16_Q(-0.00598483),
  DSP16_Q(0.01017375),
  DSP16_Q(-0.02147023),
  DSP16_Q(0.45453774),
  DSP16_Q(0.02389992),
  DSP16_Q(-0.01081854),
  DSP16_Q(0.00629391),
  DSP16_Q(-0.00388816),
  DSP16_Q(0.00238027),
  DSP16_Q(-0.00137630),
  DSP16_Q(0.00070931),
  DSP16_Q(-0.00029226),
  DSP16_Q(0.00006986),
  DSP16_Q(-0.00000014),
  DSP16_Q(-0.00005418),
  DSP16_Q(0.00025053),
  DSP16_Q(-0.00062855),
  DSP16_Q(0.00123928),
  DSP16_Q(-0.00216154),
  DSP16_Q(0.00354353),
  DSP16_Q(-0.00572734),
  DSP16_Q(0.00973829),
  DSP16_Q(-0.02057046),
  DSP16_Q(0.45470975),
  DSP16_Q(0.02278932),
  DSP16_Q(-0.01032722),
  DSP16_Q(0.00600966),
  DSP16_Q(-0.00371271),
  DSP16_Q(0.00227269),
  DSP16_Q(-0.00131385),
  DSP16_Q(0.00067689),
  DSP16_Q(-0.00027870),
  DSP16_Q(0.00006648),
  DSP16_Q(-0.00000012),
  DSP16_Q(-0.00005194),
  DSP16_Q(0.00023957),
  DSP16_Q(-0.00060058),
  DSP16_Q(0.00118369),
  DSP16_Q(-0.00206416),
  DSP16_Q(0.00338359),
  DSP16_Q(-0.00546899),
  DSP16_Q(0.00930115),
  DSP16_Q(-0.01966551),
  DSP16_Q(0.45487397),
  DSP16_Q(0.02168306),
  DSP16_Q(-0.00983674),
  DSP16_Q(0.00572574),
  DSP16_Q(-0.00353745),
  DSP16_Q(0.00216524),
  DSP16_Q(-0.00125150),
  DSP16_Q(0.00064454),
  DSP16_Q(-0.00026518),
  DSP16_Q(0.00006312),
  DSP16_Q(-0.00000011),
  DSP16_Q(-0.00004967),
  DSP16_Q(0.00022854),
  DSP16_Q(-0.00057248),
  DSP16_Q(0.00112787),
  DSP16_Q(-0.00196643),
  DSP16_Q(0.00322312),
  DSP16_Q(-0.00520977),
  DSP16_Q(0.00886235),
  DSP16_Q(-0.01875541),
  DSP16_Q(0.45503041),
  DSP16_Q(0.02058117),
  DSP16_Q(-0.00934710),
  DSP16_Q(0.00544216),
  DSP16_Q(-0.00336239),
  DSP16_Q(0.00205793),
  DSP16_Q(-0.00118925),
  DSP16_Q(0.00061226),
  DSP16_Q(-0.00025172),
  DSP16_Q(0.00005979),
  DSP16_Q(-0.00000009),
  DSP16_Q(-0.00004737),
  DSP16_Q(0.00021744),
  DSP16_Q(-0.00054424),
  DSP16_Q(0.00107184),
  DSP16_Q(-0.00186838),
  DSP16_Q(0.00306214),
  DSP16_Q(-0.00494971),
  DSP16_Q(0.00842191),
  DSP16_Q(-0.01784016),
  DSP16_Q(0.45517906),
  DSP16_Q(0.01948368),
  DSP16_Q(-0.00885834),
  DSP16_Q(0.00515894),
  DSP16_Q(-0.00318753),
  DSP16_Q(0.00195076),
  DSP16_Q(-0.00112711),
  DSP16_Q(0.00058006),
  DSP16_Q(-0.00023831),
  DSP16_Q(0.00005649),
  DSP16_Q(-0.00000008),
  DSP16_Q(-0.00004504),
  DSP16_Q(0.00020626),
  DSP16_Q(-0.00051588),
  DSP16_Q(0.00101560),
  DSP16_Q(-0.00177000),
  DSP16_Q(0.00290065),
  DSP16_Q(-0.00468881),
  DSP16_Q(0.00797985),
  DSP16_Q(-0.01691978),
  DSP16_Q(0.45531991),
  DSP16_Q(0.01839060),
  DSP16_Q(-0.00837048),
  DSP16_Q(0.00487608),
  DSP16_Q(-0.00301288),
  DSP16_Q(0.00184373),
  DSP16_Q(-0.00106507),
  DSP16_Q(0.00054794),
  DSP16_Q(-0.00022495),
  DSP16_Q(0.00005320),
  DSP16_Q(-0.00000007),
  DSP16_Q(-0.00004269),
  DSP16_Q(0.00019503),
  DSP16_Q(-0.00048739),
  DSP16_Q(0.00095916),
  DSP16_Q(-0.00167129),
  DSP16_Q(0.00273866),
  DSP16_Q(-0.00442708),
  DSP16_Q(0.00753618),
  DSP16_Q(-0.01599428),
  DSP16_Q(0.45545296),
  DSP16_Q(0.01730197),
  DSP16_Q(-0.00788353),
  DSP16_Q(0.00459361),
  DSP16_Q(-0.00283846),
  DSP16_Q(0.00173686),
  DSP16_Q(-0.00100315),
  DSP16_Q(0.00051590),
  DSP16_Q(-0.00021164),
  DSP16_Q(0.00004995),
  DSP16_Q(-0.00000006),
  DSP16_Q(-0.00004031),
  DSP16_Q(0.00018372),
  DSP16_Q(-0.00045877),
  DSP16_Q(0.00090251),
  DSP16_Q(-0.00157227),
  DSP16_Q(0.00257617),
  DSP16_Q(-0.00416454),
  DSP16_Q(0.00709092),
  DSP16_Q(-0.01506368),
  DSP16_Q(0.45557821),
  DSP16_Q(0.01621780),
  DSP16_Q(-0.00739751),
  DSP16_Q(0.00431153),
  DSP16_Q(-0.00266426),
  DSP16_Q(0.00163014),
  DSP16_Q(-0.00094133),
  DSP16_Q(0.00048394),
  DSP16_Q(-0.00019838),
  DSP16_Q(0.00004672),
  DSP16_Q(-0.00000005),
  DSP16_Q(-0.00003791),
  DSP16_Q(0.00017234),
  DSP16_Q(-0.00043003),
  DSP16_Q(0.00084565),
  DSP16_Q(-0.00147293),
  DSP16_Q(0.00241320),
  DSP16_Q(-0.00390120),
  DSP16_Q(0.00664410),
  DSP16_Q(-0.01412801),
  DSP16_Q(0.45569564),
  DSP16_Q(0.01513811),
  DSP16_Q(-0.00691246),
  DSP16_Q(0.00402987),
  DSP16_Q(-0.00249031),
  DSP16_Q(0.00152359),
  DSP16_Q(-0.00087964),
  DSP16_Q(0.00045206),
  DSP16_Q(-0.00018518),
  DSP16_Q(0.00004352),
  DSP16_Q(-0.00000004),
  DSP16_Q(-0.00003548),
  DSP16_Q(0.00016090),
  DSP16_Q(-0.00040116),
  DSP16_Q(0.00078859),
  DSP16_Q(-0.00137328),
  DSP16_Q(0.00224974),
  DSP16_Q(-0.00363706),
  DSP16_Q(0.00619572),
  DSP16_Q(-0.01318726),
  DSP16_Q(0.45580527),
  DSP16_Q(0.01406293),
  DSP16_Q(-0.00642839),
  DSP16_Q(0.00374862),
  DSP16_Q(-0.00231660),
  DSP16_Q(0.00141720),
  DSP16_Q(-0.00081806),
  DSP16_Q(0.00042026),
  DSP16_Q(-0.00017203),
  DSP16_Q(0.00004034),
  DSP16_Q(-0.00000003),
  DSP16_Q(-0.00003302),
  DSP16_Q(0.00014939),
  DSP16_Q(-0.00037218),
  DSP16_Q(0.00073134),
  DSP16_Q(-0.00127332),
  DSP16_Q(0.00208582),
  DSP16_Q(-0.00337215),
  DSP16_Q(0.00574580),
  DSP16_Q(-0.01224147),
  DSP16_Q(0.45590708),
  DSP16_Q(0.01299228),
  DSP16_Q(-0.00594532),
  DSP16_Q(0.00346782),
  DSP16_Q(-0.00214315),
  DSP16_Q(0.00131099),
  DSP16_Q(-0.00075661),
  DSP16_Q(0.00038856),
  DSP16_Q(-0.00015893),
  DSP16_Q(0.00003719),
  DSP16_Q(-0.00000002),
  DSP16_Q(-0.00003053),
  DSP16_Q(0.00013781),
  DSP16_Q(-0.00034307),
  DSP16_Q(0.00067389),
  DSP16_Q(-0.00117307),
  DSP16_Q(0.00192143),
  DSP16_Q(-0.00310648),
  DSP16_Q(0.00529437),
  DSP16_Q(-0.01129064),
  DSP16_Q(0.45600107),
  DSP16_Q(0.01192618),
  DSP16_Q(-0.00546327),
  DSP16_Q(0.00318746),
  DSP16_Q(-0.00196996),
  DSP16_Q(0.00120495),
  DSP16_Q(-0.00069528),
  DSP16_Q(0.00035693),
  DSP16_Q(-0.00014589),
  DSP16_Q(0.00003406),
  DSP16_Q(-0.00000002),
  DSP16_Q(-0.00002802),
  DSP16_Q(0.00012617),
  DSP16_Q(-0.00031384),
  DSP16_Q(0.00061624),
  DSP16_Q(-0.00107251),
  DSP16_Q(0.00175658),
  DSP16_Q(-0.00284004),
  DSP16_Q(0.00484145),
  DSP16_Q(-0.01033480),
  DSP16_Q(0.45608723),
  DSP16_Q(0.01086466),
  DSP16_Q(-0.00498227),
  DSP16_Q(0.00290757),
  DSP16_Q(-0.00179705),
  DSP16_Q(0.00109910),
  DSP16_Q(-0.00063408),
  DSP16_Q(0.00032540),
  DSP16_Q(-0.00013290),
  DSP16_Q(0.00003096),
  DSP16_Q(-0.00000001),
  DSP16_Q(-0.00002548),
  DSP16_Q(0.00011445),
  DSP16_Q(-0.00028449),
  DSP16_Q(0.00055841),
  DSP16_Q(-0.00097167),
  DSP16_Q(0.00159128),
  DSP16_Q(-0.00257287),
  DSP16_Q(0.00438704),
  DSP16_Q(-0.00937396),
  DSP16_Q(0.45616558),
  DSP16_Q(0.00980772),
  DSP16_Q(-0.00450234),
  DSP16_Q(0.00262815),
  DSP16_Q(-0.00162441),
  DSP16_Q(0.00099344),
  DSP16_Q(-0.00057301),
  DSP16_Q(0.00029395),
  DSP16_Q(-0.00011997),
  DSP16_Q(0.00002789),
  DSP16_Q(-0.00000001),
  DSP16_Q(-0.00002291),
  DSP16_Q(0.00010268),
  DSP16_Q(-0.00025502),
  DSP16_Q(0.00050039),
  DSP16_Q(-0.00087054),
  DSP16_Q(0.00142554),
  DSP16_Q(-0.00230496),
  DSP16_Q(0.00393117),
  DSP16_Q(-0.00840813),
  DSP16_Q(0.45623609),
  DSP16_Q(0.00875541),
  DSP16_Q(-0.00402349),
  DSP16_Q(0.00234923),
  DSP16_Q(-0.00145207),
  DSP16_Q(0.00088797),
  DSP16_Q(-0.00051208),
  DSP16_Q(0.00026260),
  DSP16_Q(-0.00010710),
  DSP16_Q(0.00002484),
  DSP16_Q(-0.00000001),
  DSP16_Q(-0.00002031),
  DSP16_Q(0.00009084),
  DSP16_Q(-0.00022544),
  DSP16_Q(0.00044218),
  DSP16_Q(-0.00076912),
  DSP16_Q(0.00125937),
  DSP16_Q(-0.00203634),
  DSP16_Q(0.00347386),
  DSP16_Q(-0.00743734),
  DSP16_Q(0.45629878),
  DSP16_Q(0.00770773),
  DSP16_Q(-0.00354574),
  DSP16_Q(0.00207081),
  DSP16_Q(-0.00128002),
  DSP16_Q(0.00078269),
  DSP16_Q(-0.00045129),
  DSP16_Q(0.00023134),
  DSP16_Q(-0.00009428),
  DSP16_Q(0.00002182),
  DSP16_Q(-0.00000000),
  DSP16_Q(-0.00001769),
  DSP16_Q(0.00007893),
  DSP16_Q(-0.00019574),
  DSP16_Q(0.00038379),
  DSP16_Q(-0.00066743),
  DSP16_Q(0.00109276),
  DSP16_Q(-0.00176701),
  DSP16_Q(0.00301513),
  DSP16_Q(-0.00646161),
  DSP16_Q(0.45635363),
  DSP16_Q(0.00666471),
  DSP16_Q(-0.00306913),
  DSP16_Q(0.00179290),
  DSP16_Q(-0.00110828),
  DSP16_Q(0.00067763),
  DSP16_Q(-0.00039063),
  DSP16_Q(0.00020018),
  DSP16_Q(-0.00008152),
  DSP16_Q(0.00001882),
  DSP16_Q(-0.00000000),
  DSP16_Q(-0.00001504),
  DSP16_Q(0.00006696),
  DSP16_Q(-0.00016593),
  DSP16_Q(0.00032522),
  DSP16_Q(-0.00056547),
  DSP16_Q(0.00092574),
  DSP16_Q(-0.00149699),
  DSP16_Q(0.00255499),
  DSP16_Q(-0.00548094),
  DSP16_Q(0.45640065),
  DSP16_Q(0.00562636),
  DSP16_Q(-0.00259366),
  DSP16_Q(0.00151553),
  DSP16_Q(-0.00093686),
  DSP16_Q(0.00057277),
  DSP16_Q(-0.00033012),
  DSP16_Q(0.00016911),
  DSP16_Q(-0.00006881),
  DSP16_Q(0.00001585),
  DSP16_Q(-0.00000000),
  DSP16_Q(-0.00001237),
  DSP16_Q(0.00005493),
  DSP16_Q(-0.00013601),
  DSP16_Q(0.00026648),
  DSP16_Q(-0.00046323),
  DSP16_Q(0.00075831),
  DSP16_Q(-0.00122628),
  DSP16_Q(0.00209347),
  DSP16_Q(-0.00449537),
  DSP16_Q(0.45643984),
  DSP16_Q(0.00459272),
  DSP16_Q(-0.00211936),
  DSP16_Q(0.00123869),
  DSP16_Q(-0.00076575),
  DSP16_Q(0.00046812),
  DSP16_Q(-0.00026976),
  DSP16_Q(0.00013813),
  DSP16_Q(-0.00005617),
  DSP16_Q(0.00001291),
  DSP16_Q(-0.00000000),
  DSP16_Q(-0.00000967),
  DSP16_Q(0.00004283),
  DSP16_Q(-0.00010598),
  DSP16_Q(0.00020756),
  DSP16_Q(-0.00036074),
  DSP16_Q(0.00059048),
  DSP16_Q(-0.00095491),
  DSP16_Q(0.00163059),
  DSP16_Q(-0.00350490),
  DSP16_Q(0.45647119),
  DSP16_Q(0.00356379),
  DSP16_Q(-0.00164624),
  DSP16_Q(0.00096241),
  DSP16_Q(-0.00059498),
  DSP16_Q(0.00036370),
  DSP16_Q(-0.00020954),
  DSP16_Q(0.00010726),
  DSP16_Q(-0.00004358),
  DSP16_Q(0.00001000),
  DSP16_Q(-0.00000000),
  DSP16_Q(-0.00000694),
  DSP16_Q(0.00003067),
  DSP16_Q(-0.00007583),
  DSP16_Q(0.00014846),
  DSP16_Q(-0.00025798),
  DSP16_Q(0.00042225),
  DSP16_Q(-0.00068287),
  DSP16_Q(0.00116635),
  DSP16_Q(-0.00250956),
  DSP16_Q(0.45649471),
  DSP16_Q(0.00253960),
  DSP16_Q(-0.00117434),
  DSP16_Q(0.00068670),
  DSP16_Q(-0.00042455),
  DSP16_Q(0.00025949),
  DSP16_Q(-0.00014948),
  DSP16_Q(0.00007649),
  DSP16_Q(-0.00003105),
  DSP16_Q(0.00000711),
  DSP16_Q(-0.00000000),
  DSP16_Q(-0.00000418),
  DSP16_Q(0.00001845),
  DSP16_Q(-0.00004558),
  DSP16_Q(0.00008920),
  DSP16_Q(-0.00015498),
  DSP16_Q(0.00025363),
  DSP16_Q(-0.00041020),
  DSP16_Q(0.00070079),
  DSP16_Q(-0.00150936),
  DSP16_Q(0.45651038),
  DSP16_Q(0.00152018),
  DSP16_Q(-0.00070367),
  DSP16_Q(0.00041158),
  DSP16_Q(-0.00025446),
  DSP16_Q(0.00015552),
  DSP16_Q(-0.00008957),
  DSP16_Q(0.00004582),
  DSP16_Q(-0.00001859),
  DSP16_Q(0.00000424),
  DSP16_Q(-0.00000000),
  DSP16_Q(-0.00000140),
  DSP16_Q(0.00000617),
  DSP16_Q(-0.00001522),
  DSP16_Q(0.00002978),
  DSP16_Q(-0.00005172),
  DSP16_Q(0.00008464),
  DSP16_Q(-0.00013689),
  DSP16_Q(0.00023392),
  DSP16_Q(-0.00050432),
  DSP16_Q(0.45651822),
  DSP16_Q(0.00050553),
  DSP16_Q(-0.00023424),
  DSP16_Q(0.00013704),
  DSP16_Q(-0.00008473),
  DSP16_Q(0.00005178),
  DSP16_Q(-0.00002982),
  DSP16_Q(0.00001525),
  DSP16_Q(-0.00000618),
  DSP16_Q(0.00000141),
  DSP16_Q(-0.00000000)
};
