/*****************************************************************************
 *
 * \file
 *
 * \brief FIR coefficients for the dsp16_resampling function from the DPSLib.
 *        These coefficients have the following caracterizations:
 *          - Input sampling frequency: 32000 Hz
 *          - Output sampling frequency: 44200 Hz
 *          - Re-sampling order: 30
 *          - Coefficients are normalized
 *          - Memory foot print: 13260 bytes
 *         It matches the following parameters:
 *         (dsp16_resampling_options_t::dsp16_custom_filter)
 *          - Cut-off frequency (fc_hz): 16000
 *          - Sampling frequency (fs_hz): ******
 *          - Actual FIR filter order (order): 6630
 *
 * Copyright (c) 2010 Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an
 *    Atmel microcontroller product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 ******************************************************************************/

A_ALIGNED static const dsp16_t dsp16_resampling_fircoef_ord30_32000_44200_norm[] = {
  DSP16_Q(0.00000000),
  DSP16_Q(-0.00000072),
  DSP16_Q(0.00000307),
  DSP16_Q(-0.00000736),
  DSP16_Q(0.00001390),
  DSP16_Q(-0.00002311),
  DSP16_Q(0.00003548),
  DSP16_Q(-0.00005173),
  DSP16_Q(0.00007293),
  DSP16_Q(-0.00010083),
  DSP16_Q(0.00013866),
  DSP16_Q(-0.00019289),
  DSP16_Q(0.00027878),
  DSP16_Q(-0.00044249),
  DSP16_Q(0.00091585),
  DSP16_Q(0.40833363),
  DSP16_Q(-0.00091163),
  DSP16_Q(0.00044140),
  DSP16_Q(-0.00027828),
  DSP16_Q(0.00019259),
  DSP16_Q(-0.00013846),
  DSP16_Q(0.00010069),
  DSP16_Q(-0.00007282),
  DSP16_Q(0.00005165),
  DSP16_Q(-0.00003542),
  DSP16_Q(0.00002306),
  DSP16_Q(-0.00001387),
  DSP16_Q(0.00000733),
  DSP16_Q(-0.00000306),
  DSP16_Q(0.00000071),
  DSP16_Q(0.00000000),
  DSP16_Q(-0.00000218),
  DSP16_Q(0.00000926),
  DSP16_Q(-0.00002214),
  DSP16_Q(0.00004181),
  DSP16_Q(-0.00006946),
  DSP16_Q(0.00010663),
  DSP16_Q(-0.00015544),
  DSP16_Q(0.00021909),
  DSP16_Q(-0.00030292),
  DSP16_Q(0.00041656),
  DSP16_Q(-0.00057953),
  DSP16_Q(0.00083781),
  DSP16_Q(-0.00133065),
  DSP16_Q(0.00276016),
  DSP16_Q(0.40830595),
  DSP16_Q(-0.00272213),
  DSP16_Q(0.00132085),
  DSP16_Q(-0.00083326),
  DSP16_Q(0.00057683),
  DSP16_Q(-0.00041475),
  DSP16_Q(0.00030161),
  DSP16_Q(-0.00021811),
  DSP16_Q(0.00015468),
  DSP16_Q(-0.00010606),
  DSP16_Q(0.00006903),
  DSP16_Q(-0.00004149),
  DSP16_Q(0.00002192),
  DSP16_Q(-0.00000913),
  DSP16_Q(0.00000212),
  DSP16_Q(0.00000000),
  DSP16_Q(-0.00000367),
  DSP16_Q(0.00001551),
  DSP16_Q(-0.00003701),
  DSP16_Q(0.00006984),
  DSP16_Q(-0.00011600),
  DSP16_Q(0.00017802),
  DSP16_Q(-0.00025944),
  DSP16_Q(0.00036565),
  DSP16_Q(-0.00050553),
  DSP16_Q(0.00069519),
  DSP16_Q(-0.00096725),
  DSP16_Q(0.00139871),
  DSP16_Q(-0.00222295),
  DSP16_Q(0.00462116),
  DSP16_Q(0.40825057),
  DSP16_Q(-0.00451553),
  DSP16_Q(0.00219573),
  DSP16_Q(-0.00138606),
  DSP16_Q(0.00095977),
  DSP16_Q(-0.00069016),
  DSP16_Q(0.00050189),
  DSP16_Q(-0.00036292),
  DSP16_Q(0.00025735),
  DSP16_Q(-0.00017642),
  DSP16_Q(0.00011479),
  DSP16_Q(-0.00006896),
  DSP16_Q(0.00003641),
  DSP16_Q(-0.00001514),
  DSP16_Q(0.00000351),
  DSP16_Q(0.00000000),
  DSP16_Q(-0.00000519),
  DSP16_Q(0.00002182),
  DSP16_Q(-0.00005197),
  DSP16_Q(0.00009801),
  DSP16_Q(-0.00016270),
  DSP16_Q(0.00024962),
  DSP16_Q(-0.00036373),
  DSP16_Q(0.00051257),
  DSP16_Q(-0.00070862),
  DSP16_Q(0.00097449),
  DSP16_Q(-0.00135598),
  DSP16_Q(0.00196137),
  DSP16_Q(-0.00311922),
  DSP16_Q(0.00649869),
  DSP16_Q(0.40816752),
  DSP16_Q(-0.00629167),
  DSP16_Q(0.00306588),
  DSP16_Q(-0.00193659),
  DSP16_Q(0.00134132),
  DSP16_Q(-0.00096463),
  DSP16_Q(0.00070150),
  DSP16_Q(-0.00050722),
  DSP16_Q(0.00035964),
  DSP16_Q(-0.00024648),
  DSP16_Q(0.00016034),
  DSP16_Q(-0.00009628),
  DSP16_Q(0.00005079),
  DSP16_Q(-0.00002109),
  DSP16_Q(0.00000486),
  DSP16_Q(0.00000000),
  DSP16_Q(-0.00000673),
  DSP16_Q(0.00002818),
  DSP16_Q(-0.00006703),
  DSP16_Q(0.00012630),
  DSP16_Q(-0.00020957),
  DSP16_Q(0.00032144),
  DSP16_Q(-0.00046829),
  DSP16_Q(0.00065983),
  DSP16_Q(-0.00091215),
  DSP16_Q(0.00125439),
  DSP16_Q(-0.00174565),
  DSP16_Q(0.00252568),
  DSP16_Q(-0.00401931),
  DSP16_Q(0.00839261),
  DSP16_Q(0.40805681),
  DSP16_Q(-0.00805043),
  DSP16_Q(0.00393116),
  DSP16_Q(-0.00248473),
  DSP16_Q(0.00172142),
  DSP16_Q(-0.00123810),
  DSP16_Q(0.00090038),
  DSP16_Q(-0.00065099),
  DSP16_Q(0.00046152),
  DSP16_Q(-0.00031625),
  DSP16_Q(0.00020566),
  DSP16_Q(-0.00012344),
  DSP16_Q(0.00006507),
  DSP16_Q(-0.00002698),
  DSP16_Q(0.00000619),
  DSP16_Q(0.00000000),
  DSP16_Q(-0.00000830),
  DSP16_Q(0.00003459),
  DSP16_Q(-0.00008216),
  DSP16_Q(0.00015470),
  DSP16_Q(-0.00025659),
  DSP16_Q(0.00039344),
  DSP16_Q(-0.00057308),
  DSP16_Q(0.00080740),
  DSP16_Q(-0.00111608),
  DSP16_Q(0.00153486),
  DSP16_Q(-0.00213618),
  DSP16_Q(0.00309154),
  DSP16_Q(-0.00492307),
  DSP16_Q(0.01030277),
  DSP16_Q(0.40791844),
  DSP16_Q(-0.00979168),
  DSP16_Q(0.00479142),
  DSP16_Q(-0.00303039),
  DSP16_Q(0.00209999),
  DSP16_Q(-0.00151053),
  DSP16_Q(0.00109851),
  DSP16_Q(-0.00079419),
  DSP16_Q(0.00056297),
  DSP16_Q(-0.00038570),
  DSP16_Q(0.00025075),
  DSP16_Q(-0.00015044),
  DSP16_Q(0.00007924),
  DSP16_Q(-0.00003281),
  DSP16_Q(0.00000749),
  DSP16_Q(0.00000001),
  DSP16_Q(-0.00000990),
  DSP16_Q(0.00004106),
  DSP16_Q(-0.00009738),
  DSP16_Q(0.00018322),
  DSP16_Q(-0.00030375),
  DSP16_Q(0.00046563),
  DSP16_Q(-0.00067810),
  DSP16_Q(0.00095525),
  DSP16_Q(-0.00132038),
  DSP16_Q(0.00181583),
  DSP16_Q(-0.00252749),
  DSP16_Q(0.00365884),
  DSP16_Q(-0.00583033),
  DSP16_Q(0.01222900),
  DSP16_Q(0.40775243),
  DSP16_Q(-0.01151528),
  DSP16_Q(0.00564653),
  DSP16_Q(-0.00357346),
  DSP16_Q(0.00247696),
  DSP16_Q(-0.00178187),
  DSP16_Q(0.00129584),
  DSP16_Q(-0.00093680),
  DSP16_Q(0.00066398),
  DSP16_Q(-0.00045481),
  DSP16_Q(0.00029559),
  DSP16_Q(-0.00017726),
  DSP16_Q(0.00009329),
  DSP16_Q(-0.00003857),
  DSP16_Q(0.00000876),
  DSP16_Q(0.00000001),
  DSP16_Q(-0.00001152),
  DSP16_Q(0.00004758),
  DSP16_Q(-0.00011267),
  DSP16_Q(0.00021184),
  DSP16_Q(-0.00035105),
  DSP16_Q(0.00053798),
  DSP16_Q(-0.00078332),
  DSP16_Q(0.00110334),
  DSP16_Q(-0.00152499),
  DSP16_Q(0.00209725),
  DSP16_Q(-0.00291950),
  DSP16_Q(0.00422747),
  DSP16_Q(-0.00674095),
  DSP16_Q(0.01417116),
  DSP16_Q(0.40755880),
  DSP16_Q(-0.01322111),
  DSP16_Q(0.00649633),
  DSP16_Q(-0.00411385),
  DSP16_Q(0.00285227),
  DSP16_Q(-0.00205205),
  DSP16_Q(0.00149234),
  DSP16_Q(-0.00107879),
  DSP16_Q(0.00076453),
  DSP16_Q(-0.00052359),
  DSP16_Q(0.00034019),
  DSP16_Q(-0.00020392),
  DSP16_Q(0.00010724),
  DSP16_Q(-0.00004426),
  DSP16_Q(0.00001001),
  DSP16_Q(0.00000002),
  DSP16_Q(-0.00001317),
  DSP16_Q(0.00005415),
  DSP16_Q(-0.00012804),
  DSP16_Q(0.00024056),
  DSP16_Q(-0.00039847),
  DSP16_Q(0.00061048),
  DSP16_Q(-0.00088871),
  DSP16_Q(0.00125165),
  DSP16_Q(-0.00172988),
  DSP16_Q(0.00237906),
  DSP16_Q(-0.00331215),
  DSP16_Q(0.00479733),
  DSP16_Q(-0.00765475),
  DSP16_Q(0.01612908),
  DSP16_Q(0.40733759),
  DSP16_Q(-0.01490905),
  DSP16_Q(0.00734070),
  DSP16_Q(-0.00465146),
  DSP16_Q(0.00322583),
  DSP16_Q(-0.00232104),
  DSP16_Q(0.00168797),
  DSP16_Q(-0.00122013),
  DSP16_Q(0.00086459),
  DSP16_Q(-0.00059200),
  DSP16_Q(0.00038453),
  DSP16_Q(-0.00023039),
  DSP16_Q(0.00012107),
  DSP16_Q(-0.00004989),
  DSP16_Q(0.00001123),
  DSP16_Q(0.00000002),
  DSP16_Q(-0.00001484),
  DSP16_Q(0.00006077),
  DSP16_Q(-0.00014349),
  DSP16_Q(0.00026938),
  DSP16_Q(-0.00044601),
  DSP16_Q(0.00068311),
  DSP16_Q(-0.00099427),
  DSP16_Q(0.00140014),
  DSP16_Q(-0.00193502),
  DSP16_Q(0.00266120),
  DSP16_Q(-0.00370534),
  DSP16_Q(0.00536829),
  DSP16_Q(-0.00857157),
  DSP16_Q(0.01810259),
  DSP16_Q(0.40708881),
  DSP16_Q(-0.01657898),
  DSP16_Q(0.00817949),
  DSP16_Q(-0.00518618),
  DSP16_Q(0.00359758),
  DSP16_Q(-0.00258877),
  DSP16_Q(0.00188269),
  DSP16_Q(-0.00136080),
  DSP16_Q(0.00096416),
  DSP16_Q(-0.00066005),
  DSP16_Q(0.00042861),
  DSP16_Q(-0.00025668),
  DSP16_Q(0.00013478),
  DSP16_Q(-0.00005546),
  DSP16_Q(0.00001242),
  DSP16_Q(0.00000003),
  DSP16_Q(-0.00001654),
  DSP16_Q(0.00006743),
  DSP16_Q(-0.00015900),
  DSP16_Q(0.00029828),
  DSP16_Q(-0.00049365),
  DSP16_Q(0.00075587),
  DSP16_Q(-0.00109996),
  DSP16_Q(0.00154880),
  DSP16_Q(-0.00214034),
  DSP16_Q(0.00294362),
  DSP16_Q(-0.00409901),
  DSP16_Q(0.00594026),
  DSP16_Q(-0.00949127),
  DSP16_Q(0.02009153),
  DSP16_Q(0.40681249),
  DSP16_Q(-0.01823078),
  DSP16_Q(0.00901257),
  DSP16_Q(-0.00571794),
  DSP16_Q(0.00396746),
  DSP16_Q(-0.00285520),
  DSP16_Q(0.00207646),
  DSP16_Q(-0.00150077),
  DSP16_Q(0.00106320),
  DSP16_Q(-0.00072771),
  DSP16_Q(0.00047240),
  DSP16_Q(-0.00028278),
  DSP16_Q(0.00014837),
  DSP16_Q(-0.00006095),
  DSP16_Q(0.00001358),
  DSP16_Q(0.00000004),
  DSP16_Q(-0.00001826),
  DSP16_Q(0.00007415),
  DSP16_Q(-0.00017458),
  DSP16_Q(0.00032727),
  DSP16_Q(-0.00054139),
  DSP16_Q(0.00082873),
  DSP16_Q(-0.00120576),
  DSP16_Q(0.00169758),
  DSP16_Q(-0.00234583),
  DSP16_Q(0.00322626),
  DSP16_Q(-0.00449307),
  DSP16_Q(0.00651313),
  DSP16_Q(-0.01041366),
  DSP16_Q(0.02209573),
  DSP16_Q(0.40650868),
  DSP16_Q(-0.01986434),
  DSP16_Q(0.00983980),
  DSP16_Q(-0.00624662),
  DSP16_Q(0.00433538),
  DSP16_Q(-0.00312027),
  DSP16_Q(0.00226925),
  DSP16_Q(-0.00164001),
  DSP16_Q(0.00116170),
  DSP16_Q(-0.00079498),
  DSP16_Q(0.00051592),
  DSP16_Q(-0.00030869),
  DSP16_Q(0.00016183),
  DSP16_Q(-0.00006638),
  DSP16_Q(0.00001472),
  DSP16_Q(0.00000005),
  DSP16_Q(-0.00002001),
  DSP16_Q(0.00008091),
  DSP16_Q(-0.00019022),
  DSP16_Q(0.00035633),
  DSP16_Q(-0.00058921),
  DSP16_Q(0.00090168),
  DSP16_Q(-0.00131166),
  DSP16_Q(0.00184646),
  DSP16_Q(-0.00255142),
  DSP16_Q(0.00350907),
  DSP16_Q(-0.00488745),
  DSP16_Q(0.00708677),
  DSP16_Q(-0.01133858),
  DSP16_Q(0.02411502),
  DSP16_Q(0.40617740),
  DSP16_Q(-0.02147954),
  DSP16_Q(0.01066106),
  DSP16_Q(-0.00677214),
  DSP16_Q(0.00470129),
  DSP16_Q(-0.00338395),
  DSP16_Q(0.00246102),
  DSP16_Q(-0.00177850),
  DSP16_Q(0.00125965),
  DSP16_Q(-0.00086184),
  DSP16_Q(0.00055915),
  DSP16_Q(-0.00033440),
  DSP16_Q(0.00017518),
  DSP16_Q(-0.00007174),
  DSP16_Q(0.00001583),
  DSP16_Q(0.00000006),
  DSP16_Q(-0.00002178),
  DSP16_Q(0.00008772),
  DSP16_Q(-0.00020593),
  DSP16_Q(0.00038547),
  DSP16_Q(-0.00063711),
  DSP16_Q(0.00097471),
  DSP16_Q(-0.00141763),
  DSP16_Q(0.00199541),
  DSP16_Q(-0.00275709),
  DSP16_Q(0.00379199),
  DSP16_Q(-0.00528207),
  DSP16_Q(0.00766108),
  DSP16_Q(-0.01226588),
  DSP16_Q(0.02614923),
  DSP16_Q(0.40581869),
  DSP16_Q(-0.02307628),
  DSP16_Q(0.01147620),
  DSP16_Q(-0.00729440),
  DSP16_Q(0.00506511),
  DSP16_Q(-0.00364617),
  DSP16_Q(0.00265174),
  DSP16_Q(-0.00191621),
  DSP16_Q(0.00135701),
  DSP16_Q(-0.00092827),
  DSP16_Q(0.00060207),
  DSP16_Q(-0.00035990),
  DSP16_Q(0.00018839),
  DSP16_Q(-0.00007703),
  DSP16_Q(0.00001691),
  DSP16_Q(0.00000008),
  DSP16_Q(-0.00002358),
  DSP16_Q(0.00009457),
  DSP16_Q(-0.00022169),
  DSP16_Q(0.00041467),
  DSP16_Q(-0.00068507),
  DSP16_Q(0.00104779),
  DSP16_Q(-0.00152365),
  DSP16_Q(0.00214440),
  DSP16_Q(-0.00296279),
  DSP16_Q(0.00407495),
  DSP16_Q(-0.00567684),
  DSP16_Q(0.00823595),
  DSP16_Q(-0.01319537),
  DSP16_Q(0.02819818),
  DSP16_Q(0.40543261),
  DSP16_Q(-0.02465445),
  DSP16_Q(0.01228511),
  DSP16_Q(-0.00781331),
  DSP16_Q(0.00542678),
  DSP16_Q(-0.00390689),
  DSP16_Q(0.00284136),
  DSP16_Q(-0.00205311),
  DSP16_Q(0.00145379),
  DSP16_Q(-0.00099428),
  DSP16_Q(0.00064469),
  DSP16_Q(-0.00038520),
  DSP16_Q(0.00020148),
  DSP16_Q(-0.00008224),
  DSP16_Q(0.00001797),
  DSP16_Q(0.00000010),
  DSP16_Q(-0.00002540),
  DSP16_Q(0.00010146),
  DSP16_Q(-0.00023750),
  DSP16_Q(0.00044392),
  DSP16_Q(-0.00073309),
  DSP16_Q(0.00112093),
  DSP16_Q(-0.00162970),
  DSP16_Q(0.00229339),
  DSP16_Q(-0.00316847),
  DSP16_Q(0.00435791),
  DSP16_Q(-0.00607170),
  DSP16_Q(0.00881126),
  DSP16_Q(-0.01412689),
  DSP16_Q(0.03026169),
  DSP16_Q(0.40501920),
  DSP16_Q(-0.02621395),
  DSP16_Q(0.01308765),
  DSP16_Q(-0.00832877),
  DSP16_Q(0.00578624),
  DSP16_Q(-0.00416605),
  DSP16_Q(0.00302986),
  DSP16_Q(-0.00218918),
  DSP16_Q(0.00154995),
  DSP16_Q(-0.00105984),
  DSP16_Q(0.00068700),
  DSP16_Q(-0.00041029),
  DSP16_Q(0.00021443),
  DSP16_Q(-0.00008739),
  DSP16_Q(0.00001900),
  DSP16_Q(0.00000012),
  DSP16_Q(-0.00002725),
  DSP16_Q(0.00010839),
  DSP16_Q(-0.00025337),
  DSP16_Q(0.00047324),
  DSP16_Q(-0.00078116),
  DSP16_Q(0.00119409),
  DSP16_Q(-0.00173576),
  DSP16_Q(0.00244236),
  DSP16_Q(-0.00337410),
  DSP16_Q(0.00464081),
  DSP16_Q(-0.00646655),
  DSP16_Q(0.00938690),
  DSP16_Q(-0.01506027),
  DSP16_Q(0.03233958),
  DSP16_Q(0.40457851),
  DSP16_Q(-0.02775467),
  DSP16_Q(0.01388370),
  DSP16_Q(-0.00884070),
  DSP16_Q(0.00614340),
  DSP16_Q(-0.00442362),
  DSP16_Q(0.00321720),
  DSP16_Q(-0.00232440),
  DSP16_Q(0.00164548),
  DSP16_Q(-0.00112494),
  DSP16_Q(0.00072898),
  DSP16_Q(-0.00043516),
  DSP16_Q(0.00022725),
  DSP16_Q(-0.00009247),
  DSP16_Q(0.00002000),
  DSP16_Q(0.00000014),
  DSP16_Q(-0.00002912),
  DSP16_Q(0.00011536),
  DSP16_Q(-0.00026928),
  DSP16_Q(0.00050259),
  DSP16_Q(-0.00082927),
  DSP16_Q(0.00126728),
  DSP16_Q(-0.00184180),
  DSP16_Q(0.00259127),
  DSP16_Q(-0.00357964),
  DSP16_Q(0.00492358),
  DSP16_Q(-0.00686132),
  DSP16_Q(0.00996275),
  DSP16_Q(-0.01599535),
  DSP16_Q(0.03443167),
  DSP16_Q(0.40411060),
  DSP16_Q(-0.02927653),
  DSP16_Q(0.01467313),
  DSP16_Q(-0.00934900),
  DSP16_Q(0.00649822),
  DSP16_Q(-0.00467955),
  DSP16_Q(0.00340334),
  DSP16_Q(-0.00245873),
  DSP16_Q(0.00174036),
  DSP16_Q(-0.00118957),
  DSP16_Q(0.00077064),
  DSP16_Q(-0.00045982),
  DSP16_Q(0.00023993),
  DSP16_Q(-0.00009747),
  DSP16_Q(0.00002097),
  DSP16_Q(0.00000016),
  DSP16_Q(-0.00003102),
  DSP16_Q(0.00012237),
  DSP16_Q(-0.00028524),
  DSP16_Q(0.00053199),
  DSP16_Q(-0.00087740),
  DSP16_Q(0.00134046),
  DSP16_Q(-0.00194780),
  DSP16_Q(0.00274010),
  DSP16_Q(-0.00378503),
  DSP16_Q(0.00520617),
  DSP16_Q(-0.00725593),
  DSP16_Q(0.01053870),
  DSP16_Q(-0.01693193),
  DSP16_Q(0.03653777),
  DSP16_Q(0.40361551),
  DSP16_Q(-0.03077942),
  DSP16_Q(0.01545583),
  DSP16_Q(-0.00985359),
  DSP16_Q(0.00685063),
  DSP16_Q(-0.00493378),
  DSP16_Q(0.00358825),
  DSP16_Q(-0.00259216),
  DSP16_Q(0.00183457),
  DSP16_Q(-0.00125373),
  DSP16_Q(0.00081196),
  DSP16_Q(-0.00048425),
  DSP16_Q(0.00025248),
  DSP16_Q(-0.00010240),
  DSP16_Q(0.00002192),
  DSP16_Q(0.00000019),
  DSP16_Q(-0.00003293),
  DSP16_Q(0.00012941),
  DSP16_Q(-0.00030124),
  DSP16_Q(0.00056143),
  DSP16_Q(-0.00092555),
  DSP16_Q(0.00141362),
  DSP16_Q(-0.00205374),
  DSP16_Q(0.00288881),
  DSP16_Q(-0.00399025),
  DSP16_Q(0.00548852),
  DSP16_Q(-0.00765030),
  DSP16_Q(0.01111465),
  DSP16_Q(-0.01786987),
  DSP16_Q(0.03865768),
  DSP16_Q(0.40309332),
  DSP16_Q(-0.03226325),
  DSP16_Q(0.01623167),
  DSP16_Q(-0.01035438),
  DSP16_Q(0.00720056),
  DSP16_Q(-0.00518626),
  DSP16_Q(0.00377189),
  DSP16_Q(-0.00272465),
  DSP16_Q(0.00192810),
  DSP16_Q(-0.00131739),
  DSP16_Q(0.00085293),
  DSP16_Q(-0.00050845),
  DSP16_Q(0.00026489),
  DSP16_Q(-0.00010726),
  DSP16_Q(0.00002284),
  DSP16_Q(0.00000023),
  DSP16_Q(-0.00003488),
  DSP16_Q(0.00013650),
  DSP16_Q(-0.00031727),
  DSP16_Q(0.00059089),
  DSP16_Q(-0.00097370),
  DSP16_Q(0.00148676),
  DSP16_Q(-0.00215961),
  DSP16_Q(0.00303737),
  DSP16_Q(-0.00419524),
  DSP16_Q(0.00577057),
  DSP16_Q(-0.00804434),
  DSP16_Q(0.01169046),
  DSP16_Q(-0.01880896),
  DSP16_Q(0.04079123),
  DSP16_Q(0.40254408),
  DSP16_Q(-0.03372795),
  DSP16_Q(0.01700054),
  DSP16_Q(-0.01085129),
  DSP16_Q(0.00754794),
  DSP16_Q(-0.00543697),
  DSP16_Q(0.00395423),
  DSP16_Q(-0.00285619),
  DSP16_Q(0.00202094),
  DSP16_Q(-0.00138054),
  DSP16_Q(0.00089355),
  DSP16_Q(-0.00053241),
  DSP16_Q(0.00027715),
  DSP16_Q(-0.00011204),
  DSP16_Q(0.00002373),
  DSP16_Q(0.00000026),
  DSP16_Q(-0.00003684),
  DSP16_Q(0.00014361),
  DSP16_Q(-0.00033334),
  DSP16_Q(0.00062037),
  DSP16_Q(-0.00102185),
  DSP16_Q(0.00155985),
  DSP16_Q(-0.00226536),
  DSP16_Q(0.00318576),
  DSP16_Q(-0.00439996),
  DSP16_Q(0.00605227),
  DSP16_Q(-0.00843799),
  DSP16_Q(0.01226603),
  DSP16_Q(-0.01974905),
  DSP16_Q(0.04293822),
  DSP16_Q(0.40196787),
  DSP16_Q(-0.03517341),
  DSP16_Q(0.01776231),
  DSP16_Q(-0.01134422),
  DSP16_Q(0.00789273),
  DSP16_Q(-0.00568584),
  DSP16_Q(0.00413524),
  DSP16_Q(-0.00298674),
  DSP16_Q(0.00211305),
  DSP16_Q(-0.00144318),
  DSP16_Q(0.00093382),
  DSP16_Q(-0.00055615),
  DSP16_Q(0.00028928),
  DSP16_Q(-0.00011675),
  DSP16_Q(0.00002460),
  DSP16_Q(0.00000030),
  DSP16_Q(-0.00003883),
  DSP16_Q(0.00015076),
  DSP16_Q(-0.00034944),
  DSP16_Q(0.00064988),
  DSP16_Q(-0.00106999),
  DSP16_Q(0.00163288),
  DSP16_Q(-0.00237100),
  DSP16_Q(0.00333393),
  DSP16_Q(-0.00460438),
  DSP16_Q(0.00633356),
  DSP16_Q(-0.00883115),
  DSP16_Q(0.01284123),
  DSP16_Q(-0.02068996),
  DSP16_Q(0.04509845),
  DSP16_Q(0.40136476),
  DSP16_Q(-0.03659956),
  DSP16_Q(0.01851688),
  DSP16_Q(-0.01183310),
  DSP16_Q(0.00823485),
  DSP16_Q(-0.00593283),
  DSP16_Q(0.00431489),
  DSP16_Q(-0.00311630),
  DSP16_Q(0.00220443),
  DSP16_Q(-0.00150530),
  DSP16_Q(0.00097372),
  DSP16_Q(-0.00057964),
  DSP16_Q(0.00030125),
  DSP16_Q(-0.00012138),
  DSP16_Q(0.00002544),
  DSP16_Q(0.00000034),
  DSP16_Q(-0.00004083),
  DSP16_Q(0.00015793),
  DSP16_Q(-0.00036557),
  DSP16_Q(0.00067939),
  DSP16_Q(-0.00111810),
  DSP16_Q(0.00170584),
  DSP16_Q(-0.00247648),
  DSP16_Q(0.00348186),
  DSP16_Q(-0.00480844),
  DSP16_Q(0.00661437),
  DSP16_Q(-0.00922374),
  DSP16_Q(0.01341596),
  DSP16_Q(-0.02163150),
  DSP16_Q(0.04727172),
  DSP16_Q(0.40073481),
  DSP16_Q(-0.03800632),
  DSP16_Q(0.01926412),
  DSP16_Q(-0.01231783),
  DSP16_Q(0.00857424),
  DSP16_Q(-0.00617790),
  DSP16_Q(0.00449313),
  DSP16_Q(-0.00324482),
  DSP16_Q(0.00229506),
  DSP16_Q(-0.00156687),
  DSP16_Q(0.00101325),
  DSP16_Q(-0.00060289),
  DSP16_Q(0.00031309),
  DSP16_Q(-0.00012594),
  DSP16_Q(0.00002626),
  DSP16_Q(0.00000039),
  DSP16_Q(-0.00004286),
  DSP16_Q(0.00016514),
  DSP16_Q(-0.00038172),
  DSP16_Q(0.00070890),
  DSP16_Q(-0.00116618),
  DSP16_Q(0.00177870),
  DSP16_Q(-0.00258179),
  DSP16_Q(0.00362953),
  DSP16_Q(-0.00501211),
  DSP16_Q(0.00689465),
  DSP16_Q(-0.00961569),
  DSP16_Q(0.01399009),
  DSP16_Q(-0.02257349),
  DSP16_Q(0.04945784),
  DSP16_Q(0.40007811),
  DSP16_Q(-0.03939361),
  DSP16_Q(0.02000394),
  DSP16_Q(-0.01279834),
  DSP16_Q(0.00891085),
  DSP16_Q(-0.00642100),
  DSP16_Q(0.00466994),
  DSP16_Q(-0.00337230),
  DSP16_Q(0.00238492),
  DSP16_Q(-0.00162790),
  DSP16_Q(0.00105240),
  DSP16_Q(-0.00062588),
  DSP16_Q(0.00032477),
  DSP16_Q(-0.00013042),
  DSP16_Q(0.00002705),
  DSP16_Q(0.00000043),
  DSP16_Q(-0.00004491),
  DSP16_Q(0.00017238),
  DSP16_Q(-0.00039789),
  DSP16_Q(0.00073841),
  DSP16_Q(-0.00121421),
  DSP16_Q(0.00185145),
  DSP16_Q(-0.00268691),
  DSP16_Q(0.00377689),
  DSP16_Q(-0.00521534),
  DSP16_Q(0.00717434),
  DSP16_Q(-0.01000691),
  DSP16_Q(0.01456352),
  DSP16_Q(-0.02351577),
  DSP16_Q(0.05165660),
  DSP16_Q(0.39939474),
  DSP16_Q(-0.04076136),
  DSP16_Q(0.02073621),
  DSP16_Q(-0.01327455),
  DSP16_Q(0.00924461),
  DSP16_Q(-0.00666209),
  DSP16_Q(0.00484529),
  DSP16_Q(-0.00349870),
  DSP16_Q(0.00247401),
  DSP16_Q(-0.00168837),
  DSP16_Q(0.00109116),
  DSP16_Q(-0.00064863),
  DSP16_Q(0.00033630),
  DSP16_Q(-0.00013483),
  DSP16_Q(0.00002781),
  DSP16_Q(0.00000049),
  DSP16_Q(-0.00004699),
  DSP16_Q(0.00017964),
  DSP16_Q(-0.00041408),
  DSP16_Q(0.00076791),
  DSP16_Q(-0.00126219),
  DSP16_Q(0.00192409),
  DSP16_Q(-0.00279182),
  DSP16_Q(0.00392391),
  DSP16_Q(-0.00541809),
  DSP16_Q(0.00745339),
  DSP16_Q(-0.01039732),
  DSP16_Q(0.01513611),
  DSP16_Q(-0.02445814),
  DSP16_Q(0.05386779),
  DSP16_Q(0.39868478),
  DSP16_Q(-0.04210950),
  DSP16_Q(0.02146084),
  DSP16_Q(-0.01374637),
  DSP16_Q(0.00957547),
  DSP16_Q(-0.00690113),
  DSP16_Q(0.00501915),
  DSP16_Q(-0.00362400),
  DSP16_Q(0.00256229),
  DSP16_Q(-0.00174827),
  DSP16_Q(0.00112953),
  DSP16_Q(-0.00067113),
  DSP16_Q(0.00034768),
  DSP16_Q(-0.00013916),
  DSP16_Q(0.00002854),
  DSP16_Q(0.00000055),
  DSP16_Q(-0.00004908),
  DSP16_Q(0.00018692),
  DSP16_Q(-0.00043028),
  DSP16_Q(0.00079740),
  DSP16_Q(-0.00131010),
  DSP16_Q(0.00199658),
  DSP16_Q(-0.00289648),
  DSP16_Q(0.00407057),
  DSP16_Q(-0.00562032),
  DSP16_Q(0.00773173),
  DSP16_Q(-0.01078685),
  DSP16_Q(0.01570776),
  DSP16_Q(-0.02540043),
  DSP16_Q(0.05609122),
  DSP16_Q(0.39794832),
  DSP16_Q(-0.04343796),
  DSP16_Q(0.02217771),
  DSP16_Q(-0.01421374),
  DSP16_Q(0.00990336),
  DSP16_Q(-0.00713807),
  DSP16_Q(0.00519147),
  DSP16_Q(-0.00374819),
  DSP16_Q(0.00264976),
  DSP16_Q(-0.00180759),
  DSP16_Q(0.00116750),
  DSP16_Q(-0.00069336),
  DSP16_Q(0.00035891),
  DSP16_Q(-0.00014341),
  DSP16_Q(0.00002925),
  DSP16_Q(0.00000061),
  DSP16_Q(-0.00005119),
  DSP16_Q(0.00019423),
  DSP16_Q(-0.00044649),
  DSP16_Q(0.00082686),
  DSP16_Q(-0.00135794),
  DSP16_Q(0.00206892),
  DSP16_Q(-0.00300089),
  DSP16_Q(0.00421683),
  DSP16_Q(-0.00582198),
  DSP16_Q(0.00800930),
  DSP16_Q(-0.01117540),
  DSP16_Q(0.01627834),
  DSP16_Q(-0.02634245),
  DSP16_Q(0.05832667),
  DSP16_Q(0.39718545),
  DSP16_Q(-0.04474667),
  DSP16_Q(0.02288673),
  DSP16_Q(-0.01467656),
  DSP16_Q(0.01022824),
  DSP16_Q(-0.00737287),
  DSP16_Q(0.00536225),
  DSP16_Q(-0.00387123),
  DSP16_Q(0.00273639),
  DSP16_Q(-0.00186632),
  DSP16_Q(0.00120507),
  DSP16_Q(-0.00071533),
  DSP16_Q(0.00036998),
  DSP16_Q(-0.00014759),
  DSP16_Q(0.00002994),
  DSP16_Q(0.00000067),
  DSP16_Q(-0.00005332),
  DSP16_Q(0.00020156),
  DSP16_Q(-0.00046271),
  DSP16_Q(0.00085630),
  DSP16_Q(-0.00140569),
  DSP16_Q(0.00214109),
  DSP16_Q(-0.00310502),
  DSP16_Q(0.00436267),
  DSP16_Q(-0.00602303),
  DSP16_Q(0.00828605),
  DSP16_Q(-0.01156289),
  DSP16_Q(0.01684775),
  DSP16_Q(-0.02728403),
  DSP16_Q(0.06057392),
  DSP16_Q(0.39639626),
  DSP16_Q(-0.04603557),
  DSP16_Q(0.02358779),
  DSP16_Q(-0.01513476),
  DSP16_Q(0.01055004),
  DSP16_Q(-0.00760550),
  DSP16_Q(0.00553143),
  DSP16_Q(-0.00399311),
  DSP16_Q(0.00282219),
  DSP16_Q(-0.00192445),
  DSP16_Q(0.00124223),
  DSP16_Q(-0.00073704),
  DSP16_Q(0.00038089),
  DSP16_Q(-0.00015168),
  DSP16_Q(0.00003060),
  DSP16_Q(0.00000074),
  DSP16_Q(-0.00005548),
  DSP16_Q(0.00020891),
  DSP16_Q(-0.00047893),
  DSP16_Q(0.00088570),
  DSP16_Q(-0.00145334),
  DSP16_Q(0.00221307),
  DSP16_Q(-0.00320884),
  DSP16_Q(0.00450804),
  DSP16_Q(-0.00622343),
  DSP16_Q(0.00856191),
  DSP16_Q(-0.01194925),
  DSP16_Q(0.01741585),
  DSP16_Q(-0.02822496),
  DSP16_Q(0.06283278),
  DSP16_Q(0.39558085),
  DSP16_Q(-0.04730461),
  DSP16_Q(0.02428079),
  DSP16_Q(-0.01558827),
  DSP16_Q(0.01086871),
  DSP16_Q(-0.00783590),
  DSP16_Q(0.00569900),
  DSP16_Q(-0.00411381),
  DSP16_Q(0.00290712),
  DSP16_Q(-0.00198197),
  DSP16_Q(0.00127897),
  DSP16_Q(-0.00075847),
  DSP16_Q(0.00039165),
  DSP16_Q(-0.00015570),
  DSP16_Q(0.00003123),
  DSP16_Q(0.00000082),
  DSP16_Q(-0.00005765),
  DSP16_Q(0.00021628),
  DSP16_Q(-0.00049515),
  DSP16_Q(0.00091506),
  DSP16_Q(-0.00150089),
  DSP16_Q(0.00228486),
  DSP16_Q(-0.00331233),
  DSP16_Q(0.00465293),
  DSP16_Q(-0.00642313),
  DSP16_Q(0.00883683),
  DSP16_Q(-0.01233439),
  DSP16_Q(0.01798253),
  DSP16_Q(-0.02916508),
  DSP16_Q(0.06510301),
  DSP16_Q(0.39473931),
  DSP16_Q(-0.04855373),
  DSP16_Q(0.02496564),
  DSP16_Q(-0.01603702),
  DSP16_Q(0.01118419),
  DSP16_Q(-0.00806404),
  DSP16_Q(0.00586491),
  DSP16_Q(-0.00423330),
  DSP16_Q(0.00299118),
  DSP16_Q(-0.00203887),
  DSP16_Q(0.00131529),
  DSP16_Q(-0.00077964),
  DSP16_Q(0.00040225),
  DSP16_Q(-0.00015964),
  DSP16_Q(0.00003184),
  DSP16_Q(0.00000090),
  DSP16_Q(-0.00005984),
  DSP16_Q(0.00022366),
  DSP16_Q(-0.00051137),
  DSP16_Q(0.00094437),
  DSP16_Q(-0.00154832),
  DSP16_Q(0.00235643),
  DSP16_Q(-0.00341547),
  DSP16_Q(0.00479729),
  DSP16_Q(-0.00662210),
  DSP16_Q(0.00911075),
  DSP16_Q(-0.01271824),
  DSP16_Q(0.01854767),
  DSP16_Q(-0.03010419),
  DSP16_Q(0.06738441),
  DSP16_Q(0.39387176),
  DSP16_Q(-0.04978287),
  DSP16_Q(0.02564224),
  DSP16_Q(-0.01648092),
  DSP16_Q(0.01149644),
  DSP16_Q(-0.00828987),
  DSP16_Q(0.00602916),
  DSP16_Q(-0.00435156),
  DSP16_Q(0.00307435),
  DSP16_Q(-0.00209514),
  DSP16_Q(0.00135117),
  DSP16_Q(-0.00080053),
  DSP16_Q(0.00041269),
  DSP16_Q(-0.00016350),
  DSP16_Q(0.00003242),
  DSP16_Q(0.00000098),
  DSP16_Q(-0.00006204),
  DSP16_Q(0.00023106),
  DSP16_Q(-0.00052758),
  DSP16_Q(0.00097363),
  DSP16_Q(-0.00159562),
  DSP16_Q(0.00242776),
  DSP16_Q(-0.00351824),
  DSP16_Q(0.00494110),
  DSP16_Q(-0.00682029),
  DSP16_Q(0.00938362),
  DSP16_Q(-0.01310070),
  DSP16_Q(0.01911116),
  DSP16_Q(-0.03104211),
  DSP16_Q(0.06967675),
  DSP16_Q(0.39297830),
  DSP16_Q(-0.05099199),
  DSP16_Q(0.02631050),
  DSP16_Q(-0.01691992),
  DSP16_Q(0.01180539),
  DSP16_Q(-0.00851337),
  DSP16_Q(0.00619170),
  DSP16_Q(-0.00446858),
  DSP16_Q(0.00315662),
  DSP16_Q(-0.00215077),
  DSP16_Q(0.00138663),
  DSP16_Q(-0.00082114),
  DSP16_Q(0.00042296),
  DSP16_Q(-0.00016729),
  DSP16_Q(0.00003298),
  DSP16_Q(0.00000107),
  DSP16_Q(-0.00006427),
  DSP16_Q(0.00023847),
  DSP16_Q(-0.00054377),
  DSP16_Q(0.00100282),
  DSP16_Q(-0.00164278),
  DSP16_Q(0.00249885),
  DSP16_Q(-0.00362062),
  DSP16_Q(0.00508433),
  DSP16_Q(-0.00701765),
  DSP16_Q(0.00965536),
  DSP16_Q(-0.01348170),
  DSP16_Q(0.01967288),
  DSP16_Q(-0.03197864),
  DSP16_Q(0.07197981),
  DSP16_Q(0.39205904),
  DSP16_Q(-0.05218103),
  DSP16_Q(0.02697031),
  DSP16_Q(-0.01735393),
  DSP16_Q(0.01211099),
  DSP16_Q(-0.00873449),
  DSP16_Q(0.00635250),
  DSP16_Q(-0.00458433),
  DSP16_Q(0.00323797),
  DSP16_Q(-0.00220575),
  DSP16_Q(0.00142164),
  DSP16_Q(-0.00084147),
  DSP16_Q(0.00043307),
  DSP16_Q(-0.00017099),
  DSP16_Q(0.00003352),
  DSP16_Q(0.00000117),
  DSP16_Q(-0.00006651),
  DSP16_Q(0.00024589),
  DSP16_Q(-0.00055995),
  DSP16_Q(0.00103195),
  DSP16_Q(-0.00168980),
  DSP16_Q(0.00256967),
  DSP16_Q(-0.00372258),
  DSP16_Q(0.00522694),
  DSP16_Q(-0.00721415),
  DSP16_Q(0.00992593),
  DSP16_Q(-0.01386116),
  DSP16_Q(0.02023270),
  DSP16_Q(-0.03291361),
  DSP16_Q(0.07429337),
  DSP16_Q(0.39111409),
  DSP16_Q(-0.05334996),
  DSP16_Q(0.02762161),
  DSP16_Q(-0.01778289),
  DSP16_Q(0.01241319),
  DSP16_Q(-0.00895319),
  DSP16_Q(0.00651154),
  DSP16_Q(-0.00469879),
  DSP16_Q(0.00331838),
  DSP16_Q(-0.00226008),
  DSP16_Q(0.00145620),
  DSP16_Q(-0.00086151),
  DSP16_Q(0.00044302),
  DSP16_Q(-0.00017461),
  DSP16_Q(0.00003403),
  DSP16_Q(0.00000126),
  DSP16_Q(-0.00006877),
  DSP16_Q(0.00025333),
  DSP16_Q(-0.00057611),
  DSP16_Q(0.00106100),
  DSP16_Q(-0.00173665),
  DSP16_Q(0.00264021),
  DSP16_Q(-0.00382410),
  DSP16_Q(0.00536890),
  DSP16_Q(-0.00740974),
  DSP16_Q(0.01019526),
  DSP16_Q(-0.01423899),
  DSP16_Q(0.02079050),
  DSP16_Q(-0.03384683),
  DSP16_Q(0.07661721),
  DSP16_Q(0.39014357),
  DSP16_Q(-0.05449872),
  DSP16_Q(0.02826428),
  DSP16_Q(-0.01820673),
  DSP16_Q(0.01271195),
  DSP16_Q(-0.00916943),
  DSP16_Q(0.00666880),
  DSP16_Q(-0.00481194),
  DSP16_Q(0.00339786),
  DSP16_Q(-0.00231374),
  DSP16_Q(0.00149032),
  DSP16_Q(-0.00088127),
  DSP16_Q(0.00045280),
  DSP16_Q(-0.00017816),
  DSP16_Q(0.00003451),
  DSP16_Q(0.00000137),
  DSP16_Q(-0.00007105),
  DSP16_Q(0.00026077),
  DSP16_Q(-0.00059225),
  DSP16_Q(0.00108997),
  DSP16_Q(-0.00178334),
  DSP16_Q(0.00271046),
  DSP16_Q(-0.00392516),
  DSP16_Q(0.00551019),
  DSP16_Q(-0.00760437),
  DSP16_Q(0.01046330),
  DSP16_Q(-0.01461511),
  DSP16_Q(0.02134618),
  DSP16_Q(-0.03477810),
  DSP16_Q(0.07895109),
  DSP16_Q(0.38914759),
  DSP16_Q(-0.05562730),
  DSP16_Q(0.02889826),
  DSP16_Q(-0.01862539),
  DSP16_Q(0.01300721),
  DSP16_Q(-0.00938318),
  DSP16_Q(0.00682424),
  DSP16_Q(-0.00492377),
  DSP16_Q(0.00347637),
  DSP16_Q(-0.00236672),
  DSP16_Q(0.00152398),
  DSP16_Q(-0.00090074),
  DSP16_Q(0.00046242),
  DSP16_Q(-0.00018162),
  DSP16_Q(0.00003497),
  DSP16_Q(0.00000148),
  DSP16_Q(-0.00007334),
  DSP16_Q(0.00026822),
  DSP16_Q(-0.00060836),
  DSP16_Q(0.00111886),
  DSP16_Q(-0.00182984),
  DSP16_Q(0.00278040),
  DSP16_Q(-0.00402573),
  DSP16_Q(0.00565077),
  DSP16_Q(-0.00779802),
  DSP16_Q(0.01072999),
  DSP16_Q(-0.01498944),
  DSP16_Q(0.02189960),
  DSP16_Q(-0.03570723),
  DSP16_Q(0.08129479),
  DSP16_Q(0.38812628),
  DSP16_Q(-0.05673564),
  DSP16_Q(0.02952346),
  DSP16_Q(-0.01903879),
  DSP16_Q(0.01329892),
  DSP16_Q(-0.00959441),
  DSP16_Q(0.00697784),
  DSP16_Q(-0.00503425),
  DSP16_Q(0.00355392),
  DSP16_Q(-0.00241902),
  DSP16_Q(0.00155717),
  DSP16_Q(-0.00091991),
  DSP16_Q(0.00047187),
  DSP16_Q(-0.00018501),
  DSP16_Q(0.00003541),
  DSP16_Q(0.00000159),
  DSP16_Q(-0.00007564),
  DSP16_Q(0.00027567),
  DSP16_Q(-0.00062444),
  DSP16_Q(0.00114765),
  DSP16_Q(-0.00187616),
  DSP16_Q(0.00285001),
  DSP16_Q(-0.00412580),
  DSP16_Q(0.00579060),
  DSP16_Q(-0.00799062),
  DSP16_Q(0.01099527),
  DSP16_Q(-0.01536191),
  DSP16_Q(0.02245065),
  DSP16_Q(-0.03663404),
  DSP16_Q(0.08364807),
  DSP16_Q(0.38707976),
  DSP16_Q(-0.05782371),
  DSP16_Q(0.03013979),
  DSP16_Q(-0.01944687),
  DSP16_Q(0.01358704),
  DSP16_Q(-0.00980307),
  DSP16_Q(0.00712957),
  DSP16_Q(-0.00514337),
  DSP16_Q(0.00363048),
  DSP16_Q(-0.00247063),
  DSP16_Q(0.00158990),
  DSP16_Q(-0.00093879),
  DSP16_Q(0.00048115),
  DSP16_Q(-0.00018831),
  DSP16_Q(0.00003583),
  DSP16_Q(0.00000172),
  DSP16_Q(-0.00007796),
  DSP16_Q(0.00028312),
  DSP16_Q(-0.00064048),
  DSP16_Q(0.00117634),
  DSP16_Q(-0.00192227),
  DSP16_Q(0.00291928),
  DSP16_Q(-0.00422533),
  DSP16_Q(0.00592967),
  DSP16_Q(-0.00818215),
  DSP16_Q(0.01125907),
  DSP16_Q(-0.01573242),
  DSP16_Q(0.02299921),
  DSP16_Q(-0.03755833),
  DSP16_Q(0.08601071),
  DSP16_Q(0.38600816),
  DSP16_Q(-0.05889150),
  DSP16_Q(0.03074718),
  DSP16_Q(-0.01984958),
  DSP16_Q(0.01387151),
  DSP16_Q(-0.01000913),
  DSP16_Q(0.00727940),
  DSP16_Q(-0.00525111),
  DSP16_Q(0.00370604),
  DSP16_Q(-0.00252154),
  DSP16_Q(0.00162216),
  DSP16_Q(-0.00095737),
  DSP16_Q(0.00049026),
  DSP16_Q(-0.00019154),
  DSP16_Q(0.00003622),
  DSP16_Q(0.00000184),
  DSP16_Q(-0.00008030),
  DSP16_Q(0.00029058),
  DSP16_Q(-0.00065649),
  DSP16_Q(0.00120492),
  DSP16_Q(-0.00196817),
  DSP16_Q(0.00298820),
  DSP16_Q(-0.00432432),
  DSP16_Q(0.00606794),
  DSP16_Q(-0.00837256),
  DSP16_Q(0.01152136),
  DSP16_Q(-0.01610090),
  DSP16_Q(0.02354517),
  DSP16_Q(-0.03847992),
  DSP16_Q(0.08838246),
  DSP16_Q(0.38491162),
  DSP16_Q(-0.05993896),
  DSP16_Q(0.03134555),
  DSP16_Q(-0.02024684),
  DSP16_Q(0.01415230),
  DSP16_Q(-0.01021255),
  DSP16_Q(0.00742732),
  DSP16_Q(-0.00535744),
  DSP16_Q(0.00378059),
  DSP16_Q(-0.00257174),
  DSP16_Q(0.00165394),
  DSP16_Q(-0.00097565),
  DSP16_Q(0.00049920),
  DSP16_Q(-0.00019468),
  DSP16_Q(0.00003659),
  DSP16_Q(0.00000198),
  DSP16_Q(-0.00008264),
  DSP16_Q(0.00029803),
  DSP16_Q(-0.00067245),
  DSP16_Q(0.00123338),
  DSP16_Q(-0.00201384),
  DSP16_Q(0.00305673),
  DSP16_Q(-0.00442273),
  DSP16_Q(0.00620537),
  DSP16_Q(-0.00856180),
  DSP16_Q(0.01178205),
  DSP16_Q(-0.01646726),
  DSP16_Q(0.02408840),
  DSP16_Q(-0.03939860),
  DSP16_Q(0.09076309),
  DSP16_Q(0.38379026),
  DSP16_Q(-0.06096608),
  DSP16_Q(0.03193481),
  DSP16_Q(-0.02063861),
  DSP16_Q(0.01442935),
  DSP16_Q(-0.01041331),
  DSP16_Q(0.00757329),
  DSP16_Q(-0.00546236),
  DSP16_Q(0.00385413),
  DSP16_Q(-0.00262122),
  DSP16_Q(0.00168524),
  DSP16_Q(-0.00099363),
  DSP16_Q(0.00050796),
  DSP16_Q(-0.00019775),
  DSP16_Q(0.00003693),
  DSP16_Q(0.00000211),
  DSP16_Q(-0.00008500),
  DSP16_Q(0.00030548),
  DSP16_Q(-0.00068837),
  DSP16_Q(0.00126173),
  DSP16_Q(-0.00205929),
  DSP16_Q(0.00312488),
  DSP16_Q(-0.00452055),
  DSP16_Q(0.00634194),
  DSP16_Q(-0.00874984),
  DSP16_Q(0.01204111),
  DSP16_Q(-0.01683143),
  DSP16_Q(0.02462878),
  DSP16_Q(-0.04031419),
  DSP16_Q(0.09315236),
  DSP16_Q(0.38264422),
  DSP16_Q(-0.06197284),
  DSP16_Q(0.03251491),
  DSP16_Q(-0.02102481),
  DSP16_Q(0.01470262),
  DSP16_Q(-0.01061137),
  DSP16_Q(0.00771730),
  DSP16_Q(-0.00556584),
  DSP16_Q(0.00392663),
  DSP16_Q(-0.00266998),
  DSP16_Q(0.00171605),
  DSP16_Q(-0.00101130),
  DSP16_Q(0.00051656),
  DSP16_Q(-0.00020073),
  DSP16_Q(0.00003726),
  DSP16_Q(0.00000226),
  DSP16_Q(-0.00008738),
  DSP16_Q(0.00031293),
  DSP16_Q(-0.00070424),
  DSP16_Q(0.00128995),
  DSP16_Q(-0.00210449),
  DSP16_Q(0.00319263),
  DSP16_Q(-0.00461775),
  DSP16_Q(0.00647762),
  DSP16_Q(-0.00893663),
  DSP16_Q(0.01229847),
  DSP16_Q(-0.01719333),
  DSP16_Q(0.02516619),
  DSP16_Q(-0.04122650),
  DSP16_Q(0.09555003),
  DSP16_Q(0.38147365),
  DSP16_Q(-0.06295921),
  DSP16_Q(0.03308576),
  DSP16_Q(-0.02140539),
  DSP16_Q(0.01497207),
  DSP16_Q(-0.01080670),
  DSP16_Q(0.00785931),
  DSP16_Q(-0.00566787),
  DSP16_Q(0.00399808),
  DSP16_Q(-0.00271801),
  DSP16_Q(0.00174638),
  DSP16_Q(-0.00102866),
  DSP16_Q(0.00052498),
  DSP16_Q(-0.00020364),
  DSP16_Q(0.00003756),
  DSP16_Q(0.00000241),
  DSP16_Q(-0.00008976),
  DSP16_Q(0.00032037),
  DSP16_Q(-0.00072006),
  DSP16_Q(0.00131803),
  DSP16_Q(-0.00214943),
  DSP16_Q(0.00325995),
  DSP16_Q(-0.00471431),
  DSP16_Q(0.00661237),
  DSP16_Q(-0.00912213),
  DSP16_Q(0.01255407),
  DSP16_Q(-0.01755287),
  DSP16_Q(0.02570052),
  DSP16_Q(-0.04213533),
  DSP16_Q(0.09795586),
  DSP16_Q(0.38027868),
  DSP16_Q(-0.06392519),
  DSP16_Q(0.03364730),
  DSP16_Q(-0.02178030),
  DSP16_Q(0.01523765),
  DSP16_Q(-0.01099926),
  DSP16_Q(0.00799931),
  DSP16_Q(-0.00576843),
  DSP16_Q(0.00406848),
  DSP16_Q(-0.00276530),
  DSP16_Q(0.00177621),
  DSP16_Q(-0.00104572),
  DSP16_Q(0.00053323),
  DSP16_Q(-0.00020646),
  DSP16_Q(0.00003784),
  DSP16_Q(0.00000256),
  DSP16_Q(-0.00009216),
  DSP16_Q(0.00032780),
  DSP16_Q(-0.00073582),
  DSP16_Q(0.00134598),
  DSP16_Q(-0.00219411),
  DSP16_Q(0.00332684),
  DSP16_Q(-0.00481021),
  DSP16_Q(0.00674617),
  DSP16_Q(-0.00930630),
  DSP16_Q(0.01280785),
  DSP16_Q(-0.01790997),
  DSP16_Q(0.02623165),
  DSP16_Q(-0.04304048),
  DSP16_Q(0.10036961),
  DSP16_Q(0.37905946),
  DSP16_Q(-0.06487076),
  DSP16_Q(0.03419946),
  DSP16_Q(-0.02214947),
  DSP16_Q(0.01549932),
  DSP16_Q(-0.01118902),
  DSP16_Q(0.00813727),
  DSP16_Q(-0.00586751),
  DSP16_Q(0.00413782),
  DSP16_Q(-0.00281184),
  DSP16_Q(0.00180554),
  DSP16_Q(-0.00106246),
  DSP16_Q(0.00054130),
  DSP16_Q(-0.00020921),
  DSP16_Q(0.00003810),
  DSP16_Q(0.00000273),
  DSP16_Q(-0.00009456),
  DSP16_Q(0.00033522),
  DSP16_Q(-0.00075152),
  DSP16_Q(0.00137377),
  DSP16_Q(-0.00223852),
  DSP16_Q(0.00339328),
  DSP16_Q(-0.00490543),
  DSP16_Q(0.00687898),
  DSP16_Q(-0.00948910),
  DSP16_Q(0.01305977),
  DSP16_Q(-0.01826456),
  DSP16_Q(0.02675946),
  DSP16_Q(-0.04394177),
  DSP16_Q(0.10279103),
  DSP16_Q(0.37781615),
  DSP16_Q(-0.06579591),
  DSP16_Q(0.03474217),
  DSP16_Q(-0.02251286),
  DSP16_Q(0.01575704),
  DSP16_Q(-0.01137596),
  DSP16_Q(0.00827317),
  DSP16_Q(-0.00596509),
  DSP16_Q(0.00420607),
  DSP16_Q(-0.00285763),
  DSP16_Q(0.00183436),
  DSP16_Q(-0.00107889),
  DSP16_Q(0.00054920),
  DSP16_Q(-0.00021188),
  DSP16_Q(0.00003833),
  DSP16_Q(0.00000289),
  DSP16_Q(-0.00009698),
  DSP16_Q(0.00034263),
  DSP16_Q(-0.00076715),
  DSP16_Q(0.00140142),
  DSP16_Q(-0.00228264),
  DSP16_Q(0.00345925),
  DSP16_Q(-0.00499994),
  DSP16_Q(0.00701078),
  DSP16_Q(-0.00967049),
  DSP16_Q(0.01330975),
  DSP16_Q(-0.01861655),
  DSP16_Q(0.02728383),
  DSP16_Q(-0.04483900),
  DSP16_Q(0.10521987),
  DSP16_Q(0.37654888),
  DSP16_Q(-0.06670064),
  DSP16_Q(0.03527538),
  DSP16_Q(-0.02287042),
  DSP16_Q(0.01601078),
  DSP16_Q(-0.01156004),
  DSP16_Q(0.00840699),
  DSP16_Q(-0.00606115),
  DSP16_Q(0.00427324),
  DSP16_Q(-0.00290266),
  DSP16_Q(0.00186268),
  DSP16_Q(-0.00109501),
  DSP16_Q(0.00055692),
  DSP16_Q(-0.00021446),
  DSP16_Q(0.00003855),
  DSP16_Q(0.00000307),
  DSP16_Q(-0.00009941),
  DSP16_Q(0.00035003),
  DSP16_Q(-0.00078271),
  DSP16_Q(0.00142890),
  DSP16_Q(-0.00232646),
  DSP16_Q(0.00352475),
  DSP16_Q(-0.00509372),
  DSP16_Q(0.00714153),
  DSP16_Q(-0.00985042),
  DSP16_Q(0.01355776),
  DSP16_Q(-0.01896587),
  DSP16_Q(0.02780465),
  DSP16_Q(-0.04573198),
  DSP16_Q(0.10765589),
  DSP16_Q(0.37525782),
  DSP16_Q(-0.06758494),
  DSP16_Q(0.03579901),
  DSP16_Q(-0.02322208),
  DSP16_Q(0.01626049),
  DSP16_Q(-0.01174123),
  DSP16_Q(0.00853871),
  DSP16_Q(-0.00615567),
  DSP16_Q(0.00433931),
  DSP16_Q(-0.00294693),
  DSP16_Q(0.00189049),
  DSP16_Q(-0.00111080),
  DSP16_Q(0.00056447),
  DSP16_Q(-0.00021697),
  DSP16_Q(0.00003874),
  DSP16_Q(0.00000325),
  DSP16_Q(-0.00010184),
  DSP16_Q(0.00035741),
  DSP16_Q(-0.00079820),
  DSP16_Q(0.00145621),
  DSP16_Q(-0.00236998),
  DSP16_Q(0.00358974),
  DSP16_Q(-0.00518675),
  DSP16_Q(0.00727121),
  DSP16_Q(-0.01002885),
  DSP16_Q(0.01380372),
  DSP16_Q(-0.01931243),
  DSP16_Q(0.02832179),
  DSP16_Q(-0.04662051),
  DSP16_Q(0.11009883),
  DSP16_Q(0.37394313),
  DSP16_Q(-0.06844882),
  DSP16_Q(0.03631301),
  DSP16_Q(-0.02356780),
  DSP16_Q(0.01650613),
  DSP16_Q(-0.01191952),
  DSP16_Q(0.00866830),
  DSP16_Q(-0.00624866),
  DSP16_Q(0.00440426),
  DSP16_Q(-0.00299042),
  DSP16_Q(0.00191779),
  DSP16_Q(-0.00112628),
  DSP16_Q(0.00057184),
  DSP16_Q(-0.00021939),
  DSP16_Q(0.00003892),
  DSP16_Q(0.00000344),
  DSP16_Q(-0.00010428),
  DSP16_Q(0.00036477),
  DSP16_Q(-0.00081361),
  DSP16_Q(0.00148335),
  DSP16_Q(-0.00241318),
  DSP16_Q(0.00365422),
  DSP16_Q(-0.00527902),
  DSP16_Q(0.00739978),
  DSP16_Q(-0.01020575),
  DSP16_Q(0.01404758),
  DSP16_Q(-0.01965616),
  DSP16_Q(0.02883514),
  DSP16_Q(-0.04750439),
  DSP16_Q(0.11254844),
  DSP16_Q(0.37260496),
  DSP16_Q(-0.06929227),
  DSP16_Q(0.03681733),
  DSP16_Q(-0.02390754),
  DSP16_Q(0.01674766),
  DSP16_Q(-0.01209485),
  DSP16_Q(0.00879575),
  DSP16_Q(-0.00634008),
  DSP16_Q(0.00446811),
  DSP16_Q(-0.00303314),
  DSP16_Q(0.00194457),
  DSP16_Q(-0.00114143),
  DSP16_Q(0.00057903),
  DSP16_Q(-0.00022174),
  DSP16_Q(0.00003907),
  DSP16_Q(0.00000363),
  DSP16_Q(-0.00010673),
  DSP16_Q(0.00037211),
  DSP16_Q(-0.00082895),
  DSP16_Q(0.00151031),
  DSP16_Q(-0.00245605),
  DSP16_Q(0.00371817),
  DSP16_Q(-0.00537049),
  DSP16_Q(0.00752722),
  DSP16_Q(-0.01038107),
  DSP16_Q(0.01428930),
  DSP16_Q(-0.01999697),
  DSP16_Q(0.02934458),
  DSP16_Q(-0.04838344),
  DSP16_Q(0.11500448),
  DSP16_Q(0.37124348),
  DSP16_Q(-0.07011531),
  DSP16_Q(0.03731190),
  DSP16_Q(-0.02424124),
  DSP16_Q(0.01698506),
  DSP16_Q(-0.01226722),
  DSP16_Q(0.00892104),
  DSP16_Q(-0.00642992),
  DSP16_Q(0.00453082),
  DSP16_Q(-0.00307506),
  DSP16_Q(0.00197082),
  DSP16_Q(-0.00115627),
  DSP16_Q(0.00058605),
  DSP16_Q(-0.00022401),
  DSP16_Q(0.00003921),
  DSP16_Q(0.00000383),
  DSP16_Q(-0.00010919),
  DSP16_Q(0.00037943),
  DSP16_Q(-0.00084420),
  DSP16_Q(0.00153708),
  DSP16_Q(-0.00249859),
  DSP16_Q(0.00378158),
  DSP16_Q(-0.00546115),
  DSP16_Q(0.00765350),
  DSP16_Q(-0.01055477),
  DSP16_Q(0.01452880),
  DSP16_Q(-0.02033480),
  DSP16_Q(0.02985000),
  DSP16_Q(-0.04925745),
  DSP16_Q(0.11746668),
  DSP16_Q(0.36985885),
  DSP16_Q(-0.07091794),
  DSP16_Q(0.03779668),
  DSP16_Q(-0.02456886),
  DSP16_Q(0.01721828),
  DSP16_Q(-0.01243660),
  DSP16_Q(0.00904414),
  DSP16_Q(-0.00651818),
  DSP16_Q(0.00459239),
  DSP16_Q(-0.00311620),
  DSP16_Q(0.00199655),
  DSP16_Q(-0.00117078),
  DSP16_Q(0.00059288),
  DSP16_Q(-0.00022620),
  DSP16_Q(0.00003932),
  DSP16_Q(0.00000404),
  DSP16_Q(-0.00011165),
  DSP16_Q(0.00038672),
  DSP16_Q(-0.00085936),
  DSP16_Q(0.00156365),
  DSP16_Q(-0.00254077),
  DSP16_Q(0.00384444),
  DSP16_Q(-0.00555097),
  DSP16_Q(0.00777857),
  DSP16_Q(-0.01072682),
  DSP16_Q(0.01476604),
  DSP16_Q(-0.02066956),
  DSP16_Q(0.03035127),
  DSP16_Q(-0.05012623),
  DSP16_Q(0.11993479),
  DSP16_Q(0.36845124),
  DSP16_Q(-0.07170016),
  DSP16_Q(0.03827161),
  DSP16_Q(-0.02489035),
  DSP16_Q(0.01744729),
  DSP16_Q(-0.01260295),
  DSP16_Q(0.00916504),
  DSP16_Q(-0.00660484),
  DSP16_Q(0.00465282),
  DSP16_Q(-0.00315655),
  DSP16_Q(0.00202176),
  DSP16_Q(-0.00118496),
  DSP16_Q(0.00059954),
  DSP16_Q(-0.00022831),
  DSP16_Q(0.00003942),
  DSP16_Q(0.00000425),
  DSP16_Q(-0.00011412),
  DSP16_Q(0.00039399),
  DSP16_Q(-0.00087442),
  DSP16_Q(0.00159003),
  DSP16_Q(-0.00258260),
  DSP16_Q(0.00390672),
  DSP16_Q(-0.00563994),
  DSP16_Q(0.00790243),
  DSP16_Q(-0.01089716),
  DSP16_Q(0.01500097),
  DSP16_Q(-0.02100118),
  DSP16_Q(0.03084829),
  DSP16_Q(-0.05098959),
  DSP16_Q(0.12240855),
  DSP16_Q(0.36702083),
  DSP16_Q(-0.07246201),
  DSP16_Q(0.03873664),
  DSP16_Q(-0.02520567),
  DSP16_Q(0.01767205),
  DSP16_Q(-0.01276625),
  DSP16_Q(0.00928371),
  DSP16_Q(-0.00668988),
  DSP16_Q(0.00471210),
  DSP16_Q(-0.00319609),
  DSP16_Q(0.00204643),
  DSP16_Q(-0.00119882),
  DSP16_Q(0.00060602),
  DSP16_Q(-0.00023034),
  DSP16_Q(0.00003949),
  DSP16_Q(0.00000447),
  DSP16_Q(-0.00011659),
  DSP16_Q(0.00040124),
  DSP16_Q(-0.00088939),
  DSP16_Q(0.00161619),
  DSP16_Q(-0.00262406),
  DSP16_Q(0.00396841),
  DSP16_Q(-0.00572803),
  DSP16_Q(0.00802504),
  DSP16_Q(-0.01106577),
  DSP16_Q(0.01523352),
  DSP16_Q(-0.02132957),
  DSP16_Q(0.03134094),
  DSP16_Q(-0.05184733),
  DSP16_Q(0.12488771),
  DSP16_Q(0.36556778),
  DSP16_Q(-0.07320348),
  DSP16_Q(0.03919174),
  DSP16_Q(-0.02551478),
  DSP16_Q(0.01789254),
  DSP16_Q(-0.01292648),
  DSP16_Q(0.00940015),
  DSP16_Q(-0.00677330),
  DSP16_Q(0.00477021),
  DSP16_Q(-0.00323482),
  DSP16_Q(0.00207057),
  DSP16_Q(-0.00121235),
  DSP16_Q(0.00061231),
  DSP16_Q(-0.00023229),
  DSP16_Q(0.00003955),
  DSP16_Q(0.00000469),
  DSP16_Q(-0.00011907),
  DSP16_Q(0.00040845),
  DSP16_Q(-0.00090427),
  DSP16_Q(0.00164215),
  DSP16_Q(-0.00266514),
  DSP16_Q(0.00402950),
  DSP16_Q(-0.00581522),
  DSP16_Q(0.00814636),
  DSP16_Q(-0.01123261),
  DSP16_Q(0.01546364),
  DSP16_Q(-0.02165467),
  DSP16_Q(0.03182910),
  DSP16_Q(-0.05269925),
  DSP16_Q(0.12737200),
  DSP16_Q(0.36409227),
  DSP16_Q(-0.07392461),
  DSP16_Q(0.03963684),
  DSP16_Q(-0.02581764),
  DSP16_Q(0.01810872),
  DSP16_Q(-0.01308362),
  DSP16_Q(0.00951434),
  DSP16_Q(-0.00685507),
  DSP16_Q(0.00482715),
  DSP16_Q(-0.00327274),
  DSP16_Q(0.00209417),
  DSP16_Q(-0.00122555),
  DSP16_Q(0.00061843),
  DSP16_Q(-0.00023417),
  DSP16_Q(0.00003959),
  DSP16_Q(0.00000492),
  DSP16_Q(-0.00012155),
  DSP16_Q(0.00041563),
  DSP16_Q(-0.00091903),
  DSP16_Q(0.00166788),
  DSP16_Q(-0.00270584),
  DSP16_Q(0.00408997),
  DSP16_Q(-0.00590150),
  DSP16_Q(0.00826637),
  DSP16_Q(-0.01139762),
  DSP16_Q(0.01569128),
  DSP16_Q(-0.02197638),
  DSP16_Q(0.03231265),
  DSP16_Q(-0.05354517),
  DSP16_Q(0.12986117),
  DSP16_Q(0.36259449),
  DSP16_Q(-0.07462542),
  DSP16_Q(0.04007192),
  DSP16_Q(-0.02611421),
  DSP16_Q(0.01832056),
  DSP16_Q(-0.01323764),
  DSP16_Q(0.00962625),
  DSP16_Q(-0.00693520),
  DSP16_Q(0.00488290),
  DSP16_Q(-0.00330985),
  DSP16_Q(0.00211724),
  DSP16_Q(-0.00123842),
  DSP16_Q(0.00062437),
  DSP16_Q(-0.00023597),
  DSP16_Q(0.00003961),
  DSP16_Q(0.00000516),
  DSP16_Q(-0.00012403),
  DSP16_Q(0.00042278),
  DSP16_Q(-0.00093369),
  DSP16_Q(0.00169338),
  DSP16_Q(-0.00274613),
  DSP16_Q(0.00414981),
  DSP16_Q(-0.00598683),
  DSP16_Q(0.00838505),
  DSP16_Q(-0.01156078),
  DSP16_Q(0.01591639),
  DSP16_Q(-0.02229465),
  DSP16_Q(0.03279149),
  DSP16_Q(-0.05438487),
  DSP16_Q(0.13235494),
  DSP16_Q(0.36107460),
  DSP16_Q(-0.07530592),
  DSP16_Q(0.04049692),
  DSP16_Q(-0.02640445),
  DSP16_Q(0.01852803),
  DSP16_Q(-0.01338852),
  DSP16_Q(0.00973587),
  DSP16_Q(-0.00701366),
  DSP16_Q(0.00493748),
  DSP16_Q(-0.00334613),
  DSP16_Q(0.00213976),
  DSP16_Q(-0.00125095),
  DSP16_Q(0.00063013),
  DSP16_Q(-0.00023769),
  DSP16_Q(0.00003962),
  DSP16_Q(0.00000541),
  DSP16_Q(-0.00012652),
  DSP16_Q(0.00042989),
  DSP16_Q(-0.00094824),
  DSP16_Q(0.00171866),
  DSP16_Q(-0.00278601),
  DSP16_Q(0.00420900),
  DSP16_Q(-0.00607120),
  DSP16_Q(0.00850236),
  DSP16_Q(-0.01172205),
  DSP16_Q(0.01613891),
  DSP16_Q(-0.02260939),
  DSP16_Q(0.03326550),
  DSP16_Q(-0.05521818),
  DSP16_Q(0.13485307),
  DSP16_Q(0.35953281),
  DSP16_Q(-0.07596616),
  DSP16_Q(0.04091182),
  DSP16_Q(-0.02668832),
  DSP16_Q(0.01873111),
  DSP16_Q(-0.01353623),
  DSP16_Q(0.00984319),
  DSP16_Q(-0.00709045),
  DSP16_Q(0.00499085),
  DSP16_Q(-0.00338159),
  DSP16_Q(0.00216174),
  DSP16_Q(-0.00126316),
  DSP16_Q(0.00063572),
  DSP16_Q(-0.00023933),
  DSP16_Q(0.00003960),
  DSP16_Q(0.00000566),
  DSP16_Q(-0.00012900),
  DSP16_Q(0.00043697),
  DSP16_Q(-0.00096267),
  DSP16_Q(0.00174369),
  DSP16_Q(-0.00282548),
  DSP16_Q(0.00426753),
  DSP16_Q(-0.00615460),
  DSP16_Q(0.00861828),
  DSP16_Q(-0.01188139),
  DSP16_Q(0.01635879),
  DSP16_Q(-0.02292053),
  DSP16_Q(0.03373456),
  DSP16_Q(-0.05604488),
  DSP16_Q(0.13735527),
  DSP16_Q(0.35796929),
  DSP16_Q(-0.07660615),
  DSP16_Q(0.04131657),
  DSP16_Q(-0.02696580),
  DSP16_Q(0.01892975),
  DSP16_Q(-0.01368077),
  DSP16_Q(0.00994818),
  DSP16_Q(-0.00716555),
  DSP16_Q(0.00504303),
  DSP16_Q(-0.00341621),
  DSP16_Q(0.00218317),
  DSP16_Q(-0.00127503),
  DSP16_Q(0.00064112),
  DSP16_Q(-0.00024089),
  DSP16_Q(0.00003957),
  DSP16_Q(0.00000592),
  DSP16_Q(-0.00013149),
  DSP16_Q(0.00044400),
  DSP16_Q(-0.00097698),
  DSP16_Q(0.00176847),
  DSP16_Q(-0.00286452),
  DSP16_Q(0.00432539),
  DSP16_Q(-0.00623699),
  DSP16_Q(0.00873277),
  DSP16_Q(-0.01203875),
  DSP16_Q(0.01657598),
  DSP16_Q(-0.02322800),
  DSP16_Q(0.03419857),
  DSP16_Q(-0.05686480),
  DSP16_Q(0.13986130),
  DSP16_Q(0.35638424),
  DSP16_Q(-0.07722594),
  DSP16_Q(0.04171114),
  DSP16_Q(-0.02723684),
  DSP16_Q(0.01912395),
  DSP16_Q(-0.01382210),
  DSP16_Q(0.01005084),
  DSP16_Q(-0.00723896),
  DSP16_Q(0.00509400),
  DSP16_Q(-0.00345000),
  DSP16_Q(0.00220405),
  DSP16_Q(-0.00128657),
  DSP16_Q(0.00064634),
  DSP16_Q(-0.00024238),
  DSP16_Q(0.00003953),
  DSP16_Q(0.00000618),
  DSP16_Q(-0.00013398),
  DSP16_Q(0.00045100),
  DSP16_Q(-0.00099117),
  DSP16_Q(0.00179300),
  DSP16_Q(-0.00290311),
  DSP16_Q(0.00438255),
  DSP16_Q(-0.00631836),
  DSP16_Q(0.00884581),
  DSP16_Q(-0.01219411),
  DSP16_Q(0.01679042),
  DSP16_Q(-0.02353171),
  DSP16_Q(0.03465740),
  DSP16_Q(-0.05767773),
  DSP16_Q(0.14237087),
  DSP16_Q(0.35477784),
  DSP16_Q(-0.07782556),
  DSP16_Q(0.04209549),
  DSP16_Q(-0.02750141),
  DSP16_Q(0.01931367),
  DSP16_Q(-0.01396020),
  DSP16_Q(0.01015116),
  DSP16_Q(-0.00731066),
  DSP16_Q(0.00514375),
  DSP16_Q(-0.00348296),
  DSP16_Q(0.00222438),
  DSP16_Q(-0.00129777),
  DSP16_Q(0.00065138),
  DSP16_Q(-0.00024380),
  DSP16_Q(0.00003946),
  DSP16_Q(0.00000645),
  DSP16_Q(-0.00013646),
  DSP16_Q(0.00045795),
  DSP16_Q(-0.00100523),
  DSP16_Q(0.00181727),
  DSP16_Q(-0.00294126),
  DSP16_Q(0.00443901),
  DSP16_Q(-0.00639869),
  DSP16_Q(0.00895737),
  DSP16_Q(-0.01234742),
  DSP16_Q(0.01700206),
  DSP16_Q(-0.02383160),
  DSP16_Q(0.03511096),
  DSP16_Q(-0.05848348),
  DSP16_Q(0.14488373),
  DSP16_Q(0.35315030),
  DSP16_Q(-0.07840505),
  DSP16_Q(0.04246960),
  DSP16_Q(-0.02775949),
  DSP16_Q(0.01949888),
  DSP16_Q(-0.01409506),
  DSP16_Q(0.01024911),
  DSP16_Q(-0.00738065),
  DSP16_Q(0.00519228),
  DSP16_Q(-0.00351507),
  DSP16_Q(0.00224416),
  DSP16_Q(-0.00130864),
  DSP16_Q(0.00065624),
  DSP16_Q(-0.00024514),
  DSP16_Q(0.00003938),
  DSP16_Q(0.00000673),
  DSP16_Q(-0.00013894),
  DSP16_Q(0.00046485),
  DSP16_Q(-0.00101916),
  DSP16_Q(0.00184128),
  DSP16_Q(-0.00297895),
  DSP16_Q(0.00449475),
  DSP16_Q(-0.00647796),
  DSP16_Q(0.00906743),
  DSP16_Q(-0.01249865),
  DSP16_Q(0.01721086),
  DSP16_Q(-0.02412760),
  DSP16_Q(0.03555913),
  DSP16_Q(-0.05928185),
  DSP16_Q(0.14739960),
  DSP16_Q(0.35150180),
  DSP16_Q(-0.07896446),
  DSP16_Q(0.04283344),
  DSP16_Q(-0.02801104),
  DSP16_Q(0.01967957),
  DSP16_Q(-0.01422667),
  DSP16_Q(0.01034468),
  DSP16_Q(-0.00744892),
  DSP16_Q(0.00523959),
  DSP16_Q(-0.00354633),
  DSP16_Q(0.00226338),
  DSP16_Q(-0.00131917),
  DSP16_Q(0.00066092),
  DSP16_Q(-0.00024640),
  DSP16_Q(0.00003929),
  DSP16_Q(0.00000701),
  DSP16_Q(-0.00014142),
  DSP16_Q(0.00047171),
  DSP16_Q(-0.00103295),
  DSP16_Q(0.00186501),
  DSP16_Q(-0.00301617),
  DSP16_Q(0.00454976),
  DSP16_Q(-0.00655615),
  DSP16_Q(0.00917596),
  DSP16_Q(-0.01264776),
  DSP16_Q(0.01741676),
  DSP16_Q(-0.02441962),
  DSP16_Q(0.03600179),
  DSP16_Q(-0.06007265),
  DSP16_Q(0.14991821),
  DSP16_Q(0.34983255),
  DSP16_Q(-0.07950382),
  DSP16_Q(0.04318699),
  DSP16_Q(-0.02825604),
  DSP16_Q(0.01985571),
  DSP16_Q(-0.01435499),
  DSP16_Q(0.01043787),
  DSP16_Q(-0.00751545),
  DSP16_Q(0.00528566),
  DSP16_Q(-0.00357675),
  DSP16_Q(0.00228205),
  DSP16_Q(-0.00132936),
  DSP16_Q(0.00066543),
  DSP16_Q(-0.00024759),
  DSP16_Q(0.00003918),
  DSP16_Q(0.00000730),
  DSP16_Q(-0.00014390),
  DSP16_Q(0.00047851),
  DSP16_Q(-0.00104660),
  DSP16_Q(0.00188846),
  DSP16_Q(-0.00305292),
  DSP16_Q(0.00460402),
  DSP16_Q(-0.00663324),
  DSP16_Q(0.00928293),
  DSP16_Q(-0.01279471),
  DSP16_Q(0.01761970),
  DSP16_Q(-0.02470761),
  DSP16_Q(0.03643885),
  DSP16_Q(-0.06085569),
  DSP16_Q(0.15243929),
  DSP16_Q(0.34814275),
  DSP16_Q(-0.08002318),
  DSP16_Q(0.04353021),
  DSP16_Q(-0.02849445),
  DSP16_Q(0.02002728),
  DSP16_Q(-0.01448003),
  DSP16_Q(0.01052865),
  DSP16_Q(-0.00758025),
  DSP16_Q(0.00533049),
  DSP16_Q(-0.00360631),
  DSP16_Q(0.00230016),
  DSP16_Q(-0.00133922),
  DSP16_Q(0.00066975),
  DSP16_Q(-0.00024870),
  DSP16_Q(0.00003905),
  DSP16_Q(0.00000760),
  DSP16_Q(-0.00014637),
  DSP16_Q(0.00048527),
  DSP16_Q(-0.00106012),
  DSP16_Q(0.00191163),
  DSP16_Q(-0.00308917),
  DSP16_Q(0.00465752),
  DSP16_Q(-0.00670921),
  DSP16_Q(0.00938831),
  DSP16_Q(-0.01293947),
  DSP16_Q(0.01781965),
  DSP16_Q(-0.02499148),
  DSP16_Q(0.03687018),
  DSP16_Q(-0.06163077),
  DSP16_Q(0.15496258),
  DSP16_Q(0.34643260),
  DSP16_Q(-0.08052260),
  DSP16_Q(0.04386309),
  DSP16_Q(-0.02872626),
  DSP16_Q(0.02019425),
  DSP16_Q(-0.01460175),
  DSP16_Q(0.01061702),
  DSP16_Q(-0.00764329),
  DSP16_Q(0.00537408),
  DSP16_Q(-0.00363502),
  DSP16_Q(0.00231771),
  DSP16_Q(-0.00134874),
  DSP16_Q(0.00067390),
  DSP16_Q(-0.00024974),
  DSP16_Q(0.00003891),
  DSP16_Q(0.00000790),
  DSP16_Q(-0.00014884),
  DSP16_Q(0.00049197),
  DSP16_Q(-0.00107348),
  DSP16_Q(0.00193450),
  DSP16_Q(-0.00312493),
  DSP16_Q(0.00471024),
  DSP16_Q(-0.00678404),
  DSP16_Q(0.00949208),
  DSP16_Q(-0.01308200),
  DSP16_Q(0.01801655),
  DSP16_Q(-0.02527118),
  DSP16_Q(0.03729568),
  DSP16_Q(-0.06239771),
  DSP16_Q(0.15748779),
  DSP16_Q(0.34470231),
  DSP16_Q(-0.08100213),
  DSP16_Q(0.04418560),
  DSP16_Q(-0.02895143),
  DSP16_Q(0.02035661),
  DSP16_Q(-0.01472014),
  DSP16_Q(0.01070297),
  DSP16_Q(-0.00770458),
  DSP16_Q(0.00541642),
  DSP16_Q(-0.00366287),
  DSP16_Q(0.00233470),
  DSP16_Q(-0.00135792),
  DSP16_Q(0.00067786),
  DSP16_Q(-0.00025071),
  DSP16_Q(0.00003876),
  DSP16_Q(0.00000821),
  DSP16_Q(-0.00015130),
  DSP16_Q(0.00049862),
  DSP16_Q(-0.00108669),
  DSP16_Q(0.00195708),
  DSP16_Q(-0.00316018),
  DSP16_Q(0.00476218),
  DSP16_Q(-0.00685771),
  DSP16_Q(0.00959422),
  DSP16_Q(-0.01322227),
  DSP16_Q(0.01821035),
  DSP16_Q(-0.02554662),
  DSP16_Q(0.03771525),
  DSP16_Q(-0.06315629),
  DSP16_Q(0.16001465),
  DSP16_Q(0.34295209),
  DSP16_Q(-0.08146182),
  DSP16_Q(0.04449773),
  DSP16_Q(-0.02916995),
  DSP16_Q(0.02051434),
  DSP16_Q(-0.01483520),
  DSP16_Q(0.01078649),
  DSP16_Q(-0.00776410),
  DSP16_Q(0.00545750),
  DSP16_Q(-0.00368985),
  DSP16_Q(0.00235113),
  DSP16_Q(-0.00136676),
  DSP16_Q(0.00068165),
  DSP16_Q(-0.00025160),
  DSP16_Q(0.00003859),
  DSP16_Q(0.00000852),
  DSP16_Q(-0.00015375),
  DSP16_Q(0.00050520),
  DSP16_Q(-0.00109975),
  DSP16_Q(0.00197935),
  DSP16_Q(-0.00319492),
  DSP16_Q(0.00481332),
  DSP16_Q(-0.00693020),
  DSP16_Q(0.00969469),
  DSP16_Q(-0.01336023),
  DSP16_Q(0.01840100),
  DSP16_Q(-0.02581774),
  DSP16_Q(0.03812877),
  DSP16_Q(-0.06390635),
  DSP16_Q(0.16254290),
  DSP16_Q(0.34118215),
  DSP16_Q(-0.08190173),
  DSP16_Q(0.04479946),
  DSP16_Q(-0.02938180),
  DSP16_Q(0.02066742),
  DSP16_Q(-0.01494690),
  DSP16_Q(0.01086756),
  DSP16_Q(-0.00782186),
  DSP16_Q(0.00549733),
  DSP16_Q(-0.00371598),
  DSP16_Q(0.00236700),
  DSP16_Q(-0.00137527),
  DSP16_Q(0.00068526),
  DSP16_Q(-0.00025242),
  DSP16_Q(0.00003841),
  DSP16_Q(0.00000884),
  DSP16_Q(-0.00015620),
  DSP16_Q(0.00051173),
  DSP16_Q(-0.00111265),
  DSP16_Q(0.00200131),
  DSP16_Q(-0.00322913),
  DSP16_Q(0.00486364),
  DSP16_Q(-0.00700150),
  DSP16_Q(0.00979347),
  DSP16_Q(-0.01349587),
  DSP16_Q(0.01858846),
  DSP16_Q(-0.02608447),
  DSP16_Q(0.03853615),
  DSP16_Q(-0.06464768),
  DSP16_Q(0.16507225),
  DSP16_Q(0.33939270),
  DSP16_Q(-0.08232193),
  DSP16_Q(0.04509078),
  DSP16_Q(-0.02958696),
  DSP16_Q(0.02081583),
  DSP16_Q(-0.01505524),
  DSP16_Q(0.01094617),
  DSP16_Q(-0.00787784),
  DSP16_Q(0.00553590),
  DSP16_Q(-0.00374124),
  DSP16_Q(0.00238230),
  DSP16_Q(-0.00138344),
  DSP16_Q(0.00068869),
  DSP16_Q(-0.00025317),
  DSP16_Q(0.00003821),
  DSP16_Q(0.00000917),
  DSP16_Q(-0.00015864),
  DSP16_Q(0.00051820),
  DSP16_Q(-0.00112539),
  DSP16_Q(0.00202296),
  DSP16_Q(-0.00326280),
  DSP16_Q(0.00491313),
  DSP16_Q(-0.00707159),
  DSP16_Q(0.00989053),
  DSP16_Q(-0.01362913),
  DSP16_Q(0.01877268),
  DSP16_Q(-0.02634674),
  DSP16_Q(0.03893727),
  DSP16_Q(-0.06538008),
  DSP16_Q(0.16760243),
  DSP16_Q(0.33758395),
  DSP16_Q(-0.08272247),
  DSP16_Q(0.04537168),
  DSP16_Q(-0.02978540),
  DSP16_Q(0.02095956),
  DSP16_Q(-0.01516019),
  DSP16_Q(0.01102233),
  DSP16_Q(-0.00793203),
  DSP16_Q(0.00557320),
  DSP16_Q(-0.00376563),
  DSP16_Q(0.00239704),
  DSP16_Q(-0.00139127),
  DSP16_Q(0.00069195),
  DSP16_Q(-0.00025385),
  DSP16_Q(0.00003800),
  DSP16_Q(0.00000950),
  DSP16_Q(-0.00016107),
  DSP16_Q(0.00052460),
  DSP16_Q(-0.00113796),
  DSP16_Q(0.00204428),
  DSP16_Q(-0.00329593),
  DSP16_Q(0.00496178),
  DSP16_Q(-0.00714045),
  DSP16_Q(0.00998586),
  DSP16_Q(-0.01375999),
  DSP16_Q(0.01895361),
  DSP16_Q(-0.02660448),
  DSP16_Q(0.03933203),
  DSP16_Q(-0.06610338),
  DSP16_Q(0.17013316),
  DSP16_Q(0.33575613),
  DSP16_Q(-0.08310343),
  DSP16_Q(0.04564214),
  DSP16_Q(-0.02997711),
  DSP16_Q(0.02109860),
  DSP16_Q(-0.01526176),
  DSP16_Q(0.01109601),
  DSP16_Q(-0.00798444),
  DSP16_Q(0.00560924),
  DSP16_Q(-0.00378915),
  DSP16_Q(0.00241122),
  DSP16_Q(-0.00139876),
  DSP16_Q(0.00069503),
  DSP16_Q(-0.00025446),
  DSP16_Q(0.00003778),
  DSP16_Q(0.00000984),
  DSP16_Q(-0.00016348),
  DSP16_Q(0.00053093),
  DSP16_Q(-0.00115037),
  DSP16_Q(0.00206528),
  DSP16_Q(-0.00332851),
  DSP16_Q(0.00500957),
  DSP16_Q(-0.00720805),
  DSP16_Q(0.01007942),
  DSP16_Q(-0.01388842),
  DSP16_Q(0.01913120),
  DSP16_Q(-0.02685763),
  DSP16_Q(0.03972032),
  DSP16_Q(-0.06681738),
  DSP16_Q(0.17266416),
  DSP16_Q(0.33390946),
  DSP16_Q(-0.08346486),
  DSP16_Q(0.04590216),
  DSP16_Q(-0.03016207),
  DSP16_Q(0.02123292),
  DSP16_Q(-0.01535992),
  DSP16_Q(0.01116721),
  DSP16_Q(-0.00803505),
  DSP16_Q(0.00564400),
  DSP16_Q(-0.00381180),
  DSP16_Q(0.00242483),
  DSP16_Q(-0.00140591),
  DSP16_Q(0.00069793),
  DSP16_Q(-0.00025500),
  DSP16_Q(0.00003755),
  DSP16_Q(0.00001018),
  DSP16_Q(-0.00016589),
  DSP16_Q(0.00053720),
  DSP16_Q(-0.00116260),
  DSP16_Q(0.00208594),
  DSP16_Q(-0.00336052),
  DSP16_Q(0.00505650),
  DSP16_Q(-0.00727438),
  DSP16_Q(0.01017119),
  DSP16_Q(-0.01401437),
  DSP16_Q(0.01930541),
  DSP16_Q(-0.02710613),
  DSP16_Q(0.04010206),
  DSP16_Q(-0.06752189),
  DSP16_Q(0.17519516),
  DSP16_Q(0.33204415),
  DSP16_Q(-0.08380685),
  DSP16_Q(0.04615174),
  DSP16_Q(-0.03034028),
  DSP16_Q(0.02136251),
  DSP16_Q(-0.01545467),
  DSP16_Q(0.01123593),
  DSP16_Q(-0.00808386),
  DSP16_Q(0.00567749),
  DSP16_Q(-0.00383358),
  DSP16_Q(0.00243787),
  DSP16_Q(-0.00141273),
  DSP16_Q(0.00070066),
  DSP16_Q(-0.00025547),
  DSP16_Q(0.00003730),
  DSP16_Q(0.00001053),
  DSP16_Q(-0.00016829),
  DSP16_Q(0.00054340),
  DSP16_Q(-0.00117465),
  DSP16_Q(0.00210626),
  DSP16_Q(-0.00339197),
  DSP16_Q(0.00510254),
  DSP16_Q(-0.00733943),
  DSP16_Q(0.01026115),
  DSP16_Q(-0.01413783),
  DSP16_Q(0.01947619),
  DSP16_Q(-0.02734990),
  DSP16_Q(0.04047712),
  DSP16_Q(-0.06821673),
  DSP16_Q(0.17772588),
  DSP16_Q(0.33016044),
  DSP16_Q(-0.08412946),
  DSP16_Q(0.04639086),
  DSP16_Q(-0.03051171),
  DSP16_Q(0.02148737),
  DSP16_Q(-0.01554600),
  DSP16_Q(0.01130216),
  DSP16_Q(-0.00813087),
  DSP16_Q(0.00570969),
  DSP16_Q(-0.00385448),
  DSP16_Q(0.00245035),
  DSP16_Q(-0.00141921),
  DSP16_Q(0.00070322),
  DSP16_Q(-0.00025587),
  DSP16_Q(0.00003705),
  DSP16_Q(0.00001089),
  DSP16_Q(-0.00017067),
  DSP16_Q(0.00054952),
  DSP16_Q(-0.00118652),
  DSP16_Q(0.00212623),
  DSP16_Q(-0.00342283),
  DSP16_Q(0.00514770),
  DSP16_Q(-0.00740318),
  DSP16_Q(0.01034928),
  DSP16_Q(-0.01425874),
  DSP16_Q(0.01964350),
  DSP16_Q(-0.02758888),
  DSP16_Q(0.04084541),
  DSP16_Q(-0.06890169),
  DSP16_Q(0.18025604),
  DSP16_Q(0.32825854),
  DSP16_Q(-0.08443278),
  DSP16_Q(0.04661953),
  DSP16_Q(-0.03067636),
  DSP16_Q(0.02160748),
  DSP16_Q(-0.01563390),
  DSP16_Q(0.01136588),
  DSP16_Q(-0.00817607),
  DSP16_Q(0.00574062),
  DSP16_Q(-0.00387451),
  DSP16_Q(0.00246226),
  DSP16_Q(-0.00142535),
  DSP16_Q(0.00070560),
  DSP16_Q(-0.00025620),
  DSP16_Q(0.00003678),
  DSP16_Q(0.00001125),
  DSP16_Q(-0.00017304),
  DSP16_Q(0.00055557),
  DSP16_Q(-0.00119821),
  DSP16_Q(0.00214585),
  DSP16_Q(-0.00345311),
  DSP16_Q(0.00519194),
  DSP16_Q(-0.00746561),
  DSP16_Q(0.01043554),
  DSP16_Q(-0.01437709),
  DSP16_Q(0.01980730),
  DSP16_Q(-0.02782301),
  DSP16_Q(0.04120684),
  DSP16_Q(-0.06957660),
  DSP16_Q(0.18278536),
  DSP16_Q(0.32633868),
  DSP16_Q(-0.08471687),
  DSP16_Q(0.04683774),
  DSP16_Q(-0.03083422),
  DSP16_Q(0.02172283),
  DSP16_Q(-0.01571836),
  DSP16_Q(0.01142710),
  DSP16_Q(-0.00821946),
  DSP16_Q(0.00577027),
  DSP16_Q(-0.00389366),
  DSP16_Q(0.00247361),
  DSP16_Q(-0.00143116),
  DSP16_Q(0.00070780),
  DSP16_Q(-0.00025646),
  DSP16_Q(0.00003650),
  DSP16_Q(0.00001162),
  DSP16_Q(-0.00017540),
  DSP16_Q(0.00056154),
  DSP16_Q(-0.00120971),
  DSP16_Q(0.00216511),
  DSP16_Q(-0.00348279),
  DSP16_Q(0.00523527),
  DSP16_Q(-0.00752670),
  DSP16_Q(0.01051992),
  DSP16_Q(-0.01449285),
  DSP16_Q(0.01996753),
  DSP16_Q(-0.02805223),
  DSP16_Q(0.04156130),
  DSP16_Q(-0.07024128),
  DSP16_Q(0.18531356),
  DSP16_Q(0.32440110),
  DSP16_Q(-0.08498183),
  DSP16_Q(0.04704550),
  DSP16_Q(-0.03098527),
  DSP16_Q(0.02183342),
  DSP16_Q(-0.01579937),
  DSP16_Q(0.01148581),
  DSP16_Q(-0.00826104),
  DSP16_Q(0.00579863),
  DSP16_Q(-0.00391194),
  DSP16_Q(0.00248439),
  DSP16_Q(-0.00143663),
  DSP16_Q(0.00070984),
  DSP16_Q(-0.00025666),
  DSP16_Q(0.00003621),
  DSP16_Q(0.00001199),
  DSP16_Q(-0.00017774),
  DSP16_Q(0.00056744),
  DSP16_Q(-0.00122102),
  DSP16_Q(0.00218401),
  DSP16_Q(-0.00351186),
  DSP16_Q(0.00527767),
  DSP16_Q(-0.00758643),
  DSP16_Q(0.01060240),
  DSP16_Q(-0.01460597),
  DSP16_Q(0.02012416),
  DSP16_Q(-0.02827646),
  DSP16_Q(0.04190869),
  DSP16_Q(-0.07089552),
  DSP16_Q(0.18784036),
  DSP16_Q(0.32244602),
  DSP16_Q(-0.08522773),
  DSP16_Q(0.04724282),
  DSP16_Q(-0.03112952),
  DSP16_Q(0.02193923),
  DSP16_Q(-0.01587693),
  DSP16_Q(0.01154200),
  DSP16_Q(-0.00830080),
  DSP16_Q(0.00582571),
  DSP16_Q(-0.00392934),
  DSP16_Q(0.00249460),
  DSP16_Q(-0.00144176),
  DSP16_Q(0.00071170),
  DSP16_Q(-0.00025678),
  DSP16_Q(0.00003591),
  DSP16_Q(0.00001236),
  DSP16_Q(-0.00018007),
  DSP16_Q(0.00057326),
  DSP16_Q(-0.00123213),
  DSP16_Q(0.00220253),
  DSP16_Q(-0.00354032),
  DSP16_Q(0.00531913),
  DSP16_Q(-0.00764479),
  DSP16_Q(0.01068294),
  DSP16_Q(-0.01471643),
  DSP16_Q(0.02027715),
  DSP16_Q(-0.02849566),
  DSP16_Q(0.04224891),
  DSP16_Q(-0.07153915),
  DSP16_Q(0.19036548),
  DSP16_Q(0.32047369),
  DSP16_Q(-0.08545467),
  DSP16_Q(0.04742969),
  DSP16_Q(-0.03126694),
  DSP16_Q(0.02204026),
  DSP16_Q(-0.01595103),
  DSP16_Q(0.01159568),
  DSP16_Q(-0.00833873),
  DSP16_Q(0.00585150),
  DSP16_Q(-0.00394586),
  DSP16_Q(0.00250425),
  DSP16_Q(-0.00144657),
  DSP16_Q(0.00071339),
  DSP16_Q(-0.00025685),
  DSP16_Q(0.00003560),
  DSP16_Q(0.00001275),
  DSP16_Q(-0.00018237),
  DSP16_Q(0.00057899),
  DSP16_Q(-0.00124304),
  DSP16_Q(0.00222068),
  DSP16_Q(-0.00356815),
  DSP16_Q(0.00535963),
  DSP16_Q(-0.00770177),
  DSP16_Q(0.01076154),
  DSP16_Q(-0.01482421),
  DSP16_Q(0.02042644),
  DSP16_Q(-0.02870976),
  DSP16_Q(0.04258188),
  DSP16_Q(-0.07217199),
  DSP16_Q(0.19288864),
  DSP16_Q(0.31848433),
  DSP16_Q(-0.08566273),
  DSP16_Q(0.04760614),
  DSP16_Q(-0.03139755),
  DSP16_Q(0.02213650),
  DSP16_Q(-0.01602167),
  DSP16_Q(0.01164683),
  DSP16_Q(-0.00837485),
  DSP16_Q(0.00587600),
  DSP16_Q(-0.00396151),
  DSP16_Q(0.00251333),
  DSP16_Q(-0.00145103),
  DSP16_Q(0.00071491),
  DSP16_Q(-0.00025685),
  DSP16_Q(0.00003529),
  DSP16_Q(0.00001313),
  DSP16_Q(-0.00018466),
  DSP16_Q(0.00058464),
  DSP16_Q(-0.00125375),
  DSP16_Q(0.00223845),
  DSP16_Q(-0.00359535),
  DSP16_Q(0.00539917),
  DSP16_Q(-0.00775734),
  DSP16_Q(0.01083816),
  DSP16_Q(-0.01492926),
  DSP16_Q(0.02057201),
  DSP16_Q(-0.02891870),
  DSP16_Q(0.04290749),
  DSP16_Q(-0.07279384),
  DSP16_Q(0.19540956),
  DSP16_Q(0.31647818),
  DSP16_Q(-0.08585200),
  DSP16_Q(0.04777215),
  DSP16_Q(-0.03152133),
  DSP16_Q(0.02222794),
  DSP16_Q(-0.01608885),
  DSP16_Q(0.01169546),
  DSP16_Q(-0.00840914),
  DSP16_Q(0.00589921),
  DSP16_Q(-0.00397627),
  DSP16_Q(0.00252184),
  DSP16_Q(-0.00145517),
  DSP16_Q(0.00071626),
  DSP16_Q(-0.00025678),
  DSP16_Q(0.00003496),
  DSP16_Q(0.00001352),
  DSP16_Q(-0.00018693),
  DSP16_Q(0.00059020),
  DSP16_Q(-0.00126426),
  DSP16_Q(0.00225584),
  DSP16_Q(-0.00362192),
  DSP16_Q(0.00543773),
  DSP16_Q(-0.00781150),
  DSP16_Q(0.01091279),
  DSP16_Q(-0.01503156),
  DSP16_Q(0.02071381),
  DSP16_Q(-0.02912242),
  DSP16_Q(0.04322565),
  DSP16_Q(-0.07340453),
  DSP16_Q(0.19792795),
  DSP16_Q(0.31445549),
  DSP16_Q(-0.08602257),
  DSP16_Q(0.04792776),
  DSP16_Q(-0.03163829),
  DSP16_Q(0.02231459),
  DSP16_Q(-0.01615255),
  DSP16_Q(0.01174155),
  DSP16_Q(-0.00844160),
  DSP16_Q(0.00592113),
  DSP16_Q(-0.00399016),
  DSP16_Q(0.00252980),
  DSP16_Q(-0.00145897),
  DSP16_Q(0.00071744),
  DSP16_Q(-0.00025665),
  DSP16_Q(0.00003462),
  DSP16_Q(0.00001392),
  DSP16_Q(-0.00018919),
  DSP16_Q(0.00059567),
  DSP16_Q(-0.00127455),
  DSP16_Q(0.00227283),
  DSP16_Q(-0.00364783),
  DSP16_Q(0.00547531),
  DSP16_Q(-0.00786421),
  DSP16_Q(0.01098540),
  DSP16_Q(-0.01513109),
  DSP16_Q(0.02085181),
  DSP16_Q(-0.02932087),
  DSP16_Q(0.04353627),
  DSP16_Q(-0.07400387),
  DSP16_Q(0.20044354),
  DSP16_Q(0.31241649),
  DSP16_Q(-0.08617455),
  DSP16_Q(0.04807297),
  DSP16_Q(-0.03174842),
  DSP16_Q(0.02239644),
  DSP16_Q(-0.01621277),
  DSP16_Q(0.01178512),
  DSP16_Q(-0.00847224),
  DSP16_Q(0.00594177),
  DSP16_Q(-0.00400318),
  DSP16_Q(0.00253719),
  DSP16_Q(-0.00146244),
  DSP16_Q(0.00071846),
  DSP16_Q(-0.00025646),
  DSP16_Q(0.00003428),
  DSP16_Q(0.00001432),
  DSP16_Q(-0.00019142),
  DSP16_Q(0.00060106),
  DSP16_Q(-0.00128463),
  DSP16_Q(0.00228942),
  DSP16_Q(-0.00367309),
  DSP16_Q(0.00551188),
  DSP16_Q(-0.00791548),
  DSP16_Q(0.01105598),
  DSP16_Q(-0.01522781),
  DSP16_Q(0.02098595),
  DSP16_Q(-0.02951398),
  DSP16_Q(0.04383926),
  DSP16_Q(-0.07459169),
  DSP16_Q(0.20295604),
  DSP16_Q(0.31036143),
  DSP16_Q(-0.08630802),
  DSP16_Q(0.04820780),
  DSP16_Q(-0.03185173),
  DSP16_Q(0.02247348),
  DSP16_Q(-0.01626952),
  DSP16_Q(0.01182615),
  DSP16_Q(-0.00850105),
  DSP16_Q(0.00596111),
  DSP16_Q(-0.00401531),
  DSP16_Q(0.00254402),
  DSP16_Q(-0.00146558),
  DSP16_Q(0.00071931),
  DSP16_Q(-0.00025621),
  DSP16_Q(0.00003393),
  DSP16_Q(0.00001472),
  DSP16_Q(-0.00019363),
  DSP16_Q(0.00060635),
  DSP16_Q(-0.00129450),
  DSP16_Q(0.00230561),
  DSP16_Q(-0.00369769),
  DSP16_Q(0.00554744),
  DSP16_Q(-0.00796529),
  DSP16_Q(0.01112450),
  DSP16_Q(-0.01532169),
  DSP16_Q(0.02111620),
  DSP16_Q(-0.02970170),
  DSP16_Q(0.04413452),
  DSP16_Q(-0.07516780),
  DSP16_Q(0.20546517),
  DSP16_Q(0.30829055),
  DSP16_Q(-0.08642310),
  DSP16_Q(0.04833226),
  DSP16_Q(-0.03194821),
  DSP16_Q(0.02254572),
  DSP16_Q(-0.01632279),
  DSP16_Q(0.01186465),
  DSP16_Q(-0.00852803),
  DSP16_Q(0.00597916),
  DSP16_Q(-0.00402657),
  DSP16_Q(0.00255028),
  DSP16_Q(-0.00146840),
  DSP16_Q(0.00071999),
  DSP16_Q(-0.00025589),
  DSP16_Q(0.00003356),
  DSP16_Q(0.00001513),
  DSP16_Q(-0.00019581),
  DSP16_Q(0.00061155),
  DSP16_Q(-0.00130415),
  DSP16_Q(0.00232139),
  DSP16_Q(-0.00372162),
  DSP16_Q(0.00558198),
  DSP16_Q(-0.00801361),
  DSP16_Q(0.01119095),
  DSP16_Q(-0.01541271),
  DSP16_Q(0.02124253),
  DSP16_Q(-0.02988398),
  DSP16_Q(0.04442196),
  DSP16_Q(-0.07573203),
  DSP16_Q(0.20797064),
  DSP16_Q(0.30620409),
  DSP16_Q(-0.08651987),
  DSP16_Q(0.04844638),
  DSP16_Q(-0.03203788),
  DSP16_Q(0.02261316),
  DSP16_Q(-0.01637258),
  DSP16_Q(0.01190061),
  DSP16_Q(-0.00855318),
  DSP16_Q(0.00599593),
  DSP16_Q(-0.00403696),
  DSP16_Q(0.00255599),
  DSP16_Q(-0.00147088),
  DSP16_Q(0.00072051),
  DSP16_Q(-0.00025551),
  DSP16_Q(0.00003320),
  DSP16_Q(0.00001555),
  DSP16_Q(-0.00019798),
  DSP16_Q(0.00061665),
  DSP16_Q(-0.00131357),
  DSP16_Q(0.00233676),
  DSP16_Q(-0.00374487),
  DSP16_Q(0.00561549),
  DSP16_Q(-0.00806044),
  DSP16_Q(0.01125529),
  DSP16_Q(-0.01550084),
  DSP16_Q(0.02136490),
  DSP16_Q(-0.03006075),
  DSP16_Q(0.04470150),
  DSP16_Q(-0.07628419),
  DSP16_Q(0.21047219),
  DSP16_Q(0.30410231),
  DSP16_Q(-0.08659846),
  DSP16_Q(0.04855017),
  DSP16_Q(-0.03212073),
  DSP16_Q(0.02267579),
  DSP16_Q(-0.01641889),
  DSP16_Q(0.01193405),
  DSP16_Q(-0.00857651),
  DSP16_Q(0.00601141),
  DSP16_Q(-0.00404647),
  DSP16_Q(0.00256114),
  DSP16_Q(-0.00147304),
  DSP16_Q(0.00072086),
  DSP16_Q(-0.00025508),
  DSP16_Q(0.00003282),
  DSP16_Q(0.00001596),
  DSP16_Q(-0.00020012),
  DSP16_Q(0.00062165),
  DSP16_Q(-0.00132276),
  DSP16_Q(0.00235170),
  DSP16_Q(-0.00376743),
  DSP16_Q(0.00564796),
  DSP16_Q(-0.00810576),
  DSP16_Q(0.01131752),
  DSP16_Q(-0.01558606),
  DSP16_Q(0.02148327),
  DSP16_Q(-0.03023198),
  DSP16_Q(0.04497305),
  DSP16_Q(-0.07682411),
  DSP16_Q(0.21296951),
  DSP16_Q(0.30198546),
  DSP16_Q(-0.08665896),
  DSP16_Q(0.04864366),
  DSP16_Q(-0.03219677),
  DSP16_Q(0.02273362),
  DSP16_Q(-0.01646172),
  DSP16_Q(0.01196494),
  DSP16_Q(-0.00859800),
  DSP16_Q(0.00602560),
  DSP16_Q(-0.00405511),
  DSP16_Q(0.00256573),
  DSP16_Q(-0.00147488),
  DSP16_Q(0.00072105),
  DSP16_Q(-0.00025458),
  DSP16_Q(0.00003244),
  DSP16_Q(0.00001638),
  DSP16_Q(-0.00020223),
  DSP16_Q(0.00062655),
  DSP16_Q(-0.00133173),
  DSP16_Q(0.00236623),
  DSP16_Q(-0.00378930),
  DSP16_Q(0.00567937),
  DSP16_Q(-0.00814956),
  DSP16_Q(0.01137762),
  DSP16_Q(-0.01566834),
  DSP16_Q(0.02159760),
  DSP16_Q(-0.03039759),
  DSP16_Q(0.04523652),
  DSP16_Q(-0.07735161),
  DSP16_Q(0.21546234),
  DSP16_Q(0.29985378),
  DSP16_Q(-0.08670148),
  DSP16_Q(0.04872688),
  DSP16_Q(-0.03226601),
  DSP16_Q(0.02278665),
  DSP16_Q(-0.01650108),
  DSP16_Q(0.01199331),
  DSP16_Q(-0.00861768),
  DSP16_Q(0.00603850),
  DSP16_Q(-0.00406288),
  DSP16_Q(0.00256976),
  DSP16_Q(-0.00147639),
  DSP16_Q(0.00072109),
  DSP16_Q(-0.00025403),
  DSP16_Q(0.00003205),
  DSP16_Q(0.00001681),
  DSP16_Q(-0.00020432),
  DSP16_Q(0.00063135),
  DSP16_Q(-0.00134046),
  DSP16_Q(0.00238032),
  DSP16_Q(-0.00381046),
  DSP16_Q(0.00570972),
  DSP16_Q(-0.00819182),
  DSP16_Q(0.01143556),
  DSP16_Q(-0.01574765),
  DSP16_Q(0.02170786),
  DSP16_Q(-0.03055755),
  DSP16_Q(0.04549183),
  DSP16_Q(-0.07786653),
  DSP16_Q(0.21795038),
  DSP16_Q(0.29770753),
  DSP16_Q(-0.08672614),
  DSP16_Q(0.04879985),
  DSP16_Q(-0.03232845),
  DSP16_Q(0.02283488),
  DSP16_Q(-0.01653696),
  DSP16_Q(0.01201914),
  DSP16_Q(-0.00863552),
  DSP16_Q(0.00605012),
  DSP16_Q(-0.00406978),
  DSP16_Q(0.00257325),
  DSP16_Q(-0.00147757),
  DSP16_Q(0.00072096),
  DSP16_Q(-0.00025341),
  DSP16_Q(0.00003166),
  DSP16_Q(0.00001723),
  DSP16_Q(-0.00020638),
  DSP16_Q(0.00063605),
  DSP16_Q(-0.00134895),
  DSP16_Q(0.00239398),
  DSP16_Q(-0.00383092),
  DSP16_Q(0.00573899),
  DSP16_Q(-0.00823252),
  DSP16_Q(0.01149133),
  DSP16_Q(-0.01582397),
  DSP16_Q(0.02181402),
  DSP16_Q(-0.03071180),
  DSP16_Q(0.04573890),
  DSP16_Q(-0.07836868),
  DSP16_Q(0.22043336),
  DSP16_Q(0.29554696),
  DSP16_Q(-0.08673305),
  DSP16_Q(0.04886261),
  DSP16_Q(-0.03238412),
  DSP16_Q(0.02287832),
  DSP16_Q(-0.01656936),
  DSP16_Q(0.01204245),
  DSP16_Q(-0.00865155),
  DSP16_Q(0.00606046),
  DSP16_Q(-0.00407582),
  DSP16_Q(0.00257617),
  DSP16_Q(-0.00147844),
  DSP16_Q(0.00072067),
  DSP16_Q(-0.00025275),
  DSP16_Q(0.00003126),
  DSP16_Q(0.00001766),
  DSP16_Q(-0.00020842),
  DSP16_Q(0.00064064),
  DSP16_Q(-0.00135721),
  DSP16_Q(0.00240720),
  DSP16_Q(-0.00385067),
  DSP16_Q(0.00576718),
  DSP16_Q(-0.00827167),
  DSP16_Q(0.01154491),
  DSP16_Q(-0.01589728),
  DSP16_Q(0.02191604),
  DSP16_Q(-0.03086029),
  DSP16_Q(0.04597764),
  DSP16_Q(-0.07885789),
  DSP16_Q(0.22291100),
  DSP16_Q(0.29337233),
  DSP16_Q(-0.08672233),
  DSP16_Q(0.04891517),
  DSP16_Q(-0.03243302),
  DSP16_Q(0.02291698),
  DSP16_Q(-0.01659830),
  DSP16_Q(0.01206323),
  DSP16_Q(-0.00866576),
  DSP16_Q(0.00606952),
  DSP16_Q(-0.00408098),
  DSP16_Q(0.00257855),
  DSP16_Q(-0.00147899),
  DSP16_Q(0.00072022),
  DSP16_Q(-0.00025202),
  DSP16_Q(0.00003085),
  DSP16_Q(0.00001810),
  DSP16_Q(-0.00021042),
  DSP16_Q(0.00064512),
  DSP16_Q(-0.00136522),
  DSP16_Q(0.00241998),
  DSP16_Q(-0.00386969),
  DSP16_Q(0.00579428),
  DSP16_Q(-0.00830924),
  DSP16_Q(0.01159629),
  DSP16_Q(-0.01596754),
  DSP16_Q(0.02201389),
  DSP16_Q(-0.03100297),
  DSP16_Q(0.04620796),
  DSP16_Q(-0.07933400),
  DSP16_Q(0.22538301),
  DSP16_Q(0.29118390),
  DSP16_Q(-0.08669409),
  DSP16_Q(0.04895758),
  DSP16_Q(-0.03247515),
  DSP16_Q(0.02295086),
  DSP16_Q(-0.01662377),
  DSP16_Q(0.01208148),
  DSP16_Q(-0.00867815),
  DSP16_Q(0.00607731),
  DSP16_Q(-0.00408529),
  DSP16_Q(0.00258038),
  DSP16_Q(-0.00147922),
  DSP16_Q(0.00071962),
  DSP16_Q(-0.00025124),
  DSP16_Q(0.00003044),
  DSP16_Q(0.00001853),
  DSP16_Q(-0.00021240),
  DSP16_Q(0.00064950),
  DSP16_Q(-0.00137299),
  DSP16_Q(0.00243231),
  DSP16_Q(-0.00388798),
  DSP16_Q(0.00582028),
  DSP16_Q(-0.00834522),
  DSP16_Q(0.01164544),
  DSP16_Q(-0.01603475),
  DSP16_Q(0.02210753),
  DSP16_Q(-0.03113979),
  DSP16_Q(0.04642980),
  DSP16_Q(-0.07979683),
  DSP16_Q(0.22784911),
  DSP16_Q(0.28898192),
  DSP16_Q(-0.08664846),
  DSP16_Q(0.04898986),
  DSP16_Q(-0.03251055),
  DSP16_Q(0.02297996),
  DSP16_Q(-0.01664578),
  DSP16_Q(0.01209722),
  DSP16_Q(-0.00868872),
  DSP16_Q(0.00608381),
  DSP16_Q(-0.00408874),
  DSP16_Q(0.00258166),
  DSP16_Q(-0.00147914),
  DSP16_Q(0.00071886),
  DSP16_Q(-0.00025040),
  DSP16_Q(0.00003003),
  DSP16_Q(0.00001897),
  DSP16_Q(-0.00021435),
  DSP16_Q(0.00065376),
  DSP16_Q(-0.00138050),
  DSP16_Q(0.00244418),
  DSP16_Q(-0.00390554),
  DSP16_Q(0.00584517),
  DSP16_Q(-0.00837960),
  DSP16_Q(0.01169235),
  DSP16_Q(-0.01609887),
  DSP16_Q(0.02219694),
  DSP16_Q(-0.03127072),
  DSP16_Q(0.04664307),
  DSP16_Q(-0.08024621),
  DSP16_Q(0.23030902),
  DSP16_Q(0.28676665),
  DSP16_Q(-0.08658555),
  DSP16_Q(0.04901207),
  DSP16_Q(-0.03253922),
  DSP16_Q(0.02300430),
  DSP16_Q(-0.01666433),
  DSP16_Q(0.01211044),
  DSP16_Q(-0.00869748),
  DSP16_Q(0.00608905),
  DSP16_Q(-0.00409132),
  DSP16_Q(0.00258240),
  DSP16_Q(-0.00147874),
  DSP16_Q(0.00071795),
  DSP16_Q(-0.00024952),
  DSP16_Q(0.00002961),
  DSP16_Q(0.00001941),
  DSP16_Q(-0.00021626),
  DSP16_Q(0.00065790),
  DSP16_Q(-0.00138777),
  DSP16_Q(0.00245560),
  DSP16_Q(-0.00392235),
  DSP16_Q(0.00586894),
  DSP16_Q(-0.00841236),
  DSP16_Q(0.01173700),
  DSP16_Q(-0.01615988),
  DSP16_Q(0.02228208),
  DSP16_Q(-0.03139569),
  DSP16_Q(0.04684770),
  DSP16_Q(-0.08068198),
  DSP16_Q(0.23276246),
  DSP16_Q(0.28453836),
  DSP16_Q(-0.08650549),
  DSP16_Q(0.04902422),
  DSP16_Q(-0.03256117),
  DSP16_Q(0.02302388),
  DSP16_Q(-0.01667943),
  DSP16_Q(0.01212115),
  DSP16_Q(-0.00870444),
  DSP16_Q(0.00609302),
  DSP16_Q(-0.00409305),
  DSP16_Q(0.00258259),
  DSP16_Q(-0.00147803),
  DSP16_Q(0.00071689),
  DSP16_Q(-0.00024858),
  DSP16_Q(0.00002919),
  DSP16_Q(0.00001985),
  DSP16_Q(-0.00021814),
  DSP16_Q(0.00066194),
  DSP16_Q(-0.00139477),
  DSP16_Q(0.00246655),
  DSP16_Q(-0.00393842),
  DSP16_Q(0.00589158),
  DSP16_Q(-0.00844350),
  DSP16_Q(0.01177938),
  DSP16_Q(-0.01621777),
  DSP16_Q(0.02236293),
  DSP16_Q(-0.03151467),
  DSP16_Q(0.04704360),
  DSP16_Q(-0.08110398),
  DSP16_Q(0.23520915),
  DSP16_Q(0.28229730),
  DSP16_Q(-0.08640842),
  DSP16_Q(0.04902636),
  DSP16_Q(-0.03257644),
  DSP16_Q(0.02303872),
  DSP16_Q(-0.01669109),
  DSP16_Q(0.01212935),
  DSP16_Q(-0.00870959),
  DSP16_Q(0.00609572),
  DSP16_Q(-0.00409393),
  DSP16_Q(0.00258225),
  DSP16_Q(-0.00147700),
  DSP16_Q(0.00071567),
  DSP16_Q(-0.00024758),
  DSP16_Q(0.00002876),
  DSP16_Q(0.00002030),
  DSP16_Q(-0.00021999),
  DSP16_Q(0.00066585),
  DSP16_Q(-0.00140152),
  DSP16_Q(0.00247704),
  DSP16_Q(-0.00395373),
  DSP16_Q(0.00591308),
  DSP16_Q(-0.00847301),
  DSP16_Q(0.01181948),
  DSP16_Q(-0.01627251),
  DSP16_Q(0.02243945),
  DSP16_Q(-0.03162762),
  DSP16_Q(0.04723071),
  DSP16_Q(-0.08151203),
  DSP16_Q(0.23764880),
  DSP16_Q(0.28004374),
  DSP16_Q(-0.08629444),
  DSP16_Q(0.04901854),
  DSP16_Q(-0.03258502),
  DSP16_Q(0.02304883),
  DSP16_Q(-0.01669932),
  DSP16_Q(0.01213505),
  DSP16_Q(-0.00871295),
  DSP16_Q(0.00609716),
  DSP16_Q(-0.00409396),
  DSP16_Q(0.00258137),
  DSP16_Q(-0.00147567),
  DSP16_Q(0.00071431),
  DSP16_Q(-0.00024654),
  DSP16_Q(0.00002833),
  DSP16_Q(0.00002074),
  DSP16_Q(-0.00022181),
  DSP16_Q(0.00066965),
  DSP16_Q(-0.00140801),
  DSP16_Q(0.00248705),
  DSP16_Q(-0.00396828),
  DSP16_Q(0.00593343),
  DSP16_Q(-0.00850086),
  DSP16_Q(0.01185727),
  DSP16_Q(-0.01632408),
  DSP16_Q(0.02251162),
  DSP16_Q(-0.03173448),
  DSP16_Q(0.04740895),
  DSP16_Q(-0.08190597),
  DSP16_Q(0.24008114),
  DSP16_Q(0.27777795),
  DSP16_Q(-0.08616371),
  DSP16_Q(0.04900080),
  DSP16_Q(-0.03258696),
  DSP16_Q(0.02305422),
  DSP16_Q(-0.01670412),
  DSP16_Q(0.01213826),
  DSP16_Q(-0.00871451),
  DSP16_Q(0.00609734),
  DSP16_Q(-0.00409315),
  DSP16_Q(0.00257995),
  DSP16_Q(-0.00147404),
  DSP16_Q(0.00071279),
  DSP16_Q(-0.00024544),
  DSP16_Q(0.00002789),
  DSP16_Q(0.00002119),
  DSP16_Q(-0.00022358),
  DSP16_Q(0.00067332),
  DSP16_Q(-0.00141424),
  DSP16_Q(0.00249659),
  DSP16_Q(-0.00398206),
  DSP16_Q(0.00595263),
  DSP16_Q(-0.00852706),
  DSP16_Q(0.01189274),
  DSP16_Q(-0.01637246),
  DSP16_Q(0.02257940),
  DSP16_Q(-0.03183522),
  DSP16_Q(0.04757825),
  DSP16_Q(-0.08228565),
  DSP16_Q(0.24250590),
  DSP16_Q(0.27550018),
  DSP16_Q(-0.08601635),
  DSP16_Q(0.04897317),
  DSP16_Q(-0.03258226),
  DSP16_Q(0.02305490),
  DSP16_Q(-0.01670550),
  DSP16_Q(0.01213898),
  DSP16_Q(-0.00871428),
  DSP16_Q(0.00609627),
  DSP16_Q(-0.00409149),
  DSP16_Q(0.00257800),
  DSP16_Q(-0.00147210),
  DSP16_Q(0.00071113),
  DSP16_Q(-0.00024430),
  DSP16_Q(0.00002746),
  DSP16_Q(0.00002164),
  DSP16_Q(-0.00022533),
  DSP16_Q(0.00067688),
  DSP16_Q(-0.00142019),
  DSP16_Q(0.00250565),
  DSP16_Q(-0.00399507),
  DSP16_Q(0.00597067),
  DSP16_Q(-0.00855158),
  DSP16_Q(0.01192588),
  DSP16_Q(-0.01641764),
  DSP16_Q(0.02264278),
  DSP16_Q(-0.03192981),
  DSP16_Q(0.04773854),
  DSP16_Q(-0.08265090),
  DSP16_Q(0.24492278),
  DSP16_Q(0.27321071),
  DSP16_Q(-0.08585249),
  DSP16_Q(0.04893572),
  DSP16_Q(-0.03257095),
  DSP16_Q(0.02305088),
  DSP16_Q(-0.01670347),
  DSP16_Q(0.01213722),
  DSP16_Q(-0.00871227),
  DSP16_Q(0.00609396),
  DSP16_Q(-0.00408899),
  DSP16_Q(0.00257552),
  DSP16_Q(-0.00146985),
  DSP16_Q(0.00070933),
  DSP16_Q(-0.00024311),
  DSP16_Q(0.00002702),
  DSP16_Q(0.00002209),
  DSP16_Q(-0.00022703),
  DSP16_Q(0.00068031),
  DSP16_Q(-0.00142588),
  DSP16_Q(0.00251423),
  DSP16_Q(-0.00400731),
  DSP16_Q(0.00598754),
  DSP16_Q(-0.00857443),
  DSP16_Q(0.01195668),
  DSP16_Q(-0.01645959),
  DSP16_Q(0.02270172),
  DSP16_Q(-0.03201819),
  DSP16_Q(0.04788975),
  DSP16_Q(-0.08300157),
  DSP16_Q(0.24733151),
  DSP16_Q(0.27090981),
  DSP16_Q(-0.08567226),
  DSP16_Q(0.04888848),
  DSP16_Q(-0.03255306),
  DSP16_Q(0.02304218),
  DSP16_Q(-0.01669805),
  DSP16_Q(0.01213299),
  DSP16_Q(-0.00870849),
  DSP16_Q(0.00609039),
  DSP16_Q(-0.00408566),
  DSP16_Q(0.00257252),
  DSP16_Q(-0.00146731),
  DSP16_Q(0.00070738),
  DSP16_Q(-0.00024187),
  DSP16_Q(0.00002657),
  DSP16_Q(0.00002254),
  DSP16_Q(-0.00022870),
  DSP16_Q(0.00068361),
  DSP16_Q(-0.00143129),
  DSP16_Q(0.00252232),
  DSP16_Q(-0.00401875),
  DSP16_Q(0.00600324),
  DSP16_Q(-0.00859558),
  DSP16_Q(0.01198512),
  DSP16_Q(-0.01649829),
  DSP16_Q(0.02275621),
  DSP16_Q(-0.03210033),
  DSP16_Q(0.04803182),
  DSP16_Q(-0.08333749),
  DSP16_Q(0.24973181),
  DSP16_Q(0.26859774),
  DSP16_Q(-0.08547582),
  DSP16_Q(0.04883150),
  DSP16_Q(-0.03252861),
  DSP16_Q(0.02302882),
  DSP16_Q(-0.01668923),
  DSP16_Q(0.01212630),
  DSP16_Q(-0.00870293),
  DSP16_Q(0.00608559),
  DSP16_Q(-0.00408149),
  DSP16_Q(0.00256899),
  DSP16_Q(-0.00146447),
  DSP16_Q(0.00070528),
  DSP16_Q(-0.00024058),
  DSP16_Q(0.00002613),
  DSP16_Q(0.00002299),
  DSP16_Q(-0.00023033),
  DSP16_Q(0.00068678),
  DSP16_Q(-0.00143643),
  DSP16_Q(0.00252991),
  DSP16_Q(-0.00402941),
  DSP16_Q(0.00601775),
  DSP16_Q(-0.00861504),
  DSP16_Q(0.01201118),
  DSP16_Q(-0.01653374),
  DSP16_Q(0.02280621),
  DSP16_Q(-0.03217619),
  DSP16_Q(0.04816467),
  DSP16_Q(-0.08365851),
  DSP16_Q(0.25212341),
  DSP16_Q(0.26627478),
  DSP16_Q(-0.08526330),
  DSP16_Q(0.04876485),
  DSP16_Q(-0.03249763),
  DSP16_Q(0.02301082),
  DSP16_Q(-0.01667705),
  DSP16_Q(0.01211714),
  DSP16_Q(-0.00869561),
  DSP16_Q(0.00607956),
  DSP16_Q(-0.00407650),
  DSP16_Q(0.00256494),
  DSP16_Q(-0.00146133),
  DSP16_Q(0.00070305),
  DSP16_Q(-0.00023925),
  DSP16_Q(0.00002568),
  DSP16_Q(0.00002344),
  DSP16_Q(-0.00023192),
  DSP16_Q(0.00068983),
  DSP16_Q(-0.00144129),
  DSP16_Q(0.00253701),
  DSP16_Q(-0.00403928),
  DSP16_Q(0.00603106),
  DSP16_Q(-0.00863279),
  DSP16_Q(0.01203487),
  DSP16_Q(-0.01656591),
  DSP16_Q(0.02285171),
  DSP16_Q(-0.03224574),
  DSP16_Q(0.04828825),
  DSP16_Q(-0.08396449),
  DSP16_Q(0.25450603),
  DSP16_Q(0.26394119),
  DSP16_Q(-0.08503484),
  DSP16_Q(0.04868856),
  DSP16_Q(-0.03246014),
  DSP16_Q(0.02298818),
  DSP16_Q(-0.01666149),
  DSP16_Q(0.01210554),
  DSP16_Q(-0.00868653),
  DSP16_Q(0.00607229),
  DSP16_Q(-0.00407068),
  DSP16_Q(0.00256038),
  DSP16_Q(-0.00145790),
  DSP16_Q(0.00070068),
  DSP16_Q(-0.00023787),
  DSP16_Q(0.00002524),
  DSP16_Q(0.00002389),
  DSP16_Q(-0.00023347),
  DSP16_Q(0.00069274),
  DSP16_Q(-0.00144587),
  DSP16_Q(0.00254361),
  DSP16_Q(-0.00404834),
  DSP16_Q(0.00604318),
  DSP16_Q(-0.00864882),
  DSP16_Q(0.01205616),
  DSP16_Q(-0.01659478),
  DSP16_Q(0.02289267),
  DSP16_Q(-0.03230895),
  DSP16_Q(0.04840249),
  DSP16_Q(-0.08425525),
  DSP16_Q(0.25687939),
  DSP16_Q(0.26159724),
  DSP16_Q(-0.08479058),
  DSP16_Q(0.04860270),
  DSP16_Q(-0.03241618),
  DSP16_Q(0.02296093),
  DSP16_Q(-0.01664259),
  DSP16_Q(0.01209151),
  DSP16_Q(-0.00867570),
  DSP16_Q(0.00606380),
  DSP16_Q(-0.00406405),
  DSP16_Q(0.00255530),
  DSP16_Q(-0.00145418),
  DSP16_Q(0.00069817),
  DSP16_Q(-0.00023644),
  DSP16_Q(0.00002479),
  DSP16_Q(0.00002434),
  DSP16_Q(-0.00023498),
  DSP16_Q(0.00069552),
  DSP16_Q(-0.00145017),
  DSP16_Q(0.00254971),
  DSP16_Q(-0.00405660),
  DSP16_Q(0.00605410),
  DSP16_Q(-0.00866313),
  DSP16_Q(0.01207504),
  DSP16_Q(-0.01662035),
  DSP16_Q(0.02292909),
  DSP16_Q(-0.03236577),
  DSP16_Q(0.04850732),
  DSP16_Q(-0.08453067),
  DSP16_Q(0.25924322),
  DSP16_Q(0.25924322),
  DSP16_Q(-0.08453067),
  DSP16_Q(0.04850732),
  DSP16_Q(-0.03236577),
  DSP16_Q(0.02292909),
  DSP16_Q(-0.01662035),
  DSP16_Q(0.01207504),
  DSP16_Q(-0.00866313),
  DSP16_Q(0.00605410),
  DSP16_Q(-0.00405660),
  DSP16_Q(0.00254971),
  DSP16_Q(-0.00145017),
  DSP16_Q(0.00069552),
  DSP16_Q(-0.00023498),
  DSP16_Q(0.00002434),
  DSP16_Q(0.00002479),
  DSP16_Q(-0.00023644),
  DSP16_Q(0.00069817),
  DSP16_Q(-0.00145418),
  DSP16_Q(0.00255530),
  DSP16_Q(-0.00406405),
  DSP16_Q(0.00606380),
  DSP16_Q(-0.00867570),
  DSP16_Q(0.01209151),
  DSP16_Q(-0.01664259),
  DSP16_Q(0.02296093),
  DSP16_Q(-0.03241618),
  DSP16_Q(0.04860270),
  DSP16_Q(-0.08479058),
  DSP16_Q(0.26159724),
  DSP16_Q(0.25687939),
  DSP16_Q(-0.08425525),
  DSP16_Q(0.04840249),
  DSP16_Q(-0.03230895),
  DSP16_Q(0.02289267),
  DSP16_Q(-0.01659478),
  DSP16_Q(0.01205616),
  DSP16_Q(-0.00864882),
  DSP16_Q(0.00604318),
  DSP16_Q(-0.00404834),
  DSP16_Q(0.00254361),
  DSP16_Q(-0.00144587),
  DSP16_Q(0.00069274),
  DSP16_Q(-0.00023347),
  DSP16_Q(0.00002389),
  DSP16_Q(0.00002524),
  DSP16_Q(-0.00023787),
  DSP16_Q(0.00070068),
  DSP16_Q(-0.00145790),
  DSP16_Q(0.00256038),
  DSP16_Q(-0.00407068),
  DSP16_Q(0.00607229),
  DSP16_Q(-0.00868653),
  DSP16_Q(0.01210554),
  DSP16_Q(-0.01666149),
  DSP16_Q(0.02298818),
  DSP16_Q(-0.03246014),
  DSP16_Q(0.04868856),
  DSP16_Q(-0.08503484),
  DSP16_Q(0.26394119),
  DSP16_Q(0.25450603),
  DSP16_Q(-0.08396449),
  DSP16_Q(0.04828825),
  DSP16_Q(-0.03224574),
  DSP16_Q(0.02285171),
  DSP16_Q(-0.01656591),
  DSP16_Q(0.01203487),
  DSP16_Q(-0.00863279),
  DSP16_Q(0.00603106),
  DSP16_Q(-0.00403928),
  DSP16_Q(0.00253701),
  DSP16_Q(-0.00144129),
  DSP16_Q(0.00068983),
  DSP16_Q(-0.00023192),
  DSP16_Q(0.00002344),
  DSP16_Q(0.00002568),
  DSP16_Q(-0.00023925),
  DSP16_Q(0.00070305),
  DSP16_Q(-0.00146133),
  DSP16_Q(0.00256494),
  DSP16_Q(-0.00407650),
  DSP16_Q(0.00607956),
  DSP16_Q(-0.00869561),
  DSP16_Q(0.01211714),
  DSP16_Q(-0.01667705),
  DSP16_Q(0.02301082),
  DSP16_Q(-0.03249763),
  DSP16_Q(0.04876485),
  DSP16_Q(-0.08526330),
  DSP16_Q(0.26627478),
  DSP16_Q(0.25212341),
  DSP16_Q(-0.08365851),
  DSP16_Q(0.04816467),
  DSP16_Q(-0.03217619),
  DSP16_Q(0.02280621),
  DSP16_Q(-0.01653374),
  DSP16_Q(0.01201118),
  DSP16_Q(-0.00861504),
  DSP16_Q(0.00601775),
  DSP16_Q(-0.00402941),
  DSP16_Q(0.00252991),
  DSP16_Q(-0.00143643),
  DSP16_Q(0.00068678),
  DSP16_Q(-0.00023033),
  DSP16_Q(0.00002299),
  DSP16_Q(0.00002613),
  DSP16_Q(-0.00024058),
  DSP16_Q(0.00070528),
  DSP16_Q(-0.00146447),
  DSP16_Q(0.00256899),
  DSP16_Q(-0.00408149),
  DSP16_Q(0.00608559),
  DSP16_Q(-0.00870293),
  DSP16_Q(0.01212630),
  DSP16_Q(-0.01668923),
  DSP16_Q(0.02302882),
  DSP16_Q(-0.03252861),
  DSP16_Q(0.04883150),
  DSP16_Q(-0.08547582),
  DSP16_Q(0.26859774),
  DSP16_Q(0.24973181),
  DSP16_Q(-0.08333749),
  DSP16_Q(0.04803182),
  DSP16_Q(-0.03210033),
  DSP16_Q(0.02275621),
  DSP16_Q(-0.01649829),
  DSP16_Q(0.01198512),
  DSP16_Q(-0.00859558),
  DSP16_Q(0.00600324),
  DSP16_Q(-0.00401875),
  DSP16_Q(0.00252232),
  DSP16_Q(-0.00143129),
  DSP16_Q(0.00068361),
  DSP16_Q(-0.00022870),
  DSP16_Q(0.00002254),
  DSP16_Q(0.00002657),
  DSP16_Q(-0.00024187),
  DSP16_Q(0.00070738),
  DSP16_Q(-0.00146731),
  DSP16_Q(0.00257252),
  DSP16_Q(-0.00408566),
  DSP16_Q(0.00609039),
  DSP16_Q(-0.00870849),
  DSP16_Q(0.01213299),
  DSP16_Q(-0.01669805),
  DSP16_Q(0.02304218),
  DSP16_Q(-0.03255306),
  DSP16_Q(0.04888848),
  DSP16_Q(-0.08567226),
  DSP16_Q(0.27090981),
  DSP16_Q(0.24733151),
  DSP16_Q(-0.08300157),
  DSP16_Q(0.04788975),
  DSP16_Q(-0.03201819),
  DSP16_Q(0.02270172),
  DSP16_Q(-0.01645959),
  DSP16_Q(0.01195668),
  DSP16_Q(-0.00857443),
  DSP16_Q(0.00598754),
  DSP16_Q(-0.00400731),
  DSP16_Q(0.00251423),
  DSP16_Q(-0.00142588),
  DSP16_Q(0.00068031),
  DSP16_Q(-0.00022703),
  DSP16_Q(0.00002209),
  DSP16_Q(0.00002702),
  DSP16_Q(-0.00024311),
  DSP16_Q(0.00070933),
  DSP16_Q(-0.00146985),
  DSP16_Q(0.00257552),
  DSP16_Q(-0.00408899),
  DSP16_Q(0.00609396),
  DSP16_Q(-0.00871227),
  DSP16_Q(0.01213722),
  DSP16_Q(-0.01670347),
  DSP16_Q(0.02305088),
  DSP16_Q(-0.03257095),
  DSP16_Q(0.04893572),
  DSP16_Q(-0.08585249),
  DSP16_Q(0.27321071),
  DSP16_Q(0.24492278),
  DSP16_Q(-0.08265090),
  DSP16_Q(0.04773854),
  DSP16_Q(-0.03192981),
  DSP16_Q(0.02264278),
  DSP16_Q(-0.01641764),
  DSP16_Q(0.01192588),
  DSP16_Q(-0.00855158),
  DSP16_Q(0.00597067),
  DSP16_Q(-0.00399507),
  DSP16_Q(0.00250565),
  DSP16_Q(-0.00142019),
  DSP16_Q(0.00067688),
  DSP16_Q(-0.00022533),
  DSP16_Q(0.00002164),
  DSP16_Q(0.00002746),
  DSP16_Q(-0.00024430),
  DSP16_Q(0.00071113),
  DSP16_Q(-0.00147210),
  DSP16_Q(0.00257800),
  DSP16_Q(-0.00409149),
  DSP16_Q(0.00609627),
  DSP16_Q(-0.00871428),
  DSP16_Q(0.01213898),
  DSP16_Q(-0.01670550),
  DSP16_Q(0.02305490),
  DSP16_Q(-0.03258226),
  DSP16_Q(0.04897317),
  DSP16_Q(-0.08601635),
  DSP16_Q(0.27550018),
  DSP16_Q(0.24250590),
  DSP16_Q(-0.08228565),
  DSP16_Q(0.04757825),
  DSP16_Q(-0.03183522),
  DSP16_Q(0.02257940),
  DSP16_Q(-0.01637246),
  DSP16_Q(0.01189274),
  DSP16_Q(-0.00852706),
  DSP16_Q(0.00595263),
  DSP16_Q(-0.00398206),
  DSP16_Q(0.00249659),
  DSP16_Q(-0.00141424),
  DSP16_Q(0.00067332),
  DSP16_Q(-0.00022358),
  DSP16_Q(0.00002119),
  DSP16_Q(0.00002789),
  DSP16_Q(-0.00024544),
  DSP16_Q(0.00071279),
  DSP16_Q(-0.00147404),
  DSP16_Q(0.00257995),
  DSP16_Q(-0.00409315),
  DSP16_Q(0.00609734),
  DSP16_Q(-0.00871451),
  DSP16_Q(0.01213826),
  DSP16_Q(-0.01670412),
  DSP16_Q(0.02305422),
  DSP16_Q(-0.03258696),
  DSP16_Q(0.04900080),
  DSP16_Q(-0.08616371),
  DSP16_Q(0.27777795),
  DSP16_Q(0.24008114),
  DSP16_Q(-0.08190597),
  DSP16_Q(0.04740895),
  DSP16_Q(-0.03173448),
  DSP16_Q(0.02251162),
  DSP16_Q(-0.01632408),
  DSP16_Q(0.01185727),
  DSP16_Q(-0.00850086),
  DSP16_Q(0.00593343),
  DSP16_Q(-0.00396828),
  DSP16_Q(0.00248705),
  DSP16_Q(-0.00140801),
  DSP16_Q(0.00066965),
  DSP16_Q(-0.00022181),
  DSP16_Q(0.00002074),
  DSP16_Q(0.00002833),
  DSP16_Q(-0.00024654),
  DSP16_Q(0.00071431),
  DSP16_Q(-0.00147567),
  DSP16_Q(0.00258137),
  DSP16_Q(-0.00409396),
  DSP16_Q(0.00609716),
  DSP16_Q(-0.00871295),
  DSP16_Q(0.01213505),
  DSP16_Q(-0.01669932),
  DSP16_Q(0.02304883),
  DSP16_Q(-0.03258502),
  DSP16_Q(0.04901854),
  DSP16_Q(-0.08629444),
  DSP16_Q(0.28004374),
  DSP16_Q(0.23764880),
  DSP16_Q(-0.08151203),
  DSP16_Q(0.04723071),
  DSP16_Q(-0.03162762),
  DSP16_Q(0.02243945),
  DSP16_Q(-0.01627251),
  DSP16_Q(0.01181948),
  DSP16_Q(-0.00847301),
  DSP16_Q(0.00591308),
  DSP16_Q(-0.00395373),
  DSP16_Q(0.00247704),
  DSP16_Q(-0.00140152),
  DSP16_Q(0.00066585),
  DSP16_Q(-0.00021999),
  DSP16_Q(0.00002030),
  DSP16_Q(0.00002876),
  DSP16_Q(-0.00024758),
  DSP16_Q(0.00071567),
  DSP16_Q(-0.00147700),
  DSP16_Q(0.00258225),
  DSP16_Q(-0.00409393),
  DSP16_Q(0.00609572),
  DSP16_Q(-0.00870959),
  DSP16_Q(0.01212935),
  DSP16_Q(-0.01669109),
  DSP16_Q(0.02303872),
  DSP16_Q(-0.03257644),
  DSP16_Q(0.04902636),
  DSP16_Q(-0.08640842),
  DSP16_Q(0.28229730),
  DSP16_Q(0.23520915),
  DSP16_Q(-0.08110398),
  DSP16_Q(0.04704360),
  DSP16_Q(-0.03151467),
  DSP16_Q(0.02236293),
  DSP16_Q(-0.01621777),
  DSP16_Q(0.01177938),
  DSP16_Q(-0.00844350),
  DSP16_Q(0.00589158),
  DSP16_Q(-0.00393842),
  DSP16_Q(0.00246655),
  DSP16_Q(-0.00139477),
  DSP16_Q(0.00066194),
  DSP16_Q(-0.00021814),
  DSP16_Q(0.00001985),
  DSP16_Q(0.00002919),
  DSP16_Q(-0.00024858),
  DSP16_Q(0.00071689),
  DSP16_Q(-0.00147803),
  DSP16_Q(0.00258259),
  DSP16_Q(-0.00409305),
  DSP16_Q(0.00609302),
  DSP16_Q(-0.00870444),
  DSP16_Q(0.01212115),
  DSP16_Q(-0.01667943),
  DSP16_Q(0.02302388),
  DSP16_Q(-0.03256117),
  DSP16_Q(0.04902422),
  DSP16_Q(-0.08650549),
  DSP16_Q(0.28453836),
  DSP16_Q(0.23276246),
  DSP16_Q(-0.08068198),
  DSP16_Q(0.04684770),
  DSP16_Q(-0.03139569),
  DSP16_Q(0.02228208),
  DSP16_Q(-0.01615988),
  DSP16_Q(0.01173700),
  DSP16_Q(-0.00841236),
  DSP16_Q(0.00586894),
  DSP16_Q(-0.00392235),
  DSP16_Q(0.00245560),
  DSP16_Q(-0.00138777),
  DSP16_Q(0.00065790),
  DSP16_Q(-0.00021626),
  DSP16_Q(0.00001941),
  DSP16_Q(0.00002961),
  DSP16_Q(-0.00024952),
  DSP16_Q(0.00071795),
  DSP16_Q(-0.00147874),
  DSP16_Q(0.00258240),
  DSP16_Q(-0.00409132),
  DSP16_Q(0.00608905),
  DSP16_Q(-0.00869748),
  DSP16_Q(0.01211044),
  DSP16_Q(-0.01666433),
  DSP16_Q(0.02300430),
  DSP16_Q(-0.03253922),
  DSP16_Q(0.04901207),
  DSP16_Q(-0.08658555),
  DSP16_Q(0.28676665),
  DSP16_Q(0.23030902),
  DSP16_Q(-0.08024621),
  DSP16_Q(0.04664307),
  DSP16_Q(-0.03127072),
  DSP16_Q(0.02219694),
  DSP16_Q(-0.01609887),
  DSP16_Q(0.01169235),
  DSP16_Q(-0.00837960),
  DSP16_Q(0.00584517),
  DSP16_Q(-0.00390554),
  DSP16_Q(0.00244418),
  DSP16_Q(-0.00138050),
  DSP16_Q(0.00065376),
  DSP16_Q(-0.00021435),
  DSP16_Q(0.00001897),
  DSP16_Q(0.00003003),
  DSP16_Q(-0.00025040),
  DSP16_Q(0.00071886),
  DSP16_Q(-0.00147914),
  DSP16_Q(0.00258166),
  DSP16_Q(-0.00408874),
  DSP16_Q(0.00608381),
  DSP16_Q(-0.00868872),
  DSP16_Q(0.01209722),
  DSP16_Q(-0.01664578),
  DSP16_Q(0.02297996),
  DSP16_Q(-0.03251055),
  DSP16_Q(0.04898986),
  DSP16_Q(-0.08664846),
  DSP16_Q(0.28898192),
  DSP16_Q(0.22784911),
  DSP16_Q(-0.07979683),
  DSP16_Q(0.04642980),
  DSP16_Q(-0.03113979),
  DSP16_Q(0.02210753),
  DSP16_Q(-0.01603475),
  DSP16_Q(0.01164544),
  DSP16_Q(-0.00834522),
  DSP16_Q(0.00582028),
  DSP16_Q(-0.00388798),
  DSP16_Q(0.00243231),
  DSP16_Q(-0.00137299),
  DSP16_Q(0.00064950),
  DSP16_Q(-0.00021240),
  DSP16_Q(0.00001853),
  DSP16_Q(0.00003044),
  DSP16_Q(-0.00025124),
  DSP16_Q(0.00071962),
  DSP16_Q(-0.00147922),
  DSP16_Q(0.00258038),
  DSP16_Q(-0.00408529),
  DSP16_Q(0.00607731),
  DSP16_Q(-0.00867815),
  DSP16_Q(0.01208148),
  DSP16_Q(-0.01662377),
  DSP16_Q(0.02295086),
  DSP16_Q(-0.03247515),
  DSP16_Q(0.04895758),
  DSP16_Q(-0.08669409),
  DSP16_Q(0.29118390),
  DSP16_Q(0.22538301),
  DSP16_Q(-0.07933400),
  DSP16_Q(0.04620796),
  DSP16_Q(-0.03100297),
  DSP16_Q(0.02201389),
  DSP16_Q(-0.01596754),
  DSP16_Q(0.01159629),
  DSP16_Q(-0.00830924),
  DSP16_Q(0.00579428),
  DSP16_Q(-0.00386969),
  DSP16_Q(0.00241998),
  DSP16_Q(-0.00136522),
  DSP16_Q(0.00064512),
  DSP16_Q(-0.00021042),
  DSP16_Q(0.00001810),
  DSP16_Q(0.00003085),
  DSP16_Q(-0.00025202),
  DSP16_Q(0.00072022),
  DSP16_Q(-0.00147899),
  DSP16_Q(0.00257855),
  DSP16_Q(-0.00408098),
  DSP16_Q(0.00606952),
  DSP16_Q(-0.00866576),
  DSP16_Q(0.01206323),
  DSP16_Q(-0.01659830),
  DSP16_Q(0.02291698),
  DSP16_Q(-0.03243302),
  DSP16_Q(0.04891517),
  DSP16_Q(-0.08672233),
  DSP16_Q(0.29337233),
  DSP16_Q(0.22291100),
  DSP16_Q(-0.07885789),
  DSP16_Q(0.04597764),
  DSP16_Q(-0.03086029),
  DSP16_Q(0.02191604),
  DSP16_Q(-0.01589728),
  DSP16_Q(0.01154491),
  DSP16_Q(-0.00827167),
  DSP16_Q(0.00576718),
  DSP16_Q(-0.00385067),
  DSP16_Q(0.00240720),
  DSP16_Q(-0.00135721),
  DSP16_Q(0.00064064),
  DSP16_Q(-0.00020842),
  DSP16_Q(0.00001766),
  DSP16_Q(0.00003126),
  DSP16_Q(-0.00025275),
  DSP16_Q(0.00072067),
  DSP16_Q(-0.00147844),
  DSP16_Q(0.00257617),
  DSP16_Q(-0.00407582),
  DSP16_Q(0.00606046),
  DSP16_Q(-0.00865155),
  DSP16_Q(0.01204245),
  DSP16_Q(-0.01656936),
  DSP16_Q(0.02287832),
  DSP16_Q(-0.03238412),
  DSP16_Q(0.04886261),
  DSP16_Q(-0.08673305),
  DSP16_Q(0.29554696),
  DSP16_Q(0.22043336),
  DSP16_Q(-0.07836868),
  DSP16_Q(0.04573890),
  DSP16_Q(-0.03071180),
  DSP16_Q(0.02181402),
  DSP16_Q(-0.01582397),
  DSP16_Q(0.01149133),
  DSP16_Q(-0.00823252),
  DSP16_Q(0.00573899),
  DSP16_Q(-0.00383092),
  DSP16_Q(0.00239398),
  DSP16_Q(-0.00134895),
  DSP16_Q(0.00063605),
  DSP16_Q(-0.00020638),
  DSP16_Q(0.00001723),
  DSP16_Q(0.00003166),
  DSP16_Q(-0.00025341),
  DSP16_Q(0.00072096),
  DSP16_Q(-0.00147757),
  DSP16_Q(0.00257325),
  DSP16_Q(-0.00406978),
  DSP16_Q(0.00605012),
  DSP16_Q(-0.00863552),
  DSP16_Q(0.01201914),
  DSP16_Q(-0.01653696),
  DSP16_Q(0.02283488),
  DSP16_Q(-0.03232845),
  DSP16_Q(0.04879985),
  DSP16_Q(-0.08672614),
  DSP16_Q(0.29770753),
  DSP16_Q(0.21795038),
  DSP16_Q(-0.07786653),
  DSP16_Q(0.04549183),
  DSP16_Q(-0.03055755),
  DSP16_Q(0.02170786),
  DSP16_Q(-0.01574765),
  DSP16_Q(0.01143556),
  DSP16_Q(-0.00819182),
  DSP16_Q(0.00570972),
  DSP16_Q(-0.00381046),
  DSP16_Q(0.00238032),
  DSP16_Q(-0.00134046),
  DSP16_Q(0.00063135),
  DSP16_Q(-0.00020432),
  DSP16_Q(0.00001681),
  DSP16_Q(0.00003205),
  DSP16_Q(-0.00025403),
  DSP16_Q(0.00072109),
  DSP16_Q(-0.00147639),
  DSP16_Q(0.00256976),
  DSP16_Q(-0.00406288),
  DSP16_Q(0.00603850),
  DSP16_Q(-0.00861768),
  DSP16_Q(0.01199331),
  DSP16_Q(-0.01650108),
  DSP16_Q(0.02278665),
  DSP16_Q(-0.03226601),
  DSP16_Q(0.04872688),
  DSP16_Q(-0.08670148),
  DSP16_Q(0.29985378),
  DSP16_Q(0.21546234),
  DSP16_Q(-0.07735161),
  DSP16_Q(0.04523652),
  DSP16_Q(-0.03039759),
  DSP16_Q(0.02159760),
  DSP16_Q(-0.01566834),
  DSP16_Q(0.01137762),
  DSP16_Q(-0.00814956),
  DSP16_Q(0.00567937),
  DSP16_Q(-0.00378930),
  DSP16_Q(0.00236623),
  DSP16_Q(-0.00133173),
  DSP16_Q(0.00062655),
  DSP16_Q(-0.00020223),
  DSP16_Q(0.00001638),
  DSP16_Q(0.00003244),
  DSP16_Q(-0.00025458),
  DSP16_Q(0.00072105),
  DSP16_Q(-0.00147488),
  DSP16_Q(0.00256573),
  DSP16_Q(-0.00405511),
  DSP16_Q(0.00602560),
  DSP16_Q(-0.00859800),
  DSP16_Q(0.01196494),
  DSP16_Q(-0.01646172),
  DSP16_Q(0.02273362),
  DSP16_Q(-0.03219677),
  DSP16_Q(0.04864366),
  DSP16_Q(-0.08665896),
  DSP16_Q(0.30198546),
  DSP16_Q(0.21296951),
  DSP16_Q(-0.07682411),
  DSP16_Q(0.04497305),
  DSP16_Q(-0.03023198),
  DSP16_Q(0.02148327),
  DSP16_Q(-0.01558606),
  DSP16_Q(0.01131752),
  DSP16_Q(-0.00810576),
  DSP16_Q(0.00564796),
  DSP16_Q(-0.00376743),
  DSP16_Q(0.00235170),
  DSP16_Q(-0.00132276),
  DSP16_Q(0.00062165),
  DSP16_Q(-0.00020012),
  DSP16_Q(0.00001596),
  DSP16_Q(0.00003282),
  DSP16_Q(-0.00025508),
  DSP16_Q(0.00072086),
  DSP16_Q(-0.00147304),
  DSP16_Q(0.00256114),
  DSP16_Q(-0.00404647),
  DSP16_Q(0.00601141),
  DSP16_Q(-0.00857651),
  DSP16_Q(0.01193405),
  DSP16_Q(-0.01641889),
  DSP16_Q(0.02267579),
  DSP16_Q(-0.03212073),
  DSP16_Q(0.04855017),
  DSP16_Q(-0.08659846),
  DSP16_Q(0.30410231),
  DSP16_Q(0.21047219),
  DSP16_Q(-0.07628419),
  DSP16_Q(0.04470150),
  DSP16_Q(-0.03006075),
  DSP16_Q(0.02136490),
  DSP16_Q(-0.01550084),
  DSP16_Q(0.01125529),
  DSP16_Q(-0.00806044),
  DSP16_Q(0.00561549),
  DSP16_Q(-0.00374487),
  DSP16_Q(0.00233676),
  DSP16_Q(-0.00131357),
  DSP16_Q(0.00061665),
  DSP16_Q(-0.00019798),
  DSP16_Q(0.00001555),
  DSP16_Q(0.00003320),
  DSP16_Q(-0.00025551),
  DSP16_Q(0.00072051),
  DSP16_Q(-0.00147088),
  DSP16_Q(0.00255599),
  DSP16_Q(-0.00403696),
  DSP16_Q(0.00599593),
  DSP16_Q(-0.00855318),
  DSP16_Q(0.01190061),
  DSP16_Q(-0.01637258),
  DSP16_Q(0.02261316),
  DSP16_Q(-0.03203788),
  DSP16_Q(0.04844638),
  DSP16_Q(-0.08651987),
  DSP16_Q(0.30620409),
  DSP16_Q(0.20797064),
  DSP16_Q(-0.07573203),
  DSP16_Q(0.04442196),
  DSP16_Q(-0.02988398),
  DSP16_Q(0.02124253),
  DSP16_Q(-0.01541271),
  DSP16_Q(0.01119095),
  DSP16_Q(-0.00801361),
  DSP16_Q(0.00558198),
  DSP16_Q(-0.00372162),
  DSP16_Q(0.00232139),
  DSP16_Q(-0.00130415),
  DSP16_Q(0.00061155),
  DSP16_Q(-0.00019581),
  DSP16_Q(0.00001513),
  DSP16_Q(0.00003356),
  DSP16_Q(-0.00025589),
  DSP16_Q(0.00071999),
  DSP16_Q(-0.00146840),
  DSP16_Q(0.00255028),
  DSP16_Q(-0.00402657),
  DSP16_Q(0.00597916),
  DSP16_Q(-0.00852803),
  DSP16_Q(0.01186465),
  DSP16_Q(-0.01632279),
  DSP16_Q(0.02254572),
  DSP16_Q(-0.03194821),
  DSP16_Q(0.04833226),
  DSP16_Q(-0.08642310),
  DSP16_Q(0.30829055),
  DSP16_Q(0.20546517),
  DSP16_Q(-0.07516780),
  DSP16_Q(0.04413452),
  DSP16_Q(-0.02970170),
  DSP16_Q(0.02111620),
  DSP16_Q(-0.01532169),
  DSP16_Q(0.01112450),
  DSP16_Q(-0.00796529),
  DSP16_Q(0.00554744),
  DSP16_Q(-0.00369769),
  DSP16_Q(0.00230561),
  DSP16_Q(-0.00129450),
  DSP16_Q(0.00060635),
  DSP16_Q(-0.00019363),
  DSP16_Q(0.00001472),
  DSP16_Q(0.00003393),
  DSP16_Q(-0.00025621),
  DSP16_Q(0.00071931),
  DSP16_Q(-0.00146558),
  DSP16_Q(0.00254402),
  DSP16_Q(-0.00401531),
  DSP16_Q(0.00596111),
  DSP16_Q(-0.00850105),
  DSP16_Q(0.01182615),
  DSP16_Q(-0.01626952),
  DSP16_Q(0.02247348),
  DSP16_Q(-0.03185173),
  DSP16_Q(0.04820780),
  DSP16_Q(-0.08630802),
  DSP16_Q(0.31036143),
  DSP16_Q(0.20295604),
  DSP16_Q(-0.07459169),
  DSP16_Q(0.04383926),
  DSP16_Q(-0.02951398),
  DSP16_Q(0.02098595),
  DSP16_Q(-0.01522781),
  DSP16_Q(0.01105598),
  DSP16_Q(-0.00791548),
  DSP16_Q(0.00551188),
  DSP16_Q(-0.00367309),
  DSP16_Q(0.00228942),
  DSP16_Q(-0.00128463),
  DSP16_Q(0.00060106),
  DSP16_Q(-0.00019142),
  DSP16_Q(0.00001432),
  DSP16_Q(0.00003428),
  DSP16_Q(-0.00025646),
  DSP16_Q(0.00071846),
  DSP16_Q(-0.00146244),
  DSP16_Q(0.00253719),
  DSP16_Q(-0.00400318),
  DSP16_Q(0.00594177),
  DSP16_Q(-0.00847224),
  DSP16_Q(0.01178512),
  DSP16_Q(-0.01621277),
  DSP16_Q(0.02239644),
  DSP16_Q(-0.03174842),
  DSP16_Q(0.04807297),
  DSP16_Q(-0.08617455),
  DSP16_Q(0.31241649),
  DSP16_Q(0.20044354),
  DSP16_Q(-0.07400387),
  DSP16_Q(0.04353627),
  DSP16_Q(-0.02932087),
  DSP16_Q(0.02085181),
  DSP16_Q(-0.01513109),
  DSP16_Q(0.01098540),
  DSP16_Q(-0.00786421),
  DSP16_Q(0.00547531),
  DSP16_Q(-0.00364783),
  DSP16_Q(0.00227283),
  DSP16_Q(-0.00127455),
  DSP16_Q(0.00059567),
  DSP16_Q(-0.00018919),
  DSP16_Q(0.00001392),
  DSP16_Q(0.00003462),
  DSP16_Q(-0.00025665),
  DSP16_Q(0.00071744),
  DSP16_Q(-0.00145897),
  DSP16_Q(0.00252980),
  DSP16_Q(-0.00399016),
  DSP16_Q(0.00592113),
  DSP16_Q(-0.00844160),
  DSP16_Q(0.01174155),
  DSP16_Q(-0.01615255),
  DSP16_Q(0.02231459),
  DSP16_Q(-0.03163829),
  DSP16_Q(0.04792776),
  DSP16_Q(-0.08602257),
  DSP16_Q(0.31445549),
  DSP16_Q(0.19792795),
  DSP16_Q(-0.07340453),
  DSP16_Q(0.04322565),
  DSP16_Q(-0.02912242),
  DSP16_Q(0.02071381),
  DSP16_Q(-0.01503156),
  DSP16_Q(0.01091279),
  DSP16_Q(-0.00781150),
  DSP16_Q(0.00543773),
  DSP16_Q(-0.00362192),
  DSP16_Q(0.00225584),
  DSP16_Q(-0.00126426),
  DSP16_Q(0.00059020),
  DSP16_Q(-0.00018693),
  DSP16_Q(0.00001352),
  DSP16_Q(0.00003496),
  DSP16_Q(-0.00025678),
  DSP16_Q(0.00071626),
  DSP16_Q(-0.00145517),
  DSP16_Q(0.00252184),
  DSP16_Q(-0.00397627),
  DSP16_Q(0.00589921),
  DSP16_Q(-0.00840914),
  DSP16_Q(0.01169546),
  DSP16_Q(-0.01608885),
  DSP16_Q(0.02222794),
  DSP16_Q(-0.03152133),
  DSP16_Q(0.04777215),
  DSP16_Q(-0.08585200),
  DSP16_Q(0.31647818),
  DSP16_Q(0.19540956),
  DSP16_Q(-0.07279384),
  DSP16_Q(0.04290749),
  DSP16_Q(-0.02891870),
  DSP16_Q(0.02057201),
  DSP16_Q(-0.01492926),
  DSP16_Q(0.01083816),
  DSP16_Q(-0.00775734),
  DSP16_Q(0.00539917),
  DSP16_Q(-0.00359535),
  DSP16_Q(0.00223845),
  DSP16_Q(-0.00125375),
  DSP16_Q(0.00058464),
  DSP16_Q(-0.00018466),
  DSP16_Q(0.00001313),
  DSP16_Q(0.00003529),
  DSP16_Q(-0.00025685),
  DSP16_Q(0.00071491),
  DSP16_Q(-0.00145103),
  DSP16_Q(0.00251333),
  DSP16_Q(-0.00396151),
  DSP16_Q(0.00587600),
  DSP16_Q(-0.00837485),
  DSP16_Q(0.01164683),
  DSP16_Q(-0.01602167),
  DSP16_Q(0.02213650),
  DSP16_Q(-0.03139755),
  DSP16_Q(0.04760614),
  DSP16_Q(-0.08566273),
  DSP16_Q(0.31848433),
  DSP16_Q(0.19288864),
  DSP16_Q(-0.07217199),
  DSP16_Q(0.04258188),
  DSP16_Q(-0.02870976),
  DSP16_Q(0.02042644),
  DSP16_Q(-0.01482421),
  DSP16_Q(0.01076154),
  DSP16_Q(-0.00770177),
  DSP16_Q(0.00535963),
  DSP16_Q(-0.00356815),
  DSP16_Q(0.00222068),
  DSP16_Q(-0.00124304),
  DSP16_Q(0.00057899),
  DSP16_Q(-0.00018237),
  DSP16_Q(0.00001275),
  DSP16_Q(0.00003560),
  DSP16_Q(-0.00025685),
  DSP16_Q(0.00071339),
  DSP16_Q(-0.00144657),
  DSP16_Q(0.00250425),
  DSP16_Q(-0.00394586),
  DSP16_Q(0.00585150),
  DSP16_Q(-0.00833873),
  DSP16_Q(0.01159568),
  DSP16_Q(-0.01595103),
  DSP16_Q(0.02204026),
  DSP16_Q(-0.03126694),
  DSP16_Q(0.04742969),
  DSP16_Q(-0.08545467),
  DSP16_Q(0.32047369),
  DSP16_Q(0.19036548),
  DSP16_Q(-0.07153915),
  DSP16_Q(0.04224891),
  DSP16_Q(-0.02849566),
  DSP16_Q(0.02027715),
  DSP16_Q(-0.01471643),
  DSP16_Q(0.01068294),
  DSP16_Q(-0.00764479),
  DSP16_Q(0.00531913),
  DSP16_Q(-0.00354032),
  DSP16_Q(0.00220253),
  DSP16_Q(-0.00123213),
  DSP16_Q(0.00057326),
  DSP16_Q(-0.00018007),
  DSP16_Q(0.00001236),
  DSP16_Q(0.00003591),
  DSP16_Q(-0.00025678),
  DSP16_Q(0.00071170),
  DSP16_Q(-0.00144176),
  DSP16_Q(0.00249460),
  DSP16_Q(-0.00392934),
  DSP16_Q(0.00582571),
  DSP16_Q(-0.00830080),
  DSP16_Q(0.01154200),
  DSP16_Q(-0.01587693),
  DSP16_Q(0.02193923),
  DSP16_Q(-0.03112952),
  DSP16_Q(0.04724282),
  DSP16_Q(-0.08522773),
  DSP16_Q(0.32244602),
  DSP16_Q(0.18784036),
  DSP16_Q(-0.07089552),
  DSP16_Q(0.04190869),
  DSP16_Q(-0.02827646),
  DSP16_Q(0.02012416),
  DSP16_Q(-0.01460597),
  DSP16_Q(0.01060240),
  DSP16_Q(-0.00758643),
  DSP16_Q(0.00527767),
  DSP16_Q(-0.00351186),
  DSP16_Q(0.00218401),
  DSP16_Q(-0.00122102),
  DSP16_Q(0.00056744),
  DSP16_Q(-0.00017774),
  DSP16_Q(0.00001199),
  DSP16_Q(0.00003621),
  DSP16_Q(-0.00025666),
  DSP16_Q(0.00070984),
  DSP16_Q(-0.00143663),
  DSP16_Q(0.00248439),
  DSP16_Q(-0.00391194),
  DSP16_Q(0.00579863),
  DSP16_Q(-0.00826104),
  DSP16_Q(0.01148581),
  DSP16_Q(-0.01579937),
  DSP16_Q(0.02183342),
  DSP16_Q(-0.03098527),
  DSP16_Q(0.04704550),
  DSP16_Q(-0.08498183),
  DSP16_Q(0.32440110),
  DSP16_Q(0.18531356),
  DSP16_Q(-0.07024128),
  DSP16_Q(0.04156130),
  DSP16_Q(-0.02805223),
  DSP16_Q(0.01996753),
  DSP16_Q(-0.01449285),
  DSP16_Q(0.01051992),
  DSP16_Q(-0.00752670),
  DSP16_Q(0.00523527),
  DSP16_Q(-0.00348279),
  DSP16_Q(0.00216511),
  DSP16_Q(-0.00120971),
  DSP16_Q(0.00056154),
  DSP16_Q(-0.00017540),
  DSP16_Q(0.00001162),
  DSP16_Q(0.00003650),
  DSP16_Q(-0.00025646),
  DSP16_Q(0.00070780),
  DSP16_Q(-0.00143116),
  DSP16_Q(0.00247361),
  DSP16_Q(-0.00389366),
  DSP16_Q(0.00577027),
  DSP16_Q(-0.00821946),
  DSP16_Q(0.01142710),
  DSP16_Q(-0.01571836),
  DSP16_Q(0.02172283),
  DSP16_Q(-0.03083422),
  DSP16_Q(0.04683774),
  DSP16_Q(-0.08471687),
  DSP16_Q(0.32633868),
  DSP16_Q(0.18278536),
  DSP16_Q(-0.06957660),
  DSP16_Q(0.04120684),
  DSP16_Q(-0.02782301),
  DSP16_Q(0.01980730),
  DSP16_Q(-0.01437709),
  DSP16_Q(0.01043554),
  DSP16_Q(-0.00746561),
  DSP16_Q(0.00519194),
  DSP16_Q(-0.00345311),
  DSP16_Q(0.00214585),
  DSP16_Q(-0.00119821),
  DSP16_Q(0.00055557),
  DSP16_Q(-0.00017304),
  DSP16_Q(0.00001125),
  DSP16_Q(0.00003678),
  DSP16_Q(-0.00025620),
  DSP16_Q(0.00070560),
  DSP16_Q(-0.00142535),
  DSP16_Q(0.00246226),
  DSP16_Q(-0.00387451),
  DSP16_Q(0.00574062),
  DSP16_Q(-0.00817607),
  DSP16_Q(0.01136588),
  DSP16_Q(-0.01563390),
  DSP16_Q(0.02160748),
  DSP16_Q(-0.03067636),
  DSP16_Q(0.04661953),
  DSP16_Q(-0.08443278),
  DSP16_Q(0.32825854),
  DSP16_Q(0.18025604),
  DSP16_Q(-0.06890169),
  DSP16_Q(0.04084541),
  DSP16_Q(-0.02758888),
  DSP16_Q(0.01964350),
  DSP16_Q(-0.01425874),
  DSP16_Q(0.01034928),
  DSP16_Q(-0.00740318),
  DSP16_Q(0.00514770),
  DSP16_Q(-0.00342283),
  DSP16_Q(0.00212623),
  DSP16_Q(-0.00118652),
  DSP16_Q(0.00054952),
  DSP16_Q(-0.00017067),
  DSP16_Q(0.00001089),
  DSP16_Q(0.00003705),
  DSP16_Q(-0.00025587),
  DSP16_Q(0.00070322),
  DSP16_Q(-0.00141921),
  DSP16_Q(0.00245035),
  DSP16_Q(-0.00385448),
  DSP16_Q(0.00570969),
  DSP16_Q(-0.00813087),
  DSP16_Q(0.01130216),
  DSP16_Q(-0.01554600),
  DSP16_Q(0.02148737),
  DSP16_Q(-0.03051171),
  DSP16_Q(0.04639086),
  DSP16_Q(-0.08412946),
  DSP16_Q(0.33016044),
  DSP16_Q(0.17772588),
  DSP16_Q(-0.06821673),
  DSP16_Q(0.04047712),
  DSP16_Q(-0.02734990),
  DSP16_Q(0.01947619),
  DSP16_Q(-0.01413783),
  DSP16_Q(0.01026115),
  DSP16_Q(-0.00733943),
  DSP16_Q(0.00510254),
  DSP16_Q(-0.00339197),
  DSP16_Q(0.00210626),
  DSP16_Q(-0.00117465),
  DSP16_Q(0.00054340),
  DSP16_Q(-0.00016829),
  DSP16_Q(0.00001053),
  DSP16_Q(0.00003730),
  DSP16_Q(-0.00025547),
  DSP16_Q(0.00070066),
  DSP16_Q(-0.00141273),
  DSP16_Q(0.00243787),
  DSP16_Q(-0.00383358),
  DSP16_Q(0.00567749),
  DSP16_Q(-0.00808386),
  DSP16_Q(0.01123593),
  DSP16_Q(-0.01545467),
  DSP16_Q(0.02136251),
  DSP16_Q(-0.03034028),
  DSP16_Q(0.04615174),
  DSP16_Q(-0.08380685),
  DSP16_Q(0.33204415),
  DSP16_Q(0.17519516),
  DSP16_Q(-0.06752189),
  DSP16_Q(0.04010206),
  DSP16_Q(-0.02710613),
  DSP16_Q(0.01930541),
  DSP16_Q(-0.01401437),
  DSP16_Q(0.01017119),
  DSP16_Q(-0.00727438),
  DSP16_Q(0.00505650),
  DSP16_Q(-0.00336052),
  DSP16_Q(0.00208594),
  DSP16_Q(-0.00116260),
  DSP16_Q(0.00053720),
  DSP16_Q(-0.00016589),
  DSP16_Q(0.00001018),
  DSP16_Q(0.00003755),
  DSP16_Q(-0.00025500),
  DSP16_Q(0.00069793),
  DSP16_Q(-0.00140591),
  DSP16_Q(0.00242483),
  DSP16_Q(-0.00381180),
  DSP16_Q(0.00564400),
  DSP16_Q(-0.00803505),
  DSP16_Q(0.01116721),
  DSP16_Q(-0.01535992),
  DSP16_Q(0.02123292),
  DSP16_Q(-0.03016207),
  DSP16_Q(0.04590216),
  DSP16_Q(-0.08346486),
  DSP16_Q(0.33390946),
  DSP16_Q(0.17266416),
  DSP16_Q(-0.06681738),
  DSP16_Q(0.03972032),
  DSP16_Q(-0.02685763),
  DSP16_Q(0.01913120),
  DSP16_Q(-0.01388842),
  DSP16_Q(0.01007942),
  DSP16_Q(-0.00720805),
  DSP16_Q(0.00500957),
  DSP16_Q(-0.00332851),
  DSP16_Q(0.00206528),
  DSP16_Q(-0.00115037),
  DSP16_Q(0.00053093),
  DSP16_Q(-0.00016348),
  DSP16_Q(0.00000984),
  DSP16_Q(0.00003778),
  DSP16_Q(-0.00025446),
  DSP16_Q(0.00069503),
  DSP16_Q(-0.00139876),
  DSP16_Q(0.00241122),
  DSP16_Q(-0.00378915),
  DSP16_Q(0.00560924),
  DSP16_Q(-0.00798444),
  DSP16_Q(0.01109601),
  DSP16_Q(-0.01526176),
  DSP16_Q(0.02109860),
  DSP16_Q(-0.02997711),
  DSP16_Q(0.04564214),
  DSP16_Q(-0.08310343),
  DSP16_Q(0.33575613),
  DSP16_Q(0.17013316),
  DSP16_Q(-0.06610338),
  DSP16_Q(0.03933203),
  DSP16_Q(-0.02660448),
  DSP16_Q(0.01895361),
  DSP16_Q(-0.01375999),
  DSP16_Q(0.00998586),
  DSP16_Q(-0.00714045),
  DSP16_Q(0.00496178),
  DSP16_Q(-0.00329593),
  DSP16_Q(0.00204428),
  DSP16_Q(-0.00113796),
  DSP16_Q(0.00052460),
  DSP16_Q(-0.00016107),
  DSP16_Q(0.00000950),
  DSP16_Q(0.00003800),
  DSP16_Q(-0.00025385),
  DSP16_Q(0.00069195),
  DSP16_Q(-0.00139127),
  DSP16_Q(0.00239704),
  DSP16_Q(-0.00376563),
  DSP16_Q(0.00557320),
  DSP16_Q(-0.00793203),
  DSP16_Q(0.01102233),
  DSP16_Q(-0.01516019),
  DSP16_Q(0.02095956),
  DSP16_Q(-0.02978540),
  DSP16_Q(0.04537168),
  DSP16_Q(-0.08272247),
  DSP16_Q(0.33758395),
  DSP16_Q(0.16760243),
  DSP16_Q(-0.06538008),
  DSP16_Q(0.03893727),
  DSP16_Q(-0.02634674),
  DSP16_Q(0.01877268),
  DSP16_Q(-0.01362913),
  DSP16_Q(0.00989053),
  DSP16_Q(-0.00707159),
  DSP16_Q(0.00491313),
  DSP16_Q(-0.00326280),
  DSP16_Q(0.00202296),
  DSP16_Q(-0.00112539),
  DSP16_Q(0.00051820),
  DSP16_Q(-0.00015864),
  DSP16_Q(0.00000917),
  DSP16_Q(0.00003821),
  DSP16_Q(-0.00025317),
  DSP16_Q(0.00068869),
  DSP16_Q(-0.00138344),
  DSP16_Q(0.00238230),
  DSP16_Q(-0.00374124),
  DSP16_Q(0.00553590),
  DSP16_Q(-0.00787784),
  DSP16_Q(0.01094617),
  DSP16_Q(-0.01505524),
  DSP16_Q(0.02081583),
  DSP16_Q(-0.02958696),
  DSP16_Q(0.04509078),
  DSP16_Q(-0.08232193),
  DSP16_Q(0.33939270),
  DSP16_Q(0.16507225),
  DSP16_Q(-0.06464768),
  DSP16_Q(0.03853615),
  DSP16_Q(-0.02608447),
  DSP16_Q(0.01858846),
  DSP16_Q(-0.01349587),
  DSP16_Q(0.00979347),
  DSP16_Q(-0.00700150),
  DSP16_Q(0.00486364),
  DSP16_Q(-0.00322913),
  DSP16_Q(0.00200131),
  DSP16_Q(-0.00111265),
  DSP16_Q(0.00051173),
  DSP16_Q(-0.00015620),
  DSP16_Q(0.00000884),
  DSP16_Q(0.00003841),
  DSP16_Q(-0.00025242),
  DSP16_Q(0.00068526),
  DSP16_Q(-0.00137527),
  DSP16_Q(0.00236700),
  DSP16_Q(-0.00371598),
  DSP16_Q(0.00549733),
  DSP16_Q(-0.00782186),
  DSP16_Q(0.01086756),
  DSP16_Q(-0.01494690),
  DSP16_Q(0.02066742),
  DSP16_Q(-0.02938180),
  DSP16_Q(0.04479946),
  DSP16_Q(-0.08190173),
  DSP16_Q(0.34118215),
  DSP16_Q(0.16254290),
  DSP16_Q(-0.06390635),
  DSP16_Q(0.03812877),
  DSP16_Q(-0.02581774),
  DSP16_Q(0.01840100),
  DSP16_Q(-0.01336023),
  DSP16_Q(0.00969469),
  DSP16_Q(-0.00693020),
  DSP16_Q(0.00481332),
  DSP16_Q(-0.00319492),
  DSP16_Q(0.00197935),
  DSP16_Q(-0.00109975),
  DSP16_Q(0.00050520),
  DSP16_Q(-0.00015375),
  DSP16_Q(0.00000852),
  DSP16_Q(0.00003859),
  DSP16_Q(-0.00025160),
  DSP16_Q(0.00068165),
  DSP16_Q(-0.00136676),
  DSP16_Q(0.00235113),
  DSP16_Q(-0.00368985),
  DSP16_Q(0.00545750),
  DSP16_Q(-0.00776410),
  DSP16_Q(0.01078649),
  DSP16_Q(-0.01483520),
  DSP16_Q(0.02051434),
  DSP16_Q(-0.02916995),
  DSP16_Q(0.04449773),
  DSP16_Q(-0.08146182),
  DSP16_Q(0.34295209),
  DSP16_Q(0.16001465),
  DSP16_Q(-0.06315629),
  DSP16_Q(0.03771525),
  DSP16_Q(-0.02554662),
  DSP16_Q(0.01821035),
  DSP16_Q(-0.01322227),
  DSP16_Q(0.00959422),
  DSP16_Q(-0.00685771),
  DSP16_Q(0.00476218),
  DSP16_Q(-0.00316018),
  DSP16_Q(0.00195708),
  DSP16_Q(-0.00108669),
  DSP16_Q(0.00049862),
  DSP16_Q(-0.00015130),
  DSP16_Q(0.00000821),
  DSP16_Q(0.00003876),
  DSP16_Q(-0.00025071),
  DSP16_Q(0.00067786),
  DSP16_Q(-0.00135792),
  DSP16_Q(0.00233470),
  DSP16_Q(-0.00366287),
  DSP16_Q(0.00541642),
  DSP16_Q(-0.00770458),
  DSP16_Q(0.01070297),
  DSP16_Q(-0.01472014),
  DSP16_Q(0.02035661),
  DSP16_Q(-0.02895143),
  DSP16_Q(0.04418560),
  DSP16_Q(-0.08100213),
  DSP16_Q(0.34470231),
  DSP16_Q(0.15748779),
  DSP16_Q(-0.06239771),
  DSP16_Q(0.03729568),
  DSP16_Q(-0.02527118),
  DSP16_Q(0.01801655),
  DSP16_Q(-0.01308200),
  DSP16_Q(0.00949208),
  DSP16_Q(-0.00678404),
  DSP16_Q(0.00471024),
  DSP16_Q(-0.00312493),
  DSP16_Q(0.00193450),
  DSP16_Q(-0.00107348),
  DSP16_Q(0.00049197),
  DSP16_Q(-0.00014884),
  DSP16_Q(0.00000790),
  DSP16_Q(0.00003891),
  DSP16_Q(-0.00024974),
  DSP16_Q(0.00067390),
  DSP16_Q(-0.00134874),
  DSP16_Q(0.00231771),
  DSP16_Q(-0.00363502),
  DSP16_Q(0.00537408),
  DSP16_Q(-0.00764329),
  DSP16_Q(0.01061702),
  DSP16_Q(-0.01460175),
  DSP16_Q(0.02019425),
  DSP16_Q(-0.02872626),
  DSP16_Q(0.04386309),
  DSP16_Q(-0.08052260),
  DSP16_Q(0.34643260),
  DSP16_Q(0.15496258),
  DSP16_Q(-0.06163077),
  DSP16_Q(0.03687018),
  DSP16_Q(-0.02499148),
  DSP16_Q(0.01781965),
  DSP16_Q(-0.01293947),
  DSP16_Q(0.00938831),
  DSP16_Q(-0.00670921),
  DSP16_Q(0.00465752),
  DSP16_Q(-0.00308917),
  DSP16_Q(0.00191163),
  DSP16_Q(-0.00106012),
  DSP16_Q(0.00048527),
  DSP16_Q(-0.00014637),
  DSP16_Q(0.00000760),
  DSP16_Q(0.00003905),
  DSP16_Q(-0.00024870),
  DSP16_Q(0.00066975),
  DSP16_Q(-0.00133922),
  DSP16_Q(0.00230016),
  DSP16_Q(-0.00360631),
  DSP16_Q(0.00533049),
  DSP16_Q(-0.00758025),
  DSP16_Q(0.01052865),
  DSP16_Q(-0.01448003),
  DSP16_Q(0.02002728),
  DSP16_Q(-0.02849445),
  DSP16_Q(0.04353021),
  DSP16_Q(-0.08002318),
  DSP16_Q(0.34814275),
  DSP16_Q(0.15243929),
  DSP16_Q(-0.06085569),
  DSP16_Q(0.03643885),
  DSP16_Q(-0.02470761),
  DSP16_Q(0.01761970),
  DSP16_Q(-0.01279471),
  DSP16_Q(0.00928293),
  DSP16_Q(-0.00663324),
  DSP16_Q(0.00460402),
  DSP16_Q(-0.00305292),
  DSP16_Q(0.00188846),
  DSP16_Q(-0.00104660),
  DSP16_Q(0.00047851),
  DSP16_Q(-0.00014390),
  DSP16_Q(0.00000730),
  DSP16_Q(0.00003918),
  DSP16_Q(-0.00024759),
  DSP16_Q(0.00066543),
  DSP16_Q(-0.00132936),
  DSP16_Q(0.00228205),
  DSP16_Q(-0.00357675),
  DSP16_Q(0.00528566),
  DSP16_Q(-0.00751545),
  DSP16_Q(0.01043787),
  DSP16_Q(-0.01435499),
  DSP16_Q(0.01985571),
  DSP16_Q(-0.02825604),
  DSP16_Q(0.04318699),
  DSP16_Q(-0.07950382),
  DSP16_Q(0.34983255),
  DSP16_Q(0.14991821),
  DSP16_Q(-0.06007265),
  DSP16_Q(0.03600179),
  DSP16_Q(-0.02441962),
  DSP16_Q(0.01741676),
  DSP16_Q(-0.01264776),
  DSP16_Q(0.00917596),
  DSP16_Q(-0.00655615),
  DSP16_Q(0.00454976),
  DSP16_Q(-0.00301617),
  DSP16_Q(0.00186501),
  DSP16_Q(-0.00103295),
  DSP16_Q(0.00047171),
  DSP16_Q(-0.00014142),
  DSP16_Q(0.00000701),
  DSP16_Q(0.00003929),
  DSP16_Q(-0.00024640),
  DSP16_Q(0.00066092),
  DSP16_Q(-0.00131917),
  DSP16_Q(0.00226338),
  DSP16_Q(-0.00354633),
  DSP16_Q(0.00523959),
  DSP16_Q(-0.00744892),
  DSP16_Q(0.01034468),
  DSP16_Q(-0.01422667),
  DSP16_Q(0.01967957),
  DSP16_Q(-0.02801104),
  DSP16_Q(0.04283344),
  DSP16_Q(-0.07896446),
  DSP16_Q(0.35150180),
  DSP16_Q(0.14739960),
  DSP16_Q(-0.05928185),
  DSP16_Q(0.03555913),
  DSP16_Q(-0.02412760),
  DSP16_Q(0.01721086),
  DSP16_Q(-0.01249865),
  DSP16_Q(0.00906743),
  DSP16_Q(-0.00647796),
  DSP16_Q(0.00449475),
  DSP16_Q(-0.00297895),
  DSP16_Q(0.00184128),
  DSP16_Q(-0.00101916),
  DSP16_Q(0.00046485),
  DSP16_Q(-0.00013894),
  DSP16_Q(0.00000673),
  DSP16_Q(0.00003938),
  DSP16_Q(-0.00024514),
  DSP16_Q(0.00065624),
  DSP16_Q(-0.00130864),
  DSP16_Q(0.00224416),
  DSP16_Q(-0.00351507),
  DSP16_Q(0.00519228),
  DSP16_Q(-0.00738065),
  DSP16_Q(0.01024911),
  DSP16_Q(-0.01409506),
  DSP16_Q(0.01949888),
  DSP16_Q(-0.02775949),
  DSP16_Q(0.04246960),
  DSP16_Q(-0.07840505),
  DSP16_Q(0.35315030),
  DSP16_Q(0.14488373),
  DSP16_Q(-0.05848348),
  DSP16_Q(0.03511096),
  DSP16_Q(-0.02383160),
  DSP16_Q(0.01700206),
  DSP16_Q(-0.01234742),
  DSP16_Q(0.00895737),
  DSP16_Q(-0.00639869),
  DSP16_Q(0.00443901),
  DSP16_Q(-0.00294126),
  DSP16_Q(0.00181727),
  DSP16_Q(-0.00100523),
  DSP16_Q(0.00045795),
  DSP16_Q(-0.00013646),
  DSP16_Q(0.00000645),
  DSP16_Q(0.00003946),
  DSP16_Q(-0.00024380),
  DSP16_Q(0.00065138),
  DSP16_Q(-0.00129777),
  DSP16_Q(0.00222438),
  DSP16_Q(-0.00348296),
  DSP16_Q(0.00514375),
  DSP16_Q(-0.00731066),
  DSP16_Q(0.01015116),
  DSP16_Q(-0.01396020),
  DSP16_Q(0.01931367),
  DSP16_Q(-0.02750141),
  DSP16_Q(0.04209549),
  DSP16_Q(-0.07782556),
  DSP16_Q(0.35477784),
  DSP16_Q(0.14237087),
  DSP16_Q(-0.05767773),
  DSP16_Q(0.03465740),
  DSP16_Q(-0.02353171),
  DSP16_Q(0.01679042),
  DSP16_Q(-0.01219411),
  DSP16_Q(0.00884581),
  DSP16_Q(-0.00631836),
  DSP16_Q(0.00438255),
  DSP16_Q(-0.00290311),
  DSP16_Q(0.00179300),
  DSP16_Q(-0.00099117),
  DSP16_Q(0.00045100),
  DSP16_Q(-0.00013398),
  DSP16_Q(0.00000618),
  DSP16_Q(0.00003953),
  DSP16_Q(-0.00024238),
  DSP16_Q(0.00064634),
  DSP16_Q(-0.00128657),
  DSP16_Q(0.00220405),
  DSP16_Q(-0.00345000),
  DSP16_Q(0.00509400),
  DSP16_Q(-0.00723896),
  DSP16_Q(0.01005084),
  DSP16_Q(-0.01382210),
  DSP16_Q(0.01912395),
  DSP16_Q(-0.02723684),
  DSP16_Q(0.04171114),
  DSP16_Q(-0.07722594),
  DSP16_Q(0.35638424),
  DSP16_Q(0.13986130),
  DSP16_Q(-0.05686480),
  DSP16_Q(0.03419857),
  DSP16_Q(-0.02322800),
  DSP16_Q(0.01657598),
  DSP16_Q(-0.01203875),
  DSP16_Q(0.00873277),
  DSP16_Q(-0.00623699),
  DSP16_Q(0.00432539),
  DSP16_Q(-0.00286452),
  DSP16_Q(0.00176847),
  DSP16_Q(-0.00097698),
  DSP16_Q(0.00044400),
  DSP16_Q(-0.00013149),
  DSP16_Q(0.00000592),
  DSP16_Q(0.00003957),
  DSP16_Q(-0.00024089),
  DSP16_Q(0.00064112),
  DSP16_Q(-0.00127503),
  DSP16_Q(0.00218317),
  DSP16_Q(-0.00341621),
  DSP16_Q(0.00504303),
  DSP16_Q(-0.00716555),
  DSP16_Q(0.00994818),
  DSP16_Q(-0.01368077),
  DSP16_Q(0.01892975),
  DSP16_Q(-0.02696580),
  DSP16_Q(0.04131657),
  DSP16_Q(-0.07660615),
  DSP16_Q(0.35796929),
  DSP16_Q(0.13735527),
  DSP16_Q(-0.05604488),
  DSP16_Q(0.03373456),
  DSP16_Q(-0.02292053),
  DSP16_Q(0.01635879),
  DSP16_Q(-0.01188139),
  DSP16_Q(0.00861828),
  DSP16_Q(-0.00615460),
  DSP16_Q(0.00426753),
  DSP16_Q(-0.00282548),
  DSP16_Q(0.00174369),
  DSP16_Q(-0.00096267),
  DSP16_Q(0.00043697),
  DSP16_Q(-0.00012900),
  DSP16_Q(0.00000566),
  DSP16_Q(0.00003960),
  DSP16_Q(-0.00023933),
  DSP16_Q(0.00063572),
  DSP16_Q(-0.00126316),
  DSP16_Q(0.00216174),
  DSP16_Q(-0.00338159),
  DSP16_Q(0.00499085),
  DSP16_Q(-0.00709045),
  DSP16_Q(0.00984319),
  DSP16_Q(-0.01353623),
  DSP16_Q(0.01873111),
  DSP16_Q(-0.02668832),
  DSP16_Q(0.04091182),
  DSP16_Q(-0.07596616),
  DSP16_Q(0.35953281),
  DSP16_Q(0.13485307),
  DSP16_Q(-0.05521818),
  DSP16_Q(0.03326550),
  DSP16_Q(-0.02260939),
  DSP16_Q(0.01613891),
  DSP16_Q(-0.01172205),
  DSP16_Q(0.00850236),
  DSP16_Q(-0.00607120),
  DSP16_Q(0.00420900),
  DSP16_Q(-0.00278601),
  DSP16_Q(0.00171866),
  DSP16_Q(-0.00094824),
  DSP16_Q(0.00042989),
  DSP16_Q(-0.00012652),
  DSP16_Q(0.00000541),
  DSP16_Q(0.00003962),
  DSP16_Q(-0.00023769),
  DSP16_Q(0.00063013),
  DSP16_Q(-0.00125095),
  DSP16_Q(0.00213976),
  DSP16_Q(-0.00334613),
  DSP16_Q(0.00493748),
  DSP16_Q(-0.00701366),
  DSP16_Q(0.00973587),
  DSP16_Q(-0.01338852),
  DSP16_Q(0.01852803),
  DSP16_Q(-0.02640445),
  DSP16_Q(0.04049692),
  DSP16_Q(-0.07530592),
  DSP16_Q(0.36107460),
  DSP16_Q(0.13235494),
  DSP16_Q(-0.05438487),
  DSP16_Q(0.03279149),
  DSP16_Q(-0.02229465),
  DSP16_Q(0.01591639),
  DSP16_Q(-0.01156078),
  DSP16_Q(0.00838505),
  DSP16_Q(-0.00598683),
  DSP16_Q(0.00414981),
  DSP16_Q(-0.00274613),
  DSP16_Q(0.00169338),
  DSP16_Q(-0.00093369),
  DSP16_Q(0.00042278),
  DSP16_Q(-0.00012403),
  DSP16_Q(0.00000516),
  DSP16_Q(0.00003961),
  DSP16_Q(-0.00023597),
  DSP16_Q(0.00062437),
  DSP16_Q(-0.00123842),
  DSP16_Q(0.00211724),
  DSP16_Q(-0.00330985),
  DSP16_Q(0.00488290),
  DSP16_Q(-0.00693520),
  DSP16_Q(0.00962625),
  DSP16_Q(-0.01323764),
  DSP16_Q(0.01832056),
  DSP16_Q(-0.02611421),
  DSP16_Q(0.04007192),
  DSP16_Q(-0.07462542),
  DSP16_Q(0.36259449),
  DSP16_Q(0.12986117),
  DSP16_Q(-0.05354517),
  DSP16_Q(0.03231265),
  DSP16_Q(-0.02197638),
  DSP16_Q(0.01569128),
  DSP16_Q(-0.01139762),
  DSP16_Q(0.00826637),
  DSP16_Q(-0.00590150),
  DSP16_Q(0.00408997),
  DSP16_Q(-0.00270584),
  DSP16_Q(0.00166788),
  DSP16_Q(-0.00091903),
  DSP16_Q(0.00041563),
  DSP16_Q(-0.00012155),
  DSP16_Q(0.00000492),
  DSP16_Q(0.00003959),
  DSP16_Q(-0.00023417),
  DSP16_Q(0.00061843),
  DSP16_Q(-0.00122555),
  DSP16_Q(0.00209417),
  DSP16_Q(-0.00327274),
  DSP16_Q(0.00482715),
  DSP16_Q(-0.00685507),
  DSP16_Q(0.00951434),
  DSP16_Q(-0.01308362),
  DSP16_Q(0.01810872),
  DSP16_Q(-0.02581764),
  DSP16_Q(0.03963684),
  DSP16_Q(-0.07392461),
  DSP16_Q(0.36409227),
  DSP16_Q(0.12737200),
  DSP16_Q(-0.05269925),
  DSP16_Q(0.03182910),
  DSP16_Q(-0.02165467),
  DSP16_Q(0.01546364),
  DSP16_Q(-0.01123261),
  DSP16_Q(0.00814636),
  DSP16_Q(-0.00581522),
  DSP16_Q(0.00402950),
  DSP16_Q(-0.00266514),
  DSP16_Q(0.00164215),
  DSP16_Q(-0.00090427),
  DSP16_Q(0.00040845),
  DSP16_Q(-0.00011907),
  DSP16_Q(0.00000469),
  DSP16_Q(0.00003955),
  DSP16_Q(-0.00023229),
  DSP16_Q(0.00061231),
  DSP16_Q(-0.00121235),
  DSP16_Q(0.00207057),
  DSP16_Q(-0.00323482),
  DSP16_Q(0.00477021),
  DSP16_Q(-0.00677330),
  DSP16_Q(0.00940015),
  DSP16_Q(-0.01292648),
  DSP16_Q(0.01789254),
  DSP16_Q(-0.02551478),
  DSP16_Q(0.03919174),
  DSP16_Q(-0.07320348),
  DSP16_Q(0.36556778),
  DSP16_Q(0.12488771),
  DSP16_Q(-0.05184733),
  DSP16_Q(0.03134094),
  DSP16_Q(-0.02132957),
  DSP16_Q(0.01523352),
  DSP16_Q(-0.01106577),
  DSP16_Q(0.00802504),
  DSP16_Q(-0.00572803),
  DSP16_Q(0.00396841),
  DSP16_Q(-0.00262406),
  DSP16_Q(0.00161619),
  DSP16_Q(-0.00088939),
  DSP16_Q(0.00040124),
  DSP16_Q(-0.00011659),
  DSP16_Q(0.00000447),
  DSP16_Q(0.00003949),
  DSP16_Q(-0.00023034),
  DSP16_Q(0.00060602),
  DSP16_Q(-0.00119882),
  DSP16_Q(0.00204643),
  DSP16_Q(-0.00319609),
  DSP16_Q(0.00471210),
  DSP16_Q(-0.00668988),
  DSP16_Q(0.00928371),
  DSP16_Q(-0.01276625),
  DSP16_Q(0.01767205),
  DSP16_Q(-0.02520567),
  DSP16_Q(0.03873664),
  DSP16_Q(-0.07246201),
  DSP16_Q(0.36702083),
  DSP16_Q(0.12240855),
  DSP16_Q(-0.05098959),
  DSP16_Q(0.03084829),
  DSP16_Q(-0.02100118),
  DSP16_Q(0.01500097),
  DSP16_Q(-0.01089716),
  DSP16_Q(0.00790243),
  DSP16_Q(-0.00563994),
  DSP16_Q(0.00390672),
  DSP16_Q(-0.00258260),
  DSP16_Q(0.00159003),
  DSP16_Q(-0.00087442),
  DSP16_Q(0.00039399),
  DSP16_Q(-0.00011412),
  DSP16_Q(0.00000425),
  DSP16_Q(0.00003942),
  DSP16_Q(-0.00022831),
  DSP16_Q(0.00059954),
  DSP16_Q(-0.00118496),
  DSP16_Q(0.00202176),
  DSP16_Q(-0.00315655),
  DSP16_Q(0.00465282),
  DSP16_Q(-0.00660484),
  DSP16_Q(0.00916504),
  DSP16_Q(-0.01260295),
  DSP16_Q(0.01744729),
  DSP16_Q(-0.02489035),
  DSP16_Q(0.03827161),
  DSP16_Q(-0.07170016),
  DSP16_Q(0.36845124),
  DSP16_Q(0.11993479),
  DSP16_Q(-0.05012623),
  DSP16_Q(0.03035127),
  DSP16_Q(-0.02066956),
  DSP16_Q(0.01476604),
  DSP16_Q(-0.01072682),
  DSP16_Q(0.00777857),
  DSP16_Q(-0.00555097),
  DSP16_Q(0.00384444),
  DSP16_Q(-0.00254077),
  DSP16_Q(0.00156365),
  DSP16_Q(-0.00085936),
  DSP16_Q(0.00038672),
  DSP16_Q(-0.00011165),
  DSP16_Q(0.00000404),
  DSP16_Q(0.00003932),
  DSP16_Q(-0.00022620),
  DSP16_Q(0.00059288),
  DSP16_Q(-0.00117078),
  DSP16_Q(0.00199655),
  DSP16_Q(-0.00311620),
  DSP16_Q(0.00459239),
  DSP16_Q(-0.00651818),
  DSP16_Q(0.00904414),
  DSP16_Q(-0.01243660),
  DSP16_Q(0.01721828),
  DSP16_Q(-0.02456886),
  DSP16_Q(0.03779668),
  DSP16_Q(-0.07091794),
  DSP16_Q(0.36985885),
  DSP16_Q(0.11746668),
  DSP16_Q(-0.04925745),
  DSP16_Q(0.02985000),
  DSP16_Q(-0.02033480),
  DSP16_Q(0.01452880),
  DSP16_Q(-0.01055477),
  DSP16_Q(0.00765350),
  DSP16_Q(-0.00546115),
  DSP16_Q(0.00378158),
  DSP16_Q(-0.00249859),
  DSP16_Q(0.00153708),
  DSP16_Q(-0.00084420),
  DSP16_Q(0.00037943),
  DSP16_Q(-0.00010919),
  DSP16_Q(0.00000383),
  DSP16_Q(0.00003921),
  DSP16_Q(-0.00022401),
  DSP16_Q(0.00058605),
  DSP16_Q(-0.00115627),
  DSP16_Q(0.00197082),
  DSP16_Q(-0.00307506),
  DSP16_Q(0.00453082),
  DSP16_Q(-0.00642992),
  DSP16_Q(0.00892104),
  DSP16_Q(-0.01226722),
  DSP16_Q(0.01698506),
  DSP16_Q(-0.02424124),
  DSP16_Q(0.03731190),
  DSP16_Q(-0.07011531),
  DSP16_Q(0.37124348),
  DSP16_Q(0.11500448),
  DSP16_Q(-0.04838344),
  DSP16_Q(0.02934458),
  DSP16_Q(-0.01999697),
  DSP16_Q(0.01428930),
  DSP16_Q(-0.01038107),
  DSP16_Q(0.00752722),
  DSP16_Q(-0.00537049),
  DSP16_Q(0.00371817),
  DSP16_Q(-0.00245605),
  DSP16_Q(0.00151031),
  DSP16_Q(-0.00082895),
  DSP16_Q(0.00037211),
  DSP16_Q(-0.00010673),
  DSP16_Q(0.00000363),
  DSP16_Q(0.00003907),
  DSP16_Q(-0.00022174),
  DSP16_Q(0.00057903),
  DSP16_Q(-0.00114143),
  DSP16_Q(0.00194457),
  DSP16_Q(-0.00303314),
  DSP16_Q(0.00446811),
  DSP16_Q(-0.00634008),
  DSP16_Q(0.00879575),
  DSP16_Q(-0.01209485),
  DSP16_Q(0.01674766),
  DSP16_Q(-0.02390754),
  DSP16_Q(0.03681733),
  DSP16_Q(-0.06929227),
  DSP16_Q(0.37260496),
  DSP16_Q(0.11254844),
  DSP16_Q(-0.04750439),
  DSP16_Q(0.02883514),
  DSP16_Q(-0.01965616),
  DSP16_Q(0.01404758),
  DSP16_Q(-0.01020575),
  DSP16_Q(0.00739978),
  DSP16_Q(-0.00527902),
  DSP16_Q(0.00365422),
  DSP16_Q(-0.00241318),
  DSP16_Q(0.00148335),
  DSP16_Q(-0.00081361),
  DSP16_Q(0.00036477),
  DSP16_Q(-0.00010428),
  DSP16_Q(0.00000344),
  DSP16_Q(0.00003892),
  DSP16_Q(-0.00021939),
  DSP16_Q(0.00057184),
  DSP16_Q(-0.00112628),
  DSP16_Q(0.00191779),
  DSP16_Q(-0.00299042),
  DSP16_Q(0.00440426),
  DSP16_Q(-0.00624866),
  DSP16_Q(0.00866830),
  DSP16_Q(-0.01191952),
  DSP16_Q(0.01650613),
  DSP16_Q(-0.02356780),
  DSP16_Q(0.03631301),
  DSP16_Q(-0.06844882),
  DSP16_Q(0.37394313),
  DSP16_Q(0.11009883),
  DSP16_Q(-0.04662051),
  DSP16_Q(0.02832179),
  DSP16_Q(-0.01931243),
  DSP16_Q(0.01380372),
  DSP16_Q(-0.01002885),
  DSP16_Q(0.00727121),
  DSP16_Q(-0.00518675),
  DSP16_Q(0.00358974),
  DSP16_Q(-0.00236998),
  DSP16_Q(0.00145621),
  DSP16_Q(-0.00079820),
  DSP16_Q(0.00035741),
  DSP16_Q(-0.00010184),
  DSP16_Q(0.00000325),
  DSP16_Q(0.00003874),
  DSP16_Q(-0.00021697),
  DSP16_Q(0.00056447),
  DSP16_Q(-0.00111080),
  DSP16_Q(0.00189049),
  DSP16_Q(-0.00294693),
  DSP16_Q(0.00433931),
  DSP16_Q(-0.00615567),
  DSP16_Q(0.00853871),
  DSP16_Q(-0.01174123),
  DSP16_Q(0.01626049),
  DSP16_Q(-0.02322208),
  DSP16_Q(0.03579901),
  DSP16_Q(-0.06758494),
  DSP16_Q(0.37525782),
  DSP16_Q(0.10765589),
  DSP16_Q(-0.04573198),
  DSP16_Q(0.02780465),
  DSP16_Q(-0.01896587),
  DSP16_Q(0.01355776),
  DSP16_Q(-0.00985042),
  DSP16_Q(0.00714153),
  DSP16_Q(-0.00509372),
  DSP16_Q(0.00352475),
  DSP16_Q(-0.00232646),
  DSP16_Q(0.00142890),
  DSP16_Q(-0.00078271),
  DSP16_Q(0.00035003),
  DSP16_Q(-0.00009941),
  DSP16_Q(0.00000307),
  DSP16_Q(0.00003855),
  DSP16_Q(-0.00021446),
  DSP16_Q(0.00055692),
  DSP16_Q(-0.00109501),
  DSP16_Q(0.00186268),
  DSP16_Q(-0.00290266),
  DSP16_Q(0.00427324),
  DSP16_Q(-0.00606115),
  DSP16_Q(0.00840699),
  DSP16_Q(-0.01156004),
  DSP16_Q(0.01601078),
  DSP16_Q(-0.02287042),
  DSP16_Q(0.03527538),
  DSP16_Q(-0.06670064),
  DSP16_Q(0.37654888),
  DSP16_Q(0.10521987),
  DSP16_Q(-0.04483900),
  DSP16_Q(0.02728383),
  DSP16_Q(-0.01861655),
  DSP16_Q(0.01330975),
  DSP16_Q(-0.00967049),
  DSP16_Q(0.00701078),
  DSP16_Q(-0.00499994),
  DSP16_Q(0.00345925),
  DSP16_Q(-0.00228264),
  DSP16_Q(0.00140142),
  DSP16_Q(-0.00076715),
  DSP16_Q(0.00034263),
  DSP16_Q(-0.00009698),
  DSP16_Q(0.00000289),
  DSP16_Q(0.00003833),
  DSP16_Q(-0.00021188),
  DSP16_Q(0.00054920),
  DSP16_Q(-0.00107889),
  DSP16_Q(0.00183436),
  DSP16_Q(-0.00285763),
  DSP16_Q(0.00420607),
  DSP16_Q(-0.00596509),
  DSP16_Q(0.00827317),
  DSP16_Q(-0.01137596),
  DSP16_Q(0.01575704),
  DSP16_Q(-0.02251286),
  DSP16_Q(0.03474217),
  DSP16_Q(-0.06579591),
  DSP16_Q(0.37781615),
  DSP16_Q(0.10279103),
  DSP16_Q(-0.04394177),
  DSP16_Q(0.02675946),
  DSP16_Q(-0.01826456),
  DSP16_Q(0.01305977),
  DSP16_Q(-0.00948910),
  DSP16_Q(0.00687898),
  DSP16_Q(-0.00490543),
  DSP16_Q(0.00339328),
  DSP16_Q(-0.00223852),
  DSP16_Q(0.00137377),
  DSP16_Q(-0.00075152),
  DSP16_Q(0.00033522),
  DSP16_Q(-0.00009456),
  DSP16_Q(0.00000273),
  DSP16_Q(0.00003810),
  DSP16_Q(-0.00020921),
  DSP16_Q(0.00054130),
  DSP16_Q(-0.00106246),
  DSP16_Q(0.00180554),
  DSP16_Q(-0.00281184),
  DSP16_Q(0.00413782),
  DSP16_Q(-0.00586751),
  DSP16_Q(0.00813727),
  DSP16_Q(-0.01118902),
  DSP16_Q(0.01549932),
  DSP16_Q(-0.02214947),
  DSP16_Q(0.03419946),
  DSP16_Q(-0.06487076),
  DSP16_Q(0.37905946),
  DSP16_Q(0.10036961),
  DSP16_Q(-0.04304048),
  DSP16_Q(0.02623165),
  DSP16_Q(-0.01790997),
  DSP16_Q(0.01280785),
  DSP16_Q(-0.00930630),
  DSP16_Q(0.00674617),
  DSP16_Q(-0.00481021),
  DSP16_Q(0.00332684),
  DSP16_Q(-0.00219411),
  DSP16_Q(0.00134598),
  DSP16_Q(-0.00073582),
  DSP16_Q(0.00032780),
  DSP16_Q(-0.00009216),
  DSP16_Q(0.00000256),
  DSP16_Q(0.00003784),
  DSP16_Q(-0.00020646),
  DSP16_Q(0.00053323),
  DSP16_Q(-0.00104572),
  DSP16_Q(0.00177621),
  DSP16_Q(-0.00276530),
  DSP16_Q(0.00406848),
  DSP16_Q(-0.00576843),
  DSP16_Q(0.00799931),
  DSP16_Q(-0.01099926),
  DSP16_Q(0.01523765),
  DSP16_Q(-0.02178030),
  DSP16_Q(0.03364730),
  DSP16_Q(-0.06392519),
  DSP16_Q(0.38027868),
  DSP16_Q(0.09795586),
  DSP16_Q(-0.04213533),
  DSP16_Q(0.02570052),
  DSP16_Q(-0.01755287),
  DSP16_Q(0.01255407),
  DSP16_Q(-0.00912213),
  DSP16_Q(0.00661237),
  DSP16_Q(-0.00471431),
  DSP16_Q(0.00325995),
  DSP16_Q(-0.00214943),
  DSP16_Q(0.00131803),
  DSP16_Q(-0.00072006),
  DSP16_Q(0.00032037),
  DSP16_Q(-0.00008976),
  DSP16_Q(0.00000241),
  DSP16_Q(0.00003756),
  DSP16_Q(-0.00020364),
  DSP16_Q(0.00052498),
  DSP16_Q(-0.00102866),
  DSP16_Q(0.00174638),
  DSP16_Q(-0.00271801),
  DSP16_Q(0.00399808),
  DSP16_Q(-0.00566787),
  DSP16_Q(0.00785931),
  DSP16_Q(-0.01080670),
  DSP16_Q(0.01497207),
  DSP16_Q(-0.02140539),
  DSP16_Q(0.03308576),
  DSP16_Q(-0.06295921),
  DSP16_Q(0.38147365),
  DSP16_Q(0.09555003),
  DSP16_Q(-0.04122650),
  DSP16_Q(0.02516619),
  DSP16_Q(-0.01719333),
  DSP16_Q(0.01229847),
  DSP16_Q(-0.00893663),
  DSP16_Q(0.00647762),
  DSP16_Q(-0.00461775),
  DSP16_Q(0.00319263),
  DSP16_Q(-0.00210449),
  DSP16_Q(0.00128995),
  DSP16_Q(-0.00070424),
  DSP16_Q(0.00031293),
  DSP16_Q(-0.00008738),
  DSP16_Q(0.00000226),
  DSP16_Q(0.00003726),
  DSP16_Q(-0.00020073),
  DSP16_Q(0.00051656),
  DSP16_Q(-0.00101130),
  DSP16_Q(0.00171605),
  DSP16_Q(-0.00266998),
  DSP16_Q(0.00392663),
  DSP16_Q(-0.00556584),
  DSP16_Q(0.00771730),
  DSP16_Q(-0.01061137),
  DSP16_Q(0.01470262),
  DSP16_Q(-0.02102481),
  DSP16_Q(0.03251491),
  DSP16_Q(-0.06197284),
  DSP16_Q(0.38264422),
  DSP16_Q(0.09315236),
  DSP16_Q(-0.04031419),
  DSP16_Q(0.02462878),
  DSP16_Q(-0.01683143),
  DSP16_Q(0.01204111),
  DSP16_Q(-0.00874984),
  DSP16_Q(0.00634194),
  DSP16_Q(-0.00452055),
  DSP16_Q(0.00312488),
  DSP16_Q(-0.00205929),
  DSP16_Q(0.00126173),
  DSP16_Q(-0.00068837),
  DSP16_Q(0.00030548),
  DSP16_Q(-0.00008500),
  DSP16_Q(0.00000211),
  DSP16_Q(0.00003693),
  DSP16_Q(-0.00019775),
  DSP16_Q(0.00050796),
  DSP16_Q(-0.00099363),
  DSP16_Q(0.00168524),
  DSP16_Q(-0.00262122),
  DSP16_Q(0.00385413),
  DSP16_Q(-0.00546236),
  DSP16_Q(0.00757329),
  DSP16_Q(-0.01041331),
  DSP16_Q(0.01442935),
  DSP16_Q(-0.02063861),
  DSP16_Q(0.03193481),
  DSP16_Q(-0.06096608),
  DSP16_Q(0.38379026),
  DSP16_Q(0.09076309),
  DSP16_Q(-0.03939860),
  DSP16_Q(0.02408840),
  DSP16_Q(-0.01646726),
  DSP16_Q(0.01178205),
  DSP16_Q(-0.00856180),
  DSP16_Q(0.00620537),
  DSP16_Q(-0.00442273),
  DSP16_Q(0.00305673),
  DSP16_Q(-0.00201384),
  DSP16_Q(0.00123338),
  DSP16_Q(-0.00067245),
  DSP16_Q(0.00029803),
  DSP16_Q(-0.00008264),
  DSP16_Q(0.00000198),
  DSP16_Q(0.00003659),
  DSP16_Q(-0.00019468),
  DSP16_Q(0.00049920),
  DSP16_Q(-0.00097565),
  DSP16_Q(0.00165394),
  DSP16_Q(-0.00257174),
  DSP16_Q(0.00378059),
  DSP16_Q(-0.00535744),
  DSP16_Q(0.00742732),
  DSP16_Q(-0.01021255),
  DSP16_Q(0.01415230),
  DSP16_Q(-0.02024684),
  DSP16_Q(0.03134555),
  DSP16_Q(-0.05993896),
  DSP16_Q(0.38491162),
  DSP16_Q(0.08838246),
  DSP16_Q(-0.03847992),
  DSP16_Q(0.02354517),
  DSP16_Q(-0.01610090),
  DSP16_Q(0.01152136),
  DSP16_Q(-0.00837256),
  DSP16_Q(0.00606794),
  DSP16_Q(-0.00432432),
  DSP16_Q(0.00298820),
  DSP16_Q(-0.00196817),
  DSP16_Q(0.00120492),
  DSP16_Q(-0.00065649),
  DSP16_Q(0.00029058),
  DSP16_Q(-0.00008030),
  DSP16_Q(0.00000184),
  DSP16_Q(0.00003622),
  DSP16_Q(-0.00019154),
  DSP16_Q(0.00049026),
  DSP16_Q(-0.00095737),
  DSP16_Q(0.00162216),
  DSP16_Q(-0.00252154),
  DSP16_Q(0.00370604),
  DSP16_Q(-0.00525111),
  DSP16_Q(0.00727940),
  DSP16_Q(-0.01000913),
  DSP16_Q(0.01387151),
  DSP16_Q(-0.01984958),
  DSP16_Q(0.03074718),
  DSP16_Q(-0.05889150),
  DSP16_Q(0.38600816),
  DSP16_Q(0.08601071),
  DSP16_Q(-0.03755833),
  DSP16_Q(0.02299921),
  DSP16_Q(-0.01573242),
  DSP16_Q(0.01125907),
  DSP16_Q(-0.00818215),
  DSP16_Q(0.00592967),
  DSP16_Q(-0.00422533),
  DSP16_Q(0.00291928),
  DSP16_Q(-0.00192227),
  DSP16_Q(0.00117634),
  DSP16_Q(-0.00064048),
  DSP16_Q(0.00028312),
  DSP16_Q(-0.00007796),
  DSP16_Q(0.00000172),
  DSP16_Q(0.00003583),
  DSP16_Q(-0.00018831),
  DSP16_Q(0.00048115),
  DSP16_Q(-0.00093879),
  DSP16_Q(0.00158990),
  DSP16_Q(-0.00247063),
  DSP16_Q(0.00363048),
  DSP16_Q(-0.00514337),
  DSP16_Q(0.00712957),
  DSP16_Q(-0.00980307),
  DSP16_Q(0.01358704),
  DSP16_Q(-0.01944687),
  DSP16_Q(0.03013979),
  DSP16_Q(-0.05782371),
  DSP16_Q(0.38707976),
  DSP16_Q(0.08364807),
  DSP16_Q(-0.03663404),
  DSP16_Q(0.02245065),
  DSP16_Q(-0.01536191),
  DSP16_Q(0.01099527),
  DSP16_Q(-0.00799062),
  DSP16_Q(0.00579060),
  DSP16_Q(-0.00412580),
  DSP16_Q(0.00285001),
  DSP16_Q(-0.00187616),
  DSP16_Q(0.00114765),
  DSP16_Q(-0.00062444),
  DSP16_Q(0.00027567),
  DSP16_Q(-0.00007564),
  DSP16_Q(0.00000159),
  DSP16_Q(0.00003541),
  DSP16_Q(-0.00018501),
  DSP16_Q(0.00047187),
  DSP16_Q(-0.00091991),
  DSP16_Q(0.00155717),
  DSP16_Q(-0.00241902),
  DSP16_Q(0.00355392),
  DSP16_Q(-0.00503425),
  DSP16_Q(0.00697784),
  DSP16_Q(-0.00959441),
  DSP16_Q(0.01329892),
  DSP16_Q(-0.01903879),
  DSP16_Q(0.02952346),
  DSP16_Q(-0.05673564),
  DSP16_Q(0.38812628),
  DSP16_Q(0.08129479),
  DSP16_Q(-0.03570723),
  DSP16_Q(0.02189960),
  DSP16_Q(-0.01498944),
  DSP16_Q(0.01072999),
  DSP16_Q(-0.00779802),
  DSP16_Q(0.00565077),
  DSP16_Q(-0.00402573),
  DSP16_Q(0.00278040),
  DSP16_Q(-0.00182984),
  DSP16_Q(0.00111886),
  DSP16_Q(-0.00060836),
  DSP16_Q(0.00026822),
  DSP16_Q(-0.00007334),
  DSP16_Q(0.00000148),
  DSP16_Q(0.00003497),
  DSP16_Q(-0.00018162),
  DSP16_Q(0.00046242),
  DSP16_Q(-0.00090074),
  DSP16_Q(0.00152398),
  DSP16_Q(-0.00236672),
  DSP16_Q(0.00347637),
  DSP16_Q(-0.00492377),
  DSP16_Q(0.00682424),
  DSP16_Q(-0.00938318),
  DSP16_Q(0.01300721),
  DSP16_Q(-0.01862539),
  DSP16_Q(0.02889826),
  DSP16_Q(-0.05562730),
  DSP16_Q(0.38914759),
  DSP16_Q(0.07895109),
  DSP16_Q(-0.03477810),
  DSP16_Q(0.02134618),
  DSP16_Q(-0.01461511),
  DSP16_Q(0.01046330),
  DSP16_Q(-0.00760437),
  DSP16_Q(0.00551019),
  DSP16_Q(-0.00392516),
  DSP16_Q(0.00271046),
  DSP16_Q(-0.00178334),
  DSP16_Q(0.00108997),
  DSP16_Q(-0.00059225),
  DSP16_Q(0.00026077),
  DSP16_Q(-0.00007105),
  DSP16_Q(0.00000137),
  DSP16_Q(0.00003451),
  DSP16_Q(-0.00017816),
  DSP16_Q(0.00045280),
  DSP16_Q(-0.00088127),
  DSP16_Q(0.00149032),
  DSP16_Q(-0.00231374),
  DSP16_Q(0.00339786),
  DSP16_Q(-0.00481194),
  DSP16_Q(0.00666880),
  DSP16_Q(-0.00916943),
  DSP16_Q(0.01271195),
  DSP16_Q(-0.01820673),
  DSP16_Q(0.02826428),
  DSP16_Q(-0.05449872),
  DSP16_Q(0.39014357),
  DSP16_Q(0.07661721),
  DSP16_Q(-0.03384683),
  DSP16_Q(0.02079050),
  DSP16_Q(-0.01423899),
  DSP16_Q(0.01019526),
  DSP16_Q(-0.00740974),
  DSP16_Q(0.00536890),
  DSP16_Q(-0.00382410),
  DSP16_Q(0.00264021),
  DSP16_Q(-0.00173665),
  DSP16_Q(0.00106100),
  DSP16_Q(-0.00057611),
  DSP16_Q(0.00025333),
  DSP16_Q(-0.00006877),
  DSP16_Q(0.00000126),
  DSP16_Q(0.00003403),
  DSP16_Q(-0.00017461),
  DSP16_Q(0.00044302),
  DSP16_Q(-0.00086151),
  DSP16_Q(0.00145620),
  DSP16_Q(-0.00226008),
  DSP16_Q(0.00331838),
  DSP16_Q(-0.00469879),
  DSP16_Q(0.00651154),
  DSP16_Q(-0.00895319),
  DSP16_Q(0.01241319),
  DSP16_Q(-0.01778289),
  DSP16_Q(0.02762161),
  DSP16_Q(-0.05334996),
  DSP16_Q(0.39111409),
  DSP16_Q(0.07429337),
  DSP16_Q(-0.03291361),
  DSP16_Q(0.02023270),
  DSP16_Q(-0.01386116),
  DSP16_Q(0.00992593),
  DSP16_Q(-0.00721415),
  DSP16_Q(0.00522694),
  DSP16_Q(-0.00372258),
  DSP16_Q(0.00256967),
  DSP16_Q(-0.00168980),
  DSP16_Q(0.00103195),
  DSP16_Q(-0.00055995),
  DSP16_Q(0.00024589),
  DSP16_Q(-0.00006651),
  DSP16_Q(0.00000117),
  DSP16_Q(0.00003352),
  DSP16_Q(-0.00017099),
  DSP16_Q(0.00043307),
  DSP16_Q(-0.00084147),
  DSP16_Q(0.00142164),
  DSP16_Q(-0.00220575),
  DSP16_Q(0.00323797),
  DSP16_Q(-0.00458433),
  DSP16_Q(0.00635250),
  DSP16_Q(-0.00873449),
  DSP16_Q(0.01211099),
  DSP16_Q(-0.01735393),
  DSP16_Q(0.02697031),
  DSP16_Q(-0.05218103),
  DSP16_Q(0.39205904),
  DSP16_Q(0.07197981),
  DSP16_Q(-0.03197864),
  DSP16_Q(0.01967288),
  DSP16_Q(-0.01348170),
  DSP16_Q(0.00965536),
  DSP16_Q(-0.00701765),
  DSP16_Q(0.00508433),
  DSP16_Q(-0.00362062),
  DSP16_Q(0.00249885),
  DSP16_Q(-0.00164278),
  DSP16_Q(0.00100282),
  DSP16_Q(-0.00054377),
  DSP16_Q(0.00023847),
  DSP16_Q(-0.00006427),
  DSP16_Q(0.00000107),
  DSP16_Q(0.00003298),
  DSP16_Q(-0.00016729),
  DSP16_Q(0.00042296),
  DSP16_Q(-0.00082114),
  DSP16_Q(0.00138663),
  DSP16_Q(-0.00215077),
  DSP16_Q(0.00315662),
  DSP16_Q(-0.00446858),
  DSP16_Q(0.00619170),
  DSP16_Q(-0.00851337),
  DSP16_Q(0.01180539),
  DSP16_Q(-0.01691992),
  DSP16_Q(0.02631050),
  DSP16_Q(-0.05099199),
  DSP16_Q(0.39297830),
  DSP16_Q(0.06967675),
  DSP16_Q(-0.03104211),
  DSP16_Q(0.01911116),
  DSP16_Q(-0.01310070),
  DSP16_Q(0.00938362),
  DSP16_Q(-0.00682029),
  DSP16_Q(0.00494110),
  DSP16_Q(-0.00351824),
  DSP16_Q(0.00242776),
  DSP16_Q(-0.00159562),
  DSP16_Q(0.00097363),
  DSP16_Q(-0.00052758),
  DSP16_Q(0.00023106),
  DSP16_Q(-0.00006204),
  DSP16_Q(0.00000098),
  DSP16_Q(0.00003242),
  DSP16_Q(-0.00016350),
  DSP16_Q(0.00041269),
  DSP16_Q(-0.00080053),
  DSP16_Q(0.00135117),
  DSP16_Q(-0.00209514),
  DSP16_Q(0.00307435),
  DSP16_Q(-0.00435156),
  DSP16_Q(0.00602916),
  DSP16_Q(-0.00828987),
  DSP16_Q(0.01149644),
  DSP16_Q(-0.01648092),
  DSP16_Q(0.02564224),
  DSP16_Q(-0.04978287),
  DSP16_Q(0.39387176),
  DSP16_Q(0.06738441),
  DSP16_Q(-0.03010419),
  DSP16_Q(0.01854767),
  DSP16_Q(-0.01271824),
  DSP16_Q(0.00911075),
  DSP16_Q(-0.00662210),
  DSP16_Q(0.00479729),
  DSP16_Q(-0.00341547),
  DSP16_Q(0.00235643),
  DSP16_Q(-0.00154832),
  DSP16_Q(0.00094437),
  DSP16_Q(-0.00051137),
  DSP16_Q(0.00022366),
  DSP16_Q(-0.00005984),
  DSP16_Q(0.00000090),
  DSP16_Q(0.00003184),
  DSP16_Q(-0.00015964),
  DSP16_Q(0.00040225),
  DSP16_Q(-0.00077964),
  DSP16_Q(0.00131529),
  DSP16_Q(-0.00203887),
  DSP16_Q(0.00299118),
  DSP16_Q(-0.00423330),
  DSP16_Q(0.00586491),
  DSP16_Q(-0.00806404),
  DSP16_Q(0.01118419),
  DSP16_Q(-0.01603702),
  DSP16_Q(0.02496564),
  DSP16_Q(-0.04855373),
  DSP16_Q(0.39473931),
  DSP16_Q(0.06510301),
  DSP16_Q(-0.02916508),
  DSP16_Q(0.01798253),
  DSP16_Q(-0.01233439),
  DSP16_Q(0.00883683),
  DSP16_Q(-0.00642313),
  DSP16_Q(0.00465293),
  DSP16_Q(-0.00331233),
  DSP16_Q(0.00228486),
  DSP16_Q(-0.00150089),
  DSP16_Q(0.00091506),
  DSP16_Q(-0.00049515),
  DSP16_Q(0.00021628),
  DSP16_Q(-0.00005765),
  DSP16_Q(0.00000082),
  DSP16_Q(0.00003123),
  DSP16_Q(-0.00015570),
  DSP16_Q(0.00039165),
  DSP16_Q(-0.00075847),
  DSP16_Q(0.00127897),
  DSP16_Q(-0.00198197),
  DSP16_Q(0.00290712),
  DSP16_Q(-0.00411381),
  DSP16_Q(0.00569900),
  DSP16_Q(-0.00783590),
  DSP16_Q(0.01086871),
  DSP16_Q(-0.01558827),
  DSP16_Q(0.02428079),
  DSP16_Q(-0.04730461),
  DSP16_Q(0.39558085),
  DSP16_Q(0.06283278),
  DSP16_Q(-0.02822496),
  DSP16_Q(0.01741585),
  DSP16_Q(-0.01194925),
  DSP16_Q(0.00856191),
  DSP16_Q(-0.00622343),
  DSP16_Q(0.00450804),
  DSP16_Q(-0.00320884),
  DSP16_Q(0.00221307),
  DSP16_Q(-0.00145334),
  DSP16_Q(0.00088570),
  DSP16_Q(-0.00047893),
  DSP16_Q(0.00020891),
  DSP16_Q(-0.00005548),
  DSP16_Q(0.00000074),
  DSP16_Q(0.00003060),
  DSP16_Q(-0.00015168),
  DSP16_Q(0.00038089),
  DSP16_Q(-0.00073704),
  DSP16_Q(0.00124223),
  DSP16_Q(-0.00192445),
  DSP16_Q(0.00282219),
  DSP16_Q(-0.00399311),
  DSP16_Q(0.00553143),
  DSP16_Q(-0.00760550),
  DSP16_Q(0.01055004),
  DSP16_Q(-0.01513476),
  DSP16_Q(0.02358779),
  DSP16_Q(-0.04603557),
  DSP16_Q(0.39639626),
  DSP16_Q(0.06057392),
  DSP16_Q(-0.02728403),
  DSP16_Q(0.01684775),
  DSP16_Q(-0.01156289),
  DSP16_Q(0.00828605),
  DSP16_Q(-0.00602303),
  DSP16_Q(0.00436267),
  DSP16_Q(-0.00310502),
  DSP16_Q(0.00214109),
  DSP16_Q(-0.00140569),
  DSP16_Q(0.00085630),
  DSP16_Q(-0.00046271),
  DSP16_Q(0.00020156),
  DSP16_Q(-0.00005332),
  DSP16_Q(0.00000067),
  DSP16_Q(0.00002994),
  DSP16_Q(-0.00014759),
  DSP16_Q(0.00036998),
  DSP16_Q(-0.00071533),
  DSP16_Q(0.00120507),
  DSP16_Q(-0.00186632),
  DSP16_Q(0.00273639),
  DSP16_Q(-0.00387123),
  DSP16_Q(0.00536225),
  DSP16_Q(-0.00737287),
  DSP16_Q(0.01022824),
  DSP16_Q(-0.01467656),
  DSP16_Q(0.02288673),
  DSP16_Q(-0.04474667),
  DSP16_Q(0.39718545),
  DSP16_Q(0.05832667),
  DSP16_Q(-0.02634245),
  DSP16_Q(0.01627834),
  DSP16_Q(-0.01117540),
  DSP16_Q(0.00800930),
  DSP16_Q(-0.00582198),
  DSP16_Q(0.00421683),
  DSP16_Q(-0.00300089),
  DSP16_Q(0.00206892),
  DSP16_Q(-0.00135794),
  DSP16_Q(0.00082686),
  DSP16_Q(-0.00044649),
  DSP16_Q(0.00019423),
  DSP16_Q(-0.00005119),
  DSP16_Q(0.00000061),
  DSP16_Q(0.00002925),
  DSP16_Q(-0.00014341),
  DSP16_Q(0.00035891),
  DSP16_Q(-0.00069336),
  DSP16_Q(0.00116750),
  DSP16_Q(-0.00180759),
  DSP16_Q(0.00264976),
  DSP16_Q(-0.00374819),
  DSP16_Q(0.00519147),
  DSP16_Q(-0.00713807),
  DSP16_Q(0.00990336),
  DSP16_Q(-0.01421374),
  DSP16_Q(0.02217771),
  DSP16_Q(-0.04343796),
  DSP16_Q(0.39794832),
  DSP16_Q(0.05609122),
  DSP16_Q(-0.02540043),
  DSP16_Q(0.01570776),
  DSP16_Q(-0.01078685),
  DSP16_Q(0.00773173),
  DSP16_Q(-0.00562032),
  DSP16_Q(0.00407057),
  DSP16_Q(-0.00289648),
  DSP16_Q(0.00199658),
  DSP16_Q(-0.00131010),
  DSP16_Q(0.00079740),
  DSP16_Q(-0.00043028),
  DSP16_Q(0.00018692),
  DSP16_Q(-0.00004908),
  DSP16_Q(0.00000055),
  DSP16_Q(0.00002854),
  DSP16_Q(-0.00013916),
  DSP16_Q(0.00034768),
  DSP16_Q(-0.00067113),
  DSP16_Q(0.00112953),
  DSP16_Q(-0.00174827),
  DSP16_Q(0.00256229),
  DSP16_Q(-0.00362400),
  DSP16_Q(0.00501915),
  DSP16_Q(-0.00690113),
  DSP16_Q(0.00957547),
  DSP16_Q(-0.01374637),
  DSP16_Q(0.02146084),
  DSP16_Q(-0.04210950),
  DSP16_Q(0.39868478),
  DSP16_Q(0.05386779),
  DSP16_Q(-0.02445814),
  DSP16_Q(0.01513611),
  DSP16_Q(-0.01039732),
  DSP16_Q(0.00745339),
  DSP16_Q(-0.00541809),
  DSP16_Q(0.00392391),
  DSP16_Q(-0.00279182),
  DSP16_Q(0.00192409),
  DSP16_Q(-0.00126219),
  DSP16_Q(0.00076791),
  DSP16_Q(-0.00041408),
  DSP16_Q(0.00017964),
  DSP16_Q(-0.00004699),
  DSP16_Q(0.00000049),
  DSP16_Q(0.00002781),
  DSP16_Q(-0.00013483),
  DSP16_Q(0.00033630),
  DSP16_Q(-0.00064863),
  DSP16_Q(0.00109116),
  DSP16_Q(-0.00168837),
  DSP16_Q(0.00247401),
  DSP16_Q(-0.00349870),
  DSP16_Q(0.00484529),
  DSP16_Q(-0.00666209),
  DSP16_Q(0.00924461),
  DSP16_Q(-0.01327455),
  DSP16_Q(0.02073621),
  DSP16_Q(-0.04076136),
  DSP16_Q(0.39939474),
  DSP16_Q(0.05165660),
  DSP16_Q(-0.02351577),
  DSP16_Q(0.01456352),
  DSP16_Q(-0.01000691),
  DSP16_Q(0.00717434),
  DSP16_Q(-0.00521534),
  DSP16_Q(0.00377689),
  DSP16_Q(-0.00268691),
  DSP16_Q(0.00185145),
  DSP16_Q(-0.00121421),
  DSP16_Q(0.00073841),
  DSP16_Q(-0.00039789),
  DSP16_Q(0.00017238),
  DSP16_Q(-0.00004491),
  DSP16_Q(0.00000043),
  DSP16_Q(0.00002705),
  DSP16_Q(-0.00013042),
  DSP16_Q(0.00032477),
  DSP16_Q(-0.00062588),
  DSP16_Q(0.00105240),
  DSP16_Q(-0.00162790),
  DSP16_Q(0.00238492),
  DSP16_Q(-0.00337230),
  DSP16_Q(0.00466994),
  DSP16_Q(-0.00642100),
  DSP16_Q(0.00891085),
  DSP16_Q(-0.01279834),
  DSP16_Q(0.02000394),
  DSP16_Q(-0.03939361),
  DSP16_Q(0.40007811),
  DSP16_Q(0.04945784),
  DSP16_Q(-0.02257349),
  DSP16_Q(0.01399009),
  DSP16_Q(-0.00961569),
  DSP16_Q(0.00689465),
  DSP16_Q(-0.00501211),
  DSP16_Q(0.00362953),
  DSP16_Q(-0.00258179),
  DSP16_Q(0.00177870),
  DSP16_Q(-0.00116618),
  DSP16_Q(0.00070890),
  DSP16_Q(-0.00038172),
  DSP16_Q(0.00016514),
  DSP16_Q(-0.00004286),
  DSP16_Q(0.00000039),
  DSP16_Q(0.00002626),
  DSP16_Q(-0.00012594),
  DSP16_Q(0.00031309),
  DSP16_Q(-0.00060289),
  DSP16_Q(0.00101325),
  DSP16_Q(-0.00156687),
  DSP16_Q(0.00229506),
  DSP16_Q(-0.00324482),
  DSP16_Q(0.00449313),
  DSP16_Q(-0.00617790),
  DSP16_Q(0.00857424),
  DSP16_Q(-0.01231783),
  DSP16_Q(0.01926412),
  DSP16_Q(-0.03800632),
  DSP16_Q(0.40073481),
  DSP16_Q(0.04727172),
  DSP16_Q(-0.02163150),
  DSP16_Q(0.01341596),
  DSP16_Q(-0.00922374),
  DSP16_Q(0.00661437),
  DSP16_Q(-0.00480844),
  DSP16_Q(0.00348186),
  DSP16_Q(-0.00247648),
  DSP16_Q(0.00170584),
  DSP16_Q(-0.00111810),
  DSP16_Q(0.00067939),
  DSP16_Q(-0.00036557),
  DSP16_Q(0.00015793),
  DSP16_Q(-0.00004083),
  DSP16_Q(0.00000034),
  DSP16_Q(0.00002544),
  DSP16_Q(-0.00012138),
  DSP16_Q(0.00030125),
  DSP16_Q(-0.00057964),
  DSP16_Q(0.00097372),
  DSP16_Q(-0.00150530),
  DSP16_Q(0.00220443),
  DSP16_Q(-0.00311630),
  DSP16_Q(0.00431489),
  DSP16_Q(-0.00593283),
  DSP16_Q(0.00823485),
  DSP16_Q(-0.01183310),
  DSP16_Q(0.01851688),
  DSP16_Q(-0.03659956),
  DSP16_Q(0.40136476),
  DSP16_Q(0.04509845),
  DSP16_Q(-0.02068996),
  DSP16_Q(0.01284123),
  DSP16_Q(-0.00883115),
  DSP16_Q(0.00633356),
  DSP16_Q(-0.00460438),
  DSP16_Q(0.00333393),
  DSP16_Q(-0.00237100),
  DSP16_Q(0.00163288),
  DSP16_Q(-0.00106999),
  DSP16_Q(0.00064988),
  DSP16_Q(-0.00034944),
  DSP16_Q(0.00015076),
  DSP16_Q(-0.00003883),
  DSP16_Q(0.00000030),
  DSP16_Q(0.00002460),
  DSP16_Q(-0.00011675),
  DSP16_Q(0.00028928),
  DSP16_Q(-0.00055615),
  DSP16_Q(0.00093382),
  DSP16_Q(-0.00144318),
  DSP16_Q(0.00211305),
  DSP16_Q(-0.00298674),
  DSP16_Q(0.00413524),
  DSP16_Q(-0.00568584),
  DSP16_Q(0.00789273),
  DSP16_Q(-0.01134422),
  DSP16_Q(0.01776231),
  DSP16_Q(-0.03517341),
  DSP16_Q(0.40196787),
  DSP16_Q(0.04293822),
  DSP16_Q(-0.01974905),
  DSP16_Q(0.01226603),
  DSP16_Q(-0.00843799),
  DSP16_Q(0.00605227),
  DSP16_Q(-0.00439996),
  DSP16_Q(0.00318576),
  DSP16_Q(-0.00226536),
  DSP16_Q(0.00155985),
  DSP16_Q(-0.00102185),
  DSP16_Q(0.00062037),
  DSP16_Q(-0.00033334),
  DSP16_Q(0.00014361),
  DSP16_Q(-0.00003684),
  DSP16_Q(0.00000026),
  DSP16_Q(0.00002373),
  DSP16_Q(-0.00011204),
  DSP16_Q(0.00027715),
  DSP16_Q(-0.00053241),
  DSP16_Q(0.00089355),
  DSP16_Q(-0.00138054),
  DSP16_Q(0.00202094),
  DSP16_Q(-0.00285619),
  DSP16_Q(0.00395423),
  DSP16_Q(-0.00543697),
  DSP16_Q(0.00754794),
  DSP16_Q(-0.01085129),
  DSP16_Q(0.01700054),
  DSP16_Q(-0.03372795),
  DSP16_Q(0.40254408),
  DSP16_Q(0.04079123),
  DSP16_Q(-0.01880896),
  DSP16_Q(0.01169046),
  DSP16_Q(-0.00804434),
  DSP16_Q(0.00577057),
  DSP16_Q(-0.00419524),
  DSP16_Q(0.00303737),
  DSP16_Q(-0.00215961),
  DSP16_Q(0.00148676),
  DSP16_Q(-0.00097370),
  DSP16_Q(0.00059089),
  DSP16_Q(-0.00031727),
  DSP16_Q(0.00013650),
  DSP16_Q(-0.00003488),
  DSP16_Q(0.00000023),
  DSP16_Q(0.00002284),
  DSP16_Q(-0.00010726),
  DSP16_Q(0.00026489),
  DSP16_Q(-0.00050845),
  DSP16_Q(0.00085293),
  DSP16_Q(-0.00131739),
  DSP16_Q(0.00192810),
  DSP16_Q(-0.00272465),
  DSP16_Q(0.00377189),
  DSP16_Q(-0.00518626),
  DSP16_Q(0.00720056),
  DSP16_Q(-0.01035438),
  DSP16_Q(0.01623167),
  DSP16_Q(-0.03226325),
  DSP16_Q(0.40309332),
  DSP16_Q(0.03865768),
  DSP16_Q(-0.01786987),
  DSP16_Q(0.01111465),
  DSP16_Q(-0.00765030),
  DSP16_Q(0.00548852),
  DSP16_Q(-0.00399025),
  DSP16_Q(0.00288881),
  DSP16_Q(-0.00205374),
  DSP16_Q(0.00141362),
  DSP16_Q(-0.00092555),
  DSP16_Q(0.00056143),
  DSP16_Q(-0.00030124),
  DSP16_Q(0.00012941),
  DSP16_Q(-0.00003293),
  DSP16_Q(0.00000019),
  DSP16_Q(0.00002192),
  DSP16_Q(-0.00010240),
  DSP16_Q(0.00025248),
  DSP16_Q(-0.00048425),
  DSP16_Q(0.00081196),
  DSP16_Q(-0.00125373),
  DSP16_Q(0.00183457),
  DSP16_Q(-0.00259216),
  DSP16_Q(0.00358825),
  DSP16_Q(-0.00493378),
  DSP16_Q(0.00685063),
  DSP16_Q(-0.00985359),
  DSP16_Q(0.01545583),
  DSP16_Q(-0.03077942),
  DSP16_Q(0.40361551),
  DSP16_Q(0.03653777),
  DSP16_Q(-0.01693193),
  DSP16_Q(0.01053870),
  DSP16_Q(-0.00725593),
  DSP16_Q(0.00520617),
  DSP16_Q(-0.00378503),
  DSP16_Q(0.00274010),
  DSP16_Q(-0.00194780),
  DSP16_Q(0.00134046),
  DSP16_Q(-0.00087740),
  DSP16_Q(0.00053199),
  DSP16_Q(-0.00028524),
  DSP16_Q(0.00012237),
  DSP16_Q(-0.00003102),
  DSP16_Q(0.00000016),
  DSP16_Q(0.00002097),
  DSP16_Q(-0.00009747),
  DSP16_Q(0.00023993),
  DSP16_Q(-0.00045982),
  DSP16_Q(0.00077064),
  DSP16_Q(-0.00118957),
  DSP16_Q(0.00174036),
  DSP16_Q(-0.00245873),
  DSP16_Q(0.00340334),
  DSP16_Q(-0.00467955),
  DSP16_Q(0.00649822),
  DSP16_Q(-0.00934900),
  DSP16_Q(0.01467313),
  DSP16_Q(-0.02927653),
  DSP16_Q(0.40411060),
  DSP16_Q(0.03443167),
  DSP16_Q(-0.01599535),
  DSP16_Q(0.00996275),
  DSP16_Q(-0.00686132),
  DSP16_Q(0.00492358),
  DSP16_Q(-0.00357964),
  DSP16_Q(0.00259127),
  DSP16_Q(-0.00184180),
  DSP16_Q(0.00126728),
  DSP16_Q(-0.00082927),
  DSP16_Q(0.00050259),
  DSP16_Q(-0.00026928),
  DSP16_Q(0.00011536),
  DSP16_Q(-0.00002912),
  DSP16_Q(0.00000014),
  DSP16_Q(0.00002000),
  DSP16_Q(-0.00009247),
  DSP16_Q(0.00022725),
  DSP16_Q(-0.00043516),
  DSP16_Q(0.00072898),
  DSP16_Q(-0.00112494),
  DSP16_Q(0.00164548),
  DSP16_Q(-0.00232440),
  DSP16_Q(0.00321720),
  DSP16_Q(-0.00442362),
  DSP16_Q(0.00614340),
  DSP16_Q(-0.00884070),
  DSP16_Q(0.01388370),
  DSP16_Q(-0.02775467),
  DSP16_Q(0.40457851),
  DSP16_Q(0.03233958),
  DSP16_Q(-0.01506027),
  DSP16_Q(0.00938690),
  DSP16_Q(-0.00646655),
  DSP16_Q(0.00464081),
  DSP16_Q(-0.00337410),
  DSP16_Q(0.00244236),
  DSP16_Q(-0.00173576),
  DSP16_Q(0.00119409),
  DSP16_Q(-0.00078116),
  DSP16_Q(0.00047324),
  DSP16_Q(-0.00025337),
  DSP16_Q(0.00010839),
  DSP16_Q(-0.00002725),
  DSP16_Q(0.00000012),
  DSP16_Q(0.00001900),
  DSP16_Q(-0.00008739),
  DSP16_Q(0.00021443),
  DSP16_Q(-0.00041029),
  DSP16_Q(0.00068700),
  DSP16_Q(-0.00105984),
  DSP16_Q(0.00154995),
  DSP16_Q(-0.00218918),
  DSP16_Q(0.00302986),
  DSP16_Q(-0.00416605),
  DSP16_Q(0.00578624),
  DSP16_Q(-0.00832877),
  DSP16_Q(0.01308765),
  DSP16_Q(-0.02621395),
  DSP16_Q(0.40501920),
  DSP16_Q(0.03026169),
  DSP16_Q(-0.01412689),
  DSP16_Q(0.00881126),
  DSP16_Q(-0.00607170),
  DSP16_Q(0.00435791),
  DSP16_Q(-0.00316847),
  DSP16_Q(0.00229339),
  DSP16_Q(-0.00162970),
  DSP16_Q(0.00112093),
  DSP16_Q(-0.00073309),
  DSP16_Q(0.00044392),
  DSP16_Q(-0.00023750),
  DSP16_Q(0.00010146),
  DSP16_Q(-0.00002540),
  DSP16_Q(0.00000010),
  DSP16_Q(0.00001797),
  DSP16_Q(-0.00008224),
  DSP16_Q(0.00020148),
  DSP16_Q(-0.00038520),
  DSP16_Q(0.00064469),
  DSP16_Q(-0.00099428),
  DSP16_Q(0.00145379),
  DSP16_Q(-0.00205311),
  DSP16_Q(0.00284136),
  DSP16_Q(-0.00390689),
  DSP16_Q(0.00542678),
  DSP16_Q(-0.00781331),
  DSP16_Q(0.01228511),
  DSP16_Q(-0.02465445),
  DSP16_Q(0.40543261),
  DSP16_Q(0.02819818),
  DSP16_Q(-0.01319537),
  DSP16_Q(0.00823595),
  DSP16_Q(-0.00567684),
  DSP16_Q(0.00407495),
  DSP16_Q(-0.00296279),
  DSP16_Q(0.00214440),
  DSP16_Q(-0.00152365),
  DSP16_Q(0.00104779),
  DSP16_Q(-0.00068507),
  DSP16_Q(0.00041467),
  DSP16_Q(-0.00022169),
  DSP16_Q(0.00009457),
  DSP16_Q(-0.00002358),
  DSP16_Q(0.00000008),
  DSP16_Q(0.00001691),
  DSP16_Q(-0.00007703),
  DSP16_Q(0.00018839),
  DSP16_Q(-0.00035990),
  DSP16_Q(0.00060207),
  DSP16_Q(-0.00092827),
  DSP16_Q(0.00135701),
  DSP16_Q(-0.00191621),
  DSP16_Q(0.00265174),
  DSP16_Q(-0.00364617),
  DSP16_Q(0.00506511),
  DSP16_Q(-0.00729440),
  DSP16_Q(0.01147620),
  DSP16_Q(-0.02307628),
  DSP16_Q(0.40581869),
  DSP16_Q(0.02614923),
  DSP16_Q(-0.01226588),
  DSP16_Q(0.00766108),
  DSP16_Q(-0.00528207),
  DSP16_Q(0.00379199),
  DSP16_Q(-0.00275709),
  DSP16_Q(0.00199541),
  DSP16_Q(-0.00141763),
  DSP16_Q(0.00097471),
  DSP16_Q(-0.00063711),
  DSP16_Q(0.00038547),
  DSP16_Q(-0.00020593),
  DSP16_Q(0.00008772),
  DSP16_Q(-0.00002178),
  DSP16_Q(0.00000006),
  DSP16_Q(0.00001583),
  DSP16_Q(-0.00007174),
  DSP16_Q(0.00017518),
  DSP16_Q(-0.00033440),
  DSP16_Q(0.00055915),
  DSP16_Q(-0.00086184),
  DSP16_Q(0.00125965),
  DSP16_Q(-0.00177850),
  DSP16_Q(0.00246102),
  DSP16_Q(-0.00338395),
  DSP16_Q(0.00470129),
  DSP16_Q(-0.00677214),
  DSP16_Q(0.01066106),
  DSP16_Q(-0.02147954),
  DSP16_Q(0.40617740),
  DSP16_Q(0.02411502),
  DSP16_Q(-0.01133858),
  DSP16_Q(0.00708677),
  DSP16_Q(-0.00488745),
  DSP16_Q(0.00350907),
  DSP16_Q(-0.00255142),
  DSP16_Q(0.00184646),
  DSP16_Q(-0.00131166),
  DSP16_Q(0.00090168),
  DSP16_Q(-0.00058921),
  DSP16_Q(0.00035633),
  DSP16_Q(-0.00019022),
  DSP16_Q(0.00008091),
  DSP16_Q(-0.00002001),
  DSP16_Q(0.00000005),
  DSP16_Q(0.00001472),
  DSP16_Q(-0.00006638),
  DSP16_Q(0.00016183),
  DSP16_Q(-0.00030869),
  DSP16_Q(0.00051592),
  DSP16_Q(-0.00079498),
  DSP16_Q(0.00116170),
  DSP16_Q(-0.00164001),
  DSP16_Q(0.00226925),
  DSP16_Q(-0.00312027),
  DSP16_Q(0.00433538),
  DSP16_Q(-0.00624662),
  DSP16_Q(0.00983980),
  DSP16_Q(-0.01986434),
  DSP16_Q(0.40650868),
  DSP16_Q(0.02209573),
  DSP16_Q(-0.01041366),
  DSP16_Q(0.00651313),
  DSP16_Q(-0.00449307),
  DSP16_Q(0.00322626),
  DSP16_Q(-0.00234583),
  DSP16_Q(0.00169758),
  DSP16_Q(-0.00120576),
  DSP16_Q(0.00082873),
  DSP16_Q(-0.00054139),
  DSP16_Q(0.00032727),
  DSP16_Q(-0.00017458),
  DSP16_Q(0.00007415),
  DSP16_Q(-0.00001826),
  DSP16_Q(0.00000004),
  DSP16_Q(0.00001358),
  DSP16_Q(-0.00006095),
  DSP16_Q(0.00014837),
  DSP16_Q(-0.00028278),
  DSP16_Q(0.00047240),
  DSP16_Q(-0.00072771),
  DSP16_Q(0.00106320),
  DSP16_Q(-0.00150077),
  DSP16_Q(0.00207646),
  DSP16_Q(-0.00285520),
  DSP16_Q(0.00396746),
  DSP16_Q(-0.00571794),
  DSP16_Q(0.00901257),
  DSP16_Q(-0.01823078),
  DSP16_Q(0.40681249),
  DSP16_Q(0.02009153),
  DSP16_Q(-0.00949127),
  DSP16_Q(0.00594026),
  DSP16_Q(-0.00409901),
  DSP16_Q(0.00294362),
  DSP16_Q(-0.00214034),
  DSP16_Q(0.00154880),
  DSP16_Q(-0.00109996),
  DSP16_Q(0.00075587),
  DSP16_Q(-0.00049365),
  DSP16_Q(0.00029828),
  DSP16_Q(-0.00015900),
  DSP16_Q(0.00006743),
  DSP16_Q(-0.00001654),
  DSP16_Q(0.00000003),
  DSP16_Q(0.00001242),
  DSP16_Q(-0.00005546),
  DSP16_Q(0.00013478),
  DSP16_Q(-0.00025668),
  DSP16_Q(0.00042861),
  DSP16_Q(-0.00066005),
  DSP16_Q(0.00096416),
  DSP16_Q(-0.00136080),
  DSP16_Q(0.00188269),
  DSP16_Q(-0.00258877),
  DSP16_Q(0.00359758),
  DSP16_Q(-0.00518618),
  DSP16_Q(0.00817949),
  DSP16_Q(-0.01657898),
  DSP16_Q(0.40708881),
  DSP16_Q(0.01810259),
  DSP16_Q(-0.00857157),
  DSP16_Q(0.00536829),
  DSP16_Q(-0.00370534),
  DSP16_Q(0.00266120),
  DSP16_Q(-0.00193502),
  DSP16_Q(0.00140014),
  DSP16_Q(-0.00099427),
  DSP16_Q(0.00068311),
  DSP16_Q(-0.00044601),
  DSP16_Q(0.00026938),
  DSP16_Q(-0.00014349),
  DSP16_Q(0.00006077),
  DSP16_Q(-0.00001484),
  DSP16_Q(0.00000002),
  DSP16_Q(0.00001123),
  DSP16_Q(-0.00004989),
  DSP16_Q(0.00012107),
  DSP16_Q(-0.00023039),
  DSP16_Q(0.00038453),
  DSP16_Q(-0.00059200),
  DSP16_Q(0.00086459),
  DSP16_Q(-0.00122013),
  DSP16_Q(0.00168797),
  DSP16_Q(-0.00232104),
  DSP16_Q(0.00322583),
  DSP16_Q(-0.00465146),
  DSP16_Q(0.00734070),
  DSP16_Q(-0.01490905),
  DSP16_Q(0.40733759),
  DSP16_Q(0.01612908),
  DSP16_Q(-0.00765475),
  DSP16_Q(0.00479733),
  DSP16_Q(-0.00331215),
  DSP16_Q(0.00237906),
  DSP16_Q(-0.00172988),
  DSP16_Q(0.00125165),
  DSP16_Q(-0.00088871),
  DSP16_Q(0.00061048),
  DSP16_Q(-0.00039847),
  DSP16_Q(0.00024056),
  DSP16_Q(-0.00012804),
  DSP16_Q(0.00005415),
  DSP16_Q(-0.00001317),
  DSP16_Q(0.00000002),
  DSP16_Q(0.00001001),
  DSP16_Q(-0.00004426),
  DSP16_Q(0.00010724),
  DSP16_Q(-0.00020392),
  DSP16_Q(0.00034019),
  DSP16_Q(-0.00052359),
  DSP16_Q(0.00076453),
  DSP16_Q(-0.00107879),
  DSP16_Q(0.00149234),
  DSP16_Q(-0.00205205),
  DSP16_Q(0.00285227),
  DSP16_Q(-0.00411385),
  DSP16_Q(0.00649633),
  DSP16_Q(-0.01322111),
  DSP16_Q(0.40755880),
  DSP16_Q(0.01417116),
  DSP16_Q(-0.00674095),
  DSP16_Q(0.00422747),
  DSP16_Q(-0.00291950),
  DSP16_Q(0.00209725),
  DSP16_Q(-0.00152499),
  DSP16_Q(0.00110334),
  DSP16_Q(-0.00078332),
  DSP16_Q(0.00053798),
  DSP16_Q(-0.00035105),
  DSP16_Q(0.00021184),
  DSP16_Q(-0.00011267),
  DSP16_Q(0.00004758),
  DSP16_Q(-0.00001152),
  DSP16_Q(0.00000001),
  DSP16_Q(0.00000876),
  DSP16_Q(-0.00003857),
  DSP16_Q(0.00009329),
  DSP16_Q(-0.00017726),
  DSP16_Q(0.00029559),
  DSP16_Q(-0.00045481),
  DSP16_Q(0.00066398),
  DSP16_Q(-0.00093680),
  DSP16_Q(0.00129584),
  DSP16_Q(-0.00178187),
  DSP16_Q(0.00247696),
  DSP16_Q(-0.00357346),
  DSP16_Q(0.00564653),
  DSP16_Q(-0.01151528),
  DSP16_Q(0.40775243),
  DSP16_Q(0.01222900),
  DSP16_Q(-0.00583033),
  DSP16_Q(0.00365884),
  DSP16_Q(-0.00252749),
  DSP16_Q(0.00181583),
  DSP16_Q(-0.00132038),
  DSP16_Q(0.00095525),
  DSP16_Q(-0.00067810),
  DSP16_Q(0.00046563),
  DSP16_Q(-0.00030375),
  DSP16_Q(0.00018322),
  DSP16_Q(-0.00009738),
  DSP16_Q(0.00004106),
  DSP16_Q(-0.00000990),
  DSP16_Q(0.00000001),
  DSP16_Q(0.00000749),
  DSP16_Q(-0.00003281),
  DSP16_Q(0.00007924),
  DSP16_Q(-0.00015044),
  DSP16_Q(0.00025075),
  DSP16_Q(-0.00038570),
  DSP16_Q(0.00056297),
  DSP16_Q(-0.00079419),
  DSP16_Q(0.00109851),
  DSP16_Q(-0.00151053),
  DSP16_Q(0.00209999),
  DSP16_Q(-0.00303039),
  DSP16_Q(0.00479142),
  DSP16_Q(-0.00979168),
  DSP16_Q(0.40791844),
  DSP16_Q(0.01030277),
  DSP16_Q(-0.00492307),
  DSP16_Q(0.00309154),
  DSP16_Q(-0.00213618),
  DSP16_Q(0.00153486),
  DSP16_Q(-0.00111608),
  DSP16_Q(0.00080740),
  DSP16_Q(-0.00057308),
  DSP16_Q(0.00039344),
  DSP16_Q(-0.00025659),
  DSP16_Q(0.00015470),
  DSP16_Q(-0.00008216),
  DSP16_Q(0.00003459),
  DSP16_Q(-0.00000830),
  DSP16_Q(0.00000000),
  DSP16_Q(0.00000619),
  DSP16_Q(-0.00002698),
  DSP16_Q(0.00006507),
  DSP16_Q(-0.00012344),
  DSP16_Q(0.00020566),
  DSP16_Q(-0.00031625),
  DSP16_Q(0.00046152),
  DSP16_Q(-0.00065099),
  DSP16_Q(0.00090038),
  DSP16_Q(-0.00123810),
  DSP16_Q(0.00172142),
  DSP16_Q(-0.00248473),
  DSP16_Q(0.00393116),
  DSP16_Q(-0.00805043),
  DSP16_Q(0.40805681),
  DSP16_Q(0.00839261),
  DSP16_Q(-0.00401931),
  DSP16_Q(0.00252568),
  DSP16_Q(-0.00174565),
  DSP16_Q(0.00125439),
  DSP16_Q(-0.00091215),
  DSP16_Q(0.00065983),
  DSP16_Q(-0.00046829),
  DSP16_Q(0.00032144),
  DSP16_Q(-0.00020957),
  DSP16_Q(0.00012630),
  DSP16_Q(-0.00006703),
  DSP16_Q(0.00002818),
  DSP16_Q(-0.00000673),
  DSP16_Q(0.00000000),
  DSP16_Q(0.00000486),
  DSP16_Q(-0.00002109),
  DSP16_Q(0.00005079),
  DSP16_Q(-0.00009628),
  DSP16_Q(0.00016034),
  DSP16_Q(-0.00024648),
  DSP16_Q(0.00035964),
  DSP16_Q(-0.00050722),
  DSP16_Q(0.00070150),
  DSP16_Q(-0.00096463),
  DSP16_Q(0.00134132),
  DSP16_Q(-0.00193659),
  DSP16_Q(0.00306588),
  DSP16_Q(-0.00629167),
  DSP16_Q(0.40816752),
  DSP16_Q(0.00649869),
  DSP16_Q(-0.00311922),
  DSP16_Q(0.00196137),
  DSP16_Q(-0.00135598),
  DSP16_Q(0.00097449),
  DSP16_Q(-0.00070862),
  DSP16_Q(0.00051257),
  DSP16_Q(-0.00036373),
  DSP16_Q(0.00024962),
  DSP16_Q(-0.00016270),
  DSP16_Q(0.00009801),
  DSP16_Q(-0.00005197),
  DSP16_Q(0.00002182),
  DSP16_Q(-0.00000519),
  DSP16_Q(0.00000000),
  DSP16_Q(0.00000351),
  DSP16_Q(-0.00001514),
  DSP16_Q(0.00003641),
  DSP16_Q(-0.00006896),
  DSP16_Q(0.00011479),
  DSP16_Q(-0.00017642),
  DSP16_Q(0.00025735),
  DSP16_Q(-0.00036292),
  DSP16_Q(0.00050189),
  DSP16_Q(-0.00069016),
  DSP16_Q(0.00095977),
  DSP16_Q(-0.00138606),
  DSP16_Q(0.00219573),
  DSP16_Q(-0.00451553),
  DSP16_Q(0.40825057),
  DSP16_Q(0.00462116),
  DSP16_Q(-0.00222295),
  DSP16_Q(0.00139871),
  DSP16_Q(-0.00096725),
  DSP16_Q(0.00069519),
  DSP16_Q(-0.00050553),
  DSP16_Q(0.00036565),
  DSP16_Q(-0.00025944),
  DSP16_Q(0.00017802),
  DSP16_Q(-0.00011600),
  DSP16_Q(0.00006984),
  DSP16_Q(-0.00003701),
  DSP16_Q(0.00001551),
  DSP16_Q(-0.00000367),
  DSP16_Q(0.00000000),
  DSP16_Q(0.00000212),
  DSP16_Q(-0.00000913),
  DSP16_Q(0.00002192),
  DSP16_Q(-0.00004149),
  DSP16_Q(0.00006903),
  DSP16_Q(-0.00010606),
  DSP16_Q(0.00015468),
  DSP16_Q(-0.00021811),
  DSP16_Q(0.00030161),
  DSP16_Q(-0.00041475),
  DSP16_Q(0.00057683),
  DSP16_Q(-0.00083326),
  DSP16_Q(0.00132085),
  DSP16_Q(-0.00272213),
  DSP16_Q(0.40830595),
  DSP16_Q(0.00276016),
  DSP16_Q(-0.00133065),
  DSP16_Q(0.00083781),
  DSP16_Q(-0.00057953),
  DSP16_Q(0.00041656),
  DSP16_Q(-0.00030292),
  DSP16_Q(0.00021909),
  DSP16_Q(-0.00015544),
  DSP16_Q(0.00010663),
  DSP16_Q(-0.00006946),
  DSP16_Q(0.00004181),
  DSP16_Q(-0.00002214),
  DSP16_Q(0.00000926),
  DSP16_Q(-0.00000218),
  DSP16_Q(0.00000000),
  DSP16_Q(0.00000071),
  DSP16_Q(-0.00000306),
  DSP16_Q(0.00000733),
  DSP16_Q(-0.00001387),
  DSP16_Q(0.00002306),
  DSP16_Q(-0.00003542),
  DSP16_Q(0.00005165),
  DSP16_Q(-0.00007282),
  DSP16_Q(0.00010069),
  DSP16_Q(-0.00013846),
  DSP16_Q(0.00019259),
  DSP16_Q(-0.00027828),
  DSP16_Q(0.00044140),
  DSP16_Q(-0.00091163),
  DSP16_Q(0.40833363),
  DSP16_Q(0.00091585),
  DSP16_Q(-0.00044249),
  DSP16_Q(0.00027878),
  DSP16_Q(-0.00019289),
  DSP16_Q(0.00013866),
  DSP16_Q(-0.00010083),
  DSP16_Q(0.00007293),
  DSP16_Q(-0.00005173),
  DSP16_Q(0.00003548),
  DSP16_Q(-0.00002311),
  DSP16_Q(0.00001390),
  DSP16_Q(-0.00000736),
  DSP16_Q(0.00000307),
  DSP16_Q(-0.00000072),
  DSP16_Q(0.00000000)
};
